//
// Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
// Copyright (c) 2013, Red Hat Inc.
// All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//

// Machine Generated File.  Do Not Edit!

#include "precompiled.hpp"
#include "adfiles/adGlobals_aarch64.hpp"
#include "adfiles/ad_aarch64.hpp"
#include "memory/allocation.inline.hpp"
#include "asm/macroAssembler.inline.hpp"
#include "code/compiledIC.hpp"
#include "code/vmreg.hpp"
#include "gc_interface/collectedHeap.inline.hpp"
#include "oops/compiledICHolder.hpp"
#include "oops/markOop.hpp"
#include "oops/method.hpp"
#include "oops/oop.inline.hpp"
#include "oops/oop.inline2.hpp"
#include "opto/cfgnode.hpp"
#include "opto/locknode.hpp"
#include "opto/opcodes.hpp"
#include "opto/regalloc.hpp"
#include "opto/regmask.hpp"
#include "opto/runtime.hpp"
#include "runtime/biasedLocking.hpp"
#include "runtime/sharedRuntime.hpp"
#include "runtime/stubRoutines.hpp"
#include "utilities/growableArray.hpp"
#include "assembler_aarch64.inline.hpp"
#include "nativeInst_aarch64.hpp"
#include "vmreg_aarch64.inline.hpp"

//SourceForm

#line 998 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"


  // Optimizaton of volatile gets and puts
  // -------------------------------------
  //
  // AArch64 has ldar<x> and stlr<x> instructions which we can safely
  // use to implement volatile reads and writes. For a volatile read
  // we simply need
  //
  //   ldar<x>
  //
  // and for a volatile write we need
  //
  //   stlr<x>
  // 
  // Alternatively, we can implement them by pairing a normal
  // load/store with a memory barrier. For a volatile read we need
  // 
  //   ldr<x>
  //   dmb ishld
  //
  // for a volatile write
  //
  //   dmb ish
  //   str<x>
  //   dmb ish
  //
  // We can also use ldaxr and stlxr to implement compare and swap CAS
  // sequences. These are normally translated to an instruction
  // sequence like the following
  //
  //   dmb      ish
  // retry:
  //   ldxr<x>   rval raddr
  //   cmp       rval rold
  //   b.ne done
  //   stlxr<x>  rval, rnew, rold
  //   cbnz      rval retry
  // done:
  //   cset      r0, eq
  //   dmb ishld
  //
  // Note that the exclusive store is already using an stlxr
  // instruction. That is required to ensure visibility to other
  // threads of the exclusive write (assuming it succeeds) before that
  // of any subsequent writes.
  //
  // The following instruction sequence is an improvement on the above
  //
  // retry:
  //   ldaxr<x>  rval raddr
  //   cmp       rval rold
  //   b.ne done
  //   stlxr<x>  rval, rnew, rold
  //   cbnz      rval retry
  // done:
  //   cset      r0, eq
  //
  // We don't need the leading dmb ish since the stlxr guarantees
  // visibility of prior writes in the case that the swap is
  // successful. Crucially we don't have to worry about the case where
  // the swap is not successful since no valid program should be
  // relying on visibility of prior changes by the attempting thread
  // in the case where the CAS fails.
  //
  // Similarly, we don't need the trailing dmb ishld if we substitute
  // an ldaxr instruction since that will provide all the guarantees we
  // require regarding observation of changes made by other threads
  // before any change to the CAS address observed by the load.
  //
  // In order to generate the desired instruction sequence we need to
  // be able to identify specific 'signature' ideal graph node
  // sequences which i) occur as a translation of a volatile reads or
  // writes or CAS operations and ii) do not occur through any other
  // translation or graph transformation. We can then provide
  // alternative aldc matching rules which translate these node
  // sequences to the desired machine code sequences. Selection of the
  // alternative rules can be implemented by predicates which identify
  // the relevant node sequences.
  //
  // The ideal graph generator translates a volatile read to the node
  // sequence
  //
  //   LoadX[mo_acquire]
  //   MemBarAcquire
  //
  // As a special case when using the compressed oops optimization we
  // may also see this variant
  //
  //   LoadN[mo_acquire]
  //   DecodeN
  //   MemBarAcquire
  //
  // A volatile write is translated to the node sequence
  //
  //   MemBarRelease
  //   StoreX[mo_release] {CardMark}-optional
  //   MemBarVolatile
  //
  // n.b. the above node patterns are generated with a strict
  // 'signature' configuration of input and output dependencies (see
  // the predicates below for exact details). The card mark may be as
  // simple as a few extra nodes or, in a few GC configurations, may
  // include more complex control flow between the leading and
  // trailing memory barriers. However, whatever the card mark
  // configuration these signatures are unique to translated volatile
  // reads/stores -- they will not appear as a result of any other
  // bytecode translation or inlining nor as a consequence of
  // optimizing transforms.
  //
  // We also want to catch inlined unsafe volatile gets and puts and
  // be able to implement them using either ldar<x>/stlr<x> or some
  // combination of ldr<x>/stlr<x> and dmb instructions.
  //
  // Inlined unsafe volatiles puts manifest as a minor variant of the
  // normal volatile put node sequence containing an extra cpuorder
  // membar
  //
  //   MemBarRelease
  //   MemBarCPUOrder
  //   StoreX[mo_release] {CardMark}-optional
  //   MemBarVolatile
  //
  // n.b. as an aside, the cpuorder membar is not itself subject to
  // matching and translation by adlc rules.  However, the rule
  // predicates need to detect its presence in order to correctly
  // select the desired adlc rules.
  //
  // Inlined unsafe volatile gets manifest as a somewhat different
  // node sequence to a normal volatile get
  //
  //   MemBarCPUOrder
  //        ||       \\
  //   MemBarAcquire LoadX[mo_acquire]
  //        ||
  //   MemBarCPUOrder
  //
  // In this case the acquire membar does not directly depend on the
  // load. However, we can be sure that the load is generated from an
  // inlined unsafe volatile get if we see it dependent on this unique
  // sequence of membar nodes. Similarly, given an acquire membar we
  // can know that it was added because of an inlined unsafe volatile
  // get if it is fed and feeds a cpuorder membar and if its feed
  // membar also feeds an acquiring load.
  //
  // Finally an inlined (Unsafe) CAS operation is translated to the
  // following ideal graph
  //
  //   MemBarRelease
  //   MemBarCPUOrder
  //   CompareAndSwapX {CardMark}-optional
  //   MemBarCPUOrder
  //   MemBarAcquire
  //
  // So, where we can identify these volatile read and write
  // signatures we can choose to plant either of the above two code
  // sequences. For a volatile read we can simply plant a normal
  // ldr<x> and translate the MemBarAcquire to a dmb. However, we can
  // also choose to inhibit translation of the MemBarAcquire and
  // inhibit planting of the ldr<x>, instead planting an ldar<x>.
  //
  // When we recognise a volatile store signature we can choose to
  // plant at a dmb ish as a translation for the MemBarRelease, a
  // normal str<x> and then a dmb ish for the MemBarVolatile.
  // Alternatively, we can inhibit translation of the MemBarRelease
  // and MemBarVolatile and instead plant a simple stlr<x>
  // instruction.
  //
  // when we recognise a CAS signature we can choose to plant a dmb
  // ish as a translation for the MemBarRelease, the conventional
  // macro-instruction sequence for the CompareAndSwap node (which
  // uses ldxr<x>) and then a dmb ishld for the MemBarAcquire.
  // Alternatively, we can elide generation of the dmb instructions
  // and plant the alternative CompareAndSwap macro-instruction
  // sequence (which uses ldaxr<x>).
  // 
  // Of course, the above only applies when we see these signature
  // configurations. We still want to plant dmb instructions in any
  // other cases where we may see a MemBarAcquire, MemBarRelease or
  // MemBarVolatile. For example, at the end of a constructor which
  // writes final/volatile fields we will see a MemBarRelease
  // instruction and this needs a 'dmb ish' lest we risk the
  // constructed object being visible without making the
  // final/volatile field writes visible.
  //
  // n.b. the translation rules below which rely on detection of the
  // volatile signatures and insert ldar<x> or stlr<x> are failsafe.
  // If we see anything other than the signature configurations we
  // always just translate the loads and stores to ldr<x> and str<x>
  // and translate acquire, release and volatile membars to the
  // relevant dmb instructions.
  //

  // graph traversal helpers used for volatile put/get and CAS
  // optimization

  // 1) general purpose helpers

  // if node n is linked to a parent MemBarNode by an intervening
  // Control and Memory ProjNode return the MemBarNode otherwise return
  // NULL.
  //
  // n may only be a Load or a MemBar.

  MemBarNode *parent_membar(const Node *n)
  {
    Node *ctl = NULL;
    Node *mem = NULL;
    Node *membar = NULL;

    if (n->is_Load()) {
      ctl = n->lookup(LoadNode::Control);
      mem = n->lookup(LoadNode::Memory);
    } else if (n->is_MemBar()) {
      ctl = n->lookup(TypeFunc::Control);
      mem = n->lookup(TypeFunc::Memory);
    } else {
	return NULL;
    }

    if (!ctl || !mem || !ctl->is_Proj() || !mem->is_Proj()) {
      return NULL;
    }

    membar = ctl->lookup(0);

    if (!membar || !membar->is_MemBar()) {
      return NULL;
    }

    if (mem->lookup(0) != membar) {
      return NULL;
    }

    return membar->as_MemBar();
  }

  // if n is linked to a child MemBarNode by intervening Control and
  // Memory ProjNodes return the MemBarNode otherwise return NULL.

  MemBarNode *child_membar(const MemBarNode *n)
  {
    ProjNode *ctl = n->proj_out(TypeFunc::Control);
    ProjNode *mem = n->proj_out(TypeFunc::Memory);

    // MemBar needs to have both a Ctl and Mem projection
    if (! ctl || ! mem)
      return NULL;

    MemBarNode *child = NULL;
    Node *x;

    for (DUIterator_Fast imax, i = ctl->fast_outs(imax); i < imax; i++) {
      x = ctl->fast_out(i);
      // if we see a membar we keep hold of it. we may also see a new
      // arena copy of the original but it will appear later
      if (x->is_MemBar()) {
	  child = x->as_MemBar();
	  break;
      }
    }

    if (child == NULL) {
      return NULL;
    }

    for (DUIterator_Fast imax, i = mem->fast_outs(imax); i < imax; i++) {
      x = mem->fast_out(i);
      // if we see a membar we keep hold of it. we may also see a new
      // arena copy of the original but it will appear later
      if (x == child) {
	return child;
      }
    }
    return NULL;
  }

  // helper predicate use to filter candidates for a leading memory
  // barrier
  //
  // returns true if barrier is a MemBarRelease or a MemBarCPUOrder
  // whose Ctl and Mem feeds come from a MemBarRelease otherwise false

  bool leading_membar(const MemBarNode *barrier)
  {
    int opcode = barrier->Opcode();
    // if this is a release membar we are ok
    if (opcode == Op_MemBarRelease) {
      return true;
    }
    // if its a cpuorder membar . . .
    if (opcode != Op_MemBarCPUOrder) {
      return false;
    }
    // then the parent has to be a release membar
    MemBarNode *parent = parent_membar(barrier);
    if (!parent) {
      return false;
    }
    opcode = parent->Opcode();
    return opcode == Op_MemBarRelease;
  }
 
  // 2) card mark detection helper

  // helper predicate which can be used to detect a volatile membar
  // introduced as part of a conditional card mark sequence either by
  // G1 or by CMS when UseCondCardMark is true.
  //
  // membar can be definitively determined to be part of a card mark
  // sequence if and only if all the following hold
  //
  // i) it is a MemBarVolatile
  //
  // ii) either UseG1GC or (UseConcMarkSweepGC && UseCondCardMark) is
  // true
  //
  // iii) the node's Mem projection feeds a StoreCM node.
  
  bool is_card_mark_membar(const MemBarNode *barrier)
  {
    if (!UseG1GC && !(UseConcMarkSweepGC && UseCondCardMark)) {
      return false;
    }

    if (barrier->Opcode() != Op_MemBarVolatile) {
      return false;
    }

    ProjNode *mem = barrier->proj_out(TypeFunc::Memory);

    for (DUIterator_Fast imax, i = mem->fast_outs(imax); i < imax ; i++) {
      Node *y = mem->fast_out(i);
      if (y->Opcode() == Op_StoreCM) {
	return true;
      }
    }
  
    return false;
  }


  // 3) helper predicates to traverse volatile put or CAS graphs which
  // may contain GC barrier subgraphs

  // Preamble
  // --------
  //
  // for volatile writes we can omit generating barriers and employ a
  // releasing store when we see a node sequence sequence with a
  // leading MemBarRelease and a trailing MemBarVolatile as follows
  //
  //   MemBarRelease
  //  {      ||      } -- optional
  //  {MemBarCPUOrder}
  //         ||     \\
  //         ||     StoreX[mo_release]
  //         | \     /
  //         | MergeMem
  //         | /
  //   MemBarVolatile
  //
  // where
  //  || and \\ represent Ctl and Mem feeds via Proj nodes
  //  | \ and / indicate further routing of the Ctl and Mem feeds
  // 
  // this is the graph we see for non-object stores. however, for a
  // volatile Object store (StoreN/P) we may see other nodes below the
  // leading membar because of the need for a GC pre- or post-write
  // barrier.
  //
  // with most GC configurations we with see this simple variant which
  // includes a post-write barrier card mark.
  //
  //   MemBarRelease______________________________
  //         ||    \\               Ctl \        \\
  //         ||    StoreN/P[mo_release] CastP2X  StoreB/CM
  //         | \     /                       . . .  /
  //         | MergeMem
  //         | /
  //         ||      /
  //   MemBarVolatile
  //
  // i.e. the leading membar feeds Ctl to a CastP2X (which converts
  // the object address to an int used to compute the card offset) and
  // Ctl+Mem to a StoreB node (which does the actual card mark).
  //
  // n.b. a StoreCM node will only appear in this configuration when
  // using CMS. StoreCM differs from a normal card mark write (StoreB)
  // because it implies a requirement to order visibility of the card
  // mark (StoreCM) relative to the object put (StoreP/N) using a
  // StoreStore memory barrier (arguably this ought to be represented
  // explicitly in the ideal graph but that is not how it works). This
  // ordering is required for both non-volatile and volatile
  // puts. Normally that means we need to translate a StoreCM using
  // the sequence
  //
  //   dmb ishst
  //   stlrb
  //
  // However, in the case of a volatile put if we can recognise this
  // configuration and plant an stlr for the object write then we can
  // omit the dmb and just plant an strb since visibility of the stlr
  // is ordered before visibility of subsequent stores. StoreCM nodes
  // also arise when using G1 or using CMS with conditional card
  // marking. In these cases (as we shall see) we don't need to insert
  // the dmb when translating StoreCM because there is already an
  // intervening StoreLoad barrier between it and the StoreP/N.
  //
  // It is also possible to perform the card mark conditionally on it
  // currently being unmarked in which case the volatile put graph
  // will look slightly different
  //
  //   MemBarRelease____________________________________________
  //         ||    \\               Ctl \     Ctl \     \\  Mem \
  //         ||    StoreN/P[mo_release] CastP2X   If   LoadB     |
  //         | \     /                              \            |
  //         | MergeMem                            . . .      StoreB
  //         | /                                                /
  //         ||     /
  //   MemBarVolatile
  //
  // It is worth noting at this stage that both the above
  // configurations can be uniquely identified by checking that the
  // memory flow includes the following subgraph:
  //
  //   MemBarRelease
  //  {MemBarCPUOrder}
  //          |  \      . . .
  //          |  StoreX[mo_release]  . . .
  //          |   /
  //         MergeMem
  //          |
  //   MemBarVolatile
  //
  // This is referred to as a *normal* subgraph. It can easily be
  // detected starting from any candidate MemBarRelease,
  // StoreX[mo_release] or MemBarVolatile.
  //
  // A simple variation on this normal case occurs for an unsafe CAS
  // operation. The basic graph for a non-object CAS is
  //
  //   MemBarRelease
  //         ||
  //   MemBarCPUOrder
  //         ||     \\   . . .
  //         ||     CompareAndSwapX
  //         ||       |
  //         ||     SCMemProj
  //         | \     /
  //         | MergeMem
  //         | /
  //   MemBarCPUOrder
  //         ||
  //   MemBarAcquire
  //
  // The same basic variations on this arrangement (mutatis mutandis)
  // occur when a card mark is introduced. i.e. we se the same basic
  // shape but the StoreP/N is replaced with CompareAndSawpP/N and the
  // tail of the graph is a pair comprising a MemBarCPUOrder +
  // MemBarAcquire.
  //
  // So, in the case of a CAS the normal graph has the variant form
  //
  //   MemBarRelease
  //   MemBarCPUOrder
  //          |   \      . . .
  //          |  CompareAndSwapX  . . .
  //          |    |
  //          |   SCMemProj
  //          |   /  . . .
  //         MergeMem
  //          |
  //   MemBarCPUOrder
  //   MemBarAcquire
  //
  // This graph can also easily be detected starting from any
  // candidate MemBarRelease, CompareAndSwapX or MemBarAcquire.
  //
  // the code below uses two helper predicates, leading_to_normal and
  // normal_to_leading to identify these normal graphs, one validating
  // the layout starting from the top membar and searching down and
  // the other validating the layout starting from the lower membar
  // and searching up.
  //
  // There are two special case GC configurations when a normal graph
  // may not be generated: when using G1 (which always employs a
  // conditional card mark); and when using CMS with conditional card
  // marking configured. These GCs are both concurrent rather than
  // stop-the world GCs. So they introduce extra Ctl+Mem flow into the
  // graph between the leading and trailing membar nodes, in
  // particular enforcing stronger memory serialisation beween the
  // object put and the corresponding conditional card mark. CMS
  // employs a post-write GC barrier while G1 employs both a pre- and
  // post-write GC barrier. Of course the extra nodes may be absent --
  // they are only inserted for object puts. This significantly
  // complicates the task of identifying whether a MemBarRelease,
  // StoreX[mo_release] or MemBarVolatile forms part of a volatile put
  // when using these GC configurations (see below). It adds similar
  // complexity to the task of identifying whether a MemBarRelease,
  // CompareAndSwapX or MemBarAcquire forms part of a CAS.
  //
  // In both cases the post-write subtree includes an auxiliary
  // MemBarVolatile (StoreLoad barrier) separating the object put and
  // the read of the corresponding card. This poses two additional
  // problems.
  //
  // Firstly, a card mark MemBarVolatile needs to be distinguished
  // from a normal trailing MemBarVolatile. Resolving this first
  // problem is straightforward: a card mark MemBarVolatile always
  // projects a Mem feed to a StoreCM node and that is a unique marker
  //
  //      MemBarVolatile (card mark)
  //       C |    \     . . .
  //         |   StoreCM   . . .
  //       . . .
  //
  // The second problem is how the code generator is to translate the
  // card mark barrier? It always needs to be translated to a "dmb
  // ish" instruction whether or not it occurs as part of a volatile
  // put. A StoreLoad barrier is needed after the object put to ensure
  // i) visibility to GC threads of the object put and ii) visibility
  // to the mutator thread of any card clearing write by a GC
  // thread. Clearly a normal store (str) will not guarantee this
  // ordering but neither will a releasing store (stlr). The latter
  // guarantees that the object put is visible but does not guarantee
  // that writes by other threads have also been observed.
  // 
  // So, returning to the task of translating the object put and the
  // leading/trailing membar nodes: what do the non-normal node graph
  // look like for these 2 special cases? and how can we determine the
  // status of a MemBarRelease, StoreX[mo_release] or MemBarVolatile
  // in both normal and non-normal cases?
  //
  // A CMS GC post-barrier wraps its card write (StoreCM) inside an If
  // which selects conditonal execution based on the value loaded
  // (LoadB) from the card. Ctl and Mem are fed to the If via an
  // intervening StoreLoad barrier (MemBarVolatile).
  //
  // So, with CMS we may see a node graph for a volatile object store
  // which looks like this
  //
  //   MemBarRelease
  //   MemBarCPUOrder_(leading)__________________
  //     C |    M \       \\                   C \
  //       |       \    StoreN/P[mo_release]  CastP2X
  //       |    Bot \    /
  //       |       MergeMem
  //       |         /
  //      MemBarVolatile (card mark)
  //     C |  ||    M |
  //       | LoadB    |
  //       |   |      |
  //       | Cmp      |\
  //       | /        | \
  //       If         |  \
  //       | \        |   \
  // IfFalse  IfTrue  |    \
  //       \     / \  |     \
  //        \   / StoreCM    |
  //         \ /      |      |
  //        Region   . . .   |
  //          | \           /
  //          |  . . .  \  / Bot
  //          |       MergeMem
  //          |          |
  //        MemBarVolatile (trailing)
  //
  // The first MergeMem merges the AliasIdxBot Mem slice from the
  // leading membar and the oopptr Mem slice from the Store into the
  // card mark membar. The trailing MergeMem merges the AliasIdxBot
  // Mem slice from the card mark membar and the AliasIdxRaw slice
  // from the StoreCM into the trailing membar (n.b. the latter
  // proceeds via a Phi associated with the If region).
  //
  // The graph for a CAS varies slightly, the obvious difference being
  // that the StoreN/P node is replaced by a CompareAndSwapP/N node
  // and the trailing MemBarVolatile by a MemBarCPUOrder +
  // MemBarAcquire pair. The other important difference is that the
  // CompareAndSwap node's SCMemProj is not merged into the card mark
  // membar - it still feeds the trailing MergeMem. This also means
  // that the card mark membar receives its Mem feed directly from the
  // leading membar rather than via a MergeMem.
  //
  //   MemBarRelease
  //   MemBarCPUOrder__(leading)_________________________
  //       ||                       \\                 C \
  //   MemBarVolatile (card mark)  CompareAndSwapN/P  CastP2X
  //     C |  ||    M |              |
  //       | LoadB    |       ______/|
  //       |   |      |      /       |
  //       | Cmp      |     /      SCMemProj
  //       | /        |    /         |
  //       If         |   /         /
  //       | \        |  /         /
  // IfFalse  IfTrue  | /         /
  //       \     / \  |/ prec    /
  //        \   / StoreCM       /
  //         \ /      |        /
  //        Region   . . .    /
  //          | \            /
  //          |  . . .  \   / Bot
  //          |       MergeMem
  //          |          |
  //        MemBarCPUOrder
  //        MemBarAcquire (trailing)
  //
  // This has a slightly different memory subgraph to the one seen
  // previously but the core of it is the same as for the CAS normal
  // sungraph
  //
  //   MemBarRelease
  //   MemBarCPUOrder____
  //      ||             \      . . .
  //   MemBarVolatile  CompareAndSwapX  . . .
  //      |  \            |
  //        . . .   SCMemProj
  //          |     /  . . .
  //         MergeMem
  //          |
  //   MemBarCPUOrder
  //   MemBarAcquire
  //
  //
  // G1 is quite a lot more complicated. The nodes inserted on behalf
  // of G1 may comprise: a pre-write graph which adds the old value to
  // the SATB queue; the releasing store itself; and, finally, a
  // post-write graph which performs a card mark.
  //
  // The pre-write graph may be omitted, but only when the put is
  // writing to a newly allocated (young gen) object and then only if
  // there is a direct memory chain to the Initialize node for the
  // object allocation. This will not happen for a volatile put since
  // any memory chain passes through the leading membar.
  //
  // The pre-write graph includes a series of 3 If tests. The outermost
  // If tests whether SATB is enabled (no else case). The next If tests
  // whether the old value is non-NULL (no else case). The third tests
  // whether the SATB queue index is > 0, if so updating the queue. The
  // else case for this third If calls out to the runtime to allocate a
  // new queue buffer.
  //
  // So with G1 the pre-write and releasing store subgraph looks like
  // this (the nested Ifs are omitted).
  //
  //  MemBarRelease (leading)____________
  //     C |  ||  M \   M \    M \  M \ . . .
  //       | LoadB   \  LoadL  LoadN   \
  //       | /        \                 \
  //       If         |\                 \
  //       | \        | \                 \
  //  IfFalse  IfTrue |  \                 \
  //       |     |    |   \                 |
  //       |     If   |   /\                |
  //       |     |          \               |
  //       |                 \              |
  //       |    . . .         \             |
  //       | /       | /       |            |
  //      Region  Phi[M]       |            |
  //       | \       |         |            |
  //       |  \_____ | ___     |            |
  //     C | C \     |   C \ M |            |
  //       | CastP2X | StoreN/P[mo_release] |
  //       |         |         |            |
  //     C |       M |       M |          M |
  //        \        |         |           /
  //                  . . . 
  //          (post write subtree elided)
  //                    . . .
  //             C \         M /
  //         MemBarVolatile (trailing)
  //
  // n.b. the LoadB in this subgraph is not the card read -- it's a
  // read of the SATB queue active flag.
  //
  // Once again the CAS graph is a minor variant on the above with the
  // expected substitutions of CompareAndSawpX for StoreN/P and
  // MemBarCPUOrder + MemBarAcquire for trailing MemBarVolatile.
  //
  // The G1 post-write subtree is also optional, this time when the
  // new value being written is either null or can be identified as a
  // newly allocated (young gen) object with no intervening control
  // flow. The latter cannot happen but the former may, in which case
  // the card mark membar is omitted and the memory feeds form the
  // leading membar and the SToreN/P are merged direct into the
  // trailing membar as per the normal subgraph. So, the only special
  // case which arises is when the post-write subgraph is generated.
  //
  // The kernel of the post-write G1 subgraph is the card mark itself
  // which includes a card mark memory barrier (MemBarVolatile), a
  // card test (LoadB), and a conditional update (If feeding a
  // StoreCM). These nodes are surrounded by a series of nested Ifs
  // which try to avoid doing the card mark. The top level If skips if
  // the object reference does not cross regions (i.e. it tests if
  // (adr ^ val) >> log2(regsize) != 0) -- intra-region references
  // need not be recorded. The next If, which skips on a NULL value,
  // may be absent (it is not generated if the type of value is >=
  // OopPtr::NotNull). The 3rd If skips writes to young regions (by
  // checking if card_val != young).  n.b. although this test requires
  // a pre-read of the card it can safely be done before the StoreLoad
  // barrier. However that does not bypass the need to reread the card
  // after the barrier.
  //
  //                (pre-write subtree elided)
  //        . . .                  . . .    . . .  . . .
  //        C |                    M |     M |    M |
  //       Region                  Phi[M] StoreN    |
  //          |                     / \      |      |
  //         / \_______            /   \     |      |
  //      C / C \      . . .            \    |      |
  //       If   CastP2X . . .            |   |      |
  //       / \                           |   |      |
  //      /   \                          |   |      |
  // IfFalse IfTrue                      |   |      |
  //   |       |                         |   |     /|
  //   |       If                        |   |    / |
  //   |      / \                        |   |   /  |
  //   |     /   \                        \  |  /   |
  //   | IfFalse IfTrue                   MergeMem  |
  //   |  . . .    / \                       /      |
  //   |          /   \                     /       |
  //   |     IfFalse IfTrue                /        |
  //   |      . . .    |                  /         |
  //   |               If                /          |
  //   |               / \              /           |
  //   |              /   \            /            |
  //   |         IfFalse IfTrue       /             |
  //   |           . . .   |         /              |
  //   |                    \       /               |
  //   |                     \     /                |
  //   |             MemBarVolatile__(card mark)    |
  //   |                ||   C |  M \  M \          |
  //   |               LoadB   If    |    |         |
  //   |                      / \    |    |         |
  //   |                     . . .   |    |         |
  //   |                          \  |    |        /
  //   |                        StoreCM   |       /
  //   |                          . . .   |      /
  //   |                        _________/      /
  //   |                       /  _____________/
  //   |   . . .       . . .  |  /            /
  //   |    |                 | /   _________/
  //   |    |               Phi[M] /        /
  //   |    |                 |   /        /
  //   |    |                 |  /        /
  //   |  Region  . . .     Phi[M]  _____/
  //   |    /                 |    /
  //   |                      |   /   
  //   | . . .   . . .        |  /
  //   | /                    | /
  // Region           |  |  Phi[M]
  //   |              |  |  / Bot
  //    \            MergeMem 
  //     \            /
  //     MemBarVolatile
  //
  // As with CMS the initial MergeMem merges the AliasIdxBot Mem slice
  // from the leading membar and the oopptr Mem slice from the Store
  // into the card mark membar i.e. the memory flow to the card mark
  // membar still looks like a normal graph.
  //
  // The trailing MergeMem merges an AliasIdxBot Mem slice with other
  // Mem slices (from the StoreCM and other card mark queue stores).
  // However in this case the AliasIdxBot Mem slice does not come
  // direct from the card mark membar. It is merged through a series
  // of Phi nodes. These are needed to merge the AliasIdxBot Mem flow
  // from the leading membar with the Mem feed from the card mark
  // membar. Each Phi corresponds to one of the Ifs which may skip
  // around the card mark membar. So when the If implementing the NULL
  // value check has been elided the total number of Phis is 2
  // otherwise it is 3.
  //
  // The CAS graph when using G1GC also includes a pre-write subgraph
  // and an optional post-write subgraph. Teh sam evarioations are
  // introduced as for CMS with conditional card marking i.e. the
  // StoreP/N is swapped for a CompareAndSwapP/N, the tariling
  // MemBarVolatile for a MemBarCPUOrder + MemBarAcquire pair and the
  // Mem feed from the CompareAndSwapP/N includes a precedence
  // dependency feed to the StoreCM and a feed via an SCMemProj to the
  // trailing membar. So, as before the configuration includes the
  // normal CAS graph as a subgraph of the memory flow.
  //
  // So, the upshot is that in all cases the volatile put graph will
  // include a *normal* memory subgraph betwen the leading membar and
  // its child membar, either a volatile put graph (including a
  // releasing StoreX) or a CAS graph (including a CompareAndSwapX).
  // When that child is not a card mark membar then it marks the end
  // of the volatile put or CAS subgraph. If the child is a card mark
  // membar then the normal subgraph will form part of a volatile put
  // subgraph if and only if the child feeds an AliasIdxBot Mem feed
  // to a trailing barrier via a MergeMem. That feed is either direct
  // (for CMS) or via 2 or 3 Phi nodes merging the leading barrier
  // memory flow (for G1).
  // 
  // The predicates controlling generation of instructions for store
  // and barrier nodes employ a few simple helper functions (described
  // below) which identify the presence or absence of all these
  // subgraph configurations and provide a means of traversing from
  // one node in the subgraph to another.

  // is_CAS(int opcode)
  //
  // return true if opcode is one of the possible CompareAndSwapX
  // values otherwise false.

  bool is_CAS(int opcode)
  {
    return (opcode == Op_CompareAndSwapI ||
	    opcode == Op_CompareAndSwapL ||
	    opcode == Op_CompareAndSwapN ||
	    opcode == Op_CompareAndSwapP);
  }

  // leading_to_normal
  //
  //graph traversal helper which detects the normal case Mem feed from
  // a release membar (or, optionally, its cpuorder child) to a
  // dependent volatile membar i.e. it ensures that one or other of
  // the following Mem flow subgraph is present.
  //
  //   MemBarRelease
  //   MemBarCPUOrder {leading}
  //          |  \      . . .
  //          |  StoreN/P[mo_release]  . . .
  //          |   /
  //         MergeMem
  //          |
  //   MemBarVolatile {trailing or card mark}
  //
  //   MemBarRelease
  //   MemBarCPUOrder {leading}
  //      |       \      . . .
  //      |     CompareAndSwapX  . . .
  //               |
  //     . . .    SCMemProj
  //           \   |
  //      |    MergeMem
  //      |       /
  //    MemBarCPUOrder
  //    MemBarAcquire {trailing}
  //
  // if the correct configuration is present returns the trailing
  // membar otherwise NULL.
  //
  // the input membar is expected to be either a cpuorder membar or a
  // release membar. in the latter case it should not have a cpu membar
  // child.
  //
  // the returned value may be a card mark or trailing membar
  //

  MemBarNode *leading_to_normal(MemBarNode *leading)
  {
    assert((leading->Opcode() == Op_MemBarRelease ||
	    leading->Opcode() == Op_MemBarCPUOrder),
	   "expecting a volatile or cpuroder membar!");

    // check the mem flow
    ProjNode *mem = leading->proj_out(TypeFunc::Memory);

    if (!mem) {
      return NULL;
    }

    Node *x = NULL;
    StoreNode * st = NULL;
    LoadStoreNode *cas = NULL;
    MergeMemNode *mm = NULL;

    for (DUIterator_Fast imax, i = mem->fast_outs(imax); i < imax; i++) {
      x = mem->fast_out(i);
      if (x->is_MergeMem()) {
	if (mm != NULL) {
	  return NULL;
	}
	// two merge mems is one too many
	mm = x->as_MergeMem();
      } else if (x->is_Store() && x->as_Store()->is_release() && x->Opcode() != Op_StoreCM) {
	// two releasing stores/CAS nodes is one too many
	if (st != NULL || cas != NULL) {
	  return NULL;
	}
	st = x->as_Store();
      } else if (is_CAS(x->Opcode())) {
	if (st != NULL || cas != NULL) {
	  return NULL;
	}
	cas = x->as_LoadStore();
      }
    }

    // must have a store or a cas
    if (!st && !cas) {
      return NULL;
    }

    // must have a merge if we also have st
    if (st && !mm) {
      return NULL;
    }

    Node *y = NULL;
    if (cas) {
      // look for an SCMemProj
      for (DUIterator_Fast imax, i = cas->fast_outs(imax); i < imax; i++) {
	x = cas->fast_out(i);
	if (x->is_Proj()) {
	  y = x;
	  break;
	}
      }
      if (y == NULL) {
	return NULL;
      }
      // the proj must feed a MergeMem
      for (DUIterator_Fast imax, i = y->fast_outs(imax); i < imax; i++) {
	x = y->fast_out(i);
	if (x->is_MergeMem()) {
	  mm = x->as_MergeMem();
	  break;
	}
      }
      if (mm == NULL)
	return NULL;
    } else {
      // ensure the store feeds the existing mergemem;
      for (DUIterator_Fast imax, i = st->fast_outs(imax); i < imax; i++) {
	if (st->fast_out(i) == mm) {
	  y = st;
	  break;
	}
      }
      if (y == NULL) {
	return NULL;
      }
    }

    MemBarNode *mbar = NULL;
    // ensure the merge feeds to the expected type of membar
    for (DUIterator_Fast imax, i = mm->fast_outs(imax); i < imax; i++) {
      x = mm->fast_out(i);
      if (x->is_MemBar()) {
	int opcode = x->Opcode();
	if (opcode == Op_MemBarVolatile && st) {
	  mbar = x->as_MemBar();
	} else if (cas && opcode == Op_MemBarCPUOrder) {
	  MemBarNode *y =  x->as_MemBar();
	  y = child_membar(y);
	  if (y != NULL && y->Opcode() == Op_MemBarAcquire) {
	    mbar = y;
	  }
	}
	break;
      }
    }

    return mbar;
  }

  // normal_to_leading
  //
  // graph traversal helper which detects the normal case Mem feed
  // from either a card mark or a trailing membar to a preceding
  // release membar (optionally its cpuorder child) i.e. it ensures
  // that one or other of the following Mem flow subgraphs is present.
  //
  //   MemBarRelease
  //   MemBarCPUOrder {leading}
  //          |  \      . . .
  //          |  StoreN/P[mo_release]  . . .
  //          |   /
  //         MergeMem
  //          |
  //   MemBarVolatile {card mark or trailing}
  //
  //   MemBarRelease
  //   MemBarCPUOrder {leading}
  //      |       \      . . .
  //      |     CompareAndSwapX  . . .
  //               |
  //     . . .    SCMemProj
  //           \   |
  //      |    MergeMem
  //      |        /
  //    MemBarCPUOrder
  //    MemBarAcquire {trailing}
  //
  // this predicate checks for the same flow as the previous predicate
  // but starting from the bottom rather than the top.
  //
  // if the configuration is present returns the cpuorder member for
  // preference or when absent the release membar otherwise NULL.
  //
  // n.b. the input membar is expected to be a MemBarVolatile but
  // need not be a card mark membar.

  MemBarNode *normal_to_leading(const MemBarNode *barrier)
  {
    // input must be a volatile membar
    assert((barrier->Opcode() == Op_MemBarVolatile ||
	    barrier->Opcode() == Op_MemBarAcquire),
	   "expecting a volatile or an acquire membar");
    Node *x;
    bool is_cas = barrier->Opcode() == Op_MemBarAcquire;

    // if we have an acquire membar then it must be fed via a CPUOrder
    // membar

    if (is_cas) {
      // skip to parent barrier which must be a cpuorder
      x = parent_membar(barrier);
      if (x->Opcode() != Op_MemBarCPUOrder)
	return NULL;
    } else {
      // start from the supplied barrier
      x = (Node *)barrier;
    }

    // the Mem feed to the membar should be a merge
    x = x ->in(TypeFunc::Memory);
    if (!x->is_MergeMem())
      return NULL;

    MergeMemNode *mm = x->as_MergeMem();

    if (is_cas) {
      // the merge should be fed from the CAS via an SCMemProj node
      x = NULL;
      for (uint idx = 1; idx < mm->req(); idx++) {
	if (mm->in(idx)->Opcode() == Op_SCMemProj) {
	  x = mm->in(idx);
	  break;
	}
      }
      if (x == NULL) {
	return NULL;
      }
      // check for a CAS feeding this proj
      x = x->in(0);
      int opcode = x->Opcode();
      if (!is_CAS(opcode)) {
	return NULL;
      }
      // the CAS should get its mem feed from the leading membar
      x = x->in(MemNode::Memory);
    } else {
      // the merge should get its Bottom mem feed from the leading membar
      x = mm->in(Compile::AliasIdxBot);      
    } 

    // ensure this is a non control projection
    if (!x->is_Proj() || x->is_CFG()) {
      return NULL;
    }
    // if it is fed by a membar that's the one we want
    x = x->in(0);

    if (!x->is_MemBar()) {
      return NULL;
    }

    MemBarNode *leading = x->as_MemBar();
    // reject invalid candidates
    if (!leading_membar(leading)) {
      return NULL;
    }

    // ok, we have a leading membar, now for the sanity clauses

    // the leading membar must feed Mem to a releasing store or CAS
    ProjNode *mem = leading->proj_out(TypeFunc::Memory);
    StoreNode *st = NULL;
    LoadStoreNode *cas = NULL;
    for (DUIterator_Fast imax, i = mem->fast_outs(imax); i < imax; i++) {
      x = mem->fast_out(i);
      if (x->is_Store() && x->as_Store()->is_release() && x->Opcode() != Op_StoreCM) {
	// two stores or CASes is one too many
	if (st != NULL || cas != NULL) {
	  return NULL;
	}
	st = x->as_Store();
      } else if (is_CAS(x->Opcode())) {
	if (st != NULL || cas != NULL) {
	  return NULL;
	}
	cas = x->as_LoadStore();
      }
    }

    // we should not have both a store and a cas
    if (st == NULL & cas == NULL) {
      return NULL;
    }

    if (st == NULL) {
      // nothing more to check
      return leading;
    } else {
      // we should not have a store if we started from an acquire
      if (is_cas) {
	return NULL;
      }

      // the store should feed the merge we used to get here
      for (DUIterator_Fast imax, i = st->fast_outs(imax); i < imax; i++) {
	if (st->fast_out(i) == mm) {
	  return leading;
	}
      }
    }

    return NULL;
  }

  // card_mark_to_trailing
  //
  // graph traversal helper which detects extra, non-normal Mem feed
  // from a card mark volatile membar to a trailing membar i.e. it
  // ensures that one of the following three GC post-write Mem flow
  // subgraphs is present.
  //
  // 1)
  //     . . .
  //       |
  //   MemBarVolatile (card mark)
  //      |          |     
  //      |        StoreCM
  //      |          |
  //      |        . . .
  //  Bot |  / 
  //   MergeMem 
  //      |
  //      |
  //    MemBarVolatile {trailing}
  //
  // 2)
  //   MemBarRelease/CPUOrder (leading)
  //    |
  //    | 
  //    |\       . . .
  //    | \        | 
  //    |  \  MemBarVolatile (card mark) 
  //    |   \   |     |
  //     \   \  |   StoreCM    . . .
  //      \   \ |
  //       \  Phi
  //        \ /
  //        Phi  . . .
  //     Bot |   /
  //       MergeMem
  //         |
  //    MemBarVolatile {trailing}
  //
  //
  // 3)
  //   MemBarRelease/CPUOrder (leading)
  //    |
  //    |\
  //    | \
  //    |  \      . . .
  //    |   \       |
  //    |\   \  MemBarVolatile (card mark)
  //    | \   \   |     |
  //    |  \   \  |   StoreCM    . . .
  //    |   \   \ |
  //     \   \  Phi
  //      \   \ /  
  //       \  Phi
  //        \ /
  //        Phi  . . .
  //     Bot |   /
  //       MergeMem
  //         |
  //         |
  //    MemBarVolatile {trailing}
  //
  // configuration 1 is only valid if UseConcMarkSweepGC &&
  // UseCondCardMark
  //
  // configurations 2 and 3 are only valid if UseG1GC.
  //
  // if a valid configuration is present returns the trailing membar
  // otherwise NULL.
  //
  // n.b. the supplied membar is expected to be a card mark
  // MemBarVolatile i.e. the caller must ensure the input node has the
  // correct operand and feeds Mem to a StoreCM node

  MemBarNode *card_mark_to_trailing(const MemBarNode *barrier)
  {
    // input must be a card mark volatile membar
    assert(is_card_mark_membar(barrier), "expecting a card mark membar");

    Node *feed = barrier->proj_out(TypeFunc::Memory);
    Node *x;
    MergeMemNode *mm = NULL;

    const int MAX_PHIS = 3;	// max phis we will search through
    int phicount = 0; 		// current search count

    bool retry_feed = true;
    while (retry_feed) {
      // see if we have a direct MergeMem feed
      for (DUIterator_Fast imax, i = feed->fast_outs(imax); i < imax; i++) {
	x = feed->fast_out(i);
	// the correct Phi will be merging a Bot memory slice
	if (x->is_MergeMem()) {
	  mm = x->as_MergeMem();
	  break;
	}
      }
      if (mm) {
	retry_feed = false;
      } else if (UseG1GC & phicount++ < MAX_PHIS) {
	// the barrier may feed indirectly via one or two Phi nodes
	PhiNode *phi = NULL;
	for (DUIterator_Fast imax, i = feed->fast_outs(imax); i < imax; i++) {
	  x = feed->fast_out(i);
	  // the correct Phi will be merging a Bot memory slice
	  if (x->is_Phi() && x->adr_type() == TypePtr::BOTTOM) {
	    phi = x->as_Phi();
	    break;
	  }
	}
	if (!phi) {
	  return NULL;
	}
	// look for another merge below this phi
	feed = phi;
      } else {
	// couldn't find a merge
	return NULL;
      }
    }

    // sanity check this feed turns up as the expected slice
    assert(mm->as_MergeMem()->in(Compile::AliasIdxBot) == feed, "expecting membar to feed AliasIdxBot slice to Merge");

    MemBarNode *trailing = NULL;
    // be sure we have a trailing membar the merge
    for (DUIterator_Fast imax, i = mm->fast_outs(imax); i < imax; i++) {
      x = mm->fast_out(i);
      if (x->is_MemBar() && x->Opcode() == Op_MemBarVolatile) {
	trailing = x->as_MemBar();
	break;
      }
    }

    return trailing;
  }

  // trailing_to_card_mark
  //
  // graph traversal helper which detects extra, non-normal Mem feed
  // from a trailing volatile membar to a preceding card mark volatile
  // membar i.e. it identifies whether one of the three possible extra
  // GC post-write Mem flow subgraphs is present
  //
  // this predicate checks for the same flow as the previous predicate
  // but starting from the bottom rather than the top.
  //
  // if the configuration is present returns the card mark membar
  // otherwise NULL
  //
  // n.b. the supplied membar is expected to be a trailing
  // MemBarVolatile i.e. the caller must ensure the input node has the
  // correct opcode

  MemBarNode *trailing_to_card_mark(const MemBarNode *trailing)
  {
    assert(trailing->Opcode() == Op_MemBarVolatile,
	   "expecting a volatile membar");
    assert(!is_card_mark_membar(trailing),
	   "not expecting a card mark membar");

    // the Mem feed to the membar should be a merge
    Node *x = trailing->in(TypeFunc::Memory);
    if (!x->is_MergeMem()) {
      return NULL;
    }

    MergeMemNode *mm = x->as_MergeMem();

    x = mm->in(Compile::AliasIdxBot);
    // with G1 we may possibly see a Phi or two before we see a Memory
    // Proj from the card mark membar

    const int MAX_PHIS = 3;	// max phis we will search through
    int phicount = 0; 		// current search count

    bool retry_feed = !x->is_Proj();

    while (retry_feed) {
      if (UseG1GC && x->is_Phi() && phicount++ < MAX_PHIS) {
	PhiNode *phi = x->as_Phi();
	ProjNode *proj = NULL;
	PhiNode *nextphi = NULL;
	bool found_leading = false;
	for (uint i = 1; i < phi->req(); i++) {
	  x = phi->in(i);
	  if (x->is_Phi()) {
	    nextphi = x->as_Phi();
	  } else if (x->is_Proj()) {
	    int opcode = x->in(0)->Opcode();
	    if (opcode == Op_MemBarVolatile) {
	      proj = x->as_Proj();
	    } else if (opcode == Op_MemBarRelease ||
		       opcode == Op_MemBarCPUOrder) {
	      // probably a leading membar
	      found_leading = true;
	    }
	  }
	}
	// if we found a correct looking proj then retry from there
	// otherwise we must see a leading and a phi or this the
	// wrong config
	if (proj != NULL) {
	  x = proj;
	  retry_feed = false;
	} else if (found_leading && nextphi != NULL) {
	  // retry from this phi to check phi2
	  x = nextphi;
	} else {
	  // not what we were looking for
	  return NULL;
	}
      } else {
	return NULL;
      }
    }
    // the proj has to come from the card mark membar
    x = x->in(0);
    if (!x->is_MemBar()) {
      return NULL;
    }

    MemBarNode *card_mark_membar = x->as_MemBar();

    if (!is_card_mark_membar(card_mark_membar)) {
      return NULL;
    }

    return card_mark_membar;
  }

  // trailing_to_leading
  //
  // graph traversal helper which checks the Mem flow up the graph
  // from a (non-card mark) trailing membar attempting to locate and
  // return an associated leading membar. it first looks for a
  // subgraph in the normal configuration (relying on helper
  // normal_to_leading). failing that it then looks for one of the
  // possible post-write card mark subgraphs linking the trailing node
  // to a the card mark membar (relying on helper
  // trailing_to_card_mark), and then checks that the card mark membar
  // is fed by a leading membar (once again relying on auxiliary
  // predicate normal_to_leading).
  //
  // if the configuration is valid returns the cpuorder member for
  // preference or when absent the release membar otherwise NULL.
  //
  // n.b. the input membar is expected to be either a volatile or
  // acquire membar but in the former case must *not* be a card mark
  // membar.

  MemBarNode *trailing_to_leading(const MemBarNode *trailing)
  {
    assert((trailing->Opcode() == Op_MemBarAcquire ||
	    trailing->Opcode() == Op_MemBarVolatile),
	   "expecting an acquire or volatile membar");
    assert((trailing->Opcode() != Op_MemBarVolatile ||
	    !is_card_mark_membar(trailing)),
	   "not expecting a card mark membar");

    MemBarNode *leading = normal_to_leading(trailing);

    if (leading) {
      return leading;
    }

    // nothing more to do if this is an acquire
    if (trailing->Opcode() == Op_MemBarAcquire) {
      return NULL;
    }

    MemBarNode *card_mark_membar = trailing_to_card_mark(trailing);

    if (!card_mark_membar) {
      return NULL;
    }

    return normal_to_leading(card_mark_membar);
  }

  // predicates controlling emit of ldr<x>/ldar<x> and associated dmb

bool unnecessary_acquire(const Node *barrier)
{
  assert(barrier->is_MemBar(), "expecting a membar");

  if (UseBarriersForVolatile) {
    // we need to plant a dmb
    return false;
  }

  // a volatile read derived from bytecode (or also from an inlined
  // SHA field read via LibraryCallKit::load_field_from_object)
  // manifests as a LoadX[mo_acquire] followed by an acquire membar
  // with a bogus read dependency on it's preceding load. so in those
  // cases we will find the load node at the PARMS offset of the
  // acquire membar.  n.b. there may be an intervening DecodeN node.
  //
  // a volatile load derived from an inlined unsafe field access
  // manifests as a cpuorder membar with Ctl and Mem projections
  // feeding both an acquire membar and a LoadX[mo_acquire]. The
  // acquire then feeds another cpuorder membar via Ctl and Mem
  // projections. The load has no output dependency on these trailing
  // membars because subsequent nodes inserted into the graph take
  // their control feed from the final membar cpuorder meaning they
  // are all ordered after the load.

  Node *x = barrier->lookup(TypeFunc::Parms);
  if (x) {
    // we are starting from an acquire and it has a fake dependency
    //
    // need to check for
    //
    //   LoadX[mo_acquire]
    //   {  |1   }
    //   {DecodeN}
    //      |Parms
    //   MemBarAcquire*
    //
    // where * tags node we were passed
    // and |k means input k
    if (x->is_DecodeNarrowPtr()) {
      x = x->in(1);
    }

    return (x->is_Load() && x->as_Load()->is_acquire());
  }
  
  // now check for an unsafe volatile get

  // need to check for
  //
  //   MemBarCPUOrder
  //        ||       \\
  //   MemBarAcquire* LoadX[mo_acquire]
  //        ||
  //   MemBarCPUOrder
  //
  // where * tags node we were passed
  // and || or \\ are Ctl+Mem feeds via intermediate Proj Nodes

  // check for a parent MemBarCPUOrder
  ProjNode *ctl;
  ProjNode *mem;
  MemBarNode *parent = parent_membar(barrier);
  if (!parent || parent->Opcode() != Op_MemBarCPUOrder)
    return false;
  ctl = parent->proj_out(TypeFunc::Control);
  mem = parent->proj_out(TypeFunc::Memory);
  if (!ctl || !mem) {
    return false;
  }
  // ensure the proj nodes both feed a LoadX[mo_acquire]
  LoadNode *ld = NULL;
  for (DUIterator_Fast imax, i = ctl->fast_outs(imax); i < imax; i++) {
    x = ctl->fast_out(i);
    // if we see a load we keep hold of it and stop searching
    if (x->is_Load()) {
      ld = x->as_Load();
      break;
    }
  }
  // it must be an acquiring load
  if (ld && ld->is_acquire()) {

    for (DUIterator_Fast imax, i = mem->fast_outs(imax); i < imax; i++) {
      x = mem->fast_out(i);
      // if we see the same load we drop it and stop searching
      if (x == ld) {
	ld = NULL;
	break;
      }
    }
    // we must have dropped the load
    if (ld == NULL) {
      // check for a child cpuorder membar
      MemBarNode *child  = child_membar(barrier->as_MemBar());
      if (child && child->Opcode() == Op_MemBarCPUOrder)
	return true;
    }
  }

  // final option for unnecessary mebar is that it is a trailing node
  // belonging to a CAS

  MemBarNode *leading = trailing_to_leading(barrier->as_MemBar());

  return leading != NULL;
}

bool needs_acquiring_load(const Node *n)
{
  assert(n->is_Load(), "expecting a load");
  if (UseBarriersForVolatile) {
    // we use a normal load and a dmb
    return false;
  }

  LoadNode *ld = n->as_Load();

  if (!ld->is_acquire()) {
    return false;
  }

  // check if this load is feeding an acquire membar
  //
  //   LoadX[mo_acquire]
  //   {  |1   }
  //   {DecodeN}
  //      |Parms
  //   MemBarAcquire*
  //
  // where * tags node we were passed
  // and |k means input k

  Node *start = ld;
  Node *mbacq = NULL;

  // if we hit a DecodeNarrowPtr we reset the start node and restart
  // the search through the outputs
 restart:

  for (DUIterator_Fast imax, i = start->fast_outs(imax); i < imax; i++) {
    Node *x = start->fast_out(i);
    if (x->is_MemBar() && x->Opcode() == Op_MemBarAcquire) {
      mbacq = x;
    } else if (!mbacq &&
	       (x->is_DecodeNarrowPtr() ||
		(x->is_Mach() && x->Opcode() == Op_DecodeN))) {
      start = x;
      goto restart;
    }
  }

  if (mbacq) {
    return true;
  }

  // now check for an unsafe volatile get

  // check if Ctl and Proj feed comes from a MemBarCPUOrder
  //
  //     MemBarCPUOrder
  //        ||       \\
  //   MemBarAcquire* LoadX[mo_acquire]
  //        ||
  //   MemBarCPUOrder

  MemBarNode *membar;

  membar = parent_membar(ld);

  if (!membar || !membar->Opcode() == Op_MemBarCPUOrder) {
    return false;
  }

  // ensure that there is a CPUOrder->Acquire->CPUOrder membar chain

  membar = child_membar(membar);

  if (!membar || !membar->Opcode() == Op_MemBarAcquire) {
    return false;
  }

  membar = child_membar(membar);
  
  if (!membar || !membar->Opcode() == Op_MemBarCPUOrder) {
    return false;
  }

  return true;
}

bool unnecessary_release(const Node *n)
{
  assert((n->is_MemBar() &&
	  n->Opcode() == Op_MemBarRelease),
	 "expecting a release membar");

  if (UseBarriersForVolatile) {
    // we need to plant a dmb
    return false;
  }

  // if there is a dependent CPUOrder barrier then use that as the
  // leading

  MemBarNode *barrier = n->as_MemBar();
  // check for an intervening cpuorder membar
  MemBarNode *b = child_membar(barrier);
  if (b && b->Opcode() == Op_MemBarCPUOrder) {
    // ok, so start the check from the dependent cpuorder barrier
    barrier = b;
  }

  // must start with a normal feed
  MemBarNode *child_barrier = leading_to_normal(barrier);

  if (!child_barrier) {
    return false;
  }

  if (!is_card_mark_membar(child_barrier)) {
    // this is the trailing membar and we are done
    return true;
  }

  // must be sure this card mark feeds a trailing membar
  MemBarNode *trailing = card_mark_to_trailing(child_barrier);
  return (trailing != NULL);
}

bool unnecessary_volatile(const Node *n)
{
  // assert n->is_MemBar();
  if (UseBarriersForVolatile) {
    // we need to plant a dmb
    return false;
  }

  MemBarNode *mbvol = n->as_MemBar();

  // first we check if this is part of a card mark. if so then we have
  // to generate a StoreLoad barrier
  
  if (is_card_mark_membar(mbvol)) {
      return false;
  }

  // ok, if it's not a card mark then we still need to check if it is
  // a trailing membar of a volatile put hgraph.

  return (trailing_to_leading(mbvol) != NULL);
}

// predicates controlling emit of str<x>/stlr<x> and associated dmbs

bool needs_releasing_store(const Node *n)
{
  // assert n->is_Store();
  if (UseBarriersForVolatile) {
    // we use a normal store and dmb combination
    return false;
  }

  StoreNode *st = n->as_Store();

  // the store must be marked as releasing
  if (!st->is_release()) {
    return false;
  }

  // the store must be fed by a membar

  Node *x = st->lookup(StoreNode::Memory);

  if (! x || !x->is_Proj()) {
    return false;
  }

  ProjNode *proj = x->as_Proj();

  x = proj->lookup(0);

  if (!x || !x->is_MemBar()) {
    return false;
  }

  MemBarNode *barrier = x->as_MemBar();

  // if the barrier is a release membar or a cpuorder mmebar fed by a
  // release membar then we need to check whether that forms part of a
  // volatile put graph.

  // reject invalid candidates
  if (!leading_membar(barrier)) {
    return false;
  }

  // does this lead a normal subgraph?
  MemBarNode *mbvol = leading_to_normal(barrier);

  if (!mbvol) {
    return false;
  }

  // all done unless this is a card mark
  if (!is_card_mark_membar(mbvol)) {
    return true;
  }
  
  // we found a card mark -- just make sure we have a trailing barrier

  return (card_mark_to_trailing(mbvol) != NULL);
}

// predicate controlling translation of CAS
//
// returns true if CAS needs to use an acquiring load otherwise false

bool needs_acquiring_load_exclusive(const Node *n)
{
  assert(is_CAS(n->Opcode()), "expecting a compare and swap");
  if (UseBarriersForVolatile) {
    return false;
  }

  // CAS nodes only ought to turn up in inlined unsafe CAS operations
#ifdef ASSERT
  LoadStoreNode *st = n->as_LoadStore();

  // the store must be fed by a membar

  Node *x = st->lookup(StoreNode::Memory);

  assert (x && x->is_Proj(), "CAS not fed by memory proj!");

  ProjNode *proj = x->as_Proj();

  x = proj->lookup(0);

  assert (x && x->is_MemBar(), "CAS not fed by membar!");

  MemBarNode *barrier = x->as_MemBar();

  // the barrier must be a cpuorder mmebar fed by a release membar

  assert(barrier->Opcode() == Op_MemBarCPUOrder,
	 "CAS not fed by cpuorder membar!");
      
  MemBarNode *b = parent_membar(barrier);
  assert ((b != NULL && b->Opcode() == Op_MemBarRelease),
	  "CAS not fed by cpuorder+release membar pair!");

  // does this lead a normal subgraph?
  MemBarNode *mbar = leading_to_normal(barrier);

  assert(mbar != NULL, "CAS not embedded in normal graph!");

  assert(mbar->Opcode() == Op_MemBarAcquire, "trailing membar should be an acquire");
#endif // ASSERT
  // so we can just return true here
  return true;
}

// predicate controlling translation of StoreCM
//
// returns true if a StoreStore must precede the card write otherwise
// false

bool unnecessary_storestore(const Node *storecm)
{
  assert(storecm->Opcode()  == Op_StoreCM, "expecting a StoreCM");

  // we only ever need to generate a dmb ishst between an object put
  // and the associated card mark when we are using CMS without
  // conditional card marking

  if (!UseConcMarkSweepGC || UseCondCardMark) {
    return true;
  }

  // if we are implementing volatile puts using barriers then the
  // object put as an str so we must insert the dmb ishst

  if (UseBarriersForVolatile) {
    return false;
  }

  // we can omit the dmb ishst if this StoreCM is part of a volatile
  // put because in thta case the put will be implemented by stlr
  //
  // we need to check for a normal subgraph feeding this StoreCM.
  // that means the StoreCM must be fed Memory from a leading membar,
  // either a MemBarRelease or its dependent MemBarCPUOrder, and the
  // leading membar must be part of a normal subgraph

  Node *x = storecm->in(StoreNode::Memory);

  if (!x->is_Proj()) {
    return false;
  }

  x = x->in(0);

  if (!x->is_MemBar()) {
    return false;
  }

  MemBarNode *leading = x->as_MemBar();

  // reject invalid candidates
  if (!leading_membar(leading)) {
    return false;
  }

  // we can omit the StoreStore if it is the head of a normal subgraph
  return (leading_to_normal(leading) != NULL);
}


#define __ _masm.

// advance declaratuons for helper functions to convert register
// indices to register objects

// the ad file has to provide implementations of certain methods
// expected by the generic code
//
// REQUIRED FUNCTIONALITY

//=============================================================================

// !!!!! Special hack to get all types of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.

int MachCallStaticJavaNode::ret_addr_offset()
{
  // call should be a simple bl
  // unless this is a method handle invoke in which case it is
  // mov(rfp, sp), bl, mov(sp, rfp)
  int off = 4;
  if (_method_handle_invoke) {
    off += 4;
  }
  return off;
}

int MachCallDynamicJavaNode::ret_addr_offset()
{
  return 16; // movz, movk, movk, bl
}

int MachCallRuntimeNode::ret_addr_offset() {
  // for generated stubs the call will be
  //   bl(addr)
  // for real runtime callouts it will be six instructions
  // see aarch64_enc_java_to_runtime
  //   adr(rscratch2, retaddr)
  //   lea(rscratch1, RuntimeAddress(addr)
  //   stp(zr, rscratch2, Address(__ pre(sp, -2 * wordSize)))
  //   blrt rscratch1
  CodeBlob *cb = CodeCache::find_blob(_entry_point);
  if (cb) {
    return MacroAssembler::far_branch_size();
  } else {
    return 6 * NativeInstruction::instruction_size;
  }
}

// Indicate if the safepoint node needs the polling page as an input

// the shared code plants the oop data at the start of the generated
// code for the safepoint node and that needs ot be at the load
// instruction itself. so we cannot plant a mov of the safepoint poll
// address followed by a load. setting this to true means the mov is
// scheduled as a prior instruction. that's better for scheduling
// anyway.

bool SafePointNode::needs_polling_address_input()
{
  return true;
}

//=============================================================================

#ifndef PRODUCT
void MachBreakpointNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  st->print("BREAKPOINT");
}
#endif

void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler _masm(&cbuf);
  __ brk(0);
}

uint MachBreakpointNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

//=============================================================================

#ifndef PRODUCT
  void MachNopNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("nop \t# %d bytes pad for loops and calls", _count);
  }
#endif

  void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc*) const {
    MacroAssembler _masm(&cbuf);
    for (int i = 0; i < _count; i++) { 
      __ nop();
    }
  }

  uint MachNopNode::size(PhaseRegAlloc*) const {
    return _count * NativeInstruction::instruction_size;
  }

//=============================================================================
const RegMask& MachConstantBaseNode::_out_RegMask = RegMask::Empty;

int Compile::ConstantTable::calculate_table_base_offset() const {
  return 0;  // absolute addressing, no offset
}

bool MachConstantBaseNode::requires_postalloc_expand() const { return false; }
void MachConstantBaseNode::postalloc_expand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  ShouldNotReachHere();
}

void MachConstantBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  // Empty encoding
}

uint MachConstantBaseNode::size(PhaseRegAlloc* ra_) const {
  return 0;
}

#ifndef PRODUCT
void MachConstantBaseNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  st->print("-- \t// MachConstantBaseNode (empty encoding)");
}
#endif

#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  Compile* C = ra_->C;

  int framesize = C->frame_slots() << LogBytesPerInt;

  if (C->need_stack_bang(framesize))
    st->print("# stack bang size=%d\n\t", framesize);

  if (framesize == 0) {
    // Is this even possible?
    st->print("stp  lr, rfp, [sp, #%d]!", -(2 * wordSize)); 
  } else if (framesize < ((1 << 9) + 2 * wordSize)) {
    st->print("sub  sp, sp, #%d\n\t", framesize);
    st->print("stp  rfp, lr, [sp, #%d]", framesize - 2 * wordSize);
  } else {
    st->print("stp  lr, rfp, [sp, #%d]!\n\t", -(2 * wordSize)); 
    st->print("mov  rscratch1, #%d\n\t", framesize - 2 * wordSize);
    st->print("sub  sp, sp, rscratch1");
  }
}
#endif

void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  MacroAssembler _masm(&cbuf);

  // n.b. frame size includes space for return pc and rfp
  long framesize = ((long)C->frame_slots()) << LogBytesPerInt;
  assert(framesize%(2*wordSize) == 0, "must preserve 2*wordSize alignment");

  // insert a nop at the start of the prolog so we can patch in a
  // branch if we need to invalidate the method later
  __ nop();

  if (C->need_stack_bang(framesize))
    __ generate_stack_overflow_check(framesize);

  __ build_frame(framesize);

  if (NotifySimulator) {
    __ notify(Assembler::method_entry);
  }

  if (VerifyStackAtCalls) {
    Unimplemented();
  }

  C->set_frame_complete(cbuf.insts_size());

  if (C->has_mach_constant_base_node()) {
    // NOTE: We set the table base offset here because users might be
    // emitted before MachConstantBaseNode.
    Compile::ConstantTable& constant_table = C->constant_table();
    constant_table.set_table_base_offset(constant_table.calculate_table_base_offset());
  }
}

uint MachPrologNode::size(PhaseRegAlloc* ra_) const
{
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachPrologNode::reloc() const
{
  return 0;
}

//=============================================================================

#ifndef PRODUCT
void MachEpilogNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  Compile* C = ra_->C;
  int framesize = C->frame_slots() << LogBytesPerInt;

  st->print("# pop frame %d\n\t",framesize);

  if (framesize == 0) {
    st->print("ldp  lr, rfp, [sp],#%d\n\t", (2 * wordSize));
  } else if (framesize < ((1 << 9) + 2 * wordSize)) {
    st->print("ldp  lr, rfp, [sp,#%d]\n\t", framesize - 2 * wordSize);
    st->print("add  sp, sp, #%d\n\t", framesize);
  } else {
    st->print("mov  rscratch1, #%d\n\t", framesize - 2 * wordSize);
    st->print("add  sp, sp, rscratch1\n\t");
    st->print("ldp  lr, rfp, [sp],#%d\n\t", (2 * wordSize));
  }

  if (do_polling() && C->is_method_compilation()) {
    st->print("# touch polling page\n\t");
    st->print("mov  rscratch1, #" INTPTR_FORMAT "\n\t", p2i(os::get_polling_page()));
    st->print("ldr zr, [rscratch1]");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  MacroAssembler _masm(&cbuf);
  int framesize = C->frame_slots() << LogBytesPerInt;

  __ remove_frame(framesize);

  if (NotifySimulator) {
    __ notify(Assembler::method_reentry);
  }

  if (do_polling() && C->is_method_compilation()) {
    __ read_polling_page(rscratch1, os::get_polling_page(), relocInfo::poll_return_type);
  }
}

uint MachEpilogNode::size(PhaseRegAlloc *ra_) const {
  // Variable size. Determine dynamically.
  return MachNode::size(ra_);
}

int MachEpilogNode::reloc() const {
  // Return number of relocatable values contained in this instruction.
  return 1; // 1 for polling page.
}

const Pipeline * MachEpilogNode::pipeline() const {
  return MachNode::pipeline_class();
}

// This method seems to be obsolete. It is declared in machnode.hpp
// and defined in all *.ad files, but it is never called. Should we
// get rid of it?
int MachEpilogNode::safepoint_offset() const {
  assert(do_polling(), "no return for this epilog node");
  return 4;
}

//=============================================================================

// Figure out which register class each belongs in: rc_int, rc_float or
// rc_stack.
enum RC { rc_bad, rc_int, rc_float, rc_stack };

static enum RC rc_class(OptoReg::Name reg) {

  if (reg == OptoReg::Bad) {
    return rc_bad;
  }

  // we have 30 int registers * 2 halves
  // (rscratch1 and rscratch2 are omitted)

  if (reg < 60) {
    return rc_int;
  }

  // we have 32 float register * 2 halves
  if (reg < 60 + 128) {
    return rc_float;
  }

  // Between float regs & stack is the flags regs.
  assert(OptoReg::is_stack(reg), "blow up if spilling flags");

  return rc_stack;
}

uint MachSpillCopyNode::implementation(CodeBuffer *cbuf, PhaseRegAlloc *ra_, bool do_size, outputStream *st) const {
  Compile* C = ra_->C;

  // Get registers to move.
  OptoReg::Name src_hi = ra_->get_reg_second(in(1));
  OptoReg::Name src_lo = ra_->get_reg_first(in(1));
  OptoReg::Name dst_hi = ra_->get_reg_second(this);
  OptoReg::Name dst_lo = ra_->get_reg_first(this);

  enum RC src_hi_rc = rc_class(src_hi);
  enum RC src_lo_rc = rc_class(src_lo);
  enum RC dst_hi_rc = rc_class(dst_hi);
  enum RC dst_lo_rc = rc_class(dst_lo);

  assert(src_lo != OptoReg::Bad && dst_lo != OptoReg::Bad, "must move at least 1 register");

  if (src_hi != OptoReg::Bad) {
    assert((src_lo&1)==0 && src_lo+1==src_hi &&
           (dst_lo&1)==0 && dst_lo+1==dst_hi,
           "expected aligned-adjacent pairs");
  }

  if (src_lo == dst_lo && src_hi == dst_hi) {
    return 0;            // Self copy, no move.
  }

  bool is64 = (src_lo & 1) == 0 && src_lo + 1 == src_hi &&
              (dst_lo & 1) == 0 && dst_lo + 1 == dst_hi;
  int src_offset = ra_->reg2offset(src_lo);
  int dst_offset = ra_->reg2offset(dst_lo);

  if (bottom_type()->isa_vect() != NULL) {
    uint ireg = ideal_reg();
    assert(ireg == Op_VecD || ireg == Op_VecX, "must be 64 bit or 128 bit vector");
    if (cbuf) {
      MacroAssembler _masm(cbuf);
      assert((src_lo_rc != rc_int && dst_lo_rc != rc_int), "sanity");
      if (src_lo_rc == rc_stack && dst_lo_rc == rc_stack) {
        // stack->stack
        assert((src_offset & 7) == 0 && (dst_offset & 7) == 0, "unaligned stack offset");
        if (ireg == Op_VecD) {
          __ unspill(rscratch1, true, src_offset);
          __ spill(rscratch1, true, dst_offset);
        } else {
          __ spill_copy128(src_offset, dst_offset);
        }
      } else if (src_lo_rc == rc_float && dst_lo_rc == rc_float) {
        __ mov(as_FloatRegister(Matcher::_regEncode[dst_lo]),
               ireg == Op_VecD ? __ T8B : __ T16B,
               as_FloatRegister(Matcher::_regEncode[src_lo]));
      } else if (src_lo_rc == rc_float && dst_lo_rc == rc_stack) {
        __ spill(as_FloatRegister(Matcher::_regEncode[src_lo]),
                       ireg == Op_VecD ? __ D : __ Q,
                       ra_->reg2offset(dst_lo));
      } else if (src_lo_rc == rc_stack && dst_lo_rc == rc_float) {
        __ unspill(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                       ireg == Op_VecD ? __ D : __ Q,
                       ra_->reg2offset(src_lo));
      } else {
        ShouldNotReachHere();
      }
    }
  } else if (cbuf) {
    MacroAssembler _masm(cbuf);
    switch (src_lo_rc) {
    case rc_int:
      if (dst_lo_rc == rc_int) {  // gpr --> gpr copy
        if (is64) {
            __ mov(as_Register(Matcher::_regEncode[dst_lo]),
                   as_Register(Matcher::_regEncode[src_lo]));
        } else {
            MacroAssembler _masm(cbuf);
            __ movw(as_Register(Matcher::_regEncode[dst_lo]),
                    as_Register(Matcher::_regEncode[src_lo]));
        }
      } else if (dst_lo_rc == rc_float) { // gpr --> fpr copy
        if (is64) {
            __ fmovd(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_Register(Matcher::_regEncode[src_lo]));
        } else {
            __ fmovs(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_Register(Matcher::_regEncode[src_lo]));
        }
      } else {                    // gpr --> stack spill
        assert(dst_lo_rc == rc_stack, "spill to bad register class");
        __ spill(as_Register(Matcher::_regEncode[src_lo]), is64, dst_offset);
      }
      break;
    case rc_float:
      if (dst_lo_rc == rc_int) {  // fpr --> gpr copy
        if (is64) {
            __ fmovd(as_Register(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
        } else {
            __ fmovs(as_Register(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
        }
      } else if (dst_lo_rc == rc_float) { // fpr --> fpr copy
          if (cbuf) {
            __ fmovd(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
        } else {
            __ fmovs(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                     as_FloatRegister(Matcher::_regEncode[src_lo]));
        }
      } else {                    // fpr --> stack spill
        assert(dst_lo_rc == rc_stack, "spill to bad register class");
        __ spill(as_FloatRegister(Matcher::_regEncode[src_lo]),
                 is64 ? __ D : __ S, dst_offset);
      }
      break;
    case rc_stack:
      if (dst_lo_rc == rc_int) {  // stack --> gpr load
        __ unspill(as_Register(Matcher::_regEncode[dst_lo]), is64, src_offset);
      } else if (dst_lo_rc == rc_float) { // stack --> fpr load
        __ unspill(as_FloatRegister(Matcher::_regEncode[dst_lo]),
                   is64 ? __ D : __ S, src_offset);
      } else {                    // stack --> stack copy
        assert(dst_lo_rc == rc_stack, "spill to bad register class");
        __ unspill(rscratch1, is64, src_offset);
        __ spill(rscratch1, is64, dst_offset);
      }
      break;
    default:
      assert(false, "bad rc_class for spill");
      ShouldNotReachHere();
    }
  }

  if (st) {
    st->print("spill ");
    if (src_lo_rc == rc_stack) {
      st->print("[sp, #%d] -> ", ra_->reg2offset(src_lo));
    } else {
      st->print("%s -> ", Matcher::regName[src_lo]);
    }
    if (dst_lo_rc == rc_stack) {
      st->print("[sp, #%d]", ra_->reg2offset(dst_lo));
    } else {
      st->print("%s", Matcher::regName[dst_lo]);
    }
    if (bottom_type()->isa_vect() != NULL) {
      st->print("\t# vector spill size = %d", ideal_reg()==Op_VecD ? 64:128);
    } else {
      st->print("\t# spill size = %d", is64 ? 64:32);
    }
  }

  return 0;

}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  if (!ra_)
    st->print("N%d = SpillCopy(N%d)", _idx, in(1)->_idx);
  else
    implementation(NULL, ra_, false, st);
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation(&cbuf, ra_, false, NULL);
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

//=============================================================================

#ifndef PRODUCT
void BoxLockNode::format(PhaseRegAlloc *ra_, outputStream *st) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("add %s, rsp, #%d]\t# box lock",
            Matcher::regName[reg], offset);
}
#endif

void BoxLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler _masm(&cbuf);

  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg    = ra_->get_encode(this);

  if (Assembler::operand_valid_for_add_sub_immediate(offset)) {
    __ add(as_Register(reg), sp, offset);
  } else {
    ShouldNotReachHere();
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const {
  // BoxLockNode is not a MachNode, so we can't just call MachNode::size(ra_).
  return 4;
}

//=============================================================================

#ifndef PRODUCT
void MachUEPNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  st->print_cr("# MachUEPNode");
  if (UseCompressedClassPointers) {
    st->print_cr("\tldrw rscratch1, j_rarg0 + oopDesc::klass_offset_in_bytes()]\t# compressed klass");
    if (Universe::narrow_klass_shift() != 0) {
      st->print_cr("\tdecode_klass_not_null rscratch1, rscratch1");
    }
  } else {
   st->print_cr("\tldr rscratch1, j_rarg0 + oopDesc::klass_offset_in_bytes()]\t# compressed klass");
  }
  st->print_cr("\tcmp r0, rscratch1\t # Inline cache check");
  st->print_cr("\tbne, SharedRuntime::_ic_miss_stub");
}
#endif

void MachUEPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  // This is the unverified entry point.
  MacroAssembler _masm(&cbuf);

  __ cmp_klass(j_rarg0, rscratch2, rscratch1);
  Label skip;
  // TODO
  // can we avoid this skip and still use a reloc?
  __ br(Assembler::EQ, skip);
  __ far_jump(RuntimeAddress(SharedRuntime::get_ic_miss_stub()));
  __ bind(skip);
}

uint MachUEPNode::size(PhaseRegAlloc* ra_) const
{
  return MachNode::size(ra_);
}

// REQUIRED EMIT CODE

//=============================================================================

// Emit exception handler code.
int HandlerImpl::emit_exception_handler(CodeBuffer& cbuf)
{
  // mov rscratch1 #exception_blob_entry_point
  // br rscratch1
  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_exception_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();
  __ far_jump(RuntimeAddress(OptoRuntime::exception_blob()->entry_point()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

// Emit deopt handler code.
int HandlerImpl::emit_deopt_handler(CodeBuffer& cbuf)
{
  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_deopt_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();

  __ adr(lr, __ pc());
  __ far_jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));

  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

// REQUIRED MATCHER CODE

//=============================================================================

const bool Matcher::match_rule_supported(int opcode) {

  // TODO 
  // identify extra cases that we might want to provide match rules for
  // e.g. Op_StrEquals and other intrinsics
  if (!has_match_rule(opcode)) {
    return false;
  }

  return true;  // Per default match rules are supported.
}

int Matcher::regnum_to_fpu_offset(int regnum)
{
  Unimplemented();
  return 0;
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
bool Matcher::is_short_branch_offset(int rule, int br_size, int offset) {
  // The passed offset is relative to address of the branch.

  return (-32768 <= offset && offset < 32768);
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Will one (StoreL ConL) be cheaper than two (StoreI ConI)?.
  // Probably always true, even if a temp register is required.
  return true;
}

// true just means we have fast l2f conversion
const bool Matcher::convL2FSupported(void) {
  return true;
}

// Vector width in bytes.
const int Matcher::vector_width_in_bytes(BasicType bt) {
  int size = MIN2(16,(int)MaxVectorSize);
  // Minimum 2 values in vector
  if (size < 2*type2aelembytes(bt)) size = 0;
  // But never < 4
  if (size < 4) size = 0;
  return size;
}

// Limits on vector size (number of elements) loaded into vector.
const int Matcher::max_vector_size(const BasicType bt) {
  return vector_width_in_bytes(bt)/type2aelembytes(bt);
}
const int Matcher::min_vector_size(const BasicType bt) {
//  For the moment limit the vector size to 8 bytes
    int size = 8 / type2aelembytes(bt);
    if (size < 2) size = 2;
    return size;
}

// Vector ideal reg.
const uint Matcher::vector_ideal_reg(int len) {
  switch(len) {
    case  8: return Op_VecD;
    case 16: return Op_VecX;
  }
  ShouldNotReachHere();
  return 0;
}

const uint Matcher::vector_shift_count_ideal_reg(int size) {
  return Op_VecX;
}

// AES support not yet implemented
const bool Matcher::pass_original_key_for_aes() {
  return false;
}

// x86 supports misaligned vectors store/load.
const bool Matcher::misaligned_vectors_ok() {
  return !AlignVector; // can be changed by flag
}

// false => size gets scaled to BytesPerLong, ok.
const bool Matcher::init_array_count_is_in_bytes = false;

// Threshold size for cleararray.
const int Matcher::init_array_short_size = 4 * BytesPerLong;

// Use conditional move (CMOVL)
const int Matcher::long_cmove_cost() {
  // long cmoves are no more expensive than int cmoves
  return 0;
}

const int Matcher::float_cmove_cost() {
  // float cmoves are no more expensive than int cmoves
  return 0;
}

// Does the CPU require late expand (see block.cpp for description of late expand)?
const bool Matcher::require_postalloc_expand = false;

// Should the Matcher clone shifts on addressing modes, expecting them
// to be subsumed into complex addressing expressions or compute them
// into registers?  True for Intel but false for most RISCs
const bool Matcher::clone_shift_expressions = false;

// Do we need to mask the count passed to shift instructions or does
// the cpu only look at the lower 5/6 bits anyway?
const bool Matcher::need_masked_shift_count = false;

// This affects two different things:
//  - how Decode nodes are matched
//  - how ImplicitNullCheck opportunities are recognized
// If true, the matcher will try to remove all Decodes and match them
// (as operands) into nodes. NullChecks are not prepared to deal with 
// Decodes by final_graph_reshaping().
// If false, final_graph_reshaping() forces the decode behind the Cmp
// for a NullCheck. The matcher matches the Decode node into a register.
// Implicit_null_check optimization moves the Decode along with the 
// memory operation back up before the NullCheck.
bool Matcher::narrow_oop_use_complex_address() {
  return Universe::narrow_oop_shift() == 0;
}

bool Matcher::narrow_klass_use_complex_address() {
// TODO
// decide whether we need to set this to true
  return false;
}

// Is it better to copy float constants, or load them directly from
// memory?  Intel can load a float constant from a direct address,
// requiring no extra registers.  Most RISCs will have to materialize
// an address into a register first, so they would do better to copy
// the constant from stack.
const bool Matcher::rematerialize_float_constants = false;

// If CPU can load and store mis-aligned doubles directly then no
// fixup is needed.  Else we split the double into 2 integer pieces
// and move it piece-by-piece.  Only happens when passing doubles into
// C code as the Java calling convention forces doubles to be aligned.
const bool Matcher::misaligned_doubles_ok = true;

// No-op on amd64
void Matcher::pd_implicit_null_fixup(MachNode *node, uint idx) {
  Unimplemented();
}

// Advertise here if the CPU requires explicit rounding operations to
// implement the UseStrictFP mode.
const bool Matcher::strict_fp_requires_explicit_rounding = false;

// Are floats converted to double when stored to stack during
// deoptimization?
bool Matcher::float_in_double() { return true; }

// Do ints take an entire long register or just half?
// The relevant question is how the int is callee-saved:
// the whole long is written but de-opt'ing will have to extract
// the relevant 32 bits.
const bool Matcher::int_in_long = true;

// Return whether or not this register is ever used as an argument.
// This function is used on startup to build the trampoline stubs in
// generateOptoStub.  Registers not mentioned will be killed by the VM
// call in the trampoline, and arguments in those registers not be
// available to the callee.
bool Matcher::can_be_java_arg(int reg)
{
  return
    reg ==  R0_num || reg == R0_H_num ||
    reg ==  R1_num || reg == R1_H_num ||
    reg ==  R2_num || reg == R2_H_num ||
    reg ==  R3_num || reg == R3_H_num ||
    reg ==  R4_num || reg == R4_H_num ||
    reg ==  R5_num || reg == R5_H_num ||
    reg ==  R6_num || reg == R6_H_num ||
    reg ==  R7_num || reg == R7_H_num ||
    reg ==  V0_num || reg == V0_H_num ||
    reg ==  V1_num || reg == V1_H_num ||
    reg ==  V2_num || reg == V2_H_num ||
    reg ==  V3_num || reg == V3_H_num ||
    reg ==  V4_num || reg == V4_H_num ||
    reg ==  V5_num || reg == V5_H_num ||
    reg ==  V6_num || reg == V6_H_num ||
    reg ==  V7_num || reg == V7_H_num;
}

bool Matcher::is_spillable_arg(int reg)
{
  return can_be_java_arg(reg);
}

bool Matcher::use_asm_for_ldiv_by_con(jlong divisor) {
  return false;
}

RegMask Matcher::divI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODI projection of divmodI.
RegMask Matcher::modI_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for DIVL projection of divmodL.
RegMask Matcher::divL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODL projection of divmodL.
RegMask Matcher::modL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

const RegMask Matcher::method_handle_invoke_SP_save_mask() {
  return FP_REG_mask();
}

// helper for encoding java_to_runtime calls on sim
//
// this is needed to compute the extra arguments required when
// planting a call to the simulator blrt instruction. the TypeFunc
// can be queried to identify the counts for integral, and floating
// arguments and the return type

static void getCallInfo(const TypeFunc *tf, int &gpcnt, int &fpcnt, int &rtype)
{
  int gps = 0;
  int fps = 0;
  const TypeTuple *domain = tf->domain();
  int max = domain->cnt();
  for (int i = TypeFunc::Parms; i < max; i++) {
    const Type *t = domain->field_at(i);
    switch(t->basic_type()) {
    case T_FLOAT:
    case T_DOUBLE:
      fps++;
    default:
      gps++;
    }
  }
  gpcnt = gps;
  fpcnt = fps;
  BasicType rt = tf->return_type();
  switch (rt) {
  case T_VOID:
    rtype = MacroAssembler::ret_type_void;
    break;
  default:
    rtype = MacroAssembler::ret_type_integral;
    break;
  case T_FLOAT:
    rtype = MacroAssembler::ret_type_float;
    break;
  case T_DOUBLE:
    rtype = MacroAssembler::ret_type_double;
    break;
  }
}

#define MOV_VOLATILE(REG, BASE, INDEX, SCALE, DISP, SCRATCH, INSN)	\
  MacroAssembler _masm(&cbuf);						\
  {									\
    guarantee(INDEX == -1, "mode not permitted for volatile");		\
    guarantee(DISP == 0, "mode not permitted for volatile");		\
    guarantee(SCALE == 0, "mode not permitted for volatile");		\
    __ INSN(REG, as_Register(BASE));					\
  }

typedef void (MacroAssembler::* mem_insn)(Register Rt, const Address &adr);
typedef void (MacroAssembler::* mem_float_insn)(FloatRegister Rt, const Address &adr);
typedef void (MacroAssembler::* mem_vector_insn)(FloatRegister Rt,
                                  MacroAssembler::SIMD_RegVariant T, const Address &adr);

  // Used for all non-volatile memory accesses.  The use of
  // $mem->opcode() to discover whether this pattern uses sign-extended
  // offsets is something of a kludge.
  static void loadStore(MacroAssembler masm, mem_insn insn,
			 Register reg, int opcode,
			 Register base, int index, int size, int disp)
  {
    Address::extend scale;

    // Hooboy, this is fugly.  We need a way to communicate to the
    // encoder that the index needs to be sign extended, so we have to
    // enumerate all the cases.
    switch (opcode) {
    case INDINDEXSCALEDOFFSETI2L:
    case INDINDEXSCALEDI2L:
    case INDINDEXSCALEDOFFSETI2LN:
    case INDINDEXSCALEDI2LN:
    case INDINDEXOFFSETI2L:
    case INDINDEXOFFSETI2LN:
      scale = Address::sxtw(size);
      break;
    default:
      scale = Address::lsl(size);
    }

    if (index == -1) {
      (masm.*insn)(reg, Address(base, disp));
    } else {
      if (disp == 0) {
	(masm.*insn)(reg, Address(base, as_Register(index), scale));
      } else {
	masm.lea(rscratch1, Address(base, disp));
	(masm.*insn)(reg, Address(rscratch1, as_Register(index), scale));
      }
    }
  }

  static void loadStore(MacroAssembler masm, mem_float_insn insn,
			 FloatRegister reg, int opcode,
			 Register base, int index, int size, int disp)
  {
    Address::extend scale;

    switch (opcode) {
    case INDINDEXSCALEDOFFSETI2L:
    case INDINDEXSCALEDI2L:
    case INDINDEXSCALEDOFFSETI2LN:
    case INDINDEXSCALEDI2LN:
      scale = Address::sxtw(size);
      break;
    default:
      scale = Address::lsl(size);
    }

     if (index == -1) {
      (masm.*insn)(reg, Address(base, disp));
    } else {
      if (disp == 0) {
	(masm.*insn)(reg, Address(base, as_Register(index), scale));
      } else {
	masm.lea(rscratch1, Address(base, disp));
	(masm.*insn)(reg, Address(rscratch1, as_Register(index), scale));
      }
    }
  }

  static void loadStore(MacroAssembler masm, mem_vector_insn insn,
                         FloatRegister reg, MacroAssembler::SIMD_RegVariant T,
                         int opcode, Register base, int index, int size, int disp)
  {
    if (index == -1) {
      (masm.*insn)(reg, T, Address(base, disp));
    } else {
      assert(disp == 0, "unsupported address mode");
      (masm.*insn)(reg, T, Address(base, as_Register(index), Address::lsl(size)));
    }
  }


#line 2813 "../generated/adfiles/ad_aarch64.cpp"


#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
  assert( INSN_COST == 100, "Expect (100) to equal 100");
  assert( BRANCH_COST == 200, "Expect (2 * INSN_COST) to equal 200");
  assert( CALL_COST == 200, "Expect (2 * INSN_COST) to equal 200");
  assert( VOLATILE_REF_COST == 1000, "Expect (10 * INSN_COST) to equal 1000");
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C', // R10
  'C', // R10_H
  'C', // R11
  'C', // R11_H
  'C', // R12
  'C', // R12_H
  'C', // R13
  'C', // R13_H
  'C', // R14
  'C', // R14_H
  'C', // R15
  'C', // R15_H
  'C', // R16
  'C', // R16_H
  'C', // R17
  'C', // R17_H
  'C', // R18
  'C', // R18_H
  'C', // R0
  'C', // R0_H
  'C', // R1
  'C', // R1_H
  'C', // R2
  'C', // R2_H
  'C', // R3
  'C', // R3_H
  'C', // R4
  'C', // R4_H
  'C', // R5
  'C', // R5_H
  'C', // R6
  'C', // R6_H
  'C', // R7
  'C', // R7_H
  'C', // R19
  'C', // R19_H
  'C', // R20
  'C', // R20_H
  'C', // R21
  'C', // R21_H
  'C', // R22
  'C', // R22_H
  'C', // R23
  'C', // R23_H
  'C', // R24
  'C', // R24_H
  'C', // R25
  'C', // R25_H
  'C', // R26
  'C', // R26_H
  'N', // R27
  'N', // R27_H
  'N', // R28
  'N', // R28_H
  'N', // R29
  'N', // R29_H
  'N', // R30
  'N', // R30_H
  'N', // R31
  'N', // R31_H
  'C', // V16
  'C', // V16_H
  'C', // V16_J
  'C', // V16_K
  'C', // V17
  'C', // V17_H
  'C', // V17_J
  'C', // V17_K
  'C', // V18
  'C', // V18_H
  'C', // V18_J
  'C', // V18_K
  'C', // V19
  'C', // V19_H
  'C', // V19_J
  'C', // V19_K
  'C', // V20
  'C', // V20_H
  'C', // V20_J
  'C', // V20_K
  'C', // V21
  'C', // V21_H
  'C', // V21_J
  'C', // V21_K
  'C', // V22
  'C', // V22_H
  'C', // V22_J
  'C', // V22_K
  'C', // V23
  'C', // V23_H
  'C', // V23_J
  'C', // V23_K
  'C', // V24
  'C', // V24_H
  'C', // V24_J
  'C', // V24_K
  'C', // V25
  'C', // V25_H
  'C', // V25_J
  'C', // V25_K
  'C', // V26
  'C', // V26_H
  'C', // V26_J
  'C', // V26_K
  'C', // V27
  'C', // V27_H
  'C', // V27_J
  'C', // V27_K
  'C', // V28
  'C', // V28_H
  'C', // V28_J
  'C', // V28_K
  'C', // V29
  'C', // V29_H
  'C', // V29_J
  'C', // V29_K
  'C', // V30
  'C', // V30_H
  'C', // V30_J
  'C', // V30_K
  'C', // V31
  'C', // V31_H
  'C', // V31_J
  'C', // V31_K
  'C', // V0
  'C', // V0_H
  'C', // V0_J
  'C', // V0_K
  'C', // V1
  'C', // V1_H
  'C', // V1_J
  'C', // V1_K
  'C', // V2
  'C', // V2_H
  'C', // V2_J
  'C', // V2_K
  'C', // V3
  'C', // V3_H
  'C', // V3_J
  'C', // V3_K
  'C', // V4
  'C', // V4_H
  'C', // V4_J
  'C', // V4_K
  'C', // V5
  'C', // V5_H
  'C', // V5_J
  'C', // V5_K
  'C', // V6
  'C', // V6_H
  'C', // V6_J
  'C', // V6_K
  'C', // V7
  'C', // V7_H
  'C', // V7_J
  'C', // V7_K
  'C', // V8
  'C', // V8_H
  'C', // V8_J
  'C', // V8_K
  'C', // V9
  'C', // V9_H
  'C', // V9_J
  'C', // V9_K
  'C', // V10
  'C', // V10_H
  'C', // V10_J
  'C', // V10_K
  'C', // V11
  'C', // V11_H
  'C', // V11_J
  'C', // V11_K
  'C', // V12
  'C', // V12_H
  'C', // V12_J
  'C', // V12_K
  'C', // V13
  'C', // V13_H
  'C', // V13_J
  'C', // V13_K
  'C', // V14
  'C', // V14_H
  'C', // V14_J
  'C', // V14_K
  'C', // V15
  'C', // V15_H
  'C', // V15_J
  'C', // V15_K
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C', // R10
  'C', // R10_H
  'C', // R11
  'C', // R11_H
  'C', // R12
  'C', // R12_H
  'C', // R13
  'C', // R13_H
  'C', // R14
  'C', // R14_H
  'C', // R15
  'C', // R15_H
  'C', // R16
  'C', // R16_H
  'C', // R17
  'C', // R17_H
  'C', // R18
  'C', // R18_H
  'C', // R0
  'C', // R0_H
  'C', // R1
  'C', // R1_H
  'C', // R2
  'C', // R2_H
  'C', // R3
  'C', // R3_H
  'C', // R4
  'C', // R4_H
  'C', // R5
  'C', // R5_H
  'C', // R6
  'C', // R6_H
  'C', // R7
  'C', // R7_H
  'E', // R19
  'E', // R19_H
  'E', // R20
  'E', // R20_H
  'E', // R21
  'E', // R21_H
  'E', // R22
  'E', // R22_H
  'E', // R23
  'E', // R23_H
  'E', // R24
  'E', // R24_H
  'E', // R25
  'E', // R25_H
  'E', // R26
  'E', // R26_H
  'E', // R27
  'E', // R27_H
  'E', // R28
  'E', // R28_H
  'N', // R29
  'N', // R29_H
  'N', // R30
  'N', // R30_H
  'N', // R31
  'N', // R31_H
  'C', // V16
  'C', // V16_H
  'C', // V16_J
  'C', // V16_K
  'C', // V17
  'C', // V17_H
  'C', // V17_J
  'C', // V17_K
  'C', // V18
  'C', // V18_H
  'C', // V18_J
  'C', // V18_K
  'C', // V19
  'C', // V19_H
  'C', // V19_J
  'C', // V19_K
  'C', // V20
  'C', // V20_H
  'C', // V20_J
  'C', // V20_K
  'C', // V21
  'C', // V21_H
  'C', // V21_J
  'C', // V21_K
  'C', // V22
  'C', // V22_H
  'C', // V22_J
  'C', // V22_K
  'C', // V23
  'C', // V23_H
  'C', // V23_J
  'C', // V23_K
  'C', // V24
  'C', // V24_H
  'C', // V24_J
  'C', // V24_K
  'C', // V25
  'C', // V25_H
  'C', // V25_J
  'C', // V25_K
  'C', // V26
  'C', // V26_H
  'C', // V26_J
  'C', // V26_K
  'C', // V27
  'C', // V27_H
  'C', // V27_J
  'C', // V27_K
  'C', // V28
  'C', // V28_H
  'C', // V28_J
  'C', // V28_K
  'C', // V29
  'C', // V29_H
  'C', // V29_J
  'C', // V29_K
  'C', // V30
  'C', // V30_H
  'C', // V30_J
  'C', // V30_K
  'C', // V31
  'C', // V31_H
  'C', // V31_J
  'C', // V31_K
  'C', // V0
  'C', // V0_H
  'C', // V0_J
  'C', // V0_K
  'C', // V1
  'C', // V1_H
  'C', // V1_J
  'C', // V1_K
  'C', // V2
  'C', // V2_H
  'C', // V2_J
  'C', // V2_K
  'C', // V3
  'C', // V3_H
  'C', // V3_J
  'C', // V3_K
  'C', // V4
  'C', // V4_H
  'C', // V4_J
  'C', // V4_K
  'C', // V5
  'C', // V5_H
  'C', // V5_J
  'C', // V5_K
  'C', // V6
  'C', // V6_H
  'C', // V6_J
  'C', // V6_K
  'C', // V7
  'C', // V7_H
  'C', // V7_J
  'C', // V7_K
  'C', // V8
  'C', // V8_H
  'C', // V8_J
  'C', // V8_K
  'C', // V9
  'C', // V9_H
  'C', // V9_J
  'C', // V9_K
  'C', // V10
  'C', // V10_H
  'C', // V10_J
  'C', // V10_K
  'C', // V11
  'C', // V11_H
  'C', // V11_J
  'C', // V11_K
  'C', // V12
  'C', // V12_H
  'C', // V12_J
  'C', // V12_K
  'C', // V13
  'C', // V13_H
  'C', // V13_J
  'C', // V13_K
  'C', // V14
  'C', // V14_H
  'C', // V14_J
  'C', // V14_K
  'C', // V15
  'C', // V15_H
  'C', // V15_J
  'C', // V15_K
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  0 // no trailing comma
};


const        int   reduceOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  immI_rule,
  /*    9 */  immI0_rule,
  /*   10 */  immI_1_rule,
  /*   11 */  immI_M1_rule,
  /*   12 */  immI_le_4_rule,
  /*   13 */  immI_31_rule,
  /*   14 */  immI_8_rule,
  /*   15 */  immI_16_rule,
  /*   16 */  immI_24_rule,
  /*   17 */  immI_32_rule,
  /*   18 */  immI_48_rule,
  /*   19 */  immI_56_rule,
  /*   20 */  immI_64_rule,
  /*   21 */  immI_255_rule,
  /*   22 */  immI_65535_rule,
  /*   23 */  immL_63_rule,
  /*   24 */  immL_255_rule,
  /*   25 */  immL_65535_rule,
  /*   26 */  immL_4294967295_rule,
  /*   27 */  immL_bitmask_rule,
  /*   28 */  immI_bitmask_rule,
  /*   29 */  immIScale_rule,
  /*   30 */  immI26_rule,
  /*   31 */  immI19_rule,
  /*   32 */  immIU12_rule,
  /*   33 */  immLU12_rule,
  /*   34 */  immIOffset_rule,
  /*   35 */  immIOffset4_rule,
  /*   36 */  immIOffset8_rule,
  /*   37 */  immIOffset16_rule,
  /*   38 */  immLoffset_rule,
  /*   39 */  immLoffset4_rule,
  /*   40 */  immLoffset8_rule,
  /*   41 */  immLoffset16_rule,
  /*   42 */  immIAddSub_rule,
  /*   43 */  immILog_rule,
  /*   44 */  immL_rule,
  /*   45 */  immL0_rule,
  /*   46 */  immL_1_rule,
  /*   47 */  immL_M1_rule,
  /*   48 */  immL_pc_off_rule,
  /*   49 */  immLAddSub_rule,
  /*   50 */  immLLog_rule,
  /*   51 */  immL_32bits_rule,
  /*   52 */  immP_rule,
  /*   53 */  immP0_rule,
  /*   54 */  immP_1_rule,
  /*   55 */  immPollPage_rule,
  /*   56 */  immByteMapBase_rule,
  /*   57 */  immP_M1_rule,
  /*   58 */  immP_M2_rule,
  /*   59 */  immD_rule,
  /*   60 */  immD0_rule,
  /*   61 */  immDPacked_rule,
  /*   62 */  immF_rule,
  /*   63 */  immF0_rule,
  /*   64 */  immFPacked_rule,
  /*   65 */  immN_rule,
  /*   66 */  immN0_rule,
  /*   67 */  immNKlass_rule,
  /*   68 */  iRegI_rule,
  /*   69 */  iRegINoSp_rule,
  /*   70 */  iRegL_rule,
  /*   71 */  iRegLNoSp_rule,
  /*   72 */  iRegP_rule,
  /*   73 */  iRegPNoSp_rule,
  /*   74 */  iRegP_R0_rule,
  /*   75 */  iRegP_R1_rule,
  /*   76 */  iRegP_R2_rule,
  /*   77 */  iRegP_R3_rule,
  /*   78 */  iRegP_R4_rule,
  /*   79 */  iRegP_R5_rule,
  /*   80 */  iRegP_R10_rule,
  /*   81 */  iRegL_R11_rule,
  /*   82 */  iRegP_FP_rule,
  /*   83 */  iRegI_R0_rule,
  /*   84 */  iRegI_R2_rule,
  /*   85 */  iRegI_R3_rule,
  /*   86 */  iRegI_R4_rule,
  /*   87 */  iRegN_rule,
  /*   88 */  iRegNNoSp_rule,
  /*   89 */  iRegIHeapbase_rule,
  /*   90 */  vRegF_rule,
  /*   91 */  vRegD_rule,
  /*   92 */  vecD_rule,
  /*   93 */  vecX_rule,
  /*   94 */  vRegD_V0_rule,
  /*   95 */  vRegD_V1_rule,
  /*   96 */  vRegD_V2_rule,
  /*   97 */  vRegD_V3_rule,
  /*   98 */  rFlagsReg_rule,
  /*   99 */  rFlagsRegU_rule,
  /*  100 */  inline_cache_RegP_rule,
  /*  101 */  interpreter_method_oop_RegP_rule,
  /*  102 */  thread_RegP_rule,
  /*  103 */  lr_RegP_rule,
  /*  104 */  indirect_rule,
  /*  105 */  indIndexScaledOffsetI_rule,
  /*  106 */  indIndexScaledOffsetL_rule,
  /*  107 */  indIndexOffsetI2L_rule,
  /*  108 */  indIndexScaledOffsetI2L_rule,
  /*  109 */  indIndexScaledI2L_rule,
  /*  110 */  indIndexScaled_rule,
  /*  111 */  indIndex_rule,
  /*  112 */  indOffI_rule,
  /*  113 */  indOffI4_rule,
  /*  114 */  indOffI8_rule,
  /*  115 */  indOffI16_rule,
  /*  116 */  indOffL_rule,
  /*  117 */  indOffL4_rule,
  /*  118 */  indOffL8_rule,
  /*  119 */  indOffL16_rule,
  /*  120 */  indirectN_rule,
  /*  121 */  indIndexScaledOffsetIN_rule,
  /*  122 */  indIndexScaledOffsetLN_rule,
  /*  123 */  indIndexOffsetI2LN_rule,
  /*  124 */  indIndexScaledOffsetI2LN_rule,
  /*  125 */  indIndexScaledI2LN_rule,
  /*  126 */  indIndexScaledN_rule,
  /*  127 */  indIndexN_rule,
  /*  128 */  indOffIN_rule,
  /*  129 */  indOffLN_rule,
  /*  130 */  thread_anchor_pc_rule,
  /*  131 */  stackSlotP_rule,
  /*  132 */  stackSlotI_rule,
  /*  133 */  stackSlotF_rule,
  /*  134 */  stackSlotD_rule,
  /*  135 */  stackSlotL_rule,
  /*  136 */  cmpOp_rule,
  /*  137 */  cmpOpU_rule,
  /*  138 */  iRegL2I_rule,
  // last operand
  /*  139 */  vmem4_rule,
  /*  140 */  vmem8_rule,
  /*  141 */  vmem16_rule,
  /*  142 */  memory_rule,
  /*  143 */  iRegIorL2I_rule,
  // last operand class
  /*  144 */  _LShiftL_iRegL_immIScale_rule,
  /*  145 */  _AddP_iRegP__LShiftL_iRegL_immIScale_rule,
  /*  146 */  _ConvI2L_iRegI__rule,
  /*  147 */  _AddP_iRegP__ConvI2L_iRegI__rule,
  /*  148 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  149 */  _AddP_iRegP__LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  150 */  _DecodeN_iRegN__rule,
  /*  151 */  _AddP__DecodeN_iRegN___LShiftL_iRegL_immIScale_rule,
  /*  152 */  _AddP__DecodeN_iRegN___ConvI2L_iRegI__rule,
  /*  153 */  _AddP__DecodeN_iRegN___LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  154 */  _LoadB_memory__rule,
  /*  155 */  _LoadUB_memory__rule,
  /*  156 */  _LoadS_memory__rule,
  /*  157 */  _LoadUS_memory__rule,
  /*  158 */  _LoadI_memory__rule,
  /*  159 */  _ConvI2L__LoadI_memory___rule,
  /*  160 */  _LoadB_indirect__rule,
  /*  161 */  _LoadUB_indirect__rule,
  /*  162 */  _LoadUS_indirect__rule,
  /*  163 */  _LoadS_indirect__rule,
  /*  164 */  _LoadI_indirect__rule,
  /*  165 */  _ConvI2L__LoadI_indirect___rule,
  /*  166 */  _LoadL_memory__rule,
  /*  167 */  _CastP2X_iRegP__rule,
  /*  168 */  _CastP2X__DecodeN_iRegN___rule,
  /*  169 */  _Binary_iRegP_iRegP_rule,
  /*  170 */  _Binary_iRegLNoSp_iRegLNoSp_rule,
  /*  171 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  172 */  _Binary_iRegNNoSp_iRegNNoSp_rule,
  /*  173 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  174 */  _Binary_iRegIorL2I_iRegIorL2I_rule,
  /*  175 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  176 */  _Binary_immI0_iRegIorL2I_rule,
  /*  177 */  _Binary_iRegIorL2I_immI0_rule,
  /*  178 */  _Binary_immI_1_immI0_rule,
  /*  179 */  _Binary_iRegL_iRegL_rule,
  /*  180 */  _Binary_iRegL_immL0_rule,
  /*  181 */  _Binary_immL0_iRegL_rule,
  /*  182 */  _Binary_iRegP_immP0_rule,
  /*  183 */  _Binary_immP0_iRegP_rule,
  /*  184 */  _Binary_iRegN_iRegN_rule,
  /*  185 */  _Binary_iRegN_immN0_rule,
  /*  186 */  _Binary_immN0_iRegN_rule,
  /*  187 */  _Binary_vRegF_vRegF_rule,
  /*  188 */  _Binary_vRegD_vRegD_rule,
  /*  189 */  _ConvL2I_iRegL__rule,
  /*  190 */  _ConvI2L_iRegIorL2I__rule,
  /*  191 */  _LShiftL__ConvI2L_iRegIorL2I__immIScale_rule,
  /*  192 */  _MulI_iRegIorL2I_iRegIorL2I_rule,
  /*  193 */  _MulL_iRegL_iRegL_rule,
  /*  194 */  _RShiftI_iRegIorL2I_immI_31_rule,
  /*  195 */  _URShiftI__RShiftI_iRegIorL2I_immI_31_immI_31_rule,
  /*  196 */  _RShiftL_iRegL_immL_63_rule,
  /*  197 */  _URShiftL__RShiftL_iRegL_immL_63_immL_63_rule,
  /*  198 */  _XorI_iRegIorL2I_immI_M1_rule,
  /*  199 */  _XorL_iRegL_immL_M1_rule,
  /*  200 */  _XorI_iRegIorL2I_iRegIorL2I_rule,
  /*  201 */  _XorL_iRegL_iRegL_rule,
  /*  202 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  203 */  _XorI__URShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  204 */  _URShiftL_iRegL_immI_rule,
  /*  205 */  _XorL__URShiftL_iRegL_immI_immL_M1_rule,
  /*  206 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  207 */  _XorI__RShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  208 */  _RShiftL_iRegL_immI_rule,
  /*  209 */  _XorL__RShiftL_iRegL_immI_immL_M1_rule,
  /*  210 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  211 */  _XorI__LShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  212 */  _LShiftL_iRegL_immI_rule,
  /*  213 */  _XorL__LShiftL_iRegL_immI_immL_M1_rule,
  /*  214 */  _XorI__URShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  215 */  _XorI_iRegIorL2I__URShiftI_iRegIorL2I_immI_rule,
  /*  216 */  _XorL__URShiftL_iRegL_immI_iRegL_rule,
  /*  217 */  _XorL_iRegL__URShiftL_iRegL_immI_rule,
  /*  218 */  _XorI__RShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  219 */  _XorI_iRegIorL2I__RShiftI_iRegIorL2I_immI_rule,
  /*  220 */  _XorL__RShiftL_iRegL_immI_iRegL_rule,
  /*  221 */  _XorL_iRegL__RShiftL_iRegL_immI_rule,
  /*  222 */  _XorI__LShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  223 */  _XorI_iRegIorL2I__LShiftI_iRegIorL2I_immI_rule,
  /*  224 */  _XorL__LShiftL_iRegL_immI_iRegL_rule,
  /*  225 */  _XorL_iRegL__LShiftL_iRegL_immI_rule,
  /*  226 */  _AndI__URShiftI_iRegIorL2I_immI_immI_bitmask_rule,
  /*  227 */  _LShiftL_iRegL_iRegI_rule,
  /*  228 */  _SubI_immI_64_iRegI_rule,
  /*  229 */  _URShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  230 */  _SubI_immI0_iRegI_rule,
  /*  231 */  _URShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  232 */  _LShiftI_iRegI_iRegI_rule,
  /*  233 */  _SubI_immI_32_iRegI_rule,
  /*  234 */  _URShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  235 */  _URShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  236 */  _URShiftL_iRegL_iRegI_rule,
  /*  237 */  _LShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  238 */  _LShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  239 */  _URShiftI_iRegI_iRegI_rule,
  /*  240 */  _LShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  241 */  _LShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  242 */  _LShiftI_iRegIorL2I_immI_16_rule,
  /*  243 */  _RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_rule,
  /*  244 */  _LShiftI_iRegIorL2I_immI_24_rule,
  /*  245 */  _RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  246 */  _URShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  247 */  _LShiftL_iRegL_immI_48_rule,
  /*  248 */  _RShiftL__LShiftL_iRegL_immI_48_immI_48_rule,
  /*  249 */  _LShiftL_iRegL_immI_32_rule,
  /*  250 */  _RShiftL__LShiftL_iRegL_immI_32_immI_32_rule,
  /*  251 */  _LShiftL_iRegL_immI_56_rule,
  /*  252 */  _RShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  253 */  _URShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  254 */  _AndI_iRegIorL2I_immI_255_rule,
  /*  255 */  _AndI_iRegIorL2I_immI_65535_rule,
  /*  256 */  _AndL_iRegL_immL_255_rule,
  /*  257 */  _AndL_iRegL_immL_65535_rule,
  /*  258 */  _AndL_iRegL_immL_4294967295_rule,
  /*  259 */  _ConvF2D_vRegF__rule,
  /*  260 */  _SqrtD__ConvF2D_vRegF___rule,
  /*  261 */  _OverflowMulI_iRegIorL2I_iRegIorL2I_rule,
  /*  262 */  _OverflowMulL_iRegL_iRegL_rule,
  /*  263 */  _CmpI_iRegIorL2I_immI0_rule,
  /*  264 */  _CmpL_iRegL_immL0_rule,
  /*  265 */  _CmpP_iRegP_immP0_rule,
  /*  266 */  _CmpN_iRegN_immN0_rule,
  /*  267 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  268 */  _CmpU_iRegIorL2I_immI0_rule,
  /*  269 */  _CmpUL_iRegL_immL0_rule,
  /*  270 */  _AndL_iRegL_immL_rule,
  /*  271 */  _CmpL__AndL_iRegL_immL_immL0_rule,
  /*  272 */  _AndI_iRegIorL2I_immI_rule,
  /*  273 */  _CmpI__AndI_iRegIorL2I_immI_immI0_rule,
  /*  274 */  _AndL_iRegL_iRegL_rule,
  /*  275 */  _AndI_iRegIorL2I_iRegIorL2I_rule,
  /*  276 */  _PartialSubtypeCheck_iRegP_R4_iRegP_R0_rule,
  /*  277 */  _Binary_iRegP_R1_iRegI_R2_rule,
  /*  278 */  _Binary_iRegP_R3_iRegI_R4_rule,
  /*  279 */  _Binary_iRegP_R1_iRegI_R4_rule,
  /*  280 */  _Binary_iRegP_R3_iRegI_R2_rule,
  /*  281 */  _Binary_iRegP_R3_immI_le_4_rule,
  /*  282 */  _Binary_iRegP_R1_iRegP_R3_rule,
  /*  283 */  _Binary_iRegP_R1_iRegI_R3_rule,
  /*  284 */  _MulVS_vecD_vecD_rule,
  /*  285 */  _MulVS_vecX_vecX_rule,
  /*  286 */  _MulVI_vecD_vecD_rule,
  /*  287 */  _MulVI_vecX_vecX_rule,
  // last internally defined operand
  /*  288 */  iRegINoSp_rule,
  /*  289 */  iRegLNoSp_rule,
  /*  290 */  iRegPNoSp_rule,
  /*  291 */  iRegPNoSp_rule,
  /*  292 */  iRegPNoSp_rule,
  /*  293 */  iRegPNoSp_rule,
  /*  294 */  iRegPNoSp_rule,
  /*  295 */  iRegNNoSp_rule,
  /*  296 */  iRegNNoSp_rule,
  /*  297 */  iRegNNoSp_rule,
  /*  298 */  vRegF_rule,
  /*  299 */  vRegF_rule,
  /*  300 */  vRegD_rule,
  /*  301 */  vRegD_rule,
  /*  302 */  iRegINoSp_rule,
  /*  303 */  iRegINoSp_rule,
  /*  304 */  iRegINoSp_rule,
  /*  305 */  iRegINoSp_rule,
  /*  306 */  iRegPNoSp_rule,
  /*  307 */  iRegLNoSp_rule,
  /*  308 */  iRegPNoSp_rule,
  /*  309 */  iRegPNoSp_rule,
  /*  310 */  iRegI_rule,
  /*  311 */  rFlagsReg_rule,
  /*  312 */  rFlagsReg_rule,
  /*  313 */  rFlagsReg_rule,
  /*  314 */  vRegF_rule,
  /*  315 */  vRegD_rule,
  /*  316 */  vRegF_rule,
  /*  317 */  vRegD_rule,
  /*  318 */  iRegINoSp_rule,
  /*  319 */  vRegF_rule,
  /*  320 */  iRegLNoSp_rule,
  /*  321 */  vRegD_rule,
  /*  322 */  iRegINoSp_rule,
  /*  323 */  vRegF_rule,
  /*  324 */  iRegLNoSp_rule,
  /*  325 */  vRegD_rule,
  /*  326 */  rFlagsReg_rule,
  /*  327 */  rFlagsReg_rule,
  /*  328 */  rFlagsReg_rule,
  /*  329 */  rFlagsReg_rule,
  /*  330 */  rFlagsReg_rule,
  /*  331 */  rFlagsReg_rule,
  /*  332 */  rFlagsReg_rule,
  /*  333 */  rFlagsReg_rule,
  /*  334 */  rFlagsReg_rule,
  /*  335 */  rFlagsReg_rule,
  /*  336 */  rFlagsReg_rule,
  /*  337 */  rFlagsReg_rule,
  /*  338 */  rFlagsReg_rule,
  /*  339 */  rFlagsReg_rule,
  /*  340 */  rFlagsReg_rule,
  /*  341 */  rFlagsReg_rule,
  /*  342 */  rFlagsRegU_rule,
  /*  343 */  rFlagsRegU_rule,
  /*  344 */  rFlagsRegU_rule,
  /*  345 */  rFlagsRegU_rule,
  /*  346 */  rFlagsReg_rule,
  /*  347 */  rFlagsReg_rule,
  /*  348 */  rFlagsReg_rule,
  /*  349 */  rFlagsReg_rule,
  /*  350 */  rFlagsRegU_rule,
  /*  351 */  rFlagsRegU_rule,
  /*  352 */  rFlagsRegU_rule,
  /*  353 */  rFlagsRegU_rule,
  /*  354 */  rFlagsRegU_rule,
  /*  355 */  rFlagsRegU_rule,
  /*  356 */  rFlagsRegU_rule,
  /*  357 */  rFlagsRegU_rule,
  /*  358 */  rFlagsReg_rule,
  /*  359 */  rFlagsReg_rule,
  /*  360 */  rFlagsReg_rule,
  /*  361 */  rFlagsReg_rule,
  /*  362 */  rFlagsReg_rule,
  /*  363 */  rFlagsReg_rule,
  /*  364 */  rFlagsReg_rule,
  /*  365 */  rFlagsReg_rule,
  /*  366 */  rFlagsReg_rule,
  /*  367 */  rFlagsReg_rule,
  /*  368 */  rFlagsReg_rule,
  /*  369 */  thread_RegP_rule,
  /*  370 */  vecX_rule,
  /*  371 */  vecX_rule,
  /*  372 */  iRegINoSp_rule,
  /*  373 */  iRegLNoSp_rule,
  /*  374 */  iRegINoSp_rule,
  /*  375 */  iRegLNoSp_rule,
  /*  376 */  iRegINoSp_rule,
  /*  377 */  iRegLNoSp_rule,
  /*  378 */  iRegINoSp_rule,
  /*  379 */  iRegLNoSp_rule,
  /*  380 */  iRegINoSp_rule,
  /*  381 */  iRegLNoSp_rule,
  /*  382 */  iRegLNoSp_rule,
  /*  383 */  iRegLNoSp_rule,
  /*  384 */  iRegINoSp_rule,
  /*  385 */  iRegPNoSp_rule,
  /*  386 */  iRegNNoSp_rule,
  /*  387 */  iRegPNoSp_rule,
  /*  388 */  iRegNNoSp_rule,
  /*  389 */  vRegF_rule,
  /*  390 */  vRegD_rule,
  /*  391 */  Universe_rule,
  /*  392 */  Universe_rule,
  /*  393 */  Universe_rule,
  /*  394 */  Universe_rule,
  /*  395 */  Universe_rule,
  /*  396 */  Universe_rule,
  /*  397 */  Universe_rule,
  /*  398 */  Universe_rule,
  /*  399 */  Universe_rule,
  /*  400 */  Universe_rule,
  /*  401 */  Universe_rule,
  /*  402 */  Universe_rule,
  /*  403 */  Universe_rule,
  /*  404 */  Universe_rule,
  /*  405 */  Universe_rule,
  /*  406 */  Universe_rule,
  /*  407 */  Universe_rule,
  /*  408 */  Universe_rule,
  /*  409 */  Universe_rule,
  /*  410 */  Universe_rule,
  /*  411 */  iRegINoSp_rule,
  /*  412 */  iRegLNoSp_rule,
  /*  413 */  iRegINoSp_rule,
  /*  414 */  iRegLNoSp_rule,
  /*  415 */  iRegINoSp_rule,
  /*  416 */  iRegINoSp_rule,
  /*  417 */  iRegLNoSp_rule,
  /*  418 */  iRegLNoSp_rule,
  /*  419 */  iRegINoSp_rule,
  /*  420 */  iRegLNoSp_rule,
  /*  421 */  iRegLNoSp_rule,
  /*  422 */  iRegPNoSp_rule,
  /*  423 */  iRegNNoSp_rule,
  /*  424 */  vRegF_rule,
  /*  425 */  vRegD_rule,
  /*  426 */  Universe_rule,
  /*  427 */  Universe_rule,
  /*  428 */  Universe_rule,
  /*  429 */  Universe_rule,
  /*  430 */  Universe_rule,
  /*  431 */  Universe_rule,
  /*  432 */  Universe_rule,
  /*  433 */  Universe_rule,
  /*  434 */  iRegINoSp_rule,
  /*  435 */  iRegLNoSp_rule,
  /*  436 */  iRegINoSp_rule,
  /*  437 */  iRegINoSp_rule,
  /*  438 */  iRegINoSp_rule,
  /*  439 */  iRegINoSp_rule,
  /*  440 */  iRegINoSp_rule,
  /*  441 */  iRegINoSp_rule,
  /*  442 */  Universe_rule,
  /*  443 */  Universe_rule,
  /*  444 */  Universe_rule,
  /*  445 */  Universe_rule,
  /*  446 */  Universe_rule,
  /*  447 */  Universe_rule,
  /*  448 */  Universe_rule,
  /*  449 */  Universe_rule,
  /*  450 */  Universe_rule,
  /*  451 */  Universe_rule,
  /*  452 */  Universe_rule,
  /*  453 */  iRegINoSp_rule,
  /*  454 */  iRegINoSp_rule,
  /*  455 */  iRegNNoSp_rule,
  /*  456 */  iRegNNoSp_rule,
  /*  457 */  iRegPNoSp_rule,
  /*  458 */  iRegPNoSp_rule,
  /*  459 */  iRegNNoSp_rule,
  /*  460 */  iRegPNoSp_rule,
  /*  461 */  iRegPNoSp_rule,
  /*  462 */  iRegINoSp_rule,
  /*  463 */  iRegINoSp_rule,
  /*  464 */  iRegINoSp_rule,
  /*  465 */  iRegINoSp_rule,
  /*  466 */  iRegINoSp_rule,
  /*  467 */  iRegINoSp_rule,
  /*  468 */  iRegINoSp_rule,
  /*  469 */  iRegINoSp_rule,
  /*  470 */  iRegINoSp_rule,
  /*  471 */  iRegLNoSp_rule,
  /*  472 */  iRegINoSp_rule,
  /*  473 */  iRegPNoSp_rule,
  /*  474 */  iRegLNoSp_rule,
  /*  475 */  Universe_rule,
  /*  476 */  iRegLNoSp_rule,
  /*  477 */  Universe_rule,
  /*  478 */  iRegINoSp_rule,
  /*  479 */  Universe_rule,
  /*  480 */  iRegINoSp_rule,
  /*  481 */  Universe_rule,
  /*  482 */  iRegINoSp_rule,
  /*  483 */  iRegINoSp_rule,
  /*  484 */  iRegINoSp_rule,
  /*  485 */  iRegINoSp_rule,
  /*  486 */  iRegINoSp_rule,
  /*  487 */  iRegINoSp_rule,
  /*  488 */  iRegINoSp_rule,
  /*  489 */  iRegINoSp_rule,
  /*  490 */  iRegLNoSp_rule,
  /*  491 */  iRegLNoSp_rule,
  /*  492 */  iRegLNoSp_rule,
  /*  493 */  iRegLNoSp_rule,
  /*  494 */  iRegLNoSp_rule,
  /*  495 */  iRegLNoSp_rule,
  /*  496 */  iRegPNoSp_rule,
  /*  497 */  iRegPNoSp_rule,
  /*  498 */  iRegPNoSp_rule,
  /*  499 */  iRegPNoSp_rule,
  /*  500 */  iRegPNoSp_rule,
  /*  501 */  iRegPNoSp_rule,
  /*  502 */  iRegNNoSp_rule,
  /*  503 */  iRegNNoSp_rule,
  /*  504 */  iRegNNoSp_rule,
  /*  505 */  iRegNNoSp_rule,
  /*  506 */  iRegNNoSp_rule,
  /*  507 */  iRegNNoSp_rule,
  /*  508 */  vRegF_rule,
  /*  509 */  vRegF_rule,
  /*  510 */  vRegD_rule,
  /*  511 */  vRegD_rule,
  /*  512 */  iRegINoSp_rule,
  /*  513 */  iRegINoSp_rule,
  /*  514 */  iRegINoSp_rule,
  /*  515 */  iRegPNoSp_rule,
  /*  516 */  iRegPNoSp_rule,
  /*  517 */  iRegPNoSp_rule,
  /*  518 */  iRegPNoSp_rule,
  /*  519 */  iRegLNoSp_rule,
  /*  520 */  iRegPNoSp_rule,
  /*  521 */  iRegLNoSp_rule,
  /*  522 */  iRegLNoSp_rule,
  /*  523 */  iRegINoSp_rule,
  /*  524 */  iRegINoSp_rule,
  /*  525 */  iRegLNoSp_rule,
  /*  526 */  iRegLNoSp_rule,
  /*  527 */  iRegINoSp_rule,
  /*  528 */  iRegLNoSp_rule,
  /*  529 */  iRegINoSp_rule,
  /*  530 */  iRegLNoSp_rule,
  /*  531 */  iRegLNoSp_rule,
  /*  532 */  iRegLNoSp_rule,
  /*  533 */  iRegLNoSp_rule,
  /*  534 */  iRegINoSp_rule,
  /*  535 */  iRegINoSp_rule,
  /*  536 */  iRegINoSp_rule,
  /*  537 */  iRegLNoSp_rule,
  /*  538 */  iRegLNoSp_rule,
  /*  539 */  iRegLNoSp_rule,
  /*  540 */  iRegINoSp_rule,
  /*  541 */  iRegINoSp_rule,
  /*  542 */  iRegINoSp_rule,
  /*  543 */  iRegINoSp_rule,
  /*  544 */  iRegLNoSp_rule,
  /*  545 */  iRegLNoSp_rule,
  /*  546 */  iRegLNoSp_rule,
  /*  547 */  iRegLNoSp_rule,
  /*  548 */  iRegINoSp_rule,
  /*  549 */  iRegLNoSp_rule,
  /*  550 */  iRegINoSp_rule,
  /*  551 */  iRegINoSp_rule,
  /*  552 */  iRegINoSp_rule,
  /*  553 */  iRegINoSp_rule,
  /*  554 */  iRegINoSp_rule,
  /*  555 */  iRegINoSp_rule,
  /*  556 */  iRegLNoSp_rule,
  /*  557 */  iRegLNoSp_rule,
  /*  558 */  iRegLNoSp_rule,
  /*  559 */  iRegLNoSp_rule,
  /*  560 */  iRegLNoSp_rule,
  /*  561 */  iRegLNoSp_rule,
  /*  562 */  iRegLNoSp_rule,
  /*  563 */  iRegLNoSp_rule,
  /*  564 */  iRegINoSp_rule,
  /*  565 */  iRegINoSp_rule,
  /*  566 */  iRegINoSp_rule,
  /*  567 */  iRegLNoSp_rule,
  /*  568 */  iRegLNoSp_rule,
  /*  569 */  iRegINoSp_rule,
  /*  570 */  iRegINoSp_rule,
  /*  571 */  iRegLNoSp_rule,
  /*  572 */  iRegLNoSp_rule,
  /*  573 */  iRegINoSp_rule,
  /*  574 */  iRegINoSp_rule,
  /*  575 */  iRegLNoSp_rule,
  /*  576 */  iRegLNoSp_rule,
  /*  577 */  iRegINoSp_rule,
  /*  578 */  iRegINoSp_rule,
  /*  579 */  iRegLNoSp_rule,
  /*  580 */  iRegLNoSp_rule,
  /*  581 */  iRegINoSp_rule,
  /*  582 */  iRegINoSp_rule,
  /*  583 */  iRegLNoSp_rule,
  /*  584 */  iRegLNoSp_rule,
  /*  585 */  iRegINoSp_rule,
  /*  586 */  iRegINoSp_rule,
  /*  587 */  iRegLNoSp_rule,
  /*  588 */  iRegLNoSp_rule,
  /*  589 */  iRegINoSp_rule,
  /*  590 */  iRegINoSp_rule,
  /*  591 */  iRegINoSp_rule,
  /*  592 */  iRegINoSp_rule,
  /*  593 */  iRegLNoSp_rule,
  /*  594 */  iRegLNoSp_rule,
  /*  595 */  iRegLNoSp_rule,
  /*  596 */  iRegLNoSp_rule,
  /*  597 */  iRegINoSp_rule,
  /*  598 */  iRegINoSp_rule,
  /*  599 */  iRegINoSp_rule,
  /*  600 */  iRegINoSp_rule,
  /*  601 */  iRegLNoSp_rule,
  /*  602 */  iRegLNoSp_rule,
  /*  603 */  iRegLNoSp_rule,
  /*  604 */  iRegLNoSp_rule,
  /*  605 */  iRegINoSp_rule,
  /*  606 */  iRegINoSp_rule,
  /*  607 */  iRegINoSp_rule,
  /*  608 */  iRegINoSp_rule,
  /*  609 */  iRegLNoSp_rule,
  /*  610 */  iRegLNoSp_rule,
  /*  611 */  iRegLNoSp_rule,
  /*  612 */  iRegLNoSp_rule,
  /*  613 */  iRegINoSp_rule,
  /*  614 */  iRegINoSp_rule,
  /*  615 */  iRegLNoSp_rule,
  /*  616 */  iRegLNoSp_rule,
  /*  617 */  iRegINoSp_rule,
  /*  618 */  iRegINoSp_rule,
  /*  619 */  iRegLNoSp_rule,
  /*  620 */  iRegLNoSp_rule,
  /*  621 */  iRegINoSp_rule,
  /*  622 */  iRegINoSp_rule,
  /*  623 */  iRegLNoSp_rule,
  /*  624 */  iRegLNoSp_rule,
  /*  625 */  iRegINoSp_rule,
  /*  626 */  iRegINoSp_rule,
  /*  627 */  iRegLNoSp_rule,
  /*  628 */  iRegLNoSp_rule,
  /*  629 */  iRegINoSp_rule,
  /*  630 */  iRegINoSp_rule,
  /*  631 */  iRegLNoSp_rule,
  /*  632 */  iRegLNoSp_rule,
  /*  633 */  iRegINoSp_rule,
  /*  634 */  iRegINoSp_rule,
  /*  635 */  iRegLNoSp_rule,
  /*  636 */  iRegLNoSp_rule,
  /*  637 */  iRegINoSp_rule,
  /*  638 */  iRegINoSp_rule,
  /*  639 */  iRegLNoSp_rule,
  /*  640 */  iRegLNoSp_rule,
  /*  641 */  iRegINoSp_rule,
  /*  642 */  iRegINoSp_rule,
  /*  643 */  iRegLNoSp_rule,
  /*  644 */  iRegLNoSp_rule,
  /*  645 */  iRegINoSp_rule,
  /*  646 */  iRegINoSp_rule,
  /*  647 */  iRegLNoSp_rule,
  /*  648 */  iRegLNoSp_rule,
  /*  649 */  iRegINoSp_rule,
  /*  650 */  iRegINoSp_rule,
  /*  651 */  iRegLNoSp_rule,
  /*  652 */  iRegLNoSp_rule,
  /*  653 */  iRegINoSp_rule,
  /*  654 */  iRegINoSp_rule,
  /*  655 */  iRegLNoSp_rule,
  /*  656 */  iRegLNoSp_rule,
  /*  657 */  iRegINoSp_rule,
  /*  658 */  iRegINoSp_rule,
  /*  659 */  iRegLNoSp_rule,
  /*  660 */  iRegLNoSp_rule,
  /*  661 */  iRegINoSp_rule,
  /*  662 */  iRegINoSp_rule,
  /*  663 */  iRegLNoSp_rule,
  /*  664 */  iRegLNoSp_rule,
  /*  665 */  iRegINoSp_rule,
  /*  666 */  iRegINoSp_rule,
  /*  667 */  iRegLNoSp_rule,
  /*  668 */  iRegLNoSp_rule,
  /*  669 */  iRegINoSp_rule,
  /*  670 */  iRegINoSp_rule,
  /*  671 */  iRegLNoSp_rule,
  /*  672 */  iRegLNoSp_rule,
  /*  673 */  iRegINoSp_rule,
  /*  674 */  iRegLNoSp_rule,
  /*  675 */  iRegINoSp_rule,
  /*  676 */  iRegLNoSp_rule,
  /*  677 */  iRegINoSp_rule,
  /*  678 */  iRegLNoSp_rule,
  /*  679 */  iRegLNoSp_rule,
  /*  680 */  iRegINoSp_rule,
  /*  681 */  iRegLNoSp_rule,
  /*  682 */  iRegINoSp_rule,
  /*  683 */  iRegINoSp_rule,
  /*  684 */  iRegLNoSp_rule,
  /*  685 */  iRegLNoSp_rule,
  /*  686 */  iRegLNoSp_rule,
  /*  687 */  iRegLNoSp_rule,
  /*  688 */  iRegINoSp_rule,
  /*  689 */  iRegINoSp_rule,
  /*  690 */  iRegLNoSp_rule,
  /*  691 */  iRegLNoSp_rule,
  /*  692 */  iRegINoSp_rule,
  /*  693 */  iRegINoSp_rule,
  /*  694 */  0,
  /*  695 */  0,
  /*  696 */  iRegLNoSp_rule,
  /*  697 */  iRegLNoSp_rule,
  /*  698 */  iRegLNoSp_rule,
  /*  699 */  iRegLNoSp_rule,
  /*  700 */  iRegINoSp_rule,
  /*  701 */  iRegINoSp_rule,
  /*  702 */  iRegINoSp_rule,
  /*  703 */  iRegINoSp_rule,
  /*  704 */  0,
  /*  705 */  0,
  /*  706 */  iRegLNoSp_rule,
  /*  707 */  iRegLNoSp_rule,
  /*  708 */  iRegLNoSp_rule,
  /*  709 */  iRegLNoSp_rule,
  /*  710 */  iRegINoSp_rule,
  /*  711 */  iRegINoSp_rule,
  /*  712 */  iRegINoSp_rule,
  /*  713 */  iRegINoSp_rule,
  /*  714 */  iRegLNoSp_rule,
  /*  715 */  iRegLNoSp_rule,
  /*  716 */  iRegLNoSp_rule,
  /*  717 */  iRegINoSp_rule,
  /*  718 */  iRegINoSp_rule,
  /*  719 */  iRegINoSp_rule,
  /*  720 */  iRegINoSp_rule,
  /*  721 */  iRegINoSp_rule,
  /*  722 */  iRegINoSp_rule,
  /*  723 */  iRegLNoSp_rule,
  /*  724 */  iRegLNoSp_rule,
  /*  725 */  iRegLNoSp_rule,
  /*  726 */  iRegLNoSp_rule,
  /*  727 */  iRegLNoSp_rule,
  /*  728 */  iRegLNoSp_rule,
  /*  729 */  iRegLNoSp_rule,
  /*  730 */  iRegLNoSp_rule,
  /*  731 */  iRegINoSp_rule,
  /*  732 */  iRegINoSp_rule,
  /*  733 */  iRegINoSp_rule,
  /*  734 */  iRegINoSp_rule,
  /*  735 */  iRegLNoSp_rule,
  /*  736 */  iRegLNoSp_rule,
  /*  737 */  iRegLNoSp_rule,
  /*  738 */  iRegLNoSp_rule,
  /*  739 */  iRegLNoSp_rule,
  /*  740 */  iRegLNoSp_rule,
  /*  741 */  iRegINoSp_rule,
  /*  742 */  iRegINoSp_rule,
  /*  743 */  iRegLNoSp_rule,
  /*  744 */  iRegLNoSp_rule,
  /*  745 */  iRegLNoSp_rule,
  /*  746 */  vRegF_rule,
  /*  747 */  vRegD_rule,
  /*  748 */  vRegF_rule,
  /*  749 */  vRegD_rule,
  /*  750 */  vRegF_rule,
  /*  751 */  vRegD_rule,
  /*  752 */  vRegF_rule,
  /*  753 */  vRegD_rule,
  /*  754 */  vRegD_rule,
  /*  755 */  vRegF_rule,
  /*  756 */  iRegINoSp_rule,
  /*  757 */  iRegINoSp_rule,
  /*  758 */  iRegINoSp_rule,
  /*  759 */  iRegINoSp_rule,
  /*  760 */  iRegINoSp_rule,
  /*  761 */  iRegINoSp_rule,
  /*  762 */  iRegLNoSp_rule,
  /*  763 */  iRegLNoSp_rule,
  /*  764 */  iRegLNoSp_rule,
  /*  765 */  iRegLNoSp_rule,
  /*  766 */  iRegLNoSp_rule,
  /*  767 */  iRegLNoSp_rule,
  /*  768 */  iRegLNoSp_rule,
  /*  769 */  iRegLNoSp_rule,
  /*  770 */  iRegINoSp_rule,
  /*  771 */  iRegINoSp_rule,
  /*  772 */  iRegINoSp_rule,
  /*  773 */  vRegF_rule,
  /*  774 */  vRegD_rule,
  /*  775 */  iRegINoSp_rule,
  /*  776 */  iRegLNoSp_rule,
  /*  777 */  vRegF_rule,
  /*  778 */  vRegF_rule,
  /*  779 */  iRegINoSp_rule,
  /*  780 */  iRegLNoSp_rule,
  /*  781 */  vRegD_rule,
  /*  782 */  vRegD_rule,
  /*  783 */  stackSlotI_rule,
  /*  784 */  stackSlotF_rule,
  /*  785 */  stackSlotL_rule,
  /*  786 */  stackSlotD_rule,
  /*  787 */  Universe_rule,
  /*  788 */  Universe_rule,
  /*  789 */  Universe_rule,
  /*  790 */  Universe_rule,
  /*  791 */  iRegINoSp_rule,
  /*  792 */  iRegINoSp_rule,
  /*  793 */  iRegINoSp_rule,
  /*  794 */  iRegINoSp_rule,
  /*  795 */  iRegINoSp_rule,
  /*  796 */  iRegINoSp_rule,
  /*  797 */  iRegINoSp_rule,
  /*  798 */  iRegINoSp_rule,
  /*  799 */  iRegINoSp_rule,
  /*  800 */  Universe_rule,
  /*  801 */  Universe_rule,
  /*  802 */  Universe_rule,
  /*  803 */  Universe_rule,
  /*  804 */  Universe_rule,
  /*  805 */  Universe_rule,
  /*  806 */  Universe_rule,
  /*  807 */  Universe_rule,
  /*  808 */  Universe_rule,
  /*  809 */  Universe_rule,
  /*  810 */  Universe_rule,
  /*  811 */  Universe_rule,
  /*  812 */  Universe_rule,
  /*  813 */  Universe_rule,
  /*  814 */  Universe_rule,
  /*  815 */  Universe_rule,
  /*  816 */  Universe_rule,
  /*  817 */  Universe_rule,
  /*  818 */  Universe_rule,
  /*  819 */  Universe_rule,
  /*  820 */  Universe_rule,
  /*  821 */  Universe_rule,
  /*  822 */  Universe_rule,
  /*  823 */  Universe_rule,
  /*  824 */  Universe_rule,
  /*  825 */  Universe_rule,
  /*  826 */  Universe_rule,
  /*  827 */  Universe_rule,
  /*  828 */  Universe_rule,
  /*  829 */  iRegP_R0_rule,
  /*  830 */  Universe_rule,
  /*  831 */  Universe_rule,
  /*  832 */  Universe_rule,
  /*  833 */  iRegP_R5_rule,
  /*  834 */  iRegI_R0_rule,
  /*  835 */  iRegI_R0_rule,
  /*  836 */  iRegI_R0_rule,
  /*  837 */  iRegI_R0_rule,
  /*  838 */  iRegI_R0_rule,
  /*  839 */  iRegI_R0_rule,
  /*  840 */  vecD_rule,
  /*  841 */  vecD_rule,
  /*  842 */  vecX_rule,
  /*  843 */  Universe_rule,
  /*  844 */  Universe_rule,
  /*  845 */  Universe_rule,
  /*  846 */  vecD_rule,
  /*  847 */  vecX_rule,
  /*  848 */  vecD_rule,
  /*  849 */  vecX_rule,
  /*  850 */  vecD_rule,
  /*  851 */  vecX_rule,
  /*  852 */  vecD_rule,
  /*  853 */  vecX_rule,
  /*  854 */  vecD_rule,
  /*  855 */  vecX_rule,
  /*  856 */  vecD_rule,
  /*  857 */  vecX_rule,
  /*  858 */  vecX_rule,
  /*  859 */  vecX_rule,
  /*  860 */  vecD_rule,
  /*  861 */  vecX_rule,
  /*  862 */  vecX_rule,
  /*  863 */  vecD_rule,
  /*  864 */  vecX_rule,
  /*  865 */  vecD_rule,
  /*  866 */  vecX_rule,
  /*  867 */  vecD_rule,
  /*  868 */  vecX_rule,
  /*  869 */  vecX_rule,
  /*  870 */  vecD_rule,
  /*  871 */  vecX_rule,
  /*  872 */  vecX_rule,
  /*  873 */  vecD_rule,
  /*  874 */  vecX_rule,
  /*  875 */  vecD_rule,
  /*  876 */  vecX_rule,
  /*  877 */  vecD_rule,
  /*  878 */  vecX_rule,
  /*  879 */  vecX_rule,
  /*  880 */  vecD_rule,
  /*  881 */  vecX_rule,
  /*  882 */  vecX_rule,
  /*  883 */  vecD_rule,
  /*  884 */  vecX_rule,
  /*  885 */  vecD_rule,
  /*  886 */  vecX_rule,
  /*  887 */  vecD_rule,
  /*  888 */  vecX_rule,
  /*  889 */  vecX_rule,
  /*  890 */  vecD_rule,
  /*  891 */  vecX_rule,
  /*  892 */  vecD_rule,
  /*  893 */  vecX_rule,
  /*  894 */  vecD_rule,
  /*  895 */  vecX_rule,
  /*  896 */  vecD_rule,
  /*  897 */  vecX_rule,
  /*  898 */  vecD_rule,
  /*  899 */  vecX_rule,
  /*  900 */  vecX_rule,
  /*  901 */  vecD_rule,
  /*  902 */  vecX_rule,
  /*  903 */  vecD_rule,
  /*  904 */  vecX_rule,
  /*  905 */  vecD_rule,
  /*  906 */  vecX_rule,
  /*  907 */  vecD_rule,
  /*  908 */  vecD_rule,
  /*  909 */  vecX_rule,
  /*  910 */  vecX_rule,
  /*  911 */  vecD_rule,
  /*  912 */  vecX_rule,
  /*  913 */  vecD_rule,
  /*  914 */  vecX_rule,
  /*  915 */  vecD_rule,
  /*  916 */  vecX_rule,
  /*  917 */  vecD_rule,
  /*  918 */  vecX_rule,
  /*  919 */  vecD_rule,
  /*  920 */  vecD_rule,
  /*  921 */  vecX_rule,
  /*  922 */  vecX_rule,
  /*  923 */  vecD_rule,
  /*  924 */  vecX_rule,
  /*  925 */  vecD_rule,
  /*  926 */  vecX_rule,
  /*  927 */  vecD_rule,
  /*  928 */  vecX_rule,
  /*  929 */  vecD_rule,
  /*  930 */  vecX_rule,
  /*  931 */  vecD_rule,
  /*  932 */  vecD_rule,
  /*  933 */  vecX_rule,
  /*  934 */  vecX_rule,
  /*  935 */  vecD_rule,
  /*  936 */  vecX_rule,
  /*  937 */  vecD_rule,
  /*  938 */  vecX_rule,
  /*  939 */  vecD_rule,
  /*  940 */  vecX_rule,
  /*  941 */  vecD_rule,
  /*  942 */  vecX_rule,
  /*  943 */  vecX_rule,
  /*  944 */  vecX_rule,
  /*  945 */  vecX_rule,
  /*  946 */  vecX_rule,
  /*  947 */  vecX_rule,
  /*  948 */  vecX_rule,
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  _AddP_iRegP__LShiftL_iRegL_immIScale_rule,
  /*  106 */  _AddP_iRegP__LShiftL_iRegL_immIScale_rule,
  /*  107 */  _AddP_iRegP__ConvI2L_iRegI__rule,
  /*  108 */  _AddP_iRegP__LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  109 */  iRegP_rule,
  /*  110 */  iRegP_rule,
  /*  111 */  iRegP_rule,
  /*  112 */  iRegP_rule,
  /*  113 */  iRegP_rule,
  /*  114 */  iRegP_rule,
  /*  115 */  iRegP_rule,
  /*  116 */  iRegP_rule,
  /*  117 */  iRegP_rule,
  /*  118 */  iRegP_rule,
  /*  119 */  iRegP_rule,
  /*  120 */  iRegN_rule,
  /*  121 */  _AddP__DecodeN_iRegN___LShiftL_iRegL_immIScale_rule,
  /*  122 */  _AddP__DecodeN_iRegN___LShiftL_iRegL_immIScale_rule,
  /*  123 */  _AddP__DecodeN_iRegN___ConvI2L_iRegI__rule,
  /*  124 */  _AddP__DecodeN_iRegN___LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  125 */  _DecodeN_iRegN__rule,
  /*  126 */  _DecodeN_iRegN__rule,
  /*  127 */  _DecodeN_iRegN__rule,
  /*  128 */  _DecodeN_iRegN__rule,
  /*  129 */  _DecodeN_iRegN__rule,
  /*  130 */  thread_RegP_rule,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  /*  134 */  0,
  /*  135 */  0,
  /*  136 */  0,
  /*  137 */  0,
  /*  138 */  iRegL_rule,
  // last operand
  /*  139 */  0,
  /*  140 */  0,
  /*  141 */  0,
  /*  142 */  0,
  /*  143 */  0,
  // last operand class
  /*  144 */  iRegL_rule,
  /*  145 */  iRegP_rule,
  /*  146 */  iRegI_rule,
  /*  147 */  iRegP_rule,
  /*  148 */  _ConvI2L_iRegI__rule,
  /*  149 */  iRegP_rule,
  /*  150 */  iRegN_rule,
  /*  151 */  _DecodeN_iRegN__rule,
  /*  152 */  _DecodeN_iRegN__rule,
  /*  153 */  _DecodeN_iRegN__rule,
  /*  154 */  memory_rule,
  /*  155 */  memory_rule,
  /*  156 */  memory_rule,
  /*  157 */  memory_rule,
  /*  158 */  memory_rule,
  /*  159 */  _LoadI_memory__rule,
  /*  160 */  indirect_rule,
  /*  161 */  indirect_rule,
  /*  162 */  indirect_rule,
  /*  163 */  indirect_rule,
  /*  164 */  indirect_rule,
  /*  165 */  _LoadI_indirect__rule,
  /*  166 */  memory_rule,
  /*  167 */  iRegP_rule,
  /*  168 */  _DecodeN_iRegN__rule,
  /*  169 */  iRegP_rule,
  /*  170 */  iRegLNoSp_rule,
  /*  171 */  iRegINoSp_rule,
  /*  172 */  iRegNNoSp_rule,
  /*  173 */  cmpOp_rule,
  /*  174 */  iRegIorL2I_rule,
  /*  175 */  cmpOpU_rule,
  /*  176 */  immI0_rule,
  /*  177 */  iRegIorL2I_rule,
  /*  178 */  immI_1_rule,
  /*  179 */  iRegL_rule,
  /*  180 */  iRegL_rule,
  /*  181 */  immL0_rule,
  /*  182 */  iRegP_rule,
  /*  183 */  immP0_rule,
  /*  184 */  iRegN_rule,
  /*  185 */  iRegN_rule,
  /*  186 */  immN0_rule,
  /*  187 */  vRegF_rule,
  /*  188 */  vRegD_rule,
  /*  189 */  iRegL_rule,
  /*  190 */  iRegIorL2I_rule,
  /*  191 */  _ConvI2L_iRegIorL2I__rule,
  /*  192 */  iRegIorL2I_rule,
  /*  193 */  iRegL_rule,
  /*  194 */  iRegIorL2I_rule,
  /*  195 */  _RShiftI_iRegIorL2I_immI_31_rule,
  /*  196 */  iRegL_rule,
  /*  197 */  _RShiftL_iRegL_immL_63_rule,
  /*  198 */  iRegIorL2I_rule,
  /*  199 */  iRegL_rule,
  /*  200 */  iRegIorL2I_rule,
  /*  201 */  iRegL_rule,
  /*  202 */  iRegIorL2I_rule,
  /*  203 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  204 */  iRegL_rule,
  /*  205 */  _URShiftL_iRegL_immI_rule,
  /*  206 */  iRegIorL2I_rule,
  /*  207 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  208 */  iRegL_rule,
  /*  209 */  _RShiftL_iRegL_immI_rule,
  /*  210 */  iRegIorL2I_rule,
  /*  211 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  212 */  iRegL_rule,
  /*  213 */  _LShiftL_iRegL_immI_rule,
  /*  214 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  215 */  iRegIorL2I_rule,
  /*  216 */  _URShiftL_iRegL_immI_rule,
  /*  217 */  iRegL_rule,
  /*  218 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  219 */  iRegIorL2I_rule,
  /*  220 */  _RShiftL_iRegL_immI_rule,
  /*  221 */  iRegL_rule,
  /*  222 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  223 */  iRegIorL2I_rule,
  /*  224 */  _LShiftL_iRegL_immI_rule,
  /*  225 */  iRegL_rule,
  /*  226 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  227 */  iRegL_rule,
  /*  228 */  immI_64_rule,
  /*  229 */  iRegL_rule,
  /*  230 */  immI0_rule,
  /*  231 */  iRegL_rule,
  /*  232 */  iRegI_rule,
  /*  233 */  immI_32_rule,
  /*  234 */  iRegI_rule,
  /*  235 */  iRegI_rule,
  /*  236 */  iRegL_rule,
  /*  237 */  iRegL_rule,
  /*  238 */  iRegL_rule,
  /*  239 */  iRegI_rule,
  /*  240 */  iRegI_rule,
  /*  241 */  iRegI_rule,
  /*  242 */  iRegIorL2I_rule,
  /*  243 */  _LShiftI_iRegIorL2I_immI_16_rule,
  /*  244 */  iRegIorL2I_rule,
  /*  245 */  _LShiftI_iRegIorL2I_immI_24_rule,
  /*  246 */  _LShiftI_iRegIorL2I_immI_24_rule,
  /*  247 */  iRegL_rule,
  /*  248 */  _LShiftL_iRegL_immI_48_rule,
  /*  249 */  iRegL_rule,
  /*  250 */  _LShiftL_iRegL_immI_32_rule,
  /*  251 */  iRegL_rule,
  /*  252 */  _LShiftL_iRegL_immI_56_rule,
  /*  253 */  _LShiftL_iRegL_immI_56_rule,
  /*  254 */  iRegIorL2I_rule,
  /*  255 */  iRegIorL2I_rule,
  /*  256 */  iRegL_rule,
  /*  257 */  iRegL_rule,
  /*  258 */  iRegL_rule,
  /*  259 */  vRegF_rule,
  /*  260 */  _ConvF2D_vRegF__rule,
  /*  261 */  iRegIorL2I_rule,
  /*  262 */  iRegL_rule,
  /*  263 */  iRegIorL2I_rule,
  /*  264 */  iRegL_rule,
  /*  265 */  iRegP_rule,
  /*  266 */  iRegN_rule,
  /*  267 */  _DecodeN_iRegN__rule,
  /*  268 */  iRegIorL2I_rule,
  /*  269 */  iRegL_rule,
  /*  270 */  iRegL_rule,
  /*  271 */  _AndL_iRegL_immL_rule,
  /*  272 */  iRegIorL2I_rule,
  /*  273 */  _AndI_iRegIorL2I_immI_rule,
  /*  274 */  iRegL_rule,
  /*  275 */  iRegIorL2I_rule,
  /*  276 */  iRegP_R4_rule,
  /*  277 */  iRegP_R1_rule,
  /*  278 */  iRegP_R3_rule,
  /*  279 */  iRegP_R1_rule,
  /*  280 */  iRegP_R3_rule,
  /*  281 */  iRegP_R3_rule,
  /*  282 */  iRegP_R1_rule,
  /*  283 */  iRegP_R1_rule,
  /*  284 */  vecD_rule,
  /*  285 */  vecX_rule,
  /*  286 */  vecD_rule,
  /*  287 */  vecX_rule,
  // last internally defined operand
  /*  288 */  immI_rule,
  /*  289 */  immL_rule,
  /*  290 */  immP_rule,
  /*  291 */  immP0_rule,
  /*  292 */  immP_1_rule,
  /*  293 */  immPollPage_rule,
  /*  294 */  immByteMapBase_rule,
  /*  295 */  immN_rule,
  /*  296 */  immN0_rule,
  /*  297 */  immNKlass_rule,
  /*  298 */  immFPacked_rule,
  /*  299 */  immF_rule,
  /*  300 */  immDPacked_rule,
  /*  301 */  immD_rule,
  /*  302 */  iRegIorL2I_rule,
  /*  303 */  iRegL_rule,
  /*  304 */  iRegIorL2I_rule,
  /*  305 */  iRegL_rule,
  /*  306 */  iRegL_rule,
  /*  307 */  iRegP_rule,
  /*  308 */  iRegPNoSp_rule,
  /*  309 */  iRegPNoSp_rule,
  /*  310 */  iRegI_rule,
  /*  311 */  memory_rule,
  /*  312 */  indirect_rule,
  /*  313 */  indirect_rule,
  /*  314 */  vRegF_rule,
  /*  315 */  vRegD_rule,
  /*  316 */  vRegF_rule,
  /*  317 */  vRegD_rule,
  /*  318 */  stackSlotF_rule,
  /*  319 */  stackSlotI_rule,
  /*  320 */  stackSlotD_rule,
  /*  321 */  stackSlotL_rule,
  /*  322 */  vRegF_rule,
  /*  323 */  iRegI_rule,
  /*  324 */  vRegD_rule,
  /*  325 */  iRegL_rule,
  /*  326 */  iRegIorL2I_rule,
  /*  327 */  iRegIorL2I_rule,
  /*  328 */  iRegL_rule,
  /*  329 */  iRegL_rule,
  /*  330 */  iRegIorL2I_rule,
  /*  331 */  iRegIorL2I_rule,
  /*  332 */  iRegL_rule,
  /*  333 */  iRegL_rule,
  /*  334 */  immI0_rule,
  /*  335 */  immI0_rule,
  /*  336 */  iRegIorL2I_rule,
  /*  337 */  iRegL_rule,
  /*  338 */  iRegI_rule,
  /*  339 */  iRegI_rule,
  /*  340 */  iRegI_rule,
  /*  341 */  iRegI_rule,
  /*  342 */  iRegI_rule,
  /*  343 */  iRegI_rule,
  /*  344 */  iRegI_rule,
  /*  345 */  iRegI_rule,
  /*  346 */  iRegL_rule,
  /*  347 */  iRegL_rule,
  /*  348 */  iRegL_rule,
  /*  349 */  iRegL_rule,
  /*  350 */  iRegL_rule,
  /*  351 */  iRegL_rule,
  /*  352 */  iRegL_rule,
  /*  353 */  iRegL_rule,
  /*  354 */  iRegP_rule,
  /*  355 */  iRegN_rule,
  /*  356 */  iRegP_rule,
  /*  357 */  iRegN_rule,
  /*  358 */  vRegF_rule,
  /*  359 */  vRegF_rule,
  /*  360 */  vRegD_rule,
  /*  361 */  vRegD_rule,
  /*  362 */  _AndL_iRegL_immL_rule,
  /*  363 */  _AndI_iRegIorL2I_immI_rule,
  /*  364 */  _AndL_iRegL_iRegL_rule,
  /*  365 */  _AndI_iRegIorL2I_iRegIorL2I_rule,
  /*  366 */  iRegP_rule,
  /*  367 */  iRegP_rule,
  /*  368 */  _PartialSubtypeCheck_iRegP_R4_iRegP_R0_rule,
  /*  369 */  0,
  /*  370 */  iRegIorL2I_rule,
  /*  371 */  iRegIorL2I_rule,
  /*  372 */  memory_rule,
  /*  373 */  _LoadB_memory__rule,
  /*  374 */  memory_rule,
  /*  375 */  _LoadUB_memory__rule,
  /*  376 */  memory_rule,
  /*  377 */  _LoadS_memory__rule,
  /*  378 */  memory_rule,
  /*  379 */  _LoadUS_memory__rule,
  /*  380 */  memory_rule,
  /*  381 */  _LoadI_memory__rule,
  /*  382 */  _ConvI2L__LoadI_memory___rule,
  /*  383 */  memory_rule,
  /*  384 */  memory_rule,
  /*  385 */  memory_rule,
  /*  386 */  memory_rule,
  /*  387 */  memory_rule,
  /*  388 */  memory_rule,
  /*  389 */  memory_rule,
  /*  390 */  memory_rule,
  /*  391 */  memory_rule,
  /*  392 */  memory_rule,
  /*  393 */  memory_rule,
  /*  394 */  memory_rule,
  /*  395 */  memory_rule,
  /*  396 */  memory_rule,
  /*  397 */  memory_rule,
  /*  398 */  memory_rule,
  /*  399 */  memory_rule,
  /*  400 */  memory_rule,
  /*  401 */  memory_rule,
  /*  402 */  memory_rule,
  /*  403 */  memory_rule,
  /*  404 */  memory_rule,
  /*  405 */  memory_rule,
  /*  406 */  memory_rule,
  /*  407 */  memory_rule,
  /*  408 */  memory_rule,
  /*  409 */  memory_rule,
  /*  410 */  memory_rule,
  /*  411 */  indirect_rule,
  /*  412 */  _LoadB_indirect__rule,
  /*  413 */  indirect_rule,
  /*  414 */  _LoadUB_indirect__rule,
  /*  415 */  indirect_rule,
  /*  416 */  indirect_rule,
  /*  417 */  _LoadUS_indirect__rule,
  /*  418 */  _LoadS_indirect__rule,
  /*  419 */  indirect_rule,
  /*  420 */  _ConvI2L__LoadI_indirect___rule,
  /*  421 */  indirect_rule,
  /*  422 */  indirect_rule,
  /*  423 */  indirect_rule,
  /*  424 */  indirect_rule,
  /*  425 */  indirect_rule,
  /*  426 */  indirect_rule,
  /*  427 */  indirect_rule,
  /*  428 */  indirect_rule,
  /*  429 */  indirect_rule,
  /*  430 */  indirect_rule,
  /*  431 */  indirect_rule,
  /*  432 */  indirect_rule,
  /*  433 */  indirect_rule,
  /*  434 */  iRegIorL2I_rule,
  /*  435 */  iRegL_rule,
  /*  436 */  iRegIorL2I_rule,
  /*  437 */  iRegIorL2I_rule,
  /*  438 */  iRegIorL2I_rule,
  /*  439 */  _LoadI_memory__rule,
  /*  440 */  iRegL_rule,
  /*  441 */  _LoadL_memory__rule,
  /*  442 */  0,
  /*  443 */  0,
  /*  444 */  0,
  /*  445 */  0,
  /*  446 */  0,
  /*  447 */  0,
  /*  448 */  0,
  /*  449 */  0,
  /*  450 */  0,
  /*  451 */  0,
  /*  452 */  0,
  /*  453 */  _CastP2X_iRegP__rule,
  /*  454 */  _CastP2X__DecodeN_iRegN___rule,
  /*  455 */  iRegP_rule,
  /*  456 */  iRegP_rule,
  /*  457 */  iRegN_rule,
  /*  458 */  iRegN_rule,
  /*  459 */  iRegP_rule,
  /*  460 */  iRegN_rule,
  /*  461 */  indirect_rule,
  /*  462 */  indirect_rule,
  /*  463 */  indirect_rule,
  /*  464 */  indirect_rule,
  /*  465 */  indirect_rule,
  /*  466 */  indirect_rule,
  /*  467 */  indirect_rule,
  /*  468 */  indirect_rule,
  /*  469 */  indirect_rule,
  /*  470 */  indirect_rule,
  /*  471 */  indirect_rule,
  /*  472 */  indirect_rule,
  /*  473 */  indirect_rule,
  /*  474 */  indirect_rule,
  /*  475 */  indirect_rule,
  /*  476 */  indirect_rule,
  /*  477 */  indirect_rule,
  /*  478 */  indirect_rule,
  /*  479 */  indirect_rule,
  /*  480 */  indirect_rule,
  /*  481 */  indirect_rule,
  /*  482 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  483 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  484 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  485 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  486 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  487 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  488 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  489 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  490 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  491 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  492 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  493 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  494 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  495 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  496 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  497 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  498 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  499 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  500 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  501 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  502 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  503 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  504 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  505 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  506 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  507 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  508 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  509 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  510 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  511 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  512 */  iRegIorL2I_rule,
  /*  513 */  iRegIorL2I_rule,
  /*  514 */  _ConvL2I_iRegL__rule,
  /*  515 */  iRegP_rule,
  /*  516 */  iRegP_rule,
  /*  517 */  iRegP_rule,
  /*  518 */  iRegP_rule,
  /*  519 */  _ConvI2L_iRegIorL2I__rule,
  /*  520 */  iRegP_rule,
  /*  521 */  iRegL_rule,
  /*  522 */  iRegL_rule,
  /*  523 */  iRegIorL2I_rule,
  /*  524 */  iRegIorL2I_rule,
  /*  525 */  iRegL_rule,
  /*  526 */  iRegL_rule,
  /*  527 */  immI0_rule,
  /*  528 */  immL0_rule,
  /*  529 */  iRegIorL2I_rule,
  /*  530 */  _ConvI2L_iRegIorL2I__rule,
  /*  531 */  _ConvI2L_iRegIorL2I__rule,
  /*  532 */  iRegL_rule,
  /*  533 */  iRegL_rule,
  /*  534 */  iRegIorL2I_rule,
  /*  535 */  _MulI_iRegIorL2I_iRegIorL2I_rule,
  /*  536 */  iRegIorL2I_rule,
  /*  537 */  iRegL_rule,
  /*  538 */  _MulL_iRegL_iRegL_rule,
  /*  539 */  iRegL_rule,
  /*  540 */  iRegIorL2I_rule,
  /*  541 */  _RShiftI_iRegIorL2I_immI_31_rule,
  /*  542 */  iRegIorL2I_rule,
  /*  543 */  _URShiftI__RShiftI_iRegIorL2I_immI_31_immI_31_rule,
  /*  544 */  iRegL_rule,
  /*  545 */  _RShiftL_iRegL_immL_63_rule,
  /*  546 */  iRegL_rule,
  /*  547 */  _URShiftL__RShiftL_iRegL_immL_63_immL_63_rule,
  /*  548 */  iRegIorL2I_rule,
  /*  549 */  iRegL_rule,
  /*  550 */  iRegIorL2I_rule,
  /*  551 */  iRegIorL2I_rule,
  /*  552 */  iRegIorL2I_rule,
  /*  553 */  iRegIorL2I_rule,
  /*  554 */  iRegIorL2I_rule,
  /*  555 */  iRegIorL2I_rule,
  /*  556 */  iRegL_rule,
  /*  557 */  iRegL_rule,
  /*  558 */  iRegL_rule,
  /*  559 */  iRegL_rule,
  /*  560 */  _CastP2X_iRegP__rule,
  /*  561 */  iRegL_rule,
  /*  562 */  iRegL_rule,
  /*  563 */  iRegL_rule,
  /*  564 */  iRegIorL2I_rule,
  /*  565 */  iRegIorL2I_rule,
  /*  566 */  _XorI_iRegIorL2I_immI_M1_rule,
  /*  567 */  iRegL_rule,
  /*  568 */  _XorL_iRegL_immL_M1_rule,
  /*  569 */  iRegIorL2I_rule,
  /*  570 */  _XorI_iRegIorL2I_immI_M1_rule,
  /*  571 */  iRegL_rule,
  /*  572 */  _XorL_iRegL_immL_M1_rule,
  /*  573 */  immI_M1_rule,
  /*  574 */  _XorI_iRegIorL2I_iRegIorL2I_rule,
  /*  575 */  immL_M1_rule,
  /*  576 */  _XorL_iRegL_iRegL_rule,
  /*  577 */  iRegIorL2I_rule,
  /*  578 */  _XorI__URShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  579 */  iRegL_rule,
  /*  580 */  _XorL__URShiftL_iRegL_immI_immL_M1_rule,
  /*  581 */  iRegIorL2I_rule,
  /*  582 */  _XorI__RShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  583 */  iRegL_rule,
  /*  584 */  _XorL__RShiftL_iRegL_immI_immL_M1_rule,
  /*  585 */  iRegIorL2I_rule,
  /*  586 */  _XorI__LShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  587 */  iRegL_rule,
  /*  588 */  _XorL__LShiftL_iRegL_immI_immL_M1_rule,
  /*  589 */  immI_M1_rule,
  /*  590 */  _XorI__URShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  591 */  immI_M1_rule,
  /*  592 */  _XorI_iRegIorL2I__URShiftI_iRegIorL2I_immI_rule,
  /*  593 */  immL_M1_rule,
  /*  594 */  _XorL__URShiftL_iRegL_immI_iRegL_rule,
  /*  595 */  immL_M1_rule,
  /*  596 */  _XorL_iRegL__URShiftL_iRegL_immI_rule,
  /*  597 */  immI_M1_rule,
  /*  598 */  _XorI__RShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  599 */  immI_M1_rule,
  /*  600 */  _XorI_iRegIorL2I__RShiftI_iRegIorL2I_immI_rule,
  /*  601 */  immL_M1_rule,
  /*  602 */  _XorL__RShiftL_iRegL_immI_iRegL_rule,
  /*  603 */  immL_M1_rule,
  /*  604 */  _XorL_iRegL__RShiftL_iRegL_immI_rule,
  /*  605 */  immI_M1_rule,
  /*  606 */  _XorI__LShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  607 */  immI_M1_rule,
  /*  608 */  _XorI_iRegIorL2I__LShiftI_iRegIorL2I_immI_rule,
  /*  609 */  immL_M1_rule,
  /*  610 */  _XorL__LShiftL_iRegL_immI_iRegL_rule,
  /*  611 */  immL_M1_rule,
  /*  612 */  _XorL_iRegL__LShiftL_iRegL_immI_rule,
  /*  613 */  iRegIorL2I_rule,
  /*  614 */  _XorI__URShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  615 */  iRegL_rule,
  /*  616 */  _XorL__URShiftL_iRegL_immI_immL_M1_rule,
  /*  617 */  iRegIorL2I_rule,
  /*  618 */  _XorI__RShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  619 */  iRegL_rule,
  /*  620 */  _XorL__RShiftL_iRegL_immI_immL_M1_rule,
  /*  621 */  iRegIorL2I_rule,
  /*  622 */  _XorI__LShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  623 */  iRegL_rule,
  /*  624 */  _XorL__LShiftL_iRegL_immI_immL_M1_rule,
  /*  625 */  iRegIorL2I_rule,
  /*  626 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  627 */  iRegL_rule,
  /*  628 */  _URShiftL_iRegL_immI_rule,
  /*  629 */  iRegIorL2I_rule,
  /*  630 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  631 */  iRegL_rule,
  /*  632 */  _RShiftL_iRegL_immI_rule,
  /*  633 */  iRegIorL2I_rule,
  /*  634 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  635 */  iRegL_rule,
  /*  636 */  _LShiftL_iRegL_immI_rule,
  /*  637 */  iRegIorL2I_rule,
  /*  638 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  639 */  iRegL_rule,
  /*  640 */  _URShiftL_iRegL_immI_rule,
  /*  641 */  iRegIorL2I_rule,
  /*  642 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  643 */  iRegL_rule,
  /*  644 */  _RShiftL_iRegL_immI_rule,
  /*  645 */  iRegIorL2I_rule,
  /*  646 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  647 */  iRegL_rule,
  /*  648 */  _LShiftL_iRegL_immI_rule,
  /*  649 */  iRegIorL2I_rule,
  /*  650 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  651 */  iRegL_rule,
  /*  652 */  _URShiftL_iRegL_immI_rule,
  /*  653 */  iRegIorL2I_rule,
  /*  654 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  655 */  iRegL_rule,
  /*  656 */  _RShiftL_iRegL_immI_rule,
  /*  657 */  iRegIorL2I_rule,
  /*  658 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  659 */  iRegL_rule,
  /*  660 */  _LShiftL_iRegL_immI_rule,
  /*  661 */  iRegIorL2I_rule,
  /*  662 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  663 */  iRegL_rule,
  /*  664 */  _URShiftL_iRegL_immI_rule,
  /*  665 */  iRegIorL2I_rule,
  /*  666 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  667 */  iRegL_rule,
  /*  668 */  _RShiftL_iRegL_immI_rule,
  /*  669 */  iRegIorL2I_rule,
  /*  670 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  671 */  iRegL_rule,
  /*  672 */  _LShiftL_iRegL_immI_rule,
  /*  673 */  iRegIorL2I_rule,
  /*  674 */  iRegL_rule,
  /*  675 */  iRegIorL2I_rule,
  /*  676 */  iRegL_rule,
  /*  677 */  iRegIorL2I_rule,
  /*  678 */  iRegL_rule,
  /*  679 */  _LShiftL_iRegL_immI_rule,
  /*  680 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  681 */  _LShiftL_iRegL_immI_rule,
  /*  682 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  683 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  684 */  _URShiftL_iRegL_immI_rule,
  /*  685 */  _AndI__URShiftI_iRegIorL2I_immI_immI_bitmask_rule,
  /*  686 */  _LShiftL_iRegL_immI_rule,
  /*  687 */  _URShiftL_iRegL_immI_rule,
  /*  688 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  689 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  690 */  _LShiftL_iRegL_immI_rule,
  /*  691 */  _URShiftL_iRegL_immI_rule,
  /*  692 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  693 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  694 */  0,
  /*  695 */  0,
  /*  696 */  _LShiftL_iRegL_iRegI_rule,
  /*  697 */  _URShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  698 */  _LShiftL_iRegL_iRegI_rule,
  /*  699 */  _URShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  700 */  _LShiftI_iRegI_iRegI_rule,
  /*  701 */  _URShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  702 */  _LShiftI_iRegI_iRegI_rule,
  /*  703 */  _URShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  704 */  0,
  /*  705 */  0,
  /*  706 */  _URShiftL_iRegL_iRegI_rule,
  /*  707 */  _LShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  708 */  _URShiftL_iRegL_iRegI_rule,
  /*  709 */  _LShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  710 */  _URShiftI_iRegI_iRegI_rule,
  /*  711 */  _LShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  712 */  _URShiftI_iRegI_iRegI_rule,
  /*  713 */  _LShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  714 */  iRegL_rule,
  /*  715 */  _ConvI2L_iRegIorL2I__rule,
  /*  716 */  iRegL_rule,
  /*  717 */  iRegIorL2I_rule,
  /*  718 */  _RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_rule,
  /*  719 */  iRegIorL2I_rule,
  /*  720 */  _RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  721 */  iRegIorL2I_rule,
  /*  722 */  _URShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  723 */  iRegL_rule,
  /*  724 */  _RShiftL__LShiftL_iRegL_immI_48_immI_48_rule,
  /*  725 */  iRegL_rule,
  /*  726 */  _RShiftL__LShiftL_iRegL_immI_32_immI_32_rule,
  /*  727 */  iRegL_rule,
  /*  728 */  _RShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  729 */  iRegL_rule,
  /*  730 */  _URShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  731 */  iRegIorL2I_rule,
  /*  732 */  _AndI_iRegIorL2I_immI_255_rule,
  /*  733 */  iRegIorL2I_rule,
  /*  734 */  _AndI_iRegIorL2I_immI_65535_rule,
  /*  735 */  iRegL_rule,
  /*  736 */  _AndL_iRegL_immL_255_rule,
  /*  737 */  iRegL_rule,
  /*  738 */  _AndL_iRegL_immL_65535_rule,
  /*  739 */  iRegL_rule,
  /*  740 */  _AndL_iRegL_immL_4294967295_rule,
  /*  741 */  iRegIorL2I_rule,
  /*  742 */  iRegIorL2I_rule,
  /*  743 */  iRegL_rule,
  /*  744 */  iRegL_rule,
  /*  745 */  iRegL_rule,
  /*  746 */  vRegF_rule,
  /*  747 */  vRegD_rule,
  /*  748 */  vRegF_rule,
  /*  749 */  vRegD_rule,
  /*  750 */  vRegF_rule,
  /*  751 */  vRegD_rule,
  /*  752 */  vRegF_rule,
  /*  753 */  vRegD_rule,
  /*  754 */  vRegD_rule,
  /*  755 */  _SqrtD__ConvF2D_vRegF___rule,
  /*  756 */  iRegIorL2I_rule,
  /*  757 */  iRegIorL2I_rule,
  /*  758 */  iRegIorL2I_rule,
  /*  759 */  iRegIorL2I_rule,
  /*  760 */  iRegIorL2I_rule,
  /*  761 */  iRegIorL2I_rule,
  /*  762 */  iRegL_rule,
  /*  763 */  iRegL_rule,
  /*  764 */  iRegL_rule,
  /*  765 */  iRegL_rule,
  /*  766 */  iRegL_rule,
  /*  767 */  iRegL_rule,
  /*  768 */  iRegIorL2I_rule,
  /*  769 */  _ConvI2L_iRegIorL2I__rule,
  /*  770 */  iRegL_rule,
  /*  771 */  iRegIorL2I_rule,
  /*  772 */  iRegP_rule,
  /*  773 */  vRegD_rule,
  /*  774 */  vRegF_rule,
  /*  775 */  vRegF_rule,
  /*  776 */  vRegF_rule,
  /*  777 */  iRegIorL2I_rule,
  /*  778 */  iRegL_rule,
  /*  779 */  vRegD_rule,
  /*  780 */  vRegD_rule,
  /*  781 */  iRegIorL2I_rule,
  /*  782 */  iRegL_rule,
  /*  783 */  vRegF_rule,
  /*  784 */  iRegI_rule,
  /*  785 */  vRegD_rule,
  /*  786 */  iRegL_rule,
  /*  787 */  iRegL_R11_rule,
  /*  788 */  immL_rule,
  /*  789 */  cmpOp_rule,
  /*  790 */  cmpOp_rule,
  /*  791 */  vRegF_rule,
  /*  792 */  vRegD_rule,
  /*  793 */  vRegF_rule,
  /*  794 */  vRegD_rule,
  /*  795 */  iRegL_rule,
  /*  796 */  iRegIorL2I_rule,
  /*  797 */  iRegIorL2I_rule,
  /*  798 */  iRegI_rule,
  /*  799 */  iRegI_rule,
  /*  800 */  0,
  /*  801 */  cmpOp_rule,
  /*  802 */  cmpOpU_rule,
  /*  803 */  cmpOp_rule,
  /*  804 */  cmpOp_rule,
  /*  805 */  cmpOp_rule,
  /*  806 */  cmpOp_rule,
  /*  807 */  cmpOp_rule,
  /*  808 */  cmpOpU_rule,
  /*  809 */  cmpOpU_rule,
  /*  810 */  cmpOp_rule,
  /*  811 */  cmpOp_rule,
  /*  812 */  cmpOp_rule,
  /*  813 */  cmpOp_rule,
  /*  814 */  cmpOp_rule,
  /*  815 */  cmpOp_rule,
  /*  816 */  cmpOp_rule,
  /*  817 */  cmpOp_rule,
  /*  818 */  cmpOp_rule,
  /*  819 */  cmpOpU_rule,
  /*  820 */  iRegP_rule,
  /*  821 */  0,
  /*  822 */  0,
  /*  823 */  0,
  /*  824 */  0,
  /*  825 */  0,
  /*  826 */  0,
  /*  827 */  iRegPNoSp_rule,
  /*  828 */  iRegPNoSp_rule,
  /*  829 */  0,
  /*  830 */  0,
  /*  831 */  0,
  /*  832 */  0,
  /*  833 */  iRegP_R4_rule,
  /*  834 */  _Binary_iRegP_R1_iRegI_R2_rule,
  /*  835 */  _Binary_iRegP_R1_iRegI_R4_rule,
  /*  836 */  _Binary_iRegP_R1_iRegI_R4_rule,
  /*  837 */  _Binary_iRegP_R1_iRegP_R3_rule,
  /*  838 */  iRegP_R1_rule,
  /*  839 */  iRegP_R2_rule,
  /*  840 */  vmem4_rule,
  /*  841 */  vmem8_rule,
  /*  842 */  vmem16_rule,
  /*  843 */  vmem4_rule,
  /*  844 */  vmem8_rule,
  /*  845 */  vmem16_rule,
  /*  846 */  iRegIorL2I_rule,
  /*  847 */  iRegIorL2I_rule,
  /*  848 */  immI_rule,
  /*  849 */  immI_rule,
  /*  850 */  iRegIorL2I_rule,
  /*  851 */  iRegIorL2I_rule,
  /*  852 */  immI_rule,
  /*  853 */  immI_rule,
  /*  854 */  iRegIorL2I_rule,
  /*  855 */  iRegIorL2I_rule,
  /*  856 */  immI_rule,
  /*  857 */  immI_rule,
  /*  858 */  iRegL_rule,
  /*  859 */  immI0_rule,
  /*  860 */  vRegF_rule,
  /*  861 */  vRegF_rule,
  /*  862 */  vRegD_rule,
  /*  863 */  vecD_rule,
  /*  864 */  vecX_rule,
  /*  865 */  vecD_rule,
  /*  866 */  vecX_rule,
  /*  867 */  vecD_rule,
  /*  868 */  vecX_rule,
  /*  869 */  vecX_rule,
  /*  870 */  vecD_rule,
  /*  871 */  vecX_rule,
  /*  872 */  vecX_rule,
  /*  873 */  vecD_rule,
  /*  874 */  vecX_rule,
  /*  875 */  vecD_rule,
  /*  876 */  vecX_rule,
  /*  877 */  vecD_rule,
  /*  878 */  vecX_rule,
  /*  879 */  vecX_rule,
  /*  880 */  vecD_rule,
  /*  881 */  vecX_rule,
  /*  882 */  vecX_rule,
  /*  883 */  vecD_rule,
  /*  884 */  vecX_rule,
  /*  885 */  vecD_rule,
  /*  886 */  vecX_rule,
  /*  887 */  vecD_rule,
  /*  888 */  vecX_rule,
  /*  889 */  vecX_rule,
  /*  890 */  vecD_rule,
  /*  891 */  vecX_rule,
  /*  892 */  vecD_rule,
  /*  893 */  vecX_rule,
  /*  894 */  vecD_rule,
  /*  895 */  vecX_rule,
  /*  896 */  vecD_rule,
  /*  897 */  vecX_rule,
  /*  898 */  vecD_rule,
  /*  899 */  vecX_rule,
  /*  900 */  vecX_rule,
  /*  901 */  vecD_rule,
  /*  902 */  vecX_rule,
  /*  903 */  vecD_rule,
  /*  904 */  vecX_rule,
  /*  905 */  vecD_rule,
  /*  906 */  vecX_rule,
  /*  907 */  vecD_rule,
  /*  908 */  vecD_rule,
  /*  909 */  vecX_rule,
  /*  910 */  vecX_rule,
  /*  911 */  vecD_rule,
  /*  912 */  vecX_rule,
  /*  913 */  vecD_rule,
  /*  914 */  vecX_rule,
  /*  915 */  vecD_rule,
  /*  916 */  vecX_rule,
  /*  917 */  vecD_rule,
  /*  918 */  vecX_rule,
  /*  919 */  vecD_rule,
  /*  920 */  vecD_rule,
  /*  921 */  vecX_rule,
  /*  922 */  vecX_rule,
  /*  923 */  vecD_rule,
  /*  924 */  vecX_rule,
  /*  925 */  vecD_rule,
  /*  926 */  vecX_rule,
  /*  927 */  vecD_rule,
  /*  928 */  vecX_rule,
  /*  929 */  vecD_rule,
  /*  930 */  vecX_rule,
  /*  931 */  vecD_rule,
  /*  932 */  vecD_rule,
  /*  933 */  vecX_rule,
  /*  934 */  vecX_rule,
  /*  935 */  vecD_rule,
  /*  936 */  vecX_rule,
  /*  937 */  vecD_rule,
  /*  938 */  vecX_rule,
  /*  939 */  vecD_rule,
  /*  940 */  vecX_rule,
  /*  941 */  vecD_rule,
  /*  942 */  vecX_rule,
  /*  943 */  vecX_rule,
  /*  944 */  vecX_rule,
  /*  945 */  vecX_rule,
  /*  946 */  vecX_rule,
  /*  947 */  vecX_rule,
  /*  948 */  vecX_rule,
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  immIU12_rule,
  /*  106 */  immLU12_rule,
  /*  107 */  immLU12_rule,
  /*  108 */  immLU12_rule,
  /*  109 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  110 */  _LShiftL_iRegL_immIScale_rule,
  /*  111 */  iRegL_rule,
  /*  112 */  immIOffset_rule,
  /*  113 */  immIOffset4_rule,
  /*  114 */  immIOffset8_rule,
  /*  115 */  immIOffset16_rule,
  /*  116 */  immLoffset_rule,
  /*  117 */  immLoffset4_rule,
  /*  118 */  immLoffset8_rule,
  /*  119 */  immLoffset16_rule,
  /*  120 */  0,
  /*  121 */  immIU12_rule,
  /*  122 */  immLU12_rule,
  /*  123 */  immLU12_rule,
  /*  124 */  immLU12_rule,
  /*  125 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  126 */  _LShiftL_iRegL_immIScale_rule,
  /*  127 */  iRegL_rule,
  /*  128 */  immIOffset_rule,
  /*  129 */  immLoffset_rule,
  /*  130 */  immL_pc_off_rule,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  /*  134 */  0,
  /*  135 */  0,
  /*  136 */  0,
  /*  137 */  0,
  /*  138 */  0,
  // last operand
  /*  139 */  0,
  /*  140 */  0,
  /*  141 */  0,
  /*  142 */  0,
  /*  143 */  0,
  // last operand class
  /*  144 */  immIScale_rule,
  /*  145 */  _LShiftL_iRegL_immIScale_rule,
  /*  146 */  0,
  /*  147 */  _ConvI2L_iRegI__rule,
  /*  148 */  immIScale_rule,
  /*  149 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  150 */  0,
  /*  151 */  _LShiftL_iRegL_immIScale_rule,
  /*  152 */  _ConvI2L_iRegI__rule,
  /*  153 */  _LShiftL__ConvI2L_iRegI__immIScale_rule,
  /*  154 */  0,
  /*  155 */  0,
  /*  156 */  0,
  /*  157 */  0,
  /*  158 */  0,
  /*  159 */  0,
  /*  160 */  0,
  /*  161 */  0,
  /*  162 */  0,
  /*  163 */  0,
  /*  164 */  0,
  /*  165 */  0,
  /*  166 */  0,
  /*  167 */  0,
  /*  168 */  0,
  /*  169 */  iRegP_rule,
  /*  170 */  iRegLNoSp_rule,
  /*  171 */  iRegINoSp_rule,
  /*  172 */  iRegNNoSp_rule,
  /*  173 */  rFlagsReg_rule,
  /*  174 */  iRegIorL2I_rule,
  /*  175 */  rFlagsRegU_rule,
  /*  176 */  iRegIorL2I_rule,
  /*  177 */  immI0_rule,
  /*  178 */  immI0_rule,
  /*  179 */  iRegL_rule,
  /*  180 */  immL0_rule,
  /*  181 */  iRegL_rule,
  /*  182 */  immP0_rule,
  /*  183 */  iRegP_rule,
  /*  184 */  iRegN_rule,
  /*  185 */  immN0_rule,
  /*  186 */  iRegN_rule,
  /*  187 */  vRegF_rule,
  /*  188 */  vRegD_rule,
  /*  189 */  0,
  /*  190 */  0,
  /*  191 */  immIScale_rule,
  /*  192 */  iRegIorL2I_rule,
  /*  193 */  iRegL_rule,
  /*  194 */  immI_31_rule,
  /*  195 */  immI_31_rule,
  /*  196 */  immL_63_rule,
  /*  197 */  immL_63_rule,
  /*  198 */  immI_M1_rule,
  /*  199 */  immL_M1_rule,
  /*  200 */  iRegIorL2I_rule,
  /*  201 */  iRegL_rule,
  /*  202 */  immI_rule,
  /*  203 */  immI_M1_rule,
  /*  204 */  immI_rule,
  /*  205 */  immL_M1_rule,
  /*  206 */  immI_rule,
  /*  207 */  immI_M1_rule,
  /*  208 */  immI_rule,
  /*  209 */  immL_M1_rule,
  /*  210 */  immI_rule,
  /*  211 */  immI_M1_rule,
  /*  212 */  immI_rule,
  /*  213 */  immL_M1_rule,
  /*  214 */  iRegIorL2I_rule,
  /*  215 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  216 */  iRegL_rule,
  /*  217 */  _URShiftL_iRegL_immI_rule,
  /*  218 */  iRegIorL2I_rule,
  /*  219 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  220 */  iRegL_rule,
  /*  221 */  _RShiftL_iRegL_immI_rule,
  /*  222 */  iRegIorL2I_rule,
  /*  223 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  224 */  iRegL_rule,
  /*  225 */  _LShiftL_iRegL_immI_rule,
  /*  226 */  immI_bitmask_rule,
  /*  227 */  iRegI_rule,
  /*  228 */  iRegI_rule,
  /*  229 */  _SubI_immI_64_iRegI_rule,
  /*  230 */  iRegI_rule,
  /*  231 */  _SubI_immI0_iRegI_rule,
  /*  232 */  iRegI_rule,
  /*  233 */  iRegI_rule,
  /*  234 */  _SubI_immI_32_iRegI_rule,
  /*  235 */  _SubI_immI0_iRegI_rule,
  /*  236 */  iRegI_rule,
  /*  237 */  _SubI_immI_64_iRegI_rule,
  /*  238 */  _SubI_immI0_iRegI_rule,
  /*  239 */  iRegI_rule,
  /*  240 */  _SubI_immI_32_iRegI_rule,
  /*  241 */  _SubI_immI0_iRegI_rule,
  /*  242 */  immI_16_rule,
  /*  243 */  immI_16_rule,
  /*  244 */  immI_24_rule,
  /*  245 */  immI_24_rule,
  /*  246 */  immI_24_rule,
  /*  247 */  immI_48_rule,
  /*  248 */  immI_48_rule,
  /*  249 */  immI_32_rule,
  /*  250 */  immI_32_rule,
  /*  251 */  immI_56_rule,
  /*  252 */  immI_56_rule,
  /*  253 */  immI_56_rule,
  /*  254 */  immI_255_rule,
  /*  255 */  immI_65535_rule,
  /*  256 */  immL_255_rule,
  /*  257 */  immL_65535_rule,
  /*  258 */  immL_4294967295_rule,
  /*  259 */  0,
  /*  260 */  0,
  /*  261 */  iRegIorL2I_rule,
  /*  262 */  iRegL_rule,
  /*  263 */  immI0_rule,
  /*  264 */  immL0_rule,
  /*  265 */  immP0_rule,
  /*  266 */  immN0_rule,
  /*  267 */  immP0_rule,
  /*  268 */  immI0_rule,
  /*  269 */  immL0_rule,
  /*  270 */  immL_rule,
  /*  271 */  immL0_rule,
  /*  272 */  immI_rule,
  /*  273 */  immI0_rule,
  /*  274 */  iRegL_rule,
  /*  275 */  iRegIorL2I_rule,
  /*  276 */  iRegP_R0_rule,
  /*  277 */  iRegI_R2_rule,
  /*  278 */  iRegI_R4_rule,
  /*  279 */  iRegI_R4_rule,
  /*  280 */  iRegI_R2_rule,
  /*  281 */  immI_le_4_rule,
  /*  282 */  iRegP_R3_rule,
  /*  283 */  iRegI_R3_rule,
  /*  284 */  vecD_rule,
  /*  285 */  vecX_rule,
  /*  286 */  vecD_rule,
  /*  287 */  vecX_rule,
  // last internally defined operand
  /*  288 */  0,
  /*  289 */  0,
  /*  290 */  0,
  /*  291 */  0,
  /*  292 */  0,
  /*  293 */  0,
  /*  294 */  0,
  /*  295 */  0,
  /*  296 */  0,
  /*  297 */  0,
  /*  298 */  0,
  /*  299 */  0,
  /*  300 */  0,
  /*  301 */  0,
  /*  302 */  0,
  /*  303 */  0,
  /*  304 */  0,
  /*  305 */  0,
  /*  306 */  0,
  /*  307 */  0,
  /*  308 */  0,
  /*  309 */  0,
  /*  310 */  0,
  /*  311 */  _Binary_iRegP_iRegP_rule,
  /*  312 */  _Binary_iRegLNoSp_iRegLNoSp_rule,
  /*  313 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  314 */  0,
  /*  315 */  0,
  /*  316 */  0,
  /*  317 */  0,
  /*  318 */  0,
  /*  319 */  0,
  /*  320 */  0,
  /*  321 */  0,
  /*  322 */  0,
  /*  323 */  0,
  /*  324 */  0,
  /*  325 */  0,
  /*  326 */  iRegIorL2I_rule,
  /*  327 */  immIAddSub_rule,
  /*  328 */  iRegL_rule,
  /*  329 */  immLAddSub_rule,
  /*  330 */  iRegIorL2I_rule,
  /*  331 */  immIAddSub_rule,
  /*  332 */  iRegL_rule,
  /*  333 */  immLAddSub_rule,
  /*  334 */  iRegIorL2I_rule,
  /*  335 */  iRegL_rule,
  /*  336 */  iRegIorL2I_rule,
  /*  337 */  iRegL_rule,
  /*  338 */  iRegI_rule,
  /*  339 */  immI0_rule,
  /*  340 */  immIAddSub_rule,
  /*  341 */  immI_rule,
  /*  342 */  iRegI_rule,
  /*  343 */  immI0_rule,
  /*  344 */  immIAddSub_rule,
  /*  345 */  immI_rule,
  /*  346 */  iRegL_rule,
  /*  347 */  immL0_rule,
  /*  348 */  immLAddSub_rule,
  /*  349 */  immL_rule,
  /*  350 */  iRegL_rule,
  /*  351 */  immL0_rule,
  /*  352 */  immLAddSub_rule,
  /*  353 */  immL_rule,
  /*  354 */  iRegP_rule,
  /*  355 */  iRegN_rule,
  /*  356 */  immP0_rule,
  /*  357 */  immN0_rule,
  /*  358 */  vRegF_rule,
  /*  359 */  immF0_rule,
  /*  360 */  vRegD_rule,
  /*  361 */  immD0_rule,
  /*  362 */  immL0_rule,
  /*  363 */  immI0_rule,
  /*  364 */  immL0_rule,
  /*  365 */  immI0_rule,
  /*  366 */  iRegP_rule,
  /*  367 */  iRegP_rule,
  /*  368 */  immP0_rule,
  /*  369 */  0,
  /*  370 */  0,
  /*  371 */  0,
  /*  372 */  0,
  /*  373 */  0,
  /*  374 */  0,
  /*  375 */  0,
  /*  376 */  0,
  /*  377 */  0,
  /*  378 */  0,
  /*  379 */  0,
  /*  380 */  0,
  /*  381 */  0,
  /*  382 */  immL_32bits_rule,
  /*  383 */  0,
  /*  384 */  0,
  /*  385 */  0,
  /*  386 */  0,
  /*  387 */  0,
  /*  388 */  0,
  /*  389 */  0,
  /*  390 */  0,
  /*  391 */  immI0_rule,
  /*  392 */  immI0_rule,
  /*  393 */  iRegIorL2I_rule,
  /*  394 */  immI0_rule,
  /*  395 */  iRegIorL2I_rule,
  /*  396 */  immI0_rule,
  /*  397 */  iRegIorL2I_rule,
  /*  398 */  immI0_rule,
  /*  399 */  iRegL_rule,
  /*  400 */  immL0_rule,
  /*  401 */  iRegP_rule,
  /*  402 */  immP0_rule,
  /*  403 */  iRegN_rule,
  /*  404 */  immN0_rule,
  /*  405 */  vRegF_rule,
  /*  406 */  vRegD_rule,
  /*  407 */  iRegN_rule,
  /*  408 */  0,
  /*  409 */  0,
  /*  410 */  0,
  /*  411 */  0,
  /*  412 */  0,
  /*  413 */  0,
  /*  414 */  0,
  /*  415 */  0,
  /*  416 */  0,
  /*  417 */  0,
  /*  418 */  0,
  /*  419 */  0,
  /*  420 */  immL_32bits_rule,
  /*  421 */  0,
  /*  422 */  0,
  /*  423 */  0,
  /*  424 */  0,
  /*  425 */  0,
  /*  426 */  iRegIorL2I_rule,
  /*  427 */  iRegIorL2I_rule,
  /*  428 */  iRegIorL2I_rule,
  /*  429 */  iRegL_rule,
  /*  430 */  iRegP_rule,
  /*  431 */  iRegN_rule,
  /*  432 */  vRegF_rule,
  /*  433 */  vRegD_rule,
  /*  434 */  0,
  /*  435 */  0,
  /*  436 */  0,
  /*  437 */  0,
  /*  438 */  0,
  /*  439 */  0,
  /*  440 */  0,
  /*  441 */  0,
  /*  442 */  0,
  /*  443 */  0,
  /*  444 */  0,
  /*  445 */  0,
  /*  446 */  0,
  /*  447 */  0,
  /*  448 */  0,
  /*  449 */  0,
  /*  450 */  0,
  /*  451 */  0,
  /*  452 */  0,
  /*  453 */  0,
  /*  454 */  0,
  /*  455 */  0,
  /*  456 */  0,
  /*  457 */  0,
  /*  458 */  0,
  /*  459 */  0,
  /*  460 */  0,
  /*  461 */  0,
  /*  462 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  463 */  _Binary_iRegLNoSp_iRegLNoSp_rule,
  /*  464 */  _Binary_iRegP_iRegP_rule,
  /*  465 */  _Binary_iRegNNoSp_iRegNNoSp_rule,
  /*  466 */  _Binary_iRegINoSp_iRegINoSp_rule,
  /*  467 */  _Binary_iRegLNoSp_iRegLNoSp_rule,
  /*  468 */  _Binary_iRegP_iRegP_rule,
  /*  469 */  _Binary_iRegNNoSp_iRegNNoSp_rule,
  /*  470 */  iRegI_rule,
  /*  471 */  iRegL_rule,
  /*  472 */  iRegN_rule,
  /*  473 */  iRegP_rule,
  /*  474 */  iRegL_rule,
  /*  475 */  iRegL_rule,
  /*  476 */  immLAddSub_rule,
  /*  477 */  immLAddSub_rule,
  /*  478 */  iRegIorL2I_rule,
  /*  479 */  iRegIorL2I_rule,
  /*  480 */  immIAddSub_rule,
  /*  481 */  immIAddSub_rule,
  /*  482 */  _Binary_iRegIorL2I_iRegIorL2I_rule,
  /*  483 */  _Binary_iRegIorL2I_iRegIorL2I_rule,
  /*  484 */  _Binary_immI0_iRegIorL2I_rule,
  /*  485 */  _Binary_immI0_iRegIorL2I_rule,
  /*  486 */  _Binary_iRegIorL2I_immI0_rule,
  /*  487 */  _Binary_iRegIorL2I_immI0_rule,
  /*  488 */  _Binary_immI_1_immI0_rule,
  /*  489 */  _Binary_immI_1_immI0_rule,
  /*  490 */  _Binary_iRegL_iRegL_rule,
  /*  491 */  _Binary_iRegL_iRegL_rule,
  /*  492 */  _Binary_iRegL_immL0_rule,
  /*  493 */  _Binary_iRegL_immL0_rule,
  /*  494 */  _Binary_immL0_iRegL_rule,
  /*  495 */  _Binary_immL0_iRegL_rule,
  /*  496 */  _Binary_iRegP_iRegP_rule,
  /*  497 */  _Binary_iRegP_iRegP_rule,
  /*  498 */  _Binary_iRegP_immP0_rule,
  /*  499 */  _Binary_iRegP_immP0_rule,
  /*  500 */  _Binary_immP0_iRegP_rule,
  /*  501 */  _Binary_immP0_iRegP_rule,
  /*  502 */  _Binary_iRegN_iRegN_rule,
  /*  503 */  _Binary_iRegN_iRegN_rule,
  /*  504 */  _Binary_iRegN_immN0_rule,
  /*  505 */  _Binary_iRegN_immN0_rule,
  /*  506 */  _Binary_immN0_iRegN_rule,
  /*  507 */  _Binary_immN0_iRegN_rule,
  /*  508 */  _Binary_vRegF_vRegF_rule,
  /*  509 */  _Binary_vRegF_vRegF_rule,
  /*  510 */  _Binary_vRegD_vRegD_rule,
  /*  511 */  _Binary_vRegD_vRegD_rule,
  /*  512 */  iRegIorL2I_rule,
  /*  513 */  immIAddSub_rule,
  /*  514 */  immIAddSub_rule,
  /*  515 */  iRegL_rule,
  /*  516 */  _ConvI2L_iRegIorL2I__rule,
  /*  517 */  _LShiftL_iRegL_immIScale_rule,
  /*  518 */  _LShiftL__ConvI2L_iRegIorL2I__immIScale_rule,
  /*  519 */  immI_rule,
  /*  520 */  immLAddSub_rule,
  /*  521 */  iRegL_rule,
  /*  522 */  immLAddSub_rule,
  /*  523 */  iRegIorL2I_rule,
  /*  524 */  immIAddSub_rule,
  /*  525 */  iRegL_rule,
  /*  526 */  immLAddSub_rule,
  /*  527 */  iRegIorL2I_rule,
  /*  528 */  iRegIorL2I_rule,
  /*  529 */  iRegIorL2I_rule,
  /*  530 */  _ConvI2L_iRegIorL2I__rule,
  /*  531 */  _ConvI2L_iRegIorL2I__rule,
  /*  532 */  iRegL_rule,
  /*  533 */  iRegL_rule,
  /*  534 */  _MulI_iRegIorL2I_iRegIorL2I_rule,
  /*  535 */  iRegIorL2I_rule,
  /*  536 */  _MulI_iRegIorL2I_iRegIorL2I_rule,
  /*  537 */  _MulL_iRegL_iRegL_rule,
  /*  538 */  iRegL_rule,
  /*  539 */  _MulL_iRegL_iRegL_rule,
  /*  540 */  iRegIorL2I_rule,
  /*  541 */  immI_31_rule,
  /*  542 */  _URShiftI__RShiftI_iRegIorL2I_immI_31_immI_31_rule,
  /*  543 */  iRegIorL2I_rule,
  /*  544 */  iRegL_rule,
  /*  545 */  immL_63_rule,
  /*  546 */  _URShiftL__RShiftL_iRegL_immL_63_immL_63_rule,
  /*  547 */  iRegL_rule,
  /*  548 */  iRegIorL2I_rule,
  /*  549 */  iRegL_rule,
  /*  550 */  iRegIorL2I_rule,
  /*  551 */  immI_rule,
  /*  552 */  iRegIorL2I_rule,
  /*  553 */  immI_rule,
  /*  554 */  iRegIorL2I_rule,
  /*  555 */  immI_rule,
  /*  556 */  iRegIorL2I_rule,
  /*  557 */  immI_rule,
  /*  558 */  iRegIorL2I_rule,
  /*  559 */  immI_rule,
  /*  560 */  immI_rule,
  /*  561 */  iRegIorL2I_rule,
  /*  562 */  immI_rule,
  /*  563 */  immL_M1_rule,
  /*  564 */  immI_M1_rule,
  /*  565 */  _XorI_iRegIorL2I_immI_M1_rule,
  /*  566 */  iRegIorL2I_rule,
  /*  567 */  _XorL_iRegL_immL_M1_rule,
  /*  568 */  iRegL_rule,
  /*  569 */  _XorI_iRegIorL2I_immI_M1_rule,
  /*  570 */  iRegIorL2I_rule,
  /*  571 */  _XorL_iRegL_immL_M1_rule,
  /*  572 */  iRegL_rule,
  /*  573 */  _XorI_iRegIorL2I_iRegIorL2I_rule,
  /*  574 */  immI_M1_rule,
  /*  575 */  _XorL_iRegL_iRegL_rule,
  /*  576 */  immL_M1_rule,
  /*  577 */  _XorI__URShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  578 */  iRegIorL2I_rule,
  /*  579 */  _XorL__URShiftL_iRegL_immI_immL_M1_rule,
  /*  580 */  iRegL_rule,
  /*  581 */  _XorI__RShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  582 */  iRegIorL2I_rule,
  /*  583 */  _XorL__RShiftL_iRegL_immI_immL_M1_rule,
  /*  584 */  iRegL_rule,
  /*  585 */  _XorI__LShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  586 */  iRegIorL2I_rule,
  /*  587 */  _XorL__LShiftL_iRegL_immI_immL_M1_rule,
  /*  588 */  iRegL_rule,
  /*  589 */  _XorI__URShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  590 */  immI_M1_rule,
  /*  591 */  _XorI_iRegIorL2I__URShiftI_iRegIorL2I_immI_rule,
  /*  592 */  immI_M1_rule,
  /*  593 */  _XorL__URShiftL_iRegL_immI_iRegL_rule,
  /*  594 */  immL_M1_rule,
  /*  595 */  _XorL_iRegL__URShiftL_iRegL_immI_rule,
  /*  596 */  immL_M1_rule,
  /*  597 */  _XorI__RShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  598 */  immI_M1_rule,
  /*  599 */  _XorI_iRegIorL2I__RShiftI_iRegIorL2I_immI_rule,
  /*  600 */  immI_M1_rule,
  /*  601 */  _XorL__RShiftL_iRegL_immI_iRegL_rule,
  /*  602 */  immL_M1_rule,
  /*  603 */  _XorL_iRegL__RShiftL_iRegL_immI_rule,
  /*  604 */  immL_M1_rule,
  /*  605 */  _XorI__LShiftI_iRegIorL2I_immI_iRegIorL2I_rule,
  /*  606 */  immI_M1_rule,
  /*  607 */  _XorI_iRegIorL2I__LShiftI_iRegIorL2I_immI_rule,
  /*  608 */  immI_M1_rule,
  /*  609 */  _XorL__LShiftL_iRegL_immI_iRegL_rule,
  /*  610 */  immL_M1_rule,
  /*  611 */  _XorL_iRegL__LShiftL_iRegL_immI_rule,
  /*  612 */  immL_M1_rule,
  /*  613 */  _XorI__URShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  614 */  iRegIorL2I_rule,
  /*  615 */  _XorL__URShiftL_iRegL_immI_immL_M1_rule,
  /*  616 */  iRegL_rule,
  /*  617 */  _XorI__RShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  618 */  iRegIorL2I_rule,
  /*  619 */  _XorL__RShiftL_iRegL_immI_immL_M1_rule,
  /*  620 */  iRegL_rule,
  /*  621 */  _XorI__LShiftI_iRegIorL2I_immI_immI_M1_rule,
  /*  622 */  iRegIorL2I_rule,
  /*  623 */  _XorL__LShiftL_iRegL_immI_immL_M1_rule,
  /*  624 */  iRegL_rule,
  /*  625 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  626 */  iRegIorL2I_rule,
  /*  627 */  _URShiftL_iRegL_immI_rule,
  /*  628 */  iRegL_rule,
  /*  629 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  630 */  iRegIorL2I_rule,
  /*  631 */  _RShiftL_iRegL_immI_rule,
  /*  632 */  iRegL_rule,
  /*  633 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  634 */  iRegIorL2I_rule,
  /*  635 */  _LShiftL_iRegL_immI_rule,
  /*  636 */  iRegL_rule,
  /*  637 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  638 */  iRegIorL2I_rule,
  /*  639 */  _URShiftL_iRegL_immI_rule,
  /*  640 */  iRegL_rule,
  /*  641 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  642 */  iRegIorL2I_rule,
  /*  643 */  _RShiftL_iRegL_immI_rule,
  /*  644 */  iRegL_rule,
  /*  645 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  646 */  iRegIorL2I_rule,
  /*  647 */  _LShiftL_iRegL_immI_rule,
  /*  648 */  iRegL_rule,
  /*  649 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  650 */  iRegIorL2I_rule,
  /*  651 */  _URShiftL_iRegL_immI_rule,
  /*  652 */  iRegL_rule,
  /*  653 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  654 */  iRegIorL2I_rule,
  /*  655 */  _RShiftL_iRegL_immI_rule,
  /*  656 */  iRegL_rule,
  /*  657 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  658 */  iRegIorL2I_rule,
  /*  659 */  _LShiftL_iRegL_immI_rule,
  /*  660 */  iRegL_rule,
  /*  661 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  662 */  iRegIorL2I_rule,
  /*  663 */  _URShiftL_iRegL_immI_rule,
  /*  664 */  iRegL_rule,
  /*  665 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  666 */  iRegIorL2I_rule,
  /*  667 */  _RShiftL_iRegL_immI_rule,
  /*  668 */  iRegL_rule,
  /*  669 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  670 */  iRegIorL2I_rule,
  /*  671 */  _LShiftL_iRegL_immI_rule,
  /*  672 */  iRegL_rule,
  /*  673 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  674 */  _URShiftL_iRegL_immI_rule,
  /*  675 */  _RShiftI_iRegIorL2I_immI_rule,
  /*  676 */  _RShiftL_iRegL_immI_rule,
  /*  677 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  678 */  _LShiftL_iRegL_immI_rule,
  /*  679 */  immI_rule,
  /*  680 */  immI_rule,
  /*  681 */  immI_rule,
  /*  682 */  immI_rule,
  /*  683 */  immI_bitmask_rule,
  /*  684 */  immL_bitmask_rule,
  /*  685 */  0,
  /*  686 */  _URShiftL_iRegL_immI_rule,
  /*  687 */  _LShiftL_iRegL_immI_rule,
  /*  688 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  689 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  690 */  _URShiftL_iRegL_immI_rule,
  /*  691 */  _LShiftL_iRegL_immI_rule,
  /*  692 */  _URShiftI_iRegIorL2I_immI_rule,
  /*  693 */  _LShiftI_iRegIorL2I_immI_rule,
  /*  694 */  0,
  /*  695 */  0,
  /*  696 */  _URShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  697 */  _LShiftL_iRegL_iRegI_rule,
  /*  698 */  _URShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  699 */  _LShiftL_iRegL_iRegI_rule,
  /*  700 */  _URShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  701 */  _LShiftI_iRegI_iRegI_rule,
  /*  702 */  _URShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  703 */  _LShiftI_iRegI_iRegI_rule,
  /*  704 */  0,
  /*  705 */  0,
  /*  706 */  _LShiftL_iRegL__SubI_immI_64_iRegI_rule,
  /*  707 */  _URShiftL_iRegL_iRegI_rule,
  /*  708 */  _LShiftL_iRegL__SubI_immI0_iRegI_rule,
  /*  709 */  _URShiftL_iRegL_iRegI_rule,
  /*  710 */  _LShiftI_iRegI__SubI_immI_32_iRegI_rule,
  /*  711 */  _URShiftI_iRegI_iRegI_rule,
  /*  712 */  _LShiftI_iRegI__SubI_immI0_iRegI_rule,
  /*  713 */  _URShiftI_iRegI_iRegI_rule,
  /*  714 */  _ConvI2L_iRegIorL2I__rule,
  /*  715 */  iRegL_rule,
  /*  716 */  _ConvI2L_iRegIorL2I__rule,
  /*  717 */  _RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16_rule,
  /*  718 */  iRegIorL2I_rule,
  /*  719 */  _RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  720 */  iRegIorL2I_rule,
  /*  721 */  _URShiftI__LShiftI_iRegIorL2I_immI_24_immI_24_rule,
  /*  722 */  iRegIorL2I_rule,
  /*  723 */  _RShiftL__LShiftL_iRegL_immI_48_immI_48_rule,
  /*  724 */  iRegL_rule,
  /*  725 */  _RShiftL__LShiftL_iRegL_immI_32_immI_32_rule,
  /*  726 */  iRegL_rule,
  /*  727 */  _RShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  728 */  iRegL_rule,
  /*  729 */  _URShiftL__LShiftL_iRegL_immI_56_immI_56_rule,
  /*  730 */  iRegL_rule,
  /*  731 */  _AndI_iRegIorL2I_immI_255_rule,
  /*  732 */  iRegIorL2I_rule,
  /*  733 */  _AndI_iRegIorL2I_immI_65535_rule,
  /*  734 */  iRegIorL2I_rule,
  /*  735 */  _AndL_iRegL_immL_255_rule,
  /*  736 */  iRegL_rule,
  /*  737 */  _AndL_iRegL_immL_65535_rule,
  /*  738 */  iRegL_rule,
  /*  739 */  _AndL_iRegL_immL_4294967295_rule,
  /*  740 */  iRegL_rule,
  /*  741 */  _AndI_iRegIorL2I_immI_255_rule,
  /*  742 */  _AndI_iRegIorL2I_immI_65535_rule,
  /*  743 */  _AndL_iRegL_immL_255_rule,
  /*  744 */  _AndL_iRegL_immL_65535_rule,
  /*  745 */  _AndL_iRegL_immL_4294967295_rule,
  /*  746 */  vRegF_rule,
  /*  747 */  vRegD_rule,
  /*  748 */  vRegF_rule,
  /*  749 */  vRegD_rule,
  /*  750 */  vRegF_rule,
  /*  751 */  vRegD_rule,
  /*  752 */  vRegF_rule,
  /*  753 */  vRegD_rule,
  /*  754 */  0,
  /*  755 */  0,
  /*  756 */  iRegIorL2I_rule,
  /*  757 */  immILog_rule,
  /*  758 */  iRegIorL2I_rule,
  /*  759 */  immILog_rule,
  /*  760 */  iRegIorL2I_rule,
  /*  761 */  immILog_rule,
  /*  762 */  iRegL_rule,
  /*  763 */  immLLog_rule,
  /*  764 */  iRegL_rule,
  /*  765 */  immLLog_rule,
  /*  766 */  iRegL_rule,
  /*  767 */  immLLog_rule,
  /*  768 */  0,
  /*  769 */  immL_32bits_rule,
  /*  770 */  0,
  /*  771 */  0,
  /*  772 */  0,
  /*  773 */  0,
  /*  774 */  0,
  /*  775 */  0,
  /*  776 */  0,
  /*  777 */  0,
  /*  778 */  0,
  /*  779 */  0,
  /*  780 */  0,
  /*  781 */  0,
  /*  782 */  0,
  /*  783 */  0,
  /*  784 */  0,
  /*  785 */  0,
  /*  786 */  0,
  /*  787 */  iRegP_R10_rule,
  /*  788 */  iRegP_R10_rule,
  /*  789 */  _OverflowMulI_iRegIorL2I_iRegIorL2I_rule,
  /*  790 */  _OverflowMulL_iRegL_iRegL_rule,
  /*  791 */  vRegF_rule,
  /*  792 */  vRegD_rule,
  /*  793 */  immF0_rule,
  /*  794 */  immD0_rule,
  /*  795 */  iRegL_rule,
  /*  796 */  iRegIorL2I_rule,
  /*  797 */  immI0_rule,
  /*  798 */  iRegI_rule,
  /*  799 */  iRegI_rule,
  /*  800 */  0,
  /*  801 */  rFlagsReg_rule,
  /*  802 */  rFlagsRegU_rule,
  /*  803 */  _CmpI_iRegIorL2I_immI0_rule,
  /*  804 */  _CmpL_iRegL_immL0_rule,
  /*  805 */  _CmpP_iRegP_immP0_rule,
  /*  806 */  _CmpN_iRegN_immN0_rule,
  /*  807 */  _CmpP__DecodeN_iRegN__immP0_rule,
  /*  808 */  _CmpU_iRegIorL2I_immI0_rule,
  /*  809 */  _CmpUL_iRegL_immL0_rule,
  /*  810 */  _CmpL_iRegL_immL0_rule,
  /*  811 */  _CmpI_iRegIorL2I_immI0_rule,
  /*  812 */  _CmpL__AndL_iRegL_immL_immL0_rule,
  /*  813 */  _CmpI__AndI_iRegIorL2I_immI_immI0_rule,
  /*  814 */  _CmpL_iRegL_immL0_rule,
  /*  815 */  _CmpI_iRegIorL2I_immI0_rule,
  /*  816 */  _CmpL__AndL_iRegL_immL_immL0_rule,
  /*  817 */  _CmpI__AndI_iRegIorL2I_immI_immI0_rule,
  /*  818 */  rFlagsReg_rule,
  /*  819 */  rFlagsRegU_rule,
  /*  820 */  0,
  /*  821 */  0,
  /*  822 */  0,
  /*  823 */  0,
  /*  824 */  0,
  /*  825 */  0,
  /*  826 */  0,
  /*  827 */  inline_cache_RegP_rule,
  /*  828 */  iRegP_R0_rule,
  /*  829 */  0,
  /*  830 */  0,
  /*  831 */  0,
  /*  832 */  0,
  /*  833 */  iRegP_R0_rule,
  /*  834 */  _Binary_iRegP_R3_iRegI_R4_rule,
  /*  835 */  _Binary_iRegP_R3_iRegI_R2_rule,
  /*  836 */  _Binary_iRegP_R3_immI_le_4_rule,
  /*  837 */  iRegI_R4_rule,
  /*  838 */  iRegP_R2_rule,
  /*  839 */  _Binary_iRegP_R1_iRegI_R3_rule,
  /*  840 */  0,
  /*  841 */  0,
  /*  842 */  0,
  /*  843 */  vecD_rule,
  /*  844 */  vecD_rule,
  /*  845 */  vecX_rule,
  /*  846 */  0,
  /*  847 */  0,
  /*  848 */  0,
  /*  849 */  0,
  /*  850 */  0,
  /*  851 */  0,
  /*  852 */  0,
  /*  853 */  0,
  /*  854 */  0,
  /*  855 */  0,
  /*  856 */  0,
  /*  857 */  0,
  /*  858 */  0,
  /*  859 */  0,
  /*  860 */  0,
  /*  861 */  0,
  /*  862 */  0,
  /*  863 */  vecD_rule,
  /*  864 */  vecX_rule,
  /*  865 */  vecD_rule,
  /*  866 */  vecX_rule,
  /*  867 */  vecD_rule,
  /*  868 */  vecX_rule,
  /*  869 */  vecX_rule,
  /*  870 */  vecD_rule,
  /*  871 */  vecX_rule,
  /*  872 */  vecX_rule,
  /*  873 */  vecD_rule,
  /*  874 */  vecX_rule,
  /*  875 */  vecD_rule,
  /*  876 */  vecX_rule,
  /*  877 */  vecD_rule,
  /*  878 */  vecX_rule,
  /*  879 */  vecX_rule,
  /*  880 */  vecD_rule,
  /*  881 */  vecX_rule,
  /*  882 */  vecX_rule,
  /*  883 */  vecD_rule,
  /*  884 */  vecX_rule,
  /*  885 */  vecD_rule,
  /*  886 */  vecX_rule,
  /*  887 */  vecD_rule,
  /*  888 */  vecX_rule,
  /*  889 */  vecX_rule,
  /*  890 */  _MulVS_vecD_vecD_rule,
  /*  891 */  _MulVS_vecX_vecX_rule,
  /*  892 */  _MulVI_vecD_vecD_rule,
  /*  893 */  _MulVI_vecX_vecX_rule,
  /*  894 */  _MulVS_vecD_vecD_rule,
  /*  895 */  _MulVS_vecX_vecX_rule,
  /*  896 */  _MulVI_vecD_vecD_rule,
  /*  897 */  _MulVI_vecX_vecX_rule,
  /*  898 */  vecD_rule,
  /*  899 */  vecX_rule,
  /*  900 */  vecX_rule,
  /*  901 */  vecD_rule,
  /*  902 */  vecX_rule,
  /*  903 */  vecD_rule,
  /*  904 */  vecX_rule,
  /*  905 */  vecD_rule,
  /*  906 */  vecX_rule,
  /*  907 */  vecX_rule,
  /*  908 */  vecX_rule,
  /*  909 */  vecX_rule,
  /*  910 */  vecX_rule,
  /*  911 */  vecX_rule,
  /*  912 */  vecX_rule,
  /*  913 */  immI_rule,
  /*  914 */  immI_rule,
  /*  915 */  immI_rule,
  /*  916 */  immI_rule,
  /*  917 */  immI_rule,
  /*  918 */  immI_rule,
  /*  919 */  vecX_rule,
  /*  920 */  vecX_rule,
  /*  921 */  vecX_rule,
  /*  922 */  vecX_rule,
  /*  923 */  vecX_rule,
  /*  924 */  vecX_rule,
  /*  925 */  immI_rule,
  /*  926 */  immI_rule,
  /*  927 */  immI_rule,
  /*  928 */  immI_rule,
  /*  929 */  immI_rule,
  /*  930 */  immI_rule,
  /*  931 */  vecX_rule,
  /*  932 */  vecX_rule,
  /*  933 */  vecX_rule,
  /*  934 */  vecX_rule,
  /*  935 */  vecX_rule,
  /*  936 */  vecX_rule,
  /*  937 */  immI_rule,
  /*  938 */  immI_rule,
  /*  939 */  immI_rule,
  /*  940 */  immI_rule,
  /*  941 */  immI_rule,
  /*  942 */  immI_rule,
  /*  943 */  vecX_rule,
  /*  944 */  vecX_rule,
  /*  945 */  vecX_rule,
  /*  946 */  immI_rule,
  /*  947 */  immI_rule,
  /*  948 */  immI_rule,
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  /*    0 */  "UNIVERSE",
  /*    1 */  "LABEL",
  /*    2 */  "SREGI",
  /*    3 */  "SREGP",
  /*    4 */  "SREGF",
  /*    5 */  "SREGD",
  /*    6 */  "SREGL",
  /*    7 */  "METHOD",
  /*    8 */  "IMMI",
  /*    9 */  "IMMI0",
  /*   10 */  "IMMI_1",
  /*   11 */  "IMMI_M1",
  /*   12 */  "IMMI_LE_4",
  /*   13 */  "IMMI_31",
  /*   14 */  "IMMI_8",
  /*   15 */  "IMMI_16",
  /*   16 */  "IMMI_24",
  /*   17 */  "IMMI_32",
  /*   18 */  "IMMI_48",
  /*   19 */  "IMMI_56",
  /*   20 */  "IMMI_64",
  /*   21 */  "IMMI_255",
  /*   22 */  "IMMI_65535",
  /*   23 */  "IMML_63",
  /*   24 */  "IMML_255",
  /*   25 */  "IMML_65535",
  /*   26 */  "IMML_4294967295",
  /*   27 */  "IMML_BITMASK",
  /*   28 */  "IMMI_BITMASK",
  /*   29 */  "IMMISCALE",
  /*   30 */  "IMMI26",
  /*   31 */  "IMMI19",
  /*   32 */  "IMMIU12",
  /*   33 */  "IMMLU12",
  /*   34 */  "IMMIOFFSET",
  /*   35 */  "IMMIOFFSET4",
  /*   36 */  "IMMIOFFSET8",
  /*   37 */  "IMMIOFFSET16",
  /*   38 */  "IMMLOFFSET",
  /*   39 */  "IMMLOFFSET4",
  /*   40 */  "IMMLOFFSET8",
  /*   41 */  "IMMLOFFSET16",
  /*   42 */  "IMMIADDSUB",
  /*   43 */  "IMMILOG",
  /*   44 */  "IMML",
  /*   45 */  "IMML0",
  /*   46 */  "IMML_1",
  /*   47 */  "IMML_M1",
  /*   48 */  "IMML_PC_OFF",
  /*   49 */  "IMMLADDSUB",
  /*   50 */  "IMMLLOG",
  /*   51 */  "IMML_32BITS",
  /*   52 */  "IMMP",
  /*   53 */  "IMMP0",
  /*   54 */  "IMMP_1",
  /*   55 */  "IMMPOLLPAGE",
  /*   56 */  "IMMBYTEMAPBASE",
  /*   57 */  "IMMP_M1",
  /*   58 */  "IMMP_M2",
  /*   59 */  "IMMD",
  /*   60 */  "IMMD0",
  /*   61 */  "IMMDPACKED",
  /*   62 */  "IMMF",
  /*   63 */  "IMMF0",
  /*   64 */  "IMMFPACKED",
  /*   65 */  "IMMN",
  /*   66 */  "IMMN0",
  /*   67 */  "IMMNKLASS",
  /*   68 */  "IREGI",
  /*   69 */  "IREGINOSP",
  /*   70 */  "IREGL",
  /*   71 */  "IREGLNOSP",
  /*   72 */  "IREGP",
  /*   73 */  "IREGPNOSP",
  /*   74 */  "IREGP_R0",
  /*   75 */  "IREGP_R1",
  /*   76 */  "IREGP_R2",
  /*   77 */  "IREGP_R3",
  /*   78 */  "IREGP_R4",
  /*   79 */  "IREGP_R5",
  /*   80 */  "IREGP_R10",
  /*   81 */  "IREGL_R11",
  /*   82 */  "IREGP_FP",
  /*   83 */  "IREGI_R0",
  /*   84 */  "IREGI_R2",
  /*   85 */  "IREGI_R3",
  /*   86 */  "IREGI_R4",
  /*   87 */  "IREGN",
  /*   88 */  "IREGNNOSP",
  /*   89 */  "IREGIHEAPBASE",
  /*   90 */  "VREGF",
  /*   91 */  "VREGD",
  /*   92 */  "VECD",
  /*   93 */  "VECX",
  /*   94 */  "VREGD_V0",
  /*   95 */  "VREGD_V1",
  /*   96 */  "VREGD_V2",
  /*   97 */  "VREGD_V3",
  /*   98 */  "RFLAGSREG",
  /*   99 */  "RFLAGSREGU",
  /*  100 */  "INLINE_CACHE_REGP",
  /*  101 */  "INTERPRETER_METHOD_OOP_REGP",
  /*  102 */  "THREAD_REGP",
  /*  103 */  "LR_REGP",
  /*  104 */  "INDIRECT",
  /*  105 */  "INDINDEXSCALEDOFFSETI",
  /*  106 */  "INDINDEXSCALEDOFFSETL",
  /*  107 */  "INDINDEXOFFSETI2L",
  /*  108 */  "INDINDEXSCALEDOFFSETI2L",
  /*  109 */  "INDINDEXSCALEDI2L",
  /*  110 */  "INDINDEXSCALED",
  /*  111 */  "INDINDEX",
  /*  112 */  "INDOFFI",
  /*  113 */  "INDOFFI4",
  /*  114 */  "INDOFFI8",
  /*  115 */  "INDOFFI16",
  /*  116 */  "INDOFFL",
  /*  117 */  "INDOFFL4",
  /*  118 */  "INDOFFL8",
  /*  119 */  "INDOFFL16",
  /*  120 */  "INDIRECTN",
  /*  121 */  "INDINDEXSCALEDOFFSETIN",
  /*  122 */  "INDINDEXSCALEDOFFSETLN",
  /*  123 */  "INDINDEXOFFSETI2LN",
  /*  124 */  "INDINDEXSCALEDOFFSETI2LN",
  /*  125 */  "INDINDEXSCALEDI2LN",
  /*  126 */  "INDINDEXSCALEDN",
  /*  127 */  "INDINDEXN",
  /*  128 */  "INDOFFIN",
  /*  129 */  "INDOFFLN",
  /*  130 */  "THREAD_ANCHOR_PC",
  /*  131 */  "STACKSLOTP",
  /*  132 */  "STACKSLOTI",
  /*  133 */  "STACKSLOTF",
  /*  134 */  "STACKSLOTD",
  /*  135 */  "STACKSLOTL",
  /*  136 */  "CMPOP",
  /*  137 */  "CMPOPU",
  /*  138 */  "IREGL2I",
  // last operand
  /*  139 */  "VMEM4",
  /*  140 */  "VMEM8",
  /*  141 */  "VMEM16",
  /*  142 */  "MEMORY",
  /*  143 */  "IREGIORL2I",
  // last operand class
  /*  144 */  "_LShiftL_iRegL_immIScale",
  /*  145 */  "_AddP_iRegP__LShiftL_iRegL_immIScale",
  /*  146 */  "_ConvI2L_iRegI_",
  /*  147 */  "_AddP_iRegP__ConvI2L_iRegI_",
  /*  148 */  "_LShiftL__ConvI2L_iRegI__immIScale",
  /*  149 */  "_AddP_iRegP__LShiftL__ConvI2L_iRegI__immIScale",
  /*  150 */  "_DecodeN_iRegN_",
  /*  151 */  "_AddP__DecodeN_iRegN___LShiftL_iRegL_immIScale",
  /*  152 */  "_AddP__DecodeN_iRegN___ConvI2L_iRegI_",
  /*  153 */  "_AddP__DecodeN_iRegN___LShiftL__ConvI2L_iRegI__immIScale",
  /*  154 */  "_LoadB_memory_",
  /*  155 */  "_LoadUB_memory_",
  /*  156 */  "_LoadS_memory_",
  /*  157 */  "_LoadUS_memory_",
  /*  158 */  "_LoadI_memory_",
  /*  159 */  "_ConvI2L__LoadI_memory__",
  /*  160 */  "_LoadB_indirect_",
  /*  161 */  "_LoadUB_indirect_",
  /*  162 */  "_LoadUS_indirect_",
  /*  163 */  "_LoadS_indirect_",
  /*  164 */  "_LoadI_indirect_",
  /*  165 */  "_ConvI2L__LoadI_indirect__",
  /*  166 */  "_LoadL_memory_",
  /*  167 */  "_CastP2X_iRegP_",
  /*  168 */  "_CastP2X__DecodeN_iRegN__",
  /*  169 */  "_Binary_iRegP_iRegP",
  /*  170 */  "_Binary_iRegLNoSp_iRegLNoSp",
  /*  171 */  "_Binary_iRegINoSp_iRegINoSp",
  /*  172 */  "_Binary_iRegNNoSp_iRegNNoSp",
  /*  173 */  "_Binary_cmpOp_rFlagsReg",
  /*  174 */  "_Binary_iRegIorL2I_iRegIorL2I",
  /*  175 */  "_Binary_cmpOpU_rFlagsRegU",
  /*  176 */  "_Binary_immI0_iRegIorL2I",
  /*  177 */  "_Binary_iRegIorL2I_immI0",
  /*  178 */  "_Binary_immI_1_immI0",
  /*  179 */  "_Binary_iRegL_iRegL",
  /*  180 */  "_Binary_iRegL_immL0",
  /*  181 */  "_Binary_immL0_iRegL",
  /*  182 */  "_Binary_iRegP_immP0",
  /*  183 */  "_Binary_immP0_iRegP",
  /*  184 */  "_Binary_iRegN_iRegN",
  /*  185 */  "_Binary_iRegN_immN0",
  /*  186 */  "_Binary_immN0_iRegN",
  /*  187 */  "_Binary_vRegF_vRegF",
  /*  188 */  "_Binary_vRegD_vRegD",
  /*  189 */  "_ConvL2I_iRegL_",
  /*  190 */  "_ConvI2L_iRegIorL2I_",
  /*  191 */  "_LShiftL__ConvI2L_iRegIorL2I__immIScale",
  /*  192 */  "_MulI_iRegIorL2I_iRegIorL2I",
  /*  193 */  "_MulL_iRegL_iRegL",
  /*  194 */  "_RShiftI_iRegIorL2I_immI_31",
  /*  195 */  "_URShiftI__RShiftI_iRegIorL2I_immI_31_immI_31",
  /*  196 */  "_RShiftL_iRegL_immL_63",
  /*  197 */  "_URShiftL__RShiftL_iRegL_immL_63_immL_63",
  /*  198 */  "_XorI_iRegIorL2I_immI_M1",
  /*  199 */  "_XorL_iRegL_immL_M1",
  /*  200 */  "_XorI_iRegIorL2I_iRegIorL2I",
  /*  201 */  "_XorL_iRegL_iRegL",
  /*  202 */  "_URShiftI_iRegIorL2I_immI",
  /*  203 */  "_XorI__URShiftI_iRegIorL2I_immI_immI_M1",
  /*  204 */  "_URShiftL_iRegL_immI",
  /*  205 */  "_XorL__URShiftL_iRegL_immI_immL_M1",
  /*  206 */  "_RShiftI_iRegIorL2I_immI",
  /*  207 */  "_XorI__RShiftI_iRegIorL2I_immI_immI_M1",
  /*  208 */  "_RShiftL_iRegL_immI",
  /*  209 */  "_XorL__RShiftL_iRegL_immI_immL_M1",
  /*  210 */  "_LShiftI_iRegIorL2I_immI",
  /*  211 */  "_XorI__LShiftI_iRegIorL2I_immI_immI_M1",
  /*  212 */  "_LShiftL_iRegL_immI",
  /*  213 */  "_XorL__LShiftL_iRegL_immI_immL_M1",
  /*  214 */  "_XorI__URShiftI_iRegIorL2I_immI_iRegIorL2I",
  /*  215 */  "_XorI_iRegIorL2I__URShiftI_iRegIorL2I_immI",
  /*  216 */  "_XorL__URShiftL_iRegL_immI_iRegL",
  /*  217 */  "_XorL_iRegL__URShiftL_iRegL_immI",
  /*  218 */  "_XorI__RShiftI_iRegIorL2I_immI_iRegIorL2I",
  /*  219 */  "_XorI_iRegIorL2I__RShiftI_iRegIorL2I_immI",
  /*  220 */  "_XorL__RShiftL_iRegL_immI_iRegL",
  /*  221 */  "_XorL_iRegL__RShiftL_iRegL_immI",
  /*  222 */  "_XorI__LShiftI_iRegIorL2I_immI_iRegIorL2I",
  /*  223 */  "_XorI_iRegIorL2I__LShiftI_iRegIorL2I_immI",
  /*  224 */  "_XorL__LShiftL_iRegL_immI_iRegL",
  /*  225 */  "_XorL_iRegL__LShiftL_iRegL_immI",
  /*  226 */  "_AndI__URShiftI_iRegIorL2I_immI_immI_bitmask",
  /*  227 */  "_LShiftL_iRegL_iRegI",
  /*  228 */  "_SubI_immI_64_iRegI",
  /*  229 */  "_URShiftL_iRegL__SubI_immI_64_iRegI",
  /*  230 */  "_SubI_immI0_iRegI",
  /*  231 */  "_URShiftL_iRegL__SubI_immI0_iRegI",
  /*  232 */  "_LShiftI_iRegI_iRegI",
  /*  233 */  "_SubI_immI_32_iRegI",
  /*  234 */  "_URShiftI_iRegI__SubI_immI_32_iRegI",
  /*  235 */  "_URShiftI_iRegI__SubI_immI0_iRegI",
  /*  236 */  "_URShiftL_iRegL_iRegI",
  /*  237 */  "_LShiftL_iRegL__SubI_immI_64_iRegI",
  /*  238 */  "_LShiftL_iRegL__SubI_immI0_iRegI",
  /*  239 */  "_URShiftI_iRegI_iRegI",
  /*  240 */  "_LShiftI_iRegI__SubI_immI_32_iRegI",
  /*  241 */  "_LShiftI_iRegI__SubI_immI0_iRegI",
  /*  242 */  "_LShiftI_iRegIorL2I_immI_16",
  /*  243 */  "_RShiftI__LShiftI_iRegIorL2I_immI_16_immI_16",
  /*  244 */  "_LShiftI_iRegIorL2I_immI_24",
  /*  245 */  "_RShiftI__LShiftI_iRegIorL2I_immI_24_immI_24",
  /*  246 */  "_URShiftI__LShiftI_iRegIorL2I_immI_24_immI_24",
  /*  247 */  "_LShiftL_iRegL_immI_48",
  /*  248 */  "_RShiftL__LShiftL_iRegL_immI_48_immI_48",
  /*  249 */  "_LShiftL_iRegL_immI_32",
  /*  250 */  "_RShiftL__LShiftL_iRegL_immI_32_immI_32",
  /*  251 */  "_LShiftL_iRegL_immI_56",
  /*  252 */  "_RShiftL__LShiftL_iRegL_immI_56_immI_56",
  /*  253 */  "_URShiftL__LShiftL_iRegL_immI_56_immI_56",
  /*  254 */  "_AndI_iRegIorL2I_immI_255",
  /*  255 */  "_AndI_iRegIorL2I_immI_65535",
  /*  256 */  "_AndL_iRegL_immL_255",
  /*  257 */  "_AndL_iRegL_immL_65535",
  /*  258 */  "_AndL_iRegL_immL_4294967295",
  /*  259 */  "_ConvF2D_vRegF_",
  /*  260 */  "_SqrtD__ConvF2D_vRegF__",
  /*  261 */  "_OverflowMulI_iRegIorL2I_iRegIorL2I",
  /*  262 */  "_OverflowMulL_iRegL_iRegL",
  /*  263 */  "_CmpI_iRegIorL2I_immI0",
  /*  264 */  "_CmpL_iRegL_immL0",
  /*  265 */  "_CmpP_iRegP_immP0",
  /*  266 */  "_CmpN_iRegN_immN0",
  /*  267 */  "_CmpP__DecodeN_iRegN__immP0",
  /*  268 */  "_CmpU_iRegIorL2I_immI0",
  /*  269 */  "_CmpUL_iRegL_immL0",
  /*  270 */  "_AndL_iRegL_immL",
  /*  271 */  "_CmpL__AndL_iRegL_immL_immL0",
  /*  272 */  "_AndI_iRegIorL2I_immI",
  /*  273 */  "_CmpI__AndI_iRegIorL2I_immI_immI0",
  /*  274 */  "_AndL_iRegL_iRegL",
  /*  275 */  "_AndI_iRegIorL2I_iRegIorL2I",
  /*  276 */  "_PartialSubtypeCheck_iRegP_R4_iRegP_R0",
  /*  277 */  "_Binary_iRegP_R1_iRegI_R2",
  /*  278 */  "_Binary_iRegP_R3_iRegI_R4",
  /*  279 */  "_Binary_iRegP_R1_iRegI_R4",
  /*  280 */  "_Binary_iRegP_R3_iRegI_R2",
  /*  281 */  "_Binary_iRegP_R3_immI_le_4",
  /*  282 */  "_Binary_iRegP_R1_iRegP_R3",
  /*  283 */  "_Binary_iRegP_R1_iRegI_R3",
  /*  284 */  "_MulVS_vecD_vecD",
  /*  285 */  "_MulVS_vecX_vecX",
  /*  286 */  "_MulVI_vecD_vecD",
  /*  287 */  "_MulVI_vecX_vecX",
  // last internally defined operand
  /*  288 */  "loadConI",
  /*  289 */  "loadConL",
  /*  290 */  "loadConP",
  /*  291 */  "loadConP0",
  /*  292 */  "loadConP1",
  /*  293 */  "loadConPollPage",
  /*  294 */  "loadByteMapBase",
  /*  295 */  "loadConN",
  /*  296 */  "loadConN0",
  /*  297 */  "loadConNKlass",
  /*  298 */  "loadConF_packed",
  /*  299 */  "loadConF",
  /*  300 */  "loadConD_packed",
  /*  301 */  "loadConD",
  /*  302 */  "countLeadingZerosI",
  /*  303 */  "countLeadingZerosL",
  /*  304 */  "countTrailingZerosI",
  /*  305 */  "countTrailingZerosL",
  /*  306 */  "castX2P",
  /*  307 */  "castP2X",
  /*  308 */  "checkCastPP",
  /*  309 */  "castPP",
  /*  310 */  "castII",
  /*  311 */  "storePConditional",
  /*  312 */  "storeLConditional",
  /*  313 */  "storeIConditional",
  /*  314 */  "negF_reg_reg",
  /*  315 */  "negD_reg_reg",
  /*  316 */  "absF_reg",
  /*  317 */  "absD_reg",
  /*  318 */  "MoveF2I_stack_reg",
  /*  319 */  "MoveI2F_stack_reg",
  /*  320 */  "MoveD2L_stack_reg",
  /*  321 */  "MoveL2D_stack_reg",
  /*  322 */  "MoveF2I_reg_reg",
  /*  323 */  "MoveI2F_reg_reg",
  /*  324 */  "MoveD2L_reg_reg",
  /*  325 */  "MoveL2D_reg_reg",
  /*  326 */  "overflowAddI_reg_reg",
  /*  327 */  "overflowAddI_reg_imm",
  /*  328 */  "overflowAddL_reg_reg",
  /*  329 */  "overflowAddL_reg_imm",
  /*  330 */  "overflowSubI_reg_reg",
  /*  331 */  "overflowSubI_reg_imm",
  /*  332 */  "overflowSubL_reg_reg",
  /*  333 */  "overflowSubL_reg_imm",
  /*  334 */  "overflowNegI_reg",
  /*  335 */  "overflowNegL_reg",
  /*  336 */  "overflowMulI_reg",
  /*  337 */  "overflowMulL_reg",
  /*  338 */  "compI_reg_reg",
  /*  339 */  "compI_reg_immI0",
  /*  340 */  "compI_reg_immIAddSub",
  /*  341 */  "compI_reg_immI",
  /*  342 */  "compU_reg_reg",
  /*  343 */  "compU_reg_immI0",
  /*  344 */  "compU_reg_immIAddSub",
  /*  345 */  "compU_reg_immI",
  /*  346 */  "compL_reg_reg",
  /*  347 */  "compL_reg_immL0",
  /*  348 */  "compL_reg_immLAddSub",
  /*  349 */  "compL_reg_immL",
  /*  350 */  "compUL_reg_reg",
  /*  351 */  "compUL_reg_immL0",
  /*  352 */  "compUL_reg_immLAddSub",
  /*  353 */  "compUL_reg_immL",
  /*  354 */  "compP_reg_reg",
  /*  355 */  "compN_reg_reg",
  /*  356 */  "testP_reg",
  /*  357 */  "testN_reg",
  /*  358 */  "compF_reg_reg",
  /*  359 */  "compF_reg_zero",
  /*  360 */  "compD_reg_reg",
  /*  361 */  "compD_reg_zero",
  /*  362 */  "cmpL_and",
  /*  363 */  "cmpI_and",
  /*  364 */  "cmpL_and_reg",
  /*  365 */  "cmpI_and_reg",
  /*  366 */  "cmpFastLock",
  /*  367 */  "cmpFastUnlock",
  /*  368 */  "partialSubtypeCheckVsZero",
  /*  369 */  "tlsLoadP",
  /*  370 */  "vshiftcntL",
  /*  371 */  "vshiftcntR",
  /*  372 */  "loadB",
  /*  373 */  "loadB2L",
  /*  374 */  "loadUB",
  /*  375 */  "loadUB2L",
  /*  376 */  "loadS",
  /*  377 */  "loadS2L",
  /*  378 */  "loadUS",
  /*  379 */  "loadUS2L",
  /*  380 */  "loadI",
  /*  381 */  "loadI2L",
  /*  382 */  "loadUI2L",
  /*  383 */  "loadL",
  /*  384 */  "loadRange",
  /*  385 */  "loadP",
  /*  386 */  "loadN",
  /*  387 */  "loadKlass",
  /*  388 */  "loadNKlass",
  /*  389 */  "loadF",
  /*  390 */  "loadD",
  /*  391 */  "storeimmCM0",
  /*  392 */  "storeimmCM0_ordered",
  /*  393 */  "storeB",
  /*  394 */  "storeimmB0",
  /*  395 */  "storeC",
  /*  396 */  "storeimmC0",
  /*  397 */  "storeI",
  /*  398 */  "storeimmI0",
  /*  399 */  "storeL",
  /*  400 */  "storeimmL0",
  /*  401 */  "storeP",
  /*  402 */  "storeimmP0",
  /*  403 */  "storeN",
  /*  404 */  "storeImmN0",
  /*  405 */  "storeF",
  /*  406 */  "storeD",
  /*  407 */  "storeNKlass",
  /*  408 */  "prefetchr",
  /*  409 */  "prefetchw",
  /*  410 */  "prefetchnta",
  /*  411 */  "loadB_volatile",
  /*  412 */  "loadB2L_volatile",
  /*  413 */  "loadUB_volatile",
  /*  414 */  "loadUB2L_volatile",
  /*  415 */  "loadS_volatile",
  /*  416 */  "loadUS_volatile",
  /*  417 */  "loadUS2L_volatile",
  /*  418 */  "loadS2L_volatile",
  /*  419 */  "loadI_volatile",
  /*  420 */  "loadUI2L_volatile",
  /*  421 */  "loadL_volatile",
  /*  422 */  "loadP_volatile",
  /*  423 */  "loadN_volatile",
  /*  424 */  "loadF_volatile",
  /*  425 */  "loadD_volatile",
  /*  426 */  "storeB_volatile",
  /*  427 */  "storeC_volatile",
  /*  428 */  "storeI_volatile",
  /*  429 */  "storeL_volatile",
  /*  430 */  "storeP_volatile",
  /*  431 */  "storeN_volatile",
  /*  432 */  "storeF_volatile",
  /*  433 */  "storeD_volatile",
  /*  434 */  "bytes_reverse_int",
  /*  435 */  "bytes_reverse_long",
  /*  436 */  "bytes_reverse_unsigned_short",
  /*  437 */  "bytes_reverse_short",
  /*  438 */  "popCountI",
  /*  439 */  "popCountI_mem",
  /*  440 */  "popCountL",
  /*  441 */  "popCountL_mem",
  /*  442 */  "load_fence",
  /*  443 */  "unnecessary_membar_acquire",
  /*  444 */  "membar_acquire",
  /*  445 */  "membar_acquire_lock",
  /*  446 */  "store_fence",
  /*  447 */  "unnecessary_membar_release",
  /*  448 */  "membar_release",
  /*  449 */  "membar_storestore",
  /*  450 */  "membar_release_lock",
  /*  451 */  "unnecessary_membar_volatile",
  /*  452 */  "membar_volatile",
  /*  453 */  "convP2I",
  /*  454 */  "convN2I",
  /*  455 */  "encodeHeapOop",
  /*  456 */  "encodeHeapOop_not_null",
  /*  457 */  "decodeHeapOop",
  /*  458 */  "decodeHeapOop_not_null",
  /*  459 */  "encodeKlass_not_null",
  /*  460 */  "decodeKlass_not_null",
  /*  461 */  "loadPLocked",
  /*  462 */  "compareAndSwapI",
  /*  463 */  "compareAndSwapL",
  /*  464 */  "compareAndSwapP",
  /*  465 */  "compareAndSwapN",
  /*  466 */  "compareAndSwapIAcq",
  /*  467 */  "compareAndSwapLAcq",
  /*  468 */  "compareAndSwapPAcq",
  /*  469 */  "compareAndSwapNAcq",
  /*  470 */  "get_and_setI",
  /*  471 */  "get_and_setL",
  /*  472 */  "get_and_setN",
  /*  473 */  "get_and_setP",
  /*  474 */  "get_and_addL",
  /*  475 */  "get_and_addL_no_res",
  /*  476 */  "get_and_addLi",
  /*  477 */  "get_and_addLi_no_res",
  /*  478 */  "get_and_addI",
  /*  479 */  "get_and_addI_no_res",
  /*  480 */  "get_and_addIi",
  /*  481 */  "get_and_addIi_no_res",
  /*  482 */  "cmovI_reg_reg",
  /*  483 */  "cmovUI_reg_reg",
  /*  484 */  "cmovI_zero_reg",
  /*  485 */  "cmovUI_zero_reg",
  /*  486 */  "cmovI_reg_zero",
  /*  487 */  "cmovUI_reg_zero",
  /*  488 */  "cmovI_reg_zero_one",
  /*  489 */  "cmovUI_reg_zero_one",
  /*  490 */  "cmovL_reg_reg",
  /*  491 */  "cmovUL_reg_reg",
  /*  492 */  "cmovL_reg_zero",
  /*  493 */  "cmovUL_reg_zero",
  /*  494 */  "cmovL_zero_reg",
  /*  495 */  "cmovUL_zero_reg",
  /*  496 */  "cmovP_reg_reg",
  /*  497 */  "cmovUP_reg_reg",
  /*  498 */  "cmovP_reg_zero",
  /*  499 */  "cmovUP_reg_zero",
  /*  500 */  "cmovP_zero_reg",
  /*  501 */  "cmovUP_zero_reg",
  /*  502 */  "cmovN_reg_reg",
  /*  503 */  "cmovUN_reg_reg",
  /*  504 */  "cmovN_reg_zero",
  /*  505 */  "cmovUN_reg_zero",
  /*  506 */  "cmovN_zero_reg",
  /*  507 */  "cmovUN_zero_reg",
  /*  508 */  "cmovF_reg",
  /*  509 */  "cmovUF_reg",
  /*  510 */  "cmovD_reg",
  /*  511 */  "cmovUD_reg",
  /*  512 */  "addI_reg_reg",
  /*  513 */  "addI_reg_imm",
  /*  514 */  "addI_reg_imm_i2l",
  /*  515 */  "addP_reg_reg",
  /*  516 */  "addP_reg_reg_ext",
  /*  517 */  "addP_reg_reg_lsl",
  /*  518 */  "addP_reg_reg_ext_shift",
  /*  519 */  "lshift_ext",
  /*  520 */  "addP_reg_imm",
  /*  521 */  "addL_reg_reg",
  /*  522 */  "addL_reg_imm",
  /*  523 */  "subI_reg_reg",
  /*  524 */  "subI_reg_imm",
  /*  525 */  "subL_reg_reg",
  /*  526 */  "subL_reg_imm",
  /*  527 */  "negI_reg",
  /*  528 */  "negL_reg",
  /*  529 */  "mulI",
  /*  530 */  "smulI",
  /*  531 */  "smulI_0",
  /*  532 */  "mulL",
  /*  533 */  "mulHiL_rReg",
  /*  534 */  "maddI",
  /*  535 */  "maddI_0",
  /*  536 */  "msubI",
  /*  537 */  "maddL",
  /*  538 */  "maddL_0",
  /*  539 */  "msubL",
  /*  540 */  "divI",
  /*  541 */  "signExtract",
  /*  542 */  "div2Round",
  /*  543 */  "div2Round_0",
  /*  544 */  "divL",
  /*  545 */  "signExtractL",
  /*  546 */  "div2RoundL",
  /*  547 */  "div2RoundL_0",
  /*  548 */  "modI",
  /*  549 */  "modL",
  /*  550 */  "lShiftI_reg_reg",
  /*  551 */  "lShiftI_reg_imm",
  /*  552 */  "urShiftI_reg_reg",
  /*  553 */  "urShiftI_reg_imm",
  /*  554 */  "rShiftI_reg_reg",
  /*  555 */  "rShiftI_reg_imm",
  /*  556 */  "lShiftL_reg_reg",
  /*  557 */  "lShiftL_reg_imm",
  /*  558 */  "urShiftL_reg_reg",
  /*  559 */  "urShiftL_reg_imm",
  /*  560 */  "urShiftP_reg_imm",
  /*  561 */  "rShiftL_reg_reg",
  /*  562 */  "rShiftL_reg_imm",
  /*  563 */  "regL_not_reg",
  /*  564 */  "regI_not_reg",
  /*  565 */  "AndI_reg_not_reg",
  /*  566 */  "AndI_reg_not_reg_0",
  /*  567 */  "AndL_reg_not_reg",
  /*  568 */  "AndL_reg_not_reg_0",
  /*  569 */  "OrI_reg_not_reg",
  /*  570 */  "OrI_reg_not_reg_0",
  /*  571 */  "OrL_reg_not_reg",
  /*  572 */  "OrL_reg_not_reg_0",
  /*  573 */  "XorI_reg_not_reg",
  /*  574 */  "XorI_reg_not_reg_0",
  /*  575 */  "XorL_reg_not_reg",
  /*  576 */  "XorL_reg_not_reg_0",
  /*  577 */  "AndI_reg_URShift_not_reg",
  /*  578 */  "AndI_reg_URShift_not_reg_0",
  /*  579 */  "AndL_reg_URShift_not_reg",
  /*  580 */  "AndL_reg_URShift_not_reg_0",
  /*  581 */  "AndI_reg_RShift_not_reg",
  /*  582 */  "AndI_reg_RShift_not_reg_0",
  /*  583 */  "AndL_reg_RShift_not_reg",
  /*  584 */  "AndL_reg_RShift_not_reg_0",
  /*  585 */  "AndI_reg_LShift_not_reg",
  /*  586 */  "AndI_reg_LShift_not_reg_0",
  /*  587 */  "AndL_reg_LShift_not_reg",
  /*  588 */  "AndL_reg_LShift_not_reg_0",
  /*  589 */  "XorI_reg_URShift_not_reg",
  /*  590 */  "XorI_reg_URShift_not_reg_1",
  /*  591 */  "XorI_reg_URShift_not_reg_0",
  /*  592 */  "XorI_reg_URShift_not_reg_2",
  /*  593 */  "XorL_reg_URShift_not_reg",
  /*  594 */  "XorL_reg_URShift_not_reg_1",
  /*  595 */  "XorL_reg_URShift_not_reg_0",
  /*  596 */  "XorL_reg_URShift_not_reg_2",
  /*  597 */  "XorI_reg_RShift_not_reg",
  /*  598 */  "XorI_reg_RShift_not_reg_1",
  /*  599 */  "XorI_reg_RShift_not_reg_0",
  /*  600 */  "XorI_reg_RShift_not_reg_2",
  /*  601 */  "XorL_reg_RShift_not_reg",
  /*  602 */  "XorL_reg_RShift_not_reg_1",
  /*  603 */  "XorL_reg_RShift_not_reg_0",
  /*  604 */  "XorL_reg_RShift_not_reg_2",
  /*  605 */  "XorI_reg_LShift_not_reg",
  /*  606 */  "XorI_reg_LShift_not_reg_1",
  /*  607 */  "XorI_reg_LShift_not_reg_0",
  /*  608 */  "XorI_reg_LShift_not_reg_2",
  /*  609 */  "XorL_reg_LShift_not_reg",
  /*  610 */  "XorL_reg_LShift_not_reg_1",
  /*  611 */  "XorL_reg_LShift_not_reg_0",
  /*  612 */  "XorL_reg_LShift_not_reg_2",
  /*  613 */  "OrI_reg_URShift_not_reg",
  /*  614 */  "OrI_reg_URShift_not_reg_0",
  /*  615 */  "OrL_reg_URShift_not_reg",
  /*  616 */  "OrL_reg_URShift_not_reg_0",
  /*  617 */  "OrI_reg_RShift_not_reg",
  /*  618 */  "OrI_reg_RShift_not_reg_0",
  /*  619 */  "OrL_reg_RShift_not_reg",
  /*  620 */  "OrL_reg_RShift_not_reg_0",
  /*  621 */  "OrI_reg_LShift_not_reg",
  /*  622 */  "OrI_reg_LShift_not_reg_0",
  /*  623 */  "OrL_reg_LShift_not_reg",
  /*  624 */  "OrL_reg_LShift_not_reg_0",
  /*  625 */  "AndI_reg_URShift_reg",
  /*  626 */  "AndI_reg_URShift_reg_0",
  /*  627 */  "AndL_reg_URShift_reg",
  /*  628 */  "AndL_reg_URShift_reg_0",
  /*  629 */  "AndI_reg_RShift_reg",
  /*  630 */  "AndI_reg_RShift_reg_0",
  /*  631 */  "AndL_reg_RShift_reg",
  /*  632 */  "AndL_reg_RShift_reg_0",
  /*  633 */  "AndI_reg_LShift_reg",
  /*  634 */  "AndI_reg_LShift_reg_0",
  /*  635 */  "AndL_reg_LShift_reg",
  /*  636 */  "AndL_reg_LShift_reg_0",
  /*  637 */  "XorI_reg_URShift_reg",
  /*  638 */  "XorI_reg_URShift_reg_0",
  /*  639 */  "XorL_reg_URShift_reg",
  /*  640 */  "XorL_reg_URShift_reg_0",
  /*  641 */  "XorI_reg_RShift_reg",
  /*  642 */  "XorI_reg_RShift_reg_0",
  /*  643 */  "XorL_reg_RShift_reg",
  /*  644 */  "XorL_reg_RShift_reg_0",
  /*  645 */  "XorI_reg_LShift_reg",
  /*  646 */  "XorI_reg_LShift_reg_0",
  /*  647 */  "XorL_reg_LShift_reg",
  /*  648 */  "XorL_reg_LShift_reg_0",
  /*  649 */  "OrI_reg_URShift_reg",
  /*  650 */  "OrI_reg_URShift_reg_0",
  /*  651 */  "OrL_reg_URShift_reg",
  /*  652 */  "OrL_reg_URShift_reg_0",
  /*  653 */  "OrI_reg_RShift_reg",
  /*  654 */  "OrI_reg_RShift_reg_0",
  /*  655 */  "OrL_reg_RShift_reg",
  /*  656 */  "OrL_reg_RShift_reg_0",
  /*  657 */  "OrI_reg_LShift_reg",
  /*  658 */  "OrI_reg_LShift_reg_0",
  /*  659 */  "OrL_reg_LShift_reg",
  /*  660 */  "OrL_reg_LShift_reg_0",
  /*  661 */  "AddI_reg_URShift_reg",
  /*  662 */  "AddI_reg_URShift_reg_0",
  /*  663 */  "AddL_reg_URShift_reg",
  /*  664 */  "AddL_reg_URShift_reg_0",
  /*  665 */  "AddI_reg_RShift_reg",
  /*  666 */  "AddI_reg_RShift_reg_0",
  /*  667 */  "AddL_reg_RShift_reg",
  /*  668 */  "AddL_reg_RShift_reg_0",
  /*  669 */  "AddI_reg_LShift_reg",
  /*  670 */  "AddI_reg_LShift_reg_0",
  /*  671 */  "AddL_reg_LShift_reg",
  /*  672 */  "AddL_reg_LShift_reg_0",
  /*  673 */  "SubI_reg_URShift_reg",
  /*  674 */  "SubL_reg_URShift_reg",
  /*  675 */  "SubI_reg_RShift_reg",
  /*  676 */  "SubL_reg_RShift_reg",
  /*  677 */  "SubI_reg_LShift_reg",
  /*  678 */  "SubL_reg_LShift_reg",
  /*  679 */  "sbfmL",
  /*  680 */  "sbfmwI",
  /*  681 */  "ubfmL",
  /*  682 */  "ubfmwI",
  /*  683 */  "ubfxwI",
  /*  684 */  "ubfxL",
  /*  685 */  "ubfxIConvI2L",
  /*  686 */  "extrOrL",
  /*  687 */  "extrOrL_0",
  /*  688 */  "extrOrI",
  /*  689 */  "extrOrI_0",
  /*  690 */  "extrAddL",
  /*  691 */  "extrAddL_0",
  /*  692 */  "extrAddI",
  /*  693 */  "extrAddI_0",
  /*  694 */  "rolL_rReg",
  /*  695 */  "rolI_rReg",
  /*  696 */  "rolL_rReg_Var_C_64",
  /*  697 */  "rolL_rReg_Var_C_64_0",
  /*  698 */  "rolL_rReg_Var_C0",
  /*  699 */  "rolL_rReg_Var_C0_0",
  /*  700 */  "rolI_rReg_Var_C_32",
  /*  701 */  "rolI_rReg_Var_C_32_0",
  /*  702 */  "rolI_rReg_Var_C0",
  /*  703 */  "rolI_rReg_Var_C0_0",
  /*  704 */  "rorL_rReg",
  /*  705 */  "rorI_rReg",
  /*  706 */  "rorL_rReg_Var_C_64",
  /*  707 */  "rorL_rReg_Var_C_64_0",
  /*  708 */  "rorL_rReg_Var_C0",
  /*  709 */  "rorL_rReg_Var_C0_0",
  /*  710 */  "rorI_rReg_Var_C_32",
  /*  711 */  "rorI_rReg_Var_C_32_0",
  /*  712 */  "rorI_rReg_Var_C0",
  /*  713 */  "rorI_rReg_Var_C0_0",
  /*  714 */  "AddExtI",
  /*  715 */  "AddExtI_0",
  /*  716 */  "SubExtI",
  /*  717 */  "AddExtI_sxth",
  /*  718 */  "AddExtI_sxth_0",
  /*  719 */  "AddExtI_sxtb",
  /*  720 */  "AddExtI_sxtb_0",
  /*  721 */  "AddExtI_uxtb",
  /*  722 */  "AddExtI_uxtb_0",
  /*  723 */  "AddExtL_sxth",
  /*  724 */  "AddExtL_sxth_0",
  /*  725 */  "AddExtL_sxtw",
  /*  726 */  "AddExtL_sxtw_0",
  /*  727 */  "AddExtL_sxtb",
  /*  728 */  "AddExtL_sxtb_0",
  /*  729 */  "AddExtL_uxtb",
  /*  730 */  "AddExtL_uxtb_0",
  /*  731 */  "AddExtI_uxtb_and",
  /*  732 */  "AddExtI_uxtb_and_0",
  /*  733 */  "AddExtI_uxth_and",
  /*  734 */  "AddExtI_uxth_and_0",
  /*  735 */  "AddExtL_uxtb_and",
  /*  736 */  "AddExtL_uxtb_and_0",
  /*  737 */  "AddExtL_uxth_and",
  /*  738 */  "AddExtL_uxth_and_0",
  /*  739 */  "AddExtL_uxtw_and",
  /*  740 */  "AddExtL_uxtw_and_0",
  /*  741 */  "SubExtI_uxtb_and",
  /*  742 */  "SubExtI_uxth_and",
  /*  743 */  "SubExtL_uxtb_and",
  /*  744 */  "SubExtL_uxth_and",
  /*  745 */  "SubExtL_uxtw_and",
  /*  746 */  "addF_reg_reg",
  /*  747 */  "addD_reg_reg",
  /*  748 */  "subF_reg_reg",
  /*  749 */  "subD_reg_reg",
  /*  750 */  "mulF_reg_reg",
  /*  751 */  "mulD_reg_reg",
  /*  752 */  "divF_reg_reg",
  /*  753 */  "divD_reg_reg",
  /*  754 */  "sqrtD_reg",
  /*  755 */  "sqrtF_reg",
  /*  756 */  "andI_reg_reg",
  /*  757 */  "andI_reg_imm",
  /*  758 */  "orI_reg_reg",
  /*  759 */  "orI_reg_imm",
  /*  760 */  "xorI_reg_reg",
  /*  761 */  "xorI_reg_imm",
  /*  762 */  "andL_reg_reg",
  /*  763 */  "andL_reg_imm",
  /*  764 */  "orL_reg_reg",
  /*  765 */  "orL_reg_imm",
  /*  766 */  "xorL_reg_reg",
  /*  767 */  "xorL_reg_imm",
  /*  768 */  "convI2L_reg_reg",
  /*  769 */  "convUI2L_reg_reg",
  /*  770 */  "convL2I_reg",
  /*  771 */  "convI2B",
  /*  772 */  "convP2B",
  /*  773 */  "convD2F_reg",
  /*  774 */  "convF2D_reg",
  /*  775 */  "convF2I_reg_reg",
  /*  776 */  "convF2L_reg_reg",
  /*  777 */  "convI2F_reg_reg",
  /*  778 */  "convL2F_reg_reg",
  /*  779 */  "convD2I_reg_reg",
  /*  780 */  "convD2L_reg_reg",
  /*  781 */  "convI2D_reg_reg",
  /*  782 */  "convL2D_reg_reg",
  /*  783 */  "MoveF2I_reg_stack",
  /*  784 */  "MoveI2F_reg_stack",
  /*  785 */  "MoveD2L_reg_stack",
  /*  786 */  "MoveL2D_reg_stack",
  /*  787 */  "clearArray_reg_reg",
  /*  788 */  "clearArray_imm_reg",
  /*  789 */  "overflowMulI_reg_branch",
  /*  790 */  "overflowMulL_reg_branch",
  /*  791 */  "compF3_reg_reg",
  /*  792 */  "compD3_reg_reg",
  /*  793 */  "compF3_reg_immF0",
  /*  794 */  "compD3_reg_immD0",
  /*  795 */  "cmpL3_reg_reg",
  /*  796 */  "cmpLTMask_reg_reg",
  /*  797 */  "cmpLTMask_reg_zero",
  /*  798 */  "minI_rReg",
  /*  799 */  "maxI_rReg",
  /*  800 */  "branch",
  /*  801 */  "branchCon",
  /*  802 */  "branchConU",
  /*  803 */  "cmpI_imm0_branch",
  /*  804 */  "cmpL_imm0_branch",
  /*  805 */  "cmpP_imm0_branch",
  /*  806 */  "cmpN_imm0_branch",
  /*  807 */  "cmpP_narrowOop_imm0_branch",
  /*  808 */  "cmpUI_imm0_branch",
  /*  809 */  "cmpUL_imm0_branch",
  /*  810 */  "cmpL_branch_sign",
  /*  811 */  "cmpI_branch_sign",
  /*  812 */  "cmpL_branch_bit",
  /*  813 */  "cmpI_branch_bit",
  /*  814 */  "far_cmpL_branch_sign",
  /*  815 */  "far_cmpI_branch_sign",
  /*  816 */  "far_cmpL_branch_bit",
  /*  817 */  "far_cmpI_branch_bit",
  /*  818 */  "branchLoopEnd",
  /*  819 */  "branchLoopEndU",
  /*  820 */  "safePoint",
  /*  821 */  "CallStaticJavaDirect",
  /*  822 */  "CallStaticJavaDirectHandle",
  /*  823 */  "CallDynamicJavaDirect",
  /*  824 */  "CallRuntimeDirect",
  /*  825 */  "CallLeafDirect",
  /*  826 */  "CallLeafNoFPDirect",
  /*  827 */  "TailCalljmpInd",
  /*  828 */  "TailjmpInd",
  /*  829 */  "CreateException",
  /*  830 */  "RethrowException",
  /*  831 */  "Ret",
  /*  832 */  "ShouldNotReachHere",
  /*  833 */  "partialSubtypeCheck",
  /*  834 */  "string_compare",
  /*  835 */  "string_indexof",
  /*  836 */  "string_indexof_con",
  /*  837 */  "string_equals",
  /*  838 */  "array_equals",
  /*  839 */  "encode_iso_array",
  /*  840 */  "loadV4",
  /*  841 */  "loadV8",
  /*  842 */  "loadV16",
  /*  843 */  "storeV4",
  /*  844 */  "storeV8",
  /*  845 */  "storeV16",
  /*  846 */  "replicate8B",
  /*  847 */  "replicate16B",
  /*  848 */  "replicate8B_imm",
  /*  849 */  "replicate16B_imm",
  /*  850 */  "replicate4S",
  /*  851 */  "replicate8S",
  /*  852 */  "replicate4S_imm",
  /*  853 */  "replicate8S_imm",
  /*  854 */  "replicate2I",
  /*  855 */  "replicate4I",
  /*  856 */  "replicate2I_imm",
  /*  857 */  "replicate4I_imm",
  /*  858 */  "replicate2L",
  /*  859 */  "replicate2L_zero",
  /*  860 */  "replicate2F",
  /*  861 */  "replicate4F",
  /*  862 */  "replicate2D",
  /*  863 */  "vadd8B",
  /*  864 */  "vadd16B",
  /*  865 */  "vadd4S",
  /*  866 */  "vadd8S",
  /*  867 */  "vadd2I",
  /*  868 */  "vadd4I",
  /*  869 */  "vadd2L",
  /*  870 */  "vadd2F",
  /*  871 */  "vadd4F",
  /*  872 */  "vadd2D",
  /*  873 */  "vsub8B",
  /*  874 */  "vsub16B",
  /*  875 */  "vsub4S",
  /*  876 */  "vsub8S",
  /*  877 */  "vsub2I",
  /*  878 */  "vsub4I",
  /*  879 */  "vsub2L",
  /*  880 */  "vsub2F",
  /*  881 */  "vsub4F",
  /*  882 */  "vsub2D",
  /*  883 */  "vmul4S",
  /*  884 */  "vmul8S",
  /*  885 */  "vmul2I",
  /*  886 */  "vmul4I",
  /*  887 */  "vmul2F",
  /*  888 */  "vmul4F",
  /*  889 */  "vmul2D",
  /*  890 */  "vmla4S",
  /*  891 */  "vmla8S",
  /*  892 */  "vmla2I",
  /*  893 */  "vmla4I",
  /*  894 */  "vmls4S",
  /*  895 */  "vmls8S",
  /*  896 */  "vmls2I",
  /*  897 */  "vmls4I",
  /*  898 */  "vdiv2F",
  /*  899 */  "vdiv4F",
  /*  900 */  "vdiv2D",
  /*  901 */  "vand8B",
  /*  902 */  "vand16B",
  /*  903 */  "vor8B",
  /*  904 */  "vor16B",
  /*  905 */  "vxor8B",
  /*  906 */  "vxor16B",
  /*  907 */  "vsll8B",
  /*  908 */  "vsll8B_0",
  /*  909 */  "vsll16B",
  /*  910 */  "vsll16B_0",
  /*  911 */  "vsrl8B",
  /*  912 */  "vsrl16B",
  /*  913 */  "vsll8B_imm",
  /*  914 */  "vsll16B_imm",
  /*  915 */  "vsra8B_imm",
  /*  916 */  "vsra16B_imm",
  /*  917 */  "vsrl8B_imm",
  /*  918 */  "vsrl16B_imm",
  /*  919 */  "vsll4S",
  /*  920 */  "vsll4S_0",
  /*  921 */  "vsll8S",
  /*  922 */  "vsll8S_0",
  /*  923 */  "vsrl4S",
  /*  924 */  "vsrl8S",
  /*  925 */  "vsll4S_imm",
  /*  926 */  "vsll8S_imm",
  /*  927 */  "vsra4S_imm",
  /*  928 */  "vsra8S_imm",
  /*  929 */  "vsrl4S_imm",
  /*  930 */  "vsrl8S_imm",
  /*  931 */  "vsll2I",
  /*  932 */  "vsll2I_0",
  /*  933 */  "vsll4I",
  /*  934 */  "vsll4I_0",
  /*  935 */  "vsrl2I",
  /*  936 */  "vsrl4I",
  /*  937 */  "vsll2I_imm",
  /*  938 */  "vsll4I_imm",
  /*  939 */  "vsra2I_imm",
  /*  940 */  "vsra4I_imm",
  /*  941 */  "vsrl2I_imm",
  /*  942 */  "vsrl4I_imm",
  /*  943 */  "vsll2L",
  /*  944 */  "vsll2L_0",
  /*  945 */  "vsrl2L",
  /*  946 */  "vsll2L_imm",
  /*  947 */  "vsra2L_imm",
  /*  948 */  "vsrl2L_imm",
  // last instruction
  "invalid rule name" // no trailing comma
};

const        bool  swallowed[] = {
  /*    0 */  false,
  /*    1 */  false,
  /*    2 */  false,
  /*    3 */  false,
  /*    4 */  false,
  /*    5 */  false,
  /*    6 */  false,
  /*    7 */  false,
  /*    8 */  true,
  /*    9 */  true,
  /*   10 */  true,
  /*   11 */  true,
  /*   12 */  true,
  /*   13 */  true,
  /*   14 */  true,
  /*   15 */  true,
  /*   16 */  true,
  /*   17 */  true,
  /*   18 */  true,
  /*   19 */  true,
  /*   20 */  true,
  /*   21 */  true,
  /*   22 */  true,
  /*   23 */  true,
  /*   24 */  true,
  /*   25 */  true,
  /*   26 */  true,
  /*   27 */  true,
  /*   28 */  true,
  /*   29 */  true,
  /*   30 */  true,
  /*   31 */  true,
  /*   32 */  true,
  /*   33 */  true,
  /*   34 */  true,
  /*   35 */  true,
  /*   36 */  true,
  /*   37 */  true,
  /*   38 */  true,
  /*   39 */  true,
  /*   40 */  true,
  /*   41 */  true,
  /*   42 */  true,
  /*   43 */  true,
  /*   44 */  true,
  /*   45 */  true,
  /*   46 */  true,
  /*   47 */  true,
  /*   48 */  true,
  /*   49 */  true,
  /*   50 */  true,
  /*   51 */  true,
  /*   52 */  true,
  /*   53 */  true,
  /*   54 */  true,
  /*   55 */  true,
  /*   56 */  true,
  /*   57 */  true,
  /*   58 */  true,
  /*   59 */  true,
  /*   60 */  true,
  /*   61 */  true,
  /*   62 */  true,
  /*   63 */  true,
  /*   64 */  true,
  /*   65 */  true,
  /*   66 */  true,
  /*   67 */  true,
  /*   68 */  false,
  /*   69 */  false,
  /*   70 */  false,
  /*   71 */  false,
  /*   72 */  false,
  /*   73 */  false,
  /*   74 */  false,
  /*   75 */  false,
  /*   76 */  false,
  /*   77 */  false,
  /*   78 */  false,
  /*   79 */  false,
  /*   80 */  false,
  /*   81 */  false,
  /*   82 */  false,
  /*   83 */  false,
  /*   84 */  false,
  /*   85 */  false,
  /*   86 */  false,
  /*   87 */  false,
  /*   88 */  false,
  /*   89 */  false,
  /*   90 */  false,
  /*   91 */  false,
  /*   92 */  false,
  /*   93 */  false,
  /*   94 */  false,
  /*   95 */  false,
  /*   96 */  false,
  /*   97 */  false,
  /*   98 */  false,
  /*   99 */  false,
  /*  100 */  false,
  /*  101 */  false,
  /*  102 */  false,
  /*  103 */  false,
  /*  104 */  false,
  /*  105 */  false,
  /*  106 */  false,
  /*  107 */  false,
  /*  108 */  false,
  /*  109 */  false,
  /*  110 */  false,
  /*  111 */  false,
  /*  112 */  false,
  /*  113 */  false,
  /*  114 */  false,
  /*  115 */  false,
  /*  116 */  false,
  /*  117 */  false,
  /*  118 */  false,
  /*  119 */  false,
  /*  120 */  false,
  /*  121 */  false,
  /*  122 */  false,
  /*  123 */  false,
  /*  124 */  false,
  /*  125 */  false,
  /*  126 */  false,
  /*  127 */  false,
  /*  128 */  false,
  /*  129 */  false,
  /*  130 */  false,
  /*  131 */  false,
  /*  132 */  false,
  /*  133 */  false,
  /*  134 */  false,
  /*  135 */  false,
  /*  136 */  true,
  /*  137 */  true,
  /*  138 */  false,
  // last operand
  /*  139 */  false,
  /*  140 */  false,
  /*  141 */  false,
  /*  142 */  false,
  /*  143 */  false,
  // last operand class
  /*  144 */  false,
  /*  145 */  false,
  /*  146 */  false,
  /*  147 */  false,
  /*  148 */  false,
  /*  149 */  false,
  /*  150 */  false,
  /*  151 */  false,
  /*  152 */  false,
  /*  153 */  false,
  /*  154 */  false,
  /*  155 */  false,
  /*  156 */  false,
  /*  157 */  false,
  /*  158 */  false,
  /*  159 */  false,
  /*  160 */  false,
  /*  161 */  false,
  /*  162 */  false,
  /*  163 */  false,
  /*  164 */  false,
  /*  165 */  false,
  /*  166 */  false,
  /*  167 */  false,
  /*  168 */  false,
  /*  169 */  false,
  /*  170 */  false,
  /*  171 */  false,
  /*  172 */  false,
  /*  173 */  false,
  /*  174 */  false,
  /*  175 */  false,
  /*  176 */  false,
  /*  177 */  false,
  /*  178 */  false,
  /*  179 */  false,
  /*  180 */  false,
  /*  181 */  false,
  /*  182 */  false,
  /*  183 */  false,
  /*  184 */  false,
  /*  185 */  false,
  /*  186 */  false,
  /*  187 */  false,
  /*  188 */  false,
  /*  189 */  false,
  /*  190 */  false,
  /*  191 */  false,
  /*  192 */  false,
  /*  193 */  false,
  /*  194 */  false,
  /*  195 */  false,
  /*  196 */  false,
  /*  197 */  false,
  /*  198 */  false,
  /*  199 */  false,
  /*  200 */  false,
  /*  201 */  false,
  /*  202 */  false,
  /*  203 */  false,
  /*  204 */  false,
  /*  205 */  false,
  /*  206 */  false,
  /*  207 */  false,
  /*  208 */  false,
  /*  209 */  false,
  /*  210 */  false,
  /*  211 */  false,
  /*  212 */  false,
  /*  213 */  false,
  /*  214 */  false,
  /*  215 */  false,
  /*  216 */  false,
  /*  217 */  false,
  /*  218 */  false,
  /*  219 */  false,
  /*  220 */  false,
  /*  221 */  false,
  /*  222 */  false,
  /*  223 */  false,
  /*  224 */  false,
  /*  225 */  false,
  /*  226 */  false,
  /*  227 */  false,
  /*  228 */  false,
  /*  229 */  false,
  /*  230 */  false,
  /*  231 */  false,
  /*  232 */  false,
  /*  233 */  false,
  /*  234 */  false,
  /*  235 */  false,
  /*  236 */  false,
  /*  237 */  false,
  /*  238 */  false,
  /*  239 */  false,
  /*  240 */  false,
  /*  241 */  false,
  /*  242 */  false,
  /*  243 */  false,
  /*  244 */  false,
  /*  245 */  false,
  /*  246 */  false,
  /*  247 */  false,
  /*  248 */  false,
  /*  249 */  false,
  /*  250 */  false,
  /*  251 */  false,
  /*  252 */  false,
  /*  253 */  false,
  /*  254 */  false,
  /*  255 */  false,
  /*  256 */  false,
  /*  257 */  false,
  /*  258 */  false,
  /*  259 */  false,
  /*  260 */  false,
  /*  261 */  false,
  /*  262 */  false,
  /*  263 */  false,
  /*  264 */  false,
  /*  265 */  false,
  /*  266 */  false,
  /*  267 */  false,
  /*  268 */  false,
  /*  269 */  false,
  /*  270 */  false,
  /*  271 */  false,
  /*  272 */  false,
  /*  273 */  false,
  /*  274 */  false,
  /*  275 */  false,
  /*  276 */  false,
  /*  277 */  false,
  /*  278 */  false,
  /*  279 */  false,
  /*  280 */  false,
  /*  281 */  false,
  /*  282 */  false,
  /*  283 */  false,
  /*  284 */  false,
  /*  285 */  false,
  /*  286 */  false,
  /*  287 */  false,
  // last internally defined operand
  /*  288 */  false,
  /*  289 */  false,
  /*  290 */  false,
  /*  291 */  false,
  /*  292 */  false,
  /*  293 */  false,
  /*  294 */  false,
  /*  295 */  false,
  /*  296 */  false,
  /*  297 */  false,
  /*  298 */  false,
  /*  299 */  false,
  /*  300 */  false,
  /*  301 */  false,
  /*  302 */  false,
  /*  303 */  false,
  /*  304 */  false,
  /*  305 */  false,
  /*  306 */  false,
  /*  307 */  false,
  /*  308 */  false,
  /*  309 */  false,
  /*  310 */  false,
  /*  311 */  false,
  /*  312 */  false,
  /*  313 */  false,
  /*  314 */  false,
  /*  315 */  false,
  /*  316 */  false,
  /*  317 */  false,
  /*  318 */  false,
  /*  319 */  false,
  /*  320 */  false,
  /*  321 */  false,
  /*  322 */  false,
  /*  323 */  false,
  /*  324 */  false,
  /*  325 */  false,
  /*  326 */  false,
  /*  327 */  false,
  /*  328 */  false,
  /*  329 */  false,
  /*  330 */  false,
  /*  331 */  false,
  /*  332 */  false,
  /*  333 */  false,
  /*  334 */  false,
  /*  335 */  false,
  /*  336 */  false,
  /*  337 */  false,
  /*  338 */  false,
  /*  339 */  false,
  /*  340 */  false,
  /*  341 */  false,
  /*  342 */  false,
  /*  343 */  false,
  /*  344 */  false,
  /*  345 */  false,
  /*  346 */  false,
  /*  347 */  false,
  /*  348 */  false,
  /*  349 */  false,
  /*  350 */  false,
  /*  351 */  false,
  /*  352 */  false,
  /*  353 */  false,
  /*  354 */  false,
  /*  355 */  false,
  /*  356 */  false,
  /*  357 */  false,
  /*  358 */  false,
  /*  359 */  false,
  /*  360 */  false,
  /*  361 */  false,
  /*  362 */  false,
  /*  363 */  false,
  /*  364 */  false,
  /*  365 */  false,
  /*  366 */  false,
  /*  367 */  false,
  /*  368 */  false,
  /*  369 */  false,
  /*  370 */  false,
  /*  371 */  false,
  /*  372 */  false,
  /*  373 */  false,
  /*  374 */  false,
  /*  375 */  false,
  /*  376 */  false,
  /*  377 */  false,
  /*  378 */  false,
  /*  379 */  false,
  /*  380 */  false,
  /*  381 */  false,
  /*  382 */  false,
  /*  383 */  false,
  /*  384 */  false,
  /*  385 */  false,
  /*  386 */  false,
  /*  387 */  false,
  /*  388 */  false,
  /*  389 */  false,
  /*  390 */  false,
  /*  391 */  false,
  /*  392 */  false,
  /*  393 */  false,
  /*  394 */  false,
  /*  395 */  false,
  /*  396 */  false,
  /*  397 */  false,
  /*  398 */  false,
  /*  399 */  false,
  /*  400 */  false,
  /*  401 */  false,
  /*  402 */  false,
  /*  403 */  false,
  /*  404 */  false,
  /*  405 */  false,
  /*  406 */  false,
  /*  407 */  false,
  /*  408 */  false,
  /*  409 */  false,
  /*  410 */  false,
  /*  411 */  false,
  /*  412 */  false,
  /*  413 */  false,
  /*  414 */  false,
  /*  415 */  false,
  /*  416 */  false,
  /*  417 */  false,
  /*  418 */  false,
  /*  419 */  false,
  /*  420 */  false,
  /*  421 */  false,
  /*  422 */  false,
  /*  423 */  false,
  /*  424 */  false,
  /*  425 */  false,
  /*  426 */  false,
  /*  427 */  false,
  /*  428 */  false,
  /*  429 */  false,
  /*  430 */  false,
  /*  431 */  false,
  /*  432 */  false,
  /*  433 */  false,
  /*  434 */  false,
  /*  435 */  false,
  /*  436 */  false,
  /*  437 */  false,
  /*  438 */  false,
  /*  439 */  false,
  /*  440 */  false,
  /*  441 */  false,
  /*  442 */  false,
  /*  443 */  false,
  /*  444 */  false,
  /*  445 */  false,
  /*  446 */  false,
  /*  447 */  false,
  /*  448 */  false,
  /*  449 */  false,
  /*  450 */  false,
  /*  451 */  false,
  /*  452 */  false,
  /*  453 */  false,
  /*  454 */  false,
  /*  455 */  false,
  /*  456 */  false,
  /*  457 */  false,
  /*  458 */  false,
  /*  459 */  false,
  /*  460 */  false,
  /*  461 */  false,
  /*  462 */  false,
  /*  463 */  false,
  /*  464 */  false,
  /*  465 */  false,
  /*  466 */  false,
  /*  467 */  false,
  /*  468 */  false,
  /*  469 */  false,
  /*  470 */  false,
  /*  471 */  false,
  /*  472 */  false,
  /*  473 */  false,
  /*  474 */  false,
  /*  475 */  false,
  /*  476 */  false,
  /*  477 */  false,
  /*  478 */  false,
  /*  479 */  false,
  /*  480 */  false,
  /*  481 */  false,
  /*  482 */  false,
  /*  483 */  false,
  /*  484 */  false,
  /*  485 */  false,
  /*  486 */  false,
  /*  487 */  false,
  /*  488 */  false,
  /*  489 */  false,
  /*  490 */  false,
  /*  491 */  false,
  /*  492 */  false,
  /*  493 */  false,
  /*  494 */  false,
  /*  495 */  false,
  /*  496 */  false,
  /*  497 */  false,
  /*  498 */  false,
  /*  499 */  false,
  /*  500 */  false,
  /*  501 */  false,
  /*  502 */  false,
  /*  503 */  false,
  /*  504 */  false,
  /*  505 */  false,
  /*  506 */  false,
  /*  507 */  false,
  /*  508 */  false,
  /*  509 */  false,
  /*  510 */  false,
  /*  511 */  false,
  /*  512 */  false,
  /*  513 */  false,
  /*  514 */  false,
  /*  515 */  false,
  /*  516 */  false,
  /*  517 */  false,
  /*  518 */  false,
  /*  519 */  false,
  /*  520 */  false,
  /*  521 */  false,
  /*  522 */  false,
  /*  523 */  false,
  /*  524 */  false,
  /*  525 */  false,
  /*  526 */  false,
  /*  527 */  false,
  /*  528 */  false,
  /*  529 */  false,
  /*  530 */  false,
  /*  531 */  false,
  /*  532 */  false,
  /*  533 */  false,
  /*  534 */  false,
  /*  535 */  false,
  /*  536 */  false,
  /*  537 */  false,
  /*  538 */  false,
  /*  539 */  false,
  /*  540 */  false,
  /*  541 */  false,
  /*  542 */  false,
  /*  543 */  false,
  /*  544 */  false,
  /*  545 */  false,
  /*  546 */  false,
  /*  547 */  false,
  /*  548 */  false,
  /*  549 */  false,
  /*  550 */  false,
  /*  551 */  false,
  /*  552 */  false,
  /*  553 */  false,
  /*  554 */  false,
  /*  555 */  false,
  /*  556 */  false,
  /*  557 */  false,
  /*  558 */  false,
  /*  559 */  false,
  /*  560 */  false,
  /*  561 */  false,
  /*  562 */  false,
  /*  563 */  false,
  /*  564 */  false,
  /*  565 */  false,
  /*  566 */  false,
  /*  567 */  false,
  /*  568 */  false,
  /*  569 */  false,
  /*  570 */  false,
  /*  571 */  false,
  /*  572 */  false,
  /*  573 */  false,
  /*  574 */  false,
  /*  575 */  false,
  /*  576 */  false,
  /*  577 */  false,
  /*  578 */  false,
  /*  579 */  false,
  /*  580 */  false,
  /*  581 */  false,
  /*  582 */  false,
  /*  583 */  false,
  /*  584 */  false,
  /*  585 */  false,
  /*  586 */  false,
  /*  587 */  false,
  /*  588 */  false,
  /*  589 */  false,
  /*  590 */  false,
  /*  591 */  false,
  /*  592 */  false,
  /*  593 */  false,
  /*  594 */  false,
  /*  595 */  false,
  /*  596 */  false,
  /*  597 */  false,
  /*  598 */  false,
  /*  599 */  false,
  /*  600 */  false,
  /*  601 */  false,
  /*  602 */  false,
  /*  603 */  false,
  /*  604 */  false,
  /*  605 */  false,
  /*  606 */  false,
  /*  607 */  false,
  /*  608 */  false,
  /*  609 */  false,
  /*  610 */  false,
  /*  611 */  false,
  /*  612 */  false,
  /*  613 */  false,
  /*  614 */  false,
  /*  615 */  false,
  /*  616 */  false,
  /*  617 */  false,
  /*  618 */  false,
  /*  619 */  false,
  /*  620 */  false,
  /*  621 */  false,
  /*  622 */  false,
  /*  623 */  false,
  /*  624 */  false,
  /*  625 */  false,
  /*  626 */  false,
  /*  627 */  false,
  /*  628 */  false,
  /*  629 */  false,
  /*  630 */  false,
  /*  631 */  false,
  /*  632 */  false,
  /*  633 */  false,
  /*  634 */  false,
  /*  635 */  false,
  /*  636 */  false,
  /*  637 */  false,
  /*  638 */  false,
  /*  639 */  false,
  /*  640 */  false,
  /*  641 */  false,
  /*  642 */  false,
  /*  643 */  false,
  /*  644 */  false,
  /*  645 */  false,
  /*  646 */  false,
  /*  647 */  false,
  /*  648 */  false,
  /*  649 */  false,
  /*  650 */  false,
  /*  651 */  false,
  /*  652 */  false,
  /*  653 */  false,
  /*  654 */  false,
  /*  655 */  false,
  /*  656 */  false,
  /*  657 */  false,
  /*  658 */  false,
  /*  659 */  false,
  /*  660 */  false,
  /*  661 */  false,
  /*  662 */  false,
  /*  663 */  false,
  /*  664 */  false,
  /*  665 */  false,
  /*  666 */  false,
  /*  667 */  false,
  /*  668 */  false,
  /*  669 */  false,
  /*  670 */  false,
  /*  671 */  false,
  /*  672 */  false,
  /*  673 */  false,
  /*  674 */  false,
  /*  675 */  false,
  /*  676 */  false,
  /*  677 */  false,
  /*  678 */  false,
  /*  679 */  false,
  /*  680 */  false,
  /*  681 */  false,
  /*  682 */  false,
  /*  683 */  false,
  /*  684 */  false,
  /*  685 */  false,
  /*  686 */  false,
  /*  687 */  false,
  /*  688 */  false,
  /*  689 */  false,
  /*  690 */  false,
  /*  691 */  false,
  /*  692 */  false,
  /*  693 */  false,
  /*  694 */  false,
  /*  695 */  false,
  /*  696 */  false,
  /*  697 */  false,
  /*  698 */  false,
  /*  699 */  false,
  /*  700 */  false,
  /*  701 */  false,
  /*  702 */  false,
  /*  703 */  false,
  /*  704 */  false,
  /*  705 */  false,
  /*  706 */  false,
  /*  707 */  false,
  /*  708 */  false,
  /*  709 */  false,
  /*  710 */  false,
  /*  711 */  false,
  /*  712 */  false,
  /*  713 */  false,
  /*  714 */  false,
  /*  715 */  false,
  /*  716 */  false,
  /*  717 */  false,
  /*  718 */  false,
  /*  719 */  false,
  /*  720 */  false,
  /*  721 */  false,
  /*  722 */  false,
  /*  723 */  false,
  /*  724 */  false,
  /*  725 */  false,
  /*  726 */  false,
  /*  727 */  false,
  /*  728 */  false,
  /*  729 */  false,
  /*  730 */  false,
  /*  731 */  false,
  /*  732 */  false,
  /*  733 */  false,
  /*  734 */  false,
  /*  735 */  false,
  /*  736 */  false,
  /*  737 */  false,
  /*  738 */  false,
  /*  739 */  false,
  /*  740 */  false,
  /*  741 */  false,
  /*  742 */  false,
  /*  743 */  false,
  /*  744 */  false,
  /*  745 */  false,
  /*  746 */  false,
  /*  747 */  false,
  /*  748 */  false,
  /*  749 */  false,
  /*  750 */  false,
  /*  751 */  false,
  /*  752 */  false,
  /*  753 */  false,
  /*  754 */  false,
  /*  755 */  false,
  /*  756 */  false,
  /*  757 */  false,
  /*  758 */  false,
  /*  759 */  false,
  /*  760 */  false,
  /*  761 */  false,
  /*  762 */  false,
  /*  763 */  false,
  /*  764 */  false,
  /*  765 */  false,
  /*  766 */  false,
  /*  767 */  false,
  /*  768 */  false,
  /*  769 */  false,
  /*  770 */  false,
  /*  771 */  false,
  /*  772 */  false,
  /*  773 */  false,
  /*  774 */  false,
  /*  775 */  false,
  /*  776 */  false,
  /*  777 */  false,
  /*  778 */  false,
  /*  779 */  false,
  /*  780 */  false,
  /*  781 */  false,
  /*  782 */  false,
  /*  783 */  false,
  /*  784 */  false,
  /*  785 */  false,
  /*  786 */  false,
  /*  787 */  false,
  /*  788 */  false,
  /*  789 */  false,
  /*  790 */  false,
  /*  791 */  false,
  /*  792 */  false,
  /*  793 */  false,
  /*  794 */  false,
  /*  795 */  false,
  /*  796 */  false,
  /*  797 */  false,
  /*  798 */  false,
  /*  799 */  false,
  /*  800 */  false,
  /*  801 */  false,
  /*  802 */  false,
  /*  803 */  false,
  /*  804 */  false,
  /*  805 */  false,
  /*  806 */  false,
  /*  807 */  false,
  /*  808 */  false,
  /*  809 */  false,
  /*  810 */  false,
  /*  811 */  false,
  /*  812 */  false,
  /*  813 */  false,
  /*  814 */  false,
  /*  815 */  false,
  /*  816 */  false,
  /*  817 */  false,
  /*  818 */  false,
  /*  819 */  false,
  /*  820 */  false,
  /*  821 */  false,
  /*  822 */  false,
  /*  823 */  false,
  /*  824 */  false,
  /*  825 */  false,
  /*  826 */  false,
  /*  827 */  false,
  /*  828 */  false,
  /*  829 */  false,
  /*  830 */  false,
  /*  831 */  false,
  /*  832 */  false,
  /*  833 */  false,
  /*  834 */  false,
  /*  835 */  false,
  /*  836 */  false,
  /*  837 */  false,
  /*  838 */  false,
  /*  839 */  false,
  /*  840 */  false,
  /*  841 */  false,
  /*  842 */  false,
  /*  843 */  false,
  /*  844 */  false,
  /*  845 */  false,
  /*  846 */  false,
  /*  847 */  false,
  /*  848 */  false,
  /*  849 */  false,
  /*  850 */  false,
  /*  851 */  false,
  /*  852 */  false,
  /*  853 */  false,
  /*  854 */  false,
  /*  855 */  false,
  /*  856 */  false,
  /*  857 */  false,
  /*  858 */  false,
  /*  859 */  false,
  /*  860 */  false,
  /*  861 */  false,
  /*  862 */  false,
  /*  863 */  false,
  /*  864 */  false,
  /*  865 */  false,
  /*  866 */  false,
  /*  867 */  false,
  /*  868 */  false,
  /*  869 */  false,
  /*  870 */  false,
  /*  871 */  false,
  /*  872 */  false,
  /*  873 */  false,
  /*  874 */  false,
  /*  875 */  false,
  /*  876 */  false,
  /*  877 */  false,
  /*  878 */  false,
  /*  879 */  false,
  /*  880 */  false,
  /*  881 */  false,
  /*  882 */  false,
  /*  883 */  false,
  /*  884 */  false,
  /*  885 */  false,
  /*  886 */  false,
  /*  887 */  false,
  /*  888 */  false,
  /*  889 */  false,
  /*  890 */  false,
  /*  891 */  false,
  /*  892 */  false,
  /*  893 */  false,
  /*  894 */  false,
  /*  895 */  false,
  /*  896 */  false,
  /*  897 */  false,
  /*  898 */  false,
  /*  899 */  false,
  /*  900 */  false,
  /*  901 */  false,
  /*  902 */  false,
  /*  903 */  false,
  /*  904 */  false,
  /*  905 */  false,
  /*  906 */  false,
  /*  907 */  false,
  /*  908 */  false,
  /*  909 */  false,
  /*  910 */  false,
  /*  911 */  false,
  /*  912 */  false,
  /*  913 */  false,
  /*  914 */  false,
  /*  915 */  false,
  /*  916 */  false,
  /*  917 */  false,
  /*  918 */  false,
  /*  919 */  false,
  /*  920 */  false,
  /*  921 */  false,
  /*  922 */  false,
  /*  923 */  false,
  /*  924 */  false,
  /*  925 */  false,
  /*  926 */  false,
  /*  927 */  false,
  /*  928 */  false,
  /*  929 */  false,
  /*  930 */  false,
  /*  931 */  false,
  /*  932 */  false,
  /*  933 */  false,
  /*  934 */  false,
  /*  935 */  false,
  /*  936 */  false,
  /*  937 */  false,
  /*  938 */  false,
  /*  939 */  false,
  /*  940 */  false,
  /*  941 */  false,
  /*  942 */  false,
  /*  943 */  false,
  /*  944 */  false,
  /*  945 */  false,
  /*  946 */  false,
  /*  947 */  false,
  /*  948 */  false,
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // RegFlags: 8
  0, // VecS: 9
  0, // VecD: 10
  0, // VecX: 11
  0, // VecY: 12
  0, // _last_machine_leaf: 13
  0, // AbsD: 14
  0, // AbsF: 15
  0, // AbsI: 16
  0, // AddD: 17
  0, // AddF: 18
  0, // AddI: 19
  0, // AddL: 20
  0, // AddP: 21
  0, // Allocate: 22
  0, // AllocateArray: 23
  0, // AndI: 24
  0, // AndL: 25
  0, // AryEq: 26
  0, // AtanD: 27
  1, // Binary: 28
  1, // Bool: 29
  0, // BoxLock: 30
  0, // ReverseBytesI: 31
  0, // ReverseBytesL: 32
  0, // ReverseBytesUS: 33
  0, // ReverseBytesS: 34
  0, // CProj: 35
  0, // CallDynamicJava: 36
  0, // CallJava: 37
  0, // CallLeaf: 38
  0, // CallLeafNoFP: 39
  0, // CallRuntime: 40
  0, // CallStaticJava: 41
  0, // CastII: 42
  0, // CastX2P: 43
  0, // CastP2X: 44
  0, // CastPP: 45
  0, // Catch: 46
  0, // CatchProj: 47
  0, // CheckCastPP: 48
  0, // ClearArray: 49
  0, // ConstraintCast: 50
  0, // CMoveD: 51
  0, // CMoveF: 52
  0, // CMoveI: 53
  0, // CMoveL: 54
  0, // CMoveP: 55
  0, // CMoveN: 56
  1, // CmpN: 57
  1, // CmpD: 58
  0, // CmpD3: 59
  1, // CmpF: 60
  0, // CmpF3: 61
  1, // CmpI: 62
  1, // CmpL: 63
  0, // CmpL3: 64
  0, // CmpLTMask: 65
  1, // CmpP: 66
  1, // CmpU: 67
  1, // CmpUL: 68
  0, // CompareAndSwapI: 69
  0, // CompareAndSwapL: 70
  0, // CompareAndSwapP: 71
  0, // CompareAndSwapN: 72
  0, // GetAndAddI: 73
  0, // GetAndAddL: 74
  0, // GetAndSetI: 75
  0, // GetAndSetL: 76
  0, // GetAndSetP: 77
  0, // GetAndSetN: 78
  0, // Con: 79
  0, // ConN: 80
  0, // ConNKlass: 81
  0, // ConD: 82
  0, // ConF: 83
  0, // ConI: 84
  0, // ConL: 85
  0, // ConP: 86
  0, // Conv2B: 87
  0, // ConvD2F: 88
  0, // ConvD2I: 89
  0, // ConvD2L: 90
  0, // ConvF2D: 91
  0, // ConvF2I: 92
  0, // ConvF2L: 93
  0, // ConvI2D: 94
  0, // ConvI2F: 95
  0, // ConvI2L: 96
  0, // ConvL2D: 97
  0, // ConvL2F: 98
  0, // ConvL2I: 99
  0, // CosD: 100
  0, // CountedLoop: 101
  0, // CountedLoopEnd: 102
  0, // CountLeadingZerosI: 103
  0, // CountLeadingZerosL: 104
  0, // CountTrailingZerosI: 105
  0, // CountTrailingZerosL: 106
  0, // CreateEx: 107
  0, // DecodeN: 108
  0, // DecodeNKlass: 109
  0, // DivD: 110
  0, // DivF: 111
  0, // DivI: 112
  0, // DivL: 113
  0, // DivMod: 114
  0, // DivModI: 115
  0, // DivModL: 116
  0, // EncodeISOArray: 117
  0, // EncodeP: 118
  0, // EncodePKlass: 119
  0, // ExpD: 120
  1, // FastLock: 121
  1, // FastUnlock: 122
  0, // Goto: 123
  0, // Halt: 124
  0, // If: 125
  0, // IfFalse: 126
  0, // IfTrue: 127
  0, // Initialize: 128
  0, // JProj: 129
  0, // Jump: 130
  0, // JumpProj: 131
  0, // LShiftI: 132
  0, // LShiftL: 133
  0, // LoadB: 134
  0, // LoadUB: 135
  0, // LoadUS: 136
  0, // LoadD: 137
  0, // LoadD_unaligned: 138
  0, // LoadF: 139
  0, // LoadI: 140
  0, // LoadKlass: 141
  0, // LoadNKlass: 142
  0, // LoadL: 143
  0, // LoadL_unaligned: 144
  0, // LoadPLocked: 145
  0, // LoadP: 146
  0, // LoadN: 147
  0, // LoadRange: 148
  0, // LoadS: 149
  0, // Lock: 150
  0, // LogD: 151
  0, // Log10D: 152
  0, // Loop: 153
  0, // LoopLimit: 154
  0, // Mach: 155
  0, // MachProj: 156
  0, // MaxI: 157
  0, // MemBarAcquire: 158
  0, // LoadFence: 159
  0, // MemBarAcquireLock: 160
  0, // MemBarCPUOrder: 161
  0, // MemBarRelease: 162
  0, // StoreFence: 163
  0, // MemBarReleaseLock: 164
  0, // MemBarVolatile: 165
  0, // MemBarStoreStore: 166
  0, // MergeMem: 167
  0, // MinI: 168
  0, // ModD: 169
  0, // ModF: 170
  0, // ModI: 171
  0, // ModL: 172
  0, // MoveI2F: 173
  0, // MoveF2I: 174
  0, // MoveL2D: 175
  0, // MoveD2L: 176
  0, // MulD: 177
  0, // MulF: 178
  0, // MulHiL: 179
  0, // MulI: 180
  0, // MulL: 181
  0, // Multi: 182
  0, // NegD: 183
  0, // NegF: 184
  0, // NeverBranch: 185
  0, // Opaque1: 186
  0, // Opaque2: 187
  0, // Opaque3: 188
  0, // ProfileBoolean: 189
  0, // OrI: 190
  0, // OrL: 191
  1, // OverflowAddI: 192
  1, // OverflowSubI: 193
  1, // OverflowMulI: 194
  1, // OverflowAddL: 195
  1, // OverflowSubL: 196
  1, // OverflowMulL: 197
  0, // PCTable: 198
  0, // Parm: 199
  0, // PartialSubtypeCheck: 200
  0, // Phi: 201
  0, // PopCountI: 202
  0, // PopCountL: 203
  0, // PowD: 204
  0, // PrefetchAllocation: 205
  0, // PrefetchRead: 206
  0, // PrefetchWrite: 207
  0, // Proj: 208
  0, // RShiftI: 209
  0, // RShiftL: 210
  0, // Region: 211
  0, // Rethrow: 212
  0, // Return: 213
  0, // Root: 214
  0, // RoundDouble: 215
  0, // RoundFloat: 216
  0, // SafePoint: 217
  0, // SafePointScalarObject: 218
  0, // SCMemProj: 219
  0, // SinD: 220
  0, // SqrtD: 221
  0, // Start: 222
  0, // StartOSR: 223
  0, // StoreB: 224
  0, // StoreC: 225
  0, // StoreCM: 226
  0, // StorePConditional: 227
  0, // StoreIConditional: 228
  0, // StoreLConditional: 229
  0, // StoreD: 230
  0, // StoreF: 231
  0, // StoreI: 232
  0, // StoreL: 233
  0, // StoreP: 234
  0, // StoreN: 235
  0, // StoreNKlass: 236
  0, // StrComp: 237
  0, // StrEquals: 238
  0, // StrIndexOf: 239
  0, // SubD: 240
  0, // SubF: 241
  0, // SubI: 242
  0, // SubL: 243
  0, // TailCall: 244
  0, // TailJump: 245
  0, // TanD: 246
  0, // ThreadLocal: 247
  0, // Unlock: 248
  0, // URShiftI: 249
  0, // URShiftL: 250
  0, // XorI: 251
  0, // XorL: 252
  0, // Vector: 253
  0, // AddVB: 254
  0, // AddVS: 255
  0, // AddVI: 256
  0, // AddVL: 257
  0, // AddVF: 258
  0, // AddVD: 259
  0, // SubVB: 260
  0, // SubVS: 261
  0, // SubVI: 262
  0, // SubVL: 263
  0, // SubVF: 264
  0, // SubVD: 265
  0, // MulVS: 266
  0, // MulVI: 267
  0, // MulVF: 268
  0, // MulVD: 269
  0, // DivVF: 270
  0, // DivVD: 271
  0, // LShiftCntV: 272
  0, // RShiftCntV: 273
  0, // LShiftVB: 274
  0, // LShiftVS: 275
  0, // LShiftVI: 276
  0, // LShiftVL: 277
  0, // RShiftVB: 278
  0, // RShiftVS: 279
  0, // RShiftVI: 280
  0, // RShiftVL: 281
  0, // URShiftVB: 282
  0, // URShiftVS: 283
  0, // URShiftVI: 284
  0, // URShiftVL: 285
  0, // AndV: 286
  0, // OrV: 287
  0, // XorV: 288
  0, // LoadVector: 289
  0, // StoreVector: 290
  0, // Pack: 291
  0, // PackB: 292
  0, // PackS: 293
  0, // PackI: 294
  0, // PackL: 295
  0, // PackF: 296
  0, // PackD: 297
  0, // Pack2L: 298
  0, // Pack2D: 299
  0, // ReplicateB: 300
  0, // ReplicateS: 301
  0, // ReplicateI: 302
  0, // ReplicateL: 303
  0, // ReplicateF: 304
  0, // ReplicateD: 305
  0, // Extract: 306
  0, // ExtractB: 307
  0, // ExtractUB: 308
  0, // ExtractC: 309
  0, // ExtractS: 310
  0, // ExtractI: 311
  0, // ExtractL: 312
  0, // ExtractF: 313
  0 // no trailing comma // ExtractD: 314
};
//  The following instructions can cisc-spill



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "R10",
  "R10_H",
  "R11",
  "R11_H",
  "R12",
  "R12_H",
  "R13",
  "R13_H",
  "R14",
  "R14_H",
  "R15",
  "R15_H",
  "R16",
  "R16_H",
  "R17",
  "R17_H",
  "R18",
  "R18_H",
  "R0",
  "R0_H",
  "R1",
  "R1_H",
  "R2",
  "R2_H",
  "R3",
  "R3_H",
  "R4",
  "R4_H",
  "R5",
  "R5_H",
  "R6",
  "R6_H",
  "R7",
  "R7_H",
  "R19",
  "R19_H",
  "R20",
  "R20_H",
  "R21",
  "R21_H",
  "R22",
  "R22_H",
  "R23",
  "R23_H",
  "R24",
  "R24_H",
  "R25",
  "R25_H",
  "R26",
  "R26_H",
  "R27",
  "R27_H",
  "R28",
  "R28_H",
  "R29",
  "R29_H",
  "R30",
  "R30_H",
  "R31",
  "R31_H",
  "V16",
  "V16_H",
  "V16_J",
  "V16_K",
  "V17",
  "V17_H",
  "V17_J",
  "V17_K",
  "V18",
  "V18_H",
  "V18_J",
  "V18_K",
  "V19",
  "V19_H",
  "V19_J",
  "V19_K",
  "V20",
  "V20_H",
  "V20_J",
  "V20_K",
  "V21",
  "V21_H",
  "V21_J",
  "V21_K",
  "V22",
  "V22_H",
  "V22_J",
  "V22_K",
  "V23",
  "V23_H",
  "V23_J",
  "V23_K",
  "V24",
  "V24_H",
  "V24_J",
  "V24_K",
  "V25",
  "V25_H",
  "V25_J",
  "V25_K",
  "V26",
  "V26_H",
  "V26_J",
  "V26_K",
  "V27",
  "V27_H",
  "V27_J",
  "V27_K",
  "V28",
  "V28_H",
  "V28_J",
  "V28_K",
  "V29",
  "V29_H",
  "V29_J",
  "V29_K",
  "V30",
  "V30_H",
  "V30_J",
  "V30_K",
  "V31",
  "V31_H",
  "V31_J",
  "V31_K",
  "V0",
  "V0_H",
  "V0_J",
  "V0_K",
  "V1",
  "V1_H",
  "V1_J",
  "V1_K",
  "V2",
  "V2_H",
  "V2_J",
  "V2_K",
  "V3",
  "V3_H",
  "V3_J",
  "V3_K",
  "V4",
  "V4_H",
  "V4_J",
  "V4_K",
  "V5",
  "V5_H",
  "V5_J",
  "V5_K",
  "V6",
  "V6_H",
  "V6_J",
  "V6_K",
  "V7",
  "V7_H",
  "V7_J",
  "V7_K",
  "V8",
  "V8_H",
  "V8_J",
  "V8_K",
  "V9",
  "V9_H",
  "V9_J",
  "V9_K",
  "V10",
  "V10_H",
  "V10_J",
  "V10_K",
  "V11",
  "V11_H",
  "V11_J",
  "V11_K",
  "V12",
  "V12_H",
  "V12_J",
  "V12_K",
  "V13",
  "V13_H",
  "V13_J",
  "V13_K",
  "V14",
  "V14_H",
  "V14_J",
  "V14_K",
  "V15",
  "V15_H",
  "V15_J",
  "V15_K",
  "RFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	r10->as_VMReg()        ,
	r10->as_VMReg()->next(),
	r11->as_VMReg()        ,
	r11->as_VMReg()->next(),
	r12->as_VMReg()        ,
	r12->as_VMReg()->next(),
	r13->as_VMReg()        ,
	r13->as_VMReg()->next(),
	r14->as_VMReg()        ,
	r14->as_VMReg()->next(),
	r15->as_VMReg()        ,
	r15->as_VMReg()->next(),
	r16->as_VMReg()        ,
	r16->as_VMReg()->next(),
	r17->as_VMReg()        ,
	r17->as_VMReg()->next(),
	r18->as_VMReg()        ,
	r18->as_VMReg()->next(),
	r0->as_VMReg()         ,
	r0->as_VMReg()->next() ,
	r1->as_VMReg()         ,
	r1->as_VMReg()->next() ,
	r2->as_VMReg()         ,
	r2->as_VMReg()->next() ,
	r3->as_VMReg()         ,
	r3->as_VMReg()->next() ,
	r4->as_VMReg()         ,
	r4->as_VMReg()->next() ,
	r5->as_VMReg()         ,
	r5->as_VMReg()->next() ,
	r6->as_VMReg()         ,
	r6->as_VMReg()->next() ,
	r7->as_VMReg()         ,
	r7->as_VMReg()->next() ,
	r19->as_VMReg()        ,
	r19->as_VMReg()->next(),
	r20->as_VMReg()        ,
	r20->as_VMReg()->next(),
	r21->as_VMReg()        ,
	r21->as_VMReg()->next(),
	r22->as_VMReg()        ,
	r22->as_VMReg()->next(),
	r23->as_VMReg()        ,
	r23->as_VMReg()->next(),
	r24->as_VMReg()        ,
	r24->as_VMReg()->next(),
	r25->as_VMReg()        ,
	r25->as_VMReg()->next(),
	r26->as_VMReg()        ,
	r26->as_VMReg()->next(),
	r27->as_VMReg()        ,
	r27->as_VMReg()->next(),
	r28->as_VMReg()        ,
	r28->as_VMReg()->next(),
	r29->as_VMReg()        ,
	r29->as_VMReg()->next(),
	r30->as_VMReg()        ,
	r30->as_VMReg()->next(),
	r31_sp->as_VMReg()     ,
	r31_sp->as_VMReg()->next(),
	v16->as_VMReg()         ,
	v16->as_VMReg()->next() ,
	v16->as_VMReg()->next(2),
	v16->as_VMReg()->next(3),
	v17->as_VMReg()         ,
	v17->as_VMReg()->next() ,
	v17->as_VMReg()->next(2),
	v17->as_VMReg()->next(3),
	v18->as_VMReg()         ,
	v18->as_VMReg()->next() ,
	v18->as_VMReg()->next(2),
	v18->as_VMReg()->next(3),
	v19->as_VMReg()         ,
	v19->as_VMReg()->next() ,
	v19->as_VMReg()->next(2),
	v19->as_VMReg()->next(3),
	v20->as_VMReg()         ,
	v20->as_VMReg()->next() ,
	v20->as_VMReg()->next(2),
	v20->as_VMReg()->next(3),
	v21->as_VMReg()         ,
	v21->as_VMReg()->next() ,
	v21->as_VMReg()->next(2),
	v21->as_VMReg()->next(3),
	v22->as_VMReg()         ,
	v22->as_VMReg()->next() ,
	v22->as_VMReg()->next(2),
	v22->as_VMReg()->next(3),
	v23->as_VMReg()         ,
	v23->as_VMReg()->next() ,
	v23->as_VMReg()->next(2),
	v23->as_VMReg()->next(3),
	v24->as_VMReg()         ,
	v24->as_VMReg()->next() ,
	v24->as_VMReg()->next(2),
	v24->as_VMReg()->next(3),
	v25->as_VMReg()         ,
	v25->as_VMReg()->next() ,
	v25->as_VMReg()->next(2),
	v25->as_VMReg()->next(3),
	v26->as_VMReg()         ,
	v26->as_VMReg()->next() ,
	v26->as_VMReg()->next(2),
	v26->as_VMReg()->next(3),
	v27->as_VMReg()         ,
	v27->as_VMReg()->next() ,
	v27->as_VMReg()->next(2),
	v27->as_VMReg()->next(3),
	v28->as_VMReg()         ,
	v28->as_VMReg()->next() ,
	v28->as_VMReg()->next(2),
	v28->as_VMReg()->next(3),
	v29->as_VMReg()         ,
	v29->as_VMReg()->next() ,
	v29->as_VMReg()->next(2),
	v29->as_VMReg()->next(3),
	v30->as_VMReg()         ,
	v30->as_VMReg()->next() ,
	v30->as_VMReg()->next(2),
	v30->as_VMReg()->next(3),
	v31->as_VMReg()         ,
	v31->as_VMReg()->next() ,
	v31->as_VMReg()->next(2),
	v31->as_VMReg()->next(3),
	v0->as_VMReg()          ,
	v0->as_VMReg()->next()  ,
	v0->as_VMReg()->next(2) ,
	v0->as_VMReg()->next(3) ,
	v1->as_VMReg()          ,
	v1->as_VMReg()->next()  ,
	v1->as_VMReg()->next(2) ,
	v1->as_VMReg()->next(3) ,
	v2->as_VMReg()          ,
	v2->as_VMReg()->next()  ,
	v2->as_VMReg()->next(2) ,
	v2->as_VMReg()->next(3) ,
	v3->as_VMReg()          ,
	v3->as_VMReg()->next()  ,
	v3->as_VMReg()->next(2) ,
	v3->as_VMReg()->next(3) ,
	v4->as_VMReg()          ,
	v4->as_VMReg()->next()  ,
	v4->as_VMReg()->next(2) ,
	v4->as_VMReg()->next(3) ,
	v5->as_VMReg()          ,
	v5->as_VMReg()->next()  ,
	v5->as_VMReg()->next(2) ,
	v5->as_VMReg()->next(3) ,
	v6->as_VMReg()          ,
	v6->as_VMReg()->next()  ,
	v6->as_VMReg()->next(2) ,
	v6->as_VMReg()->next(3) ,
	v7->as_VMReg()          ,
	v7->as_VMReg()->next()  ,
	v7->as_VMReg()->next(2) ,
	v7->as_VMReg()->next(3) ,
	v8->as_VMReg()          ,
	v8->as_VMReg()->next()  ,
	v8->as_VMReg()->next(2) ,
	v8->as_VMReg()->next(3) ,
	v9->as_VMReg()          ,
	v9->as_VMReg()->next()  ,
	v9->as_VMReg()->next(2) ,
	v9->as_VMReg()->next(3) ,
	v10->as_VMReg()         ,
	v10->as_VMReg()->next() ,
	v10->as_VMReg()->next(2),
	v10->as_VMReg()->next(3),
	v11->as_VMReg()         ,
	v11->as_VMReg()->next() ,
	v11->as_VMReg()->next(2),
	v11->as_VMReg()->next(3),
	v12->as_VMReg()         ,
	v12->as_VMReg()->next() ,
	v12->as_VMReg()->next(2),
	v12->as_VMReg()->next(3),
	v13->as_VMReg()         ,
	v13->as_VMReg()->next() ,
	v13->as_VMReg()->next(2),
	v13->as_VMReg()->next(3),
	v14->as_VMReg()         ,
	v14->as_VMReg()->next() ,
	v14->as_VMReg()->next(2),
	v14->as_VMReg()->next(3),
	v15->as_VMReg()         ,
	v15->as_VMReg()->next() ,
	v15->as_VMReg()->next(2),
	v15->as_VMReg()->next(3),
	VMRegImpl::Bad() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\xA',  // R10
  (unsigned char)'\xA',  // R10_H
  (unsigned char)'\xB',  // R11
  (unsigned char)'\xB',  // R11_H
  (unsigned char)'\xC',  // R12
  (unsigned char)'\xC',  // R12_H
  (unsigned char)'\xD',  // R13
  (unsigned char)'\xD',  // R13_H
  (unsigned char)'\xE',  // R14
  (unsigned char)'\xE',  // R14_H
  (unsigned char)'\xF',  // R15
  (unsigned char)'\xF',  // R15_H
  (unsigned char)'\x10',  // R16
  (unsigned char)'\x10',  // R16_H
  (unsigned char)'\x11',  // R17
  (unsigned char)'\x11',  // R17_H
  (unsigned char)'\x12',  // R18
  (unsigned char)'\x12',  // R18_H
  (unsigned char)'\x0',  // R0
  (unsigned char)'\x0',  // R0_H
  (unsigned char)'\x1',  // R1
  (unsigned char)'\x1',  // R1_H
  (unsigned char)'\x2',  // R2
  (unsigned char)'\x2',  // R2_H
  (unsigned char)'\x3',  // R3
  (unsigned char)'\x3',  // R3_H
  (unsigned char)'\x4',  // R4
  (unsigned char)'\x4',  // R4_H
  (unsigned char)'\x5',  // R5
  (unsigned char)'\x5',  // R5_H
  (unsigned char)'\x6',  // R6
  (unsigned char)'\x6',  // R6_H
  (unsigned char)'\x7',  // R7
  (unsigned char)'\x7',  // R7_H
  (unsigned char)'\x13',  // R19
  (unsigned char)'\x13',  // R19_H
  (unsigned char)'\x14',  // R20
  (unsigned char)'\x14',  // R20_H
  (unsigned char)'\x15',  // R21
  (unsigned char)'\x15',  // R21_H
  (unsigned char)'\x16',  // R22
  (unsigned char)'\x16',  // R22_H
  (unsigned char)'\x17',  // R23
  (unsigned char)'\x17',  // R23_H
  (unsigned char)'\x18',  // R24
  (unsigned char)'\x18',  // R24_H
  (unsigned char)'\x19',  // R25
  (unsigned char)'\x19',  // R25_H
  (unsigned char)'\x1A',  // R26
  (unsigned char)'\x1A',  // R26_H
  (unsigned char)'\x1B',  // R27
  (unsigned char)'\x1B',  // R27_H
  (unsigned char)'\x1C',  // R28
  (unsigned char)'\x1C',  // R28_H
  (unsigned char)'\x1D',  // R29
  (unsigned char)'\x1D',  // R29_H
  (unsigned char)'\x1E',  // R30
  (unsigned char)'\x1E',  // R30_H
  (unsigned char)'\x1F',  // R31
  (unsigned char)'\x1F',  // R31_H
  (unsigned char)'\x10',  // V16
  (unsigned char)'\x10',  // V16_H
  (unsigned char)'\x10',  // V16_J
  (unsigned char)'\x10',  // V16_K
  (unsigned char)'\x11',  // V17
  (unsigned char)'\x11',  // V17_H
  (unsigned char)'\x11',  // V17_J
  (unsigned char)'\x11',  // V17_K
  (unsigned char)'\x12',  // V18
  (unsigned char)'\x12',  // V18_H
  (unsigned char)'\x12',  // V18_J
  (unsigned char)'\x12',  // V18_K
  (unsigned char)'\x13',  // V19
  (unsigned char)'\x13',  // V19_H
  (unsigned char)'\x13',  // V19_J
  (unsigned char)'\x13',  // V19_K
  (unsigned char)'\x14',  // V20
  (unsigned char)'\x14',  // V20_H
  (unsigned char)'\x14',  // V20_J
  (unsigned char)'\x14',  // V20_K
  (unsigned char)'\x15',  // V21
  (unsigned char)'\x15',  // V21_H
  (unsigned char)'\x15',  // V21_J
  (unsigned char)'\x15',  // V21_K
  (unsigned char)'\x16',  // V22
  (unsigned char)'\x16',  // V22_H
  (unsigned char)'\x16',  // V22_J
  (unsigned char)'\x16',  // V22_K
  (unsigned char)'\x17',  // V23
  (unsigned char)'\x17',  // V23_H
  (unsigned char)'\x17',  // V23_J
  (unsigned char)'\x17',  // V23_K
  (unsigned char)'\x18',  // V24
  (unsigned char)'\x18',  // V24_H
  (unsigned char)'\x18',  // V24_J
  (unsigned char)'\x18',  // V24_K
  (unsigned char)'\x19',  // V25
  (unsigned char)'\x19',  // V25_H
  (unsigned char)'\x19',  // V25_J
  (unsigned char)'\x19',  // V25_K
  (unsigned char)'\x1A',  // V26
  (unsigned char)'\x1A',  // V26_H
  (unsigned char)'\x1A',  // V26_J
  (unsigned char)'\x1A',  // V26_K
  (unsigned char)'\x1B',  // V27
  (unsigned char)'\x1B',  // V27_H
  (unsigned char)'\x1B',  // V27_J
  (unsigned char)'\x1B',  // V27_K
  (unsigned char)'\x1C',  // V28
  (unsigned char)'\x1C',  // V28_H
  (unsigned char)'\x1C',  // V28_J
  (unsigned char)'\x1C',  // V28_K
  (unsigned char)'\x1D',  // V29
  (unsigned char)'\x1D',  // V29_H
  (unsigned char)'\x1D',  // V29_J
  (unsigned char)'\x1D',  // V29_K
  (unsigned char)'\x1E',  // V30
  (unsigned char)'\x1E',  // V30_H
  (unsigned char)'\x1E',  // V30_J
  (unsigned char)'\x1E',  // V30_K
  (unsigned char)'\x1F',  // V31
  (unsigned char)'\x1F',  // V31_H
  (unsigned char)'\x1F',  // V31_J
  (unsigned char)'\x1F',  // V31_K
  (unsigned char)'\x0',  // V0
  (unsigned char)'\x0',  // V0_H
  (unsigned char)'\x0',  // V0_J
  (unsigned char)'\x0',  // V0_K
  (unsigned char)'\x1',  // V1
  (unsigned char)'\x1',  // V1_H
  (unsigned char)'\x1',  // V1_J
  (unsigned char)'\x1',  // V1_K
  (unsigned char)'\x2',  // V2
  (unsigned char)'\x2',  // V2_H
  (unsigned char)'\x2',  // V2_J
  (unsigned char)'\x2',  // V2_K
  (unsigned char)'\x3',  // V3
  (unsigned char)'\x3',  // V3_H
  (unsigned char)'\x3',  // V3_J
  (unsigned char)'\x3',  // V3_K
  (unsigned char)'\x4',  // V4
  (unsigned char)'\x4',  // V4_H
  (unsigned char)'\x4',  // V4_J
  (unsigned char)'\x4',  // V4_K
  (unsigned char)'\x5',  // V5
  (unsigned char)'\x5',  // V5_H
  (unsigned char)'\x5',  // V5_J
  (unsigned char)'\x5',  // V5_K
  (unsigned char)'\x6',  // V6
  (unsigned char)'\x6',  // V6_H
  (unsigned char)'\x6',  // V6_J
  (unsigned char)'\x6',  // V6_K
  (unsigned char)'\x7',  // V7
  (unsigned char)'\x7',  // V7_H
  (unsigned char)'\x7',  // V7_J
  (unsigned char)'\x7',  // V7_K
  (unsigned char)'\x8',  // V8
  (unsigned char)'\x8',  // V8_H
  (unsigned char)'\x8',  // V8_J
  (unsigned char)'\x8',  // V8_K
  (unsigned char)'\x9',  // V9
  (unsigned char)'\x9',  // V9_H
  (unsigned char)'\x9',  // V9_J
  (unsigned char)'\x9',  // V9_K
  (unsigned char)'\xA',  // V10
  (unsigned char)'\xA',  // V10_H
  (unsigned char)'\xA',  // V10_J
  (unsigned char)'\xA',  // V10_K
  (unsigned char)'\xB',  // V11
  (unsigned char)'\xB',  // V11_H
  (unsigned char)'\xB',  // V11_J
  (unsigned char)'\xB',  // V11_K
  (unsigned char)'\xC',  // V12
  (unsigned char)'\xC',  // V12_H
  (unsigned char)'\xC',  // V12_J
  (unsigned char)'\xC',  // V12_K
  (unsigned char)'\xD',  // V13
  (unsigned char)'\xD',  // V13_H
  (unsigned char)'\xD',  // V13_J
  (unsigned char)'\xD',  // V13_K
  (unsigned char)'\xE',  // V14
  (unsigned char)'\xE',  // V14_H
  (unsigned char)'\xE',  // V14_J
  (unsigned char)'\xE',  // V14_K
  (unsigned char)'\xF',  // V15
  (unsigned char)'\xF',  // V15_H
  (unsigned char)'\xF',  // V15_J
  (unsigned char)'\xF',  // V15_K
  (unsigned char)'\x20' // no trailing comma  // RFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone(Compile* C) const {
  return  new (C) labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone(Compile* C) const {
  return  new (C) methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *iRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *iRegINoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG32_mask();
}

const RegMask *iRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *iRegLNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG_mask();
}

const RegMask *iRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *iRegPNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_PTR_REG_mask();
}

const RegMask *iRegP_R0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R0_REG_mask();
}

const RegMask *iRegP_R1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R1_REG_mask();
}

const RegMask *iRegP_R2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R2_REG_mask();
}

const RegMask *iRegP_R3Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R3_REG_mask();
}

const RegMask *iRegP_R4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R4_REG_mask();
}

const RegMask *iRegP_R5Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R5_REG_mask();
}

const RegMask *iRegP_R10Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R10_REG_mask();
}

const RegMask *iRegL_R11Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &R11_REG_mask();
}

const RegMask *iRegP_FPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_REG_mask();
}

const RegMask *iRegI_R0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R0_REG_mask();
}

const RegMask *iRegI_R2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R2_REG_mask();
}

const RegMask *iRegI_R3Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R3_REG_mask();
}

const RegMask *iRegI_R4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_R4_REG_mask();
}

const RegMask *iRegNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *iRegNNoSpOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NO_SPECIAL_REG32_mask();
}

const RegMask *iRegIHeapbaseOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &HEAPBASE_REG_mask();
}

const RegMask *vRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_mask();
}

const RegMask *vRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_mask();
}

const RegMask *vecDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORD_REG_mask();
}

const RegMask *vecXOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORX_REG_mask();
}

const RegMask *vRegD_V0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V0_REG_mask();
}

const RegMask *vRegD_V1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V1_REG_mask();
}

const RegMask *vRegD_V2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V2_REG_mask();
}

const RegMask *vRegD_V3Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &V3_REG_mask();
}

const RegMask *rFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *rFlagsRegUOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *inline_cache_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &METHOD_REG_mask();
}

const RegMask *interpreter_method_oop_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &METHOD_REG_mask();
}

const RegMask *thread_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &THREAD_REG_mask();
}

const RegMask *lr_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LR_REG_mask();
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indIndexScaledOffsetIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaledOffsetLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexOffsetI2LOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG32_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaledOffsetI2LOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG32_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaledI2LOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG32_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaledOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &PTR_REG_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indOffIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffI4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffI8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffI16Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffL4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffL8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffL16Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indirectNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indIndexScaledOffsetINOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG32_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaledOffsetLNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG32_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexOffsetI2LNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indIndexScaledOffsetI2LNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indIndexScaledI2LNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indIndexScaledNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG32_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG32_mask();
  case 1: return &ANY_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indOffINOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *indOffLNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG32_mask();
}

const RegMask *thread_anchor_pcOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *iRegL2IOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

//------------------Define members for classes derived from MachNode----------
// Build short branch version of this instruction
MachNode *far_cmpL_branch_signNode::short_branch_version(Compile* C) {
  cmpL_branch_signNode *node = new (C) cmpL_branch_signNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpI_branch_signNode::short_branch_version(Compile* C) {
  cmpI_branch_signNode *node = new (C) cmpI_branch_signNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpL_branch_bitNode::short_branch_version(Compile* C) {
  cmpL_branch_bitNode *node = new (C) cmpL_branch_bitNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *far_cmpI_branch_bitNode::short_branch_version(Compile* C) {
  cmpI_branch_bitNode *node = new (C) cmpI_branch_bitNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode( MachNode* node, Compile* C) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone(C);
  }
}

void overflowMulI_reg_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void overflowMulI_reg_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void overflowMulL_reg_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void overflowMulL_reg_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchConNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchConNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchConUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchConUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpN_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpN_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpP_narrowOop_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpP_narrowOop_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUI_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUI_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_imm0_branchNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_imm0_branchNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_branch_signNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_branch_signNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_branch_signNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_branch_signNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_branch_bitNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_branch_bitNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpI_branch_bitNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpI_branch_bitNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpL_branch_signNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpL_branch_signNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_branch_signNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_branch_signNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(4));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpL_branch_bitNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpL_branch_bitNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void far_cmpI_branch_bitNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void far_cmpI_branch_bitNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(5));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchLoopEndNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchLoopEndNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void branchLoopEndUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void branchLoopEndUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallStaticJavaDirectHandleNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int loadBNode::reloc() const {
  return 1;
}

int loadB2LNode::reloc() const {
  return 1;
}

int loadUBNode::reloc() const {
  return 1;
}

int loadUB2LNode::reloc() const {
  return 1;
}

int loadSNode::reloc() const {
  return 1;
}

int loadS2LNode::reloc() const {
  return 1;
}

int loadUSNode::reloc() const {
  return 1;
}

int loadUS2LNode::reloc() const {
  return 1;
}

int loadINode::reloc() const {
  return 1;
}

int loadI2LNode::reloc() const {
  return 1;
}

int loadUI2LNode::reloc() const {
  return 1;
}

int loadLNode::reloc() const {
  return 1;
}

int loadRangeNode::reloc() const {
  return 1;
}

int loadPNode::reloc() const {
  return 1;
}

int loadNNode::reloc() const {
  return 1;
}

int loadKlassNode::reloc() const {
  return 1;
}

int loadNKlassNode::reloc() const {
  return 1;
}

int loadFNode::reloc() const {
  return 1;
}

int loadDNode::reloc() const {
  return 1;
}

int loadConPNode::reloc() const {
  return 1;
}

int loadConP0Node::reloc() const {
  return 1;
}

int loadConP1Node::reloc() const {
  return 1;
}

int loadConPollPageNode::reloc() const {
  return 1;
}

int loadByteMapBaseNode::reloc() const {
  return 1;
}

int loadConF_packedNode::reloc() const {
  return 2;
}

int loadConFNode::reloc() const {
  return 2;
}

int loadConD_packedNode::reloc() const {
  return 2;
}

int loadConDNode::reloc() const {
  return 2;
}

int storeimmCM0Node::reloc() const {
  return 2;
}

int storeimmCM0_orderedNode::reloc() const {
  return 2;
}

int storeBNode::reloc() const {
  return 3;
}

int storeimmB0Node::reloc() const {
  return 2;
}

int storeCNode::reloc() const {
  return 3;
}

int storeimmC0Node::reloc() const {
  return 2;
}

int storeINode::reloc() const {
  return 3;
}

int storeimmI0Node::reloc() const {
  return 2;
}

int storeLNode::reloc() const {
  return 2;
}

int storeimmL0Node::reloc() const {
  return 2;
}

int storePNode::reloc() const {
  return 2;
}

int storeimmP0Node::reloc() const {
  return 3;
}

int storeNNode::reloc() const {
  return 2;
}

int storeImmN0Node::reloc() const {
  return 2;
}

int storeFNode::reloc() const {
  return 2;
}

int storeDNode::reloc() const {
  return 2;
}

int storeNKlassNode::reloc() const {
  return 2;
}

int prefetchrNode::reloc() const {
  return 1;
}

int prefetchwNode::reloc() const {
  return 1;
}

int prefetchntaNode::reloc() const {
  return 1;
}

int storeB_volatileNode::reloc() const {
  return 1;
}

int storeC_volatileNode::reloc() const {
  return 1;
}

int storeI_volatileNode::reloc() const {
  return 1;
}

int bytes_reverse_intNode::reloc() const {
  return 1;
}

int bytes_reverse_unsigned_shortNode::reloc() const {
  return 1;
}

int bytes_reverse_shortNode::reloc() const {
  return 1;
}

int countLeadingZerosINode::reloc() const {
  return 1;
}

int countTrailingZerosINode::reloc() const {
  return 1;
}

int popCountINode::reloc() const {
  return 1;
}

int popCountI_memNode::reloc() const {
  return 1;
}

int popCountL_memNode::reloc() const {
  return 1;
}

int storePConditionalNode::reloc() const {
  return 1;
}

int get_and_addINode::reloc() const {
  return 1;
}

int get_and_addI_no_resNode::reloc() const {
  return 1;
}

int cmovI_reg_regNode::reloc() const {
  return 2;
}

int cmovUI_reg_regNode::reloc() const {
  return 2;
}

int cmovI_zero_regNode::reloc() const {
  return 1;
}

int cmovUI_zero_regNode::reloc() const {
  return 1;
}

int cmovI_reg_zeroNode::reloc() const {
  return 1;
}

int cmovUI_reg_zeroNode::reloc() const {
  return 1;
}

int cmovP_reg_zeroNode::reloc() const {
  return 1;
}

int cmovUP_reg_zeroNode::reloc() const {
  return 1;
}

int cmovP_zero_regNode::reloc() const {
  return 1;
}

int cmovUP_zero_regNode::reloc() const {
  return 1;
}

int addI_reg_regNode::reloc() const {
  return 2;
}

int addI_reg_immNode::reloc() const {
  return 1;
}

int addP_reg_reg_extNode::reloc() const {
  return 1;
}

int addP_reg_reg_ext_shiftNode::reloc() const {
  return 1;
}

int lshift_extNode::reloc() const {
  return 1;
}

int subI_reg_regNode::reloc() const {
  return 2;
}

int subI_reg_immNode::reloc() const {
  return 1;
}

int negI_regNode::reloc() const {
  return 1;
}

int negL_regNode::reloc() const {
  return 1;
}

int mulINode::reloc() const {
  return 2;
}

int smulINode::reloc() const {
  return 2;
}

int smulI_0Node::reloc() const {
  return 2;
}

int maddINode::reloc() const {
  return 3;
}

int maddI_0Node::reloc() const {
  return 3;
}

int msubINode::reloc() const {
  return 3;
}

int divINode::reloc() const {
  return 2;
}

int signExtractNode::reloc() const {
  return 1;
}

int div2RoundNode::reloc() const {
  return 2;
}

int div2Round_0Node::reloc() const {
  return 2;
}

int modINode::reloc() const {
  return 2;
}

int lShiftI_reg_regNode::reloc() const {
  return 2;
}

int lShiftI_reg_immNode::reloc() const {
  return 1;
}

int urShiftI_reg_regNode::reloc() const {
  return 2;
}

int urShiftI_reg_immNode::reloc() const {
  return 1;
}

int rShiftI_reg_regNode::reloc() const {
  return 2;
}

int rShiftI_reg_immNode::reloc() const {
  return 1;
}

int lShiftL_reg_regNode::reloc() const {
  return 1;
}

int urShiftL_reg_regNode::reloc() const {
  return 1;
}

int rShiftL_reg_regNode::reloc() const {
  return 1;
}

int regI_not_regNode::reloc() const {
  return 1;
}

int AndI_reg_not_regNode::reloc() const {
  return 2;
}

int AndI_reg_not_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_not_regNode::reloc() const {
  return 2;
}

int OrI_reg_not_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_not_regNode::reloc() const {
  return 2;
}

int XorI_reg_not_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_URShift_not_regNode::reloc() const {
  return 2;
}

int AndI_reg_URShift_not_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_RShift_not_regNode::reloc() const {
  return 2;
}

int AndI_reg_RShift_not_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_LShift_not_regNode::reloc() const {
  return 2;
}

int AndI_reg_LShift_not_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_URShift_not_regNode::reloc() const {
  return 2;
}

int XorI_reg_URShift_not_reg_1Node::reloc() const {
  return 2;
}

int XorI_reg_URShift_not_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_URShift_not_reg_2Node::reloc() const {
  return 2;
}

int XorI_reg_RShift_not_regNode::reloc() const {
  return 2;
}

int XorI_reg_RShift_not_reg_1Node::reloc() const {
  return 2;
}

int XorI_reg_RShift_not_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_RShift_not_reg_2Node::reloc() const {
  return 2;
}

int XorI_reg_LShift_not_regNode::reloc() const {
  return 2;
}

int XorI_reg_LShift_not_reg_1Node::reloc() const {
  return 2;
}

int XorI_reg_LShift_not_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_LShift_not_reg_2Node::reloc() const {
  return 2;
}

int OrI_reg_URShift_not_regNode::reloc() const {
  return 2;
}

int OrI_reg_URShift_not_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_RShift_not_regNode::reloc() const {
  return 2;
}

int OrI_reg_RShift_not_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_LShift_not_regNode::reloc() const {
  return 2;
}

int OrI_reg_LShift_not_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_URShift_regNode::reloc() const {
  return 2;
}

int AndI_reg_URShift_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_RShift_regNode::reloc() const {
  return 2;
}

int AndI_reg_RShift_reg_0Node::reloc() const {
  return 2;
}

int AndI_reg_LShift_regNode::reloc() const {
  return 2;
}

int AndI_reg_LShift_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_URShift_regNode::reloc() const {
  return 2;
}

int XorI_reg_URShift_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_RShift_regNode::reloc() const {
  return 2;
}

int XorI_reg_RShift_reg_0Node::reloc() const {
  return 2;
}

int XorI_reg_LShift_regNode::reloc() const {
  return 2;
}

int XorI_reg_LShift_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_URShift_regNode::reloc() const {
  return 2;
}

int OrI_reg_URShift_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_RShift_regNode::reloc() const {
  return 2;
}

int OrI_reg_RShift_reg_0Node::reloc() const {
  return 2;
}

int OrI_reg_LShift_regNode::reloc() const {
  return 2;
}

int OrI_reg_LShift_reg_0Node::reloc() const {
  return 2;
}

int AddI_reg_URShift_regNode::reloc() const {
  return 2;
}

int AddI_reg_URShift_reg_0Node::reloc() const {
  return 2;
}

int AddI_reg_RShift_regNode::reloc() const {
  return 2;
}

int AddI_reg_RShift_reg_0Node::reloc() const {
  return 2;
}

int AddI_reg_LShift_regNode::reloc() const {
  return 2;
}

int AddI_reg_LShift_reg_0Node::reloc() const {
  return 2;
}

int SubI_reg_URShift_regNode::reloc() const {
  return 2;
}

int SubI_reg_RShift_regNode::reloc() const {
  return 2;
}

int SubI_reg_LShift_regNode::reloc() const {
  return 2;
}

int sbfmwINode::reloc() const {
  return 1;
}

int ubfmwINode::reloc() const {
  return 1;
}

int ubfxwINode::reloc() const {
  return 1;
}

int ubfxIConvI2LNode::reloc() const {
  return 1;
}

int extrOrINode::reloc() const {
  return 2;
}

int extrOrI_0Node::reloc() const {
  return 2;
}

int extrAddINode::reloc() const {
  return 2;
}

int extrAddI_0Node::reloc() const {
  return 2;
}

int AddExtINode::reloc() const {
  return 1;
}

int AddExtI_0Node::reloc() const {
  return 1;
}

int SubExtINode::reloc() const {
  return 1;
}

int AddExtI_sxthNode::reloc() const {
  return 2;
}

int AddExtI_sxth_0Node::reloc() const {
  return 2;
}

int AddExtI_sxtbNode::reloc() const {
  return 2;
}

int AddExtI_sxtb_0Node::reloc() const {
  return 2;
}

int AddExtI_uxtbNode::reloc() const {
  return 2;
}

int AddExtI_uxtb_0Node::reloc() const {
  return 2;
}

int AddExtI_uxtb_andNode::reloc() const {
  return 2;
}

int AddExtI_uxtb_and_0Node::reloc() const {
  return 2;
}

int AddExtI_uxth_andNode::reloc() const {
  return 2;
}

int AddExtI_uxth_and_0Node::reloc() const {
  return 2;
}

int SubExtI_uxtb_andNode::reloc() const {
  return 2;
}

int SubExtI_uxth_andNode::reloc() const {
  return 2;
}

int andI_reg_regNode::reloc() const {
  return 2;
}

int andI_reg_immNode::reloc() const {
  return 1;
}

int orI_reg_regNode::reloc() const {
  return 2;
}

int orI_reg_immNode::reloc() const {
  return 1;
}

int xorI_reg_regNode::reloc() const {
  return 2;
}

int xorI_reg_immNode::reloc() const {
  return 1;
}

int convI2L_reg_regNode::reloc() const {
  return 1;
}

int convUI2L_reg_regNode::reloc() const {
  return 1;
}

int convI2BNode::reloc() const {
  return 1;
}

int convI2F_reg_regNode::reloc() const {
  return 1;
}

int convI2D_reg_regNode::reloc() const {
  return 1;
}

int overflowAddI_reg_regNode::reloc() const {
  return 2;
}

int overflowAddI_reg_immNode::reloc() const {
  return 1;
}

int overflowSubI_reg_regNode::reloc() const {
  return 2;
}

int overflowSubI_reg_immNode::reloc() const {
  return 1;
}

int overflowNegI_regNode::reloc() const {
  return 1;
}

int overflowMulI_regNode::reloc() const {
  return 2;
}

int overflowMulI_reg_branchNode::reloc() const {
  return 2;
}

int testP_regNode::reloc() const {
  return 1;
}

int compF_reg_zeroNode::reloc() const {
  return 1;
}

int compD_reg_zeroNode::reloc() const {
  return 1;
}

int compF3_reg_immF0Node::reloc() const {
  return 1;
}

int compD3_reg_immD0Node::reloc() const {
  return 1;
}

int cmpLTMask_reg_regNode::reloc() const {
  return 2;
}

int cmpLTMask_reg_zeroNode::reloc() const {
  return 1;
}

int cmpI_imm0_branchNode::reloc() const {
  return 1;
}

int cmpP_imm0_branchNode::reloc() const {
  return 1;
}

int cmpP_narrowOop_imm0_branchNode::reloc() const {
  return 1;
}

int cmpUI_imm0_branchNode::reloc() const {
  return 1;
}

int cmpI_branch_signNode::reloc() const {
  return 1;
}

int cmpI_branch_bitNode::reloc() const {
  return 1;
}

int far_cmpI_branch_signNode::reloc() const {
  return 1;
}

int far_cmpI_branch_bitNode::reloc() const {
  return 1;
}

int cmpI_andNode::reloc() const {
  return 1;
}

int cmpI_and_regNode::reloc() const {
  return 2;
}

int safePointNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectHandleNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectNode::reloc() const {
  return 1;
}

int CallRuntimeDirectNode::reloc() const {
  return 1;
}

int CallLeafDirectNode::reloc() const {
  return 1;
}

int CallLeafNoFPDirectNode::reloc() const {
  return 1;
}

int TailCalljmpIndNode::reloc() const {
  return 1;
}

int TailjmpIndNode::reloc() const {
  return 1;
}

int RethrowExceptionNode::reloc() const {
  return 1;
}

int RetNode::reloc() const {
  return 1;
}

int partialSubtypeCheckVsZeroNode::reloc() const {
  return 1;
}

int loadV4Node::reloc() const {
  return 1;
}

int loadV8Node::reloc() const {
  return 1;
}

int loadV16Node::reloc() const {
  return 1;
}

int storeV4Node::reloc() const {
  return 2;
}

int storeV8Node::reloc() const {
  return 2;
}

int storeV16Node::reloc() const {
  return 2;
}

int replicate8BNode::reloc() const {
  return 1;
}

int replicate16BNode::reloc() const {
  return 1;
}

int replicate4SNode::reloc() const {
  return 1;
}

int replicate8SNode::reloc() const {
  return 1;
}

int replicate2INode::reloc() const {
  return 1;
}

int replicate4INode::reloc() const {
  return 1;
}

int vshiftcntLNode::reloc() const {
  return 1;
}

int vshiftcntRNode::reloc() const {
  return 1;
}


void loadBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3797 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrsbw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10826 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3803 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrsb, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10843 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadUBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3815 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrb, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10860 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadUB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3815 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrb, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10877 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3821 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrshw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10894 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3827 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrsh, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10911 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadUSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3839 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrh, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10928 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadUS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3839 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrh, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10945 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3851 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10962 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3857 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrsw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10979 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadUI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 3851 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 10997 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3863 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11014 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadRangeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3851 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11031 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3863 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11048 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3851 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11065 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3863 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11082 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3851 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrw, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11099 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3869 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister dst_reg = as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrs, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11116 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3875 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister dst_reg = as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrd, dst_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11133 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4358 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    u_int32_t con = (u_int32_t)opnd_array(1)->constant();
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    if (con == 0) {
      __ movw(dst_reg, zr);
    } else {
      __ movw(dst_reg, con);
    }
  
#line 11155 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 4369 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    u_int64_t con = (u_int64_t)opnd_array(1)->constantL();
    if (con == 0) {
      __ mov(dst_reg, zr);
    } else {
      __ mov(dst_reg, con);
    }
  
#line 11177 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 4380 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL || con == (address)1) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      if (rtype == relocInfo::oop_type) {
        __ movoop(dst_reg, (jobject)con, /*immediate*/true);
      } else if (rtype == relocInfo::metadata_type) {
        __ mov_metadata(dst_reg, (Metadata*)con);
      } else {
        assert(rtype == relocInfo::none, "unexpected reloc type");
	if (con < (address)(uintptr_t)os::vm_page_size()) {
	  __ mov(dst_reg, con);
	} else {
	  unsigned long offset;
	  __ adrp(dst_reg, con, offset);
	  __ add(dst_reg, dst_reg, offset);
	}
      }
    }
  
#line 11213 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 4405 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ mov(dst_reg, zr);
  
#line 11230 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConP1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 4411 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ mov(dst_reg, (u_int64_t)1);
  
#line 11247 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConPollPageNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 4417 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    address page = (address)opnd_array(1)->constant();
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    unsigned long off;
    __ adrp(dst_reg, Address(page, relocInfo::poll_type), off);
    assert(off == 0, "assumed offset == 0");
  
#line 11267 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadByteMapBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 4426 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    __ load_byte_map_base(opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 11283 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 4431 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      assert(rtype == relocInfo::oop_type, "unexpected reloc type");
      __ set_narrow_oop(dst_reg, (jobject)con);
    }
  
#line 11307 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 4444 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ mov(dst_reg, zr);
  
#line 11324 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 4450 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    address con = (address)opnd_array(1)->constant();
    if (con == NULL) {
      ShouldNotReachHere();
    } else {
      relocInfo::relocType rtype = opnd_array(1)->constant_reloc();
      assert(rtype == relocInfo::metadata_type, "unexpected reloc type");
      __ set_narrow_klass(dst_reg, (Klass *)con);
    }
  
#line 11348 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConF_packedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 8236 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmovs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), (double)opnd_array(1)->constantF());
  
#line 11364 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 8254 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ldrs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 11380 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConFNode::eval_constant(Compile* C) {
  {

#line 8255 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"
    _constant = C->constant_table().add(this, opnd_array(1));
#line 11389 "../generated/adfiles/ad_aarch64.cpp"
  }
}
void loadConD_packedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 8267 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmovd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->constantD());
  
#line 11404 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 8284 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ldrd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 11420 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadConDNode::eval_constant(Compile* C) {
  {

#line 8285 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"
    _constant = C->constant_table().add(this, opnd_array(1));
#line 11429 "../generated/adfiles/ad_aarch64.cpp"
  }
}
void storeimmCM0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3905 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::strb, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11446 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeimmCM0_orderedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3911 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    __ membar(Assembler::StoreStore);
    loadStore(_masm, &MacroAssembler::strb, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11465 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3899 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::strb, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11483 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeimmB0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3905 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::strb, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11501 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3918 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::strh, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11519 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeimmC0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3924 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::strh, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11537 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3930 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::strw, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11555 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeimmI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3936 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::strw, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11573 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3942 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // we sometimes get asked to store the stack pointer into the
    // current thread -- we cannot do that directly on AArch64
    if (src_reg == r31_sp) {
      MacroAssembler _masm(&cbuf);
      assert(as_Register(opnd_array(1)->base(ra_,this,idx1)) == rthread, "unexpected store for sp");
      __ mov(rscratch2, sp);
      src_reg = rscratch2;
    }
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::str, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11599 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeimmL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3956 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::str, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11617 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3942 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // we sometimes get asked to store the stack pointer into the
    // current thread -- we cannot do that directly on AArch64
    if (src_reg == r31_sp) {
      MacroAssembler _masm(&cbuf);
      assert(as_Register(opnd_array(1)->base(ra_,this,idx1)) == rthread, "unexpected store for sp");
      __ mov(rscratch2, sp);
      src_reg = rscratch2;
    }
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::str, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11643 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeimmP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3956 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    loadStore(_masm, &MacroAssembler::str, zr, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11661 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3930 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::strw, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11679 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeImmN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 3930 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(R27_enc);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::strw, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11697 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3962 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::strs, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11715 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3968 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::strd, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11733 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3930 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::strw, src_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 11751 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void prefetchrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {

#line 4298 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    if (index == -1) {
      __ prfm(Address(base, disp), PLDL1KEEP);
    } else {
      Register index_reg = as_Register(index);
      if (disp == 0) {
        __ prfm(Address(base, index_reg, Address::lsl(scale)), PLDL1KEEP);
      } else {
        __ lea(rscratch1, Address(base, disp));
	__ prfm(Address(rscratch1, index_reg, Address::lsl(scale)), PLDL1KEEP);
      }
    }
  
#line 11781 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void prefetchwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {

#line 4317 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    if (index == -1) {
      __ prfm(Address(base, disp), PSTL1KEEP);
    } else {
      Register index_reg = as_Register(index);
      if (disp == 0) {
        __ prfm(Address(base, index_reg, Address::lsl(scale)), PSTL1KEEP);
      } else {
        __ lea(rscratch1, Address(base, disp));
	__ prfm(Address(rscratch1, index_reg, Address::lsl(scale)), PSTL1KEEP);
      }
    }
  
#line 11811 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void prefetchntaNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {

#line 4336 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    if (index == -1) {
      __ prfm(Address(base, disp), PSTL1STRM);
    } else {
      Register index_reg = as_Register(index);
      if (disp == 0) {
        __ prfm(Address(base, index_reg, Address::lsl(scale)), PSTL1STRM);
        __ nop();
      } else {
        __ lea(rscratch1, Address(base, disp));
	__ prfm(Address(rscratch1, index_reg, Address::lsl(scale)), PSTL1STRM);
      }
    }
  
#line 11842 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadB_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4087 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    MOV_VOLATILE(dst_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarb);
    __ sxtb(dst_reg, dst_reg);
  
#line 11860 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadB2L_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4087 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    MOV_VOLATILE(dst_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarb);
    __ sxtb(dst_reg, dst_reg);
  
#line 11878 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadUB_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4099 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarb);
  
#line 11894 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadUB2L_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4099 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarb);
  
#line 11910 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadS_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4104 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    MOV_VOLATILE(dst_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarh);
    __ sxthw(dst_reg, dst_reg);
  
#line 11928 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadUS_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4118 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarh);
  
#line 11944 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadUS2L_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4123 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarh);
  
#line 11960 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadS2L_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4111 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    MOV_VOLATILE(dst_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarh);
    __ sxth(dst_reg, dst_reg);
  
#line 11978 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadI_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4133 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarw);
  
#line 11994 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadUI2L_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 4133 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarw);
  
#line 12011 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadL_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4138 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldar);
  
#line 12027 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadP_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4138 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldar);
  
#line 12043 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadN_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4133 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarw);
  
#line 12059 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadF_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4143 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(rscratch1, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldarw);
    __ fmovs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), rscratch1);
  
#line 12076 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadD_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4149 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(rscratch1, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
	     rscratch1, ldar);
    __ fmovd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), rscratch1);
  
#line 12093 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeB_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 4058 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
		 rscratch1, stlrb);
    if (VM_Version::cpu_cpuFeatures() & VM_Version::CPU_DMB_ATOMICS)
      __ dmb(__ ISH);
  
#line 12112 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeC_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 4065 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
		 rscratch1, stlrh);
    if (VM_Version::cpu_cpuFeatures() & VM_Version::CPU_DMB_ATOMICS)
      __ dmb(__ ISH);
  
#line 12131 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeI_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 4072 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
		 rscratch1, stlrw);
    if (VM_Version::cpu_cpuFeatures() & VM_Version::CPU_DMB_ATOMICS)
      __ dmb(__ ISH);
  
#line 12150 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeL_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 4155 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // we sometimes get asked to store the stack pointer into the
    // current thread -- we cannot do that directly on AArch64
    if (src_reg == r31_sp) {
 	MacroAssembler _masm(&cbuf);
      assert(as_Register(opnd_array(1)->base(ra_,this,idx1)) == rthread, "unexpected store for sp");
      __ mov(rscratch2, sp);
      src_reg = rscratch2;
    }
    MOV_VOLATILE(src_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
		 rscratch1, stlr);
    if (VM_Version::cpu_cpuFeatures() & VM_Version::CPU_DMB_ATOMICS)
      __ dmb(__ ISH);
  
#line 12178 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeP_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 4155 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // we sometimes get asked to store the stack pointer into the
    // current thread -- we cannot do that directly on AArch64
    if (src_reg == r31_sp) {
 	MacroAssembler _masm(&cbuf);
      assert(as_Register(opnd_array(1)->base(ra_,this,idx1)) == rthread, "unexpected store for sp");
      __ mov(rscratch2, sp);
      src_reg = rscratch2;
    }
    MOV_VOLATILE(src_reg, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
		 rscratch1, stlr);
    if (VM_Version::cpu_cpuFeatures() & VM_Version::CPU_DMB_ATOMICS)
      __ dmb(__ ISH);
  
#line 12206 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeN_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 4072 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MOV_VOLATILE(as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */), opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
		 rscratch1, stlrw);
    if (VM_Version::cpu_cpuFeatures() & VM_Version::CPU_DMB_ATOMICS)
      __ dmb(__ ISH);
  
#line 12225 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeF_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 4171 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    {
      MacroAssembler _masm(&cbuf);
      FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
      __ fmovs(rscratch2, src_reg);
    }
    MOV_VOLATILE(rscratch2, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
		 rscratch1, stlrw);
    if (VM_Version::cpu_cpuFeatures() & VM_Version::CPU_DMB_ATOMICS)
      __ dmb(__ ISH);
  
#line 12249 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeD_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 4183 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    {
      MacroAssembler _masm(&cbuf);
      FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
      __ fmovd(rscratch2, src_reg);
    }
    MOV_VOLATILE(rscratch2, opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1),
		 rscratch1, stlr);
    if (VM_Version::cpu_cpuFeatures() & VM_Version::CPU_DMB_ATOMICS)
      __ dmb(__ ISH);
  
#line 12273 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void bytes_reverse_intNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8889 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ revw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12289 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void bytes_reverse_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8902 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ rev(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12305 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void bytes_reverse_unsigned_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8915 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ rev16w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12321 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void bytes_reverse_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8929 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ rev16w(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ sbfmw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */), 0U, 15U);
  
#line 12338 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void countLeadingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8945 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ clzw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12354 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void countLeadingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8957 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ clz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 12370 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void countTrailingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8970 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ rbitw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ clzw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 12387 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void countTrailingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 8984 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ rbit(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    __ clz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 12404 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void popCountINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    MacroAssembler _masm(&cbuf);

#line 9006 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ movw(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */); // ensure top 32 bits 0
    __ mov(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cnt(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ addv(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ mov(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0);
  
#line 12425 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void popCountI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    MacroAssembler _masm(&cbuf);

#line 9027 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister tmp_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrs, tmp_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ cnt(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ addv(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ mov(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0);
  
#line 12447 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void popCountLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    MacroAssembler _masm(&cbuf);

#line 9050 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mov(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cnt(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ addv(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ mov(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0);
  
#line 12467 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void popCountL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    MacroAssembler _masm(&cbuf);

#line 9070 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister tmp_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* tmp */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldrd, tmp_reg, opnd_array(1)->opcode(),
               as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
    __ cnt(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ addv(opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T8B, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */);
    __ mov(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_FloatRegister(ra_,this,idx2)/* tmp */, __ T1D, 0);
  
#line 12489 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void load_fenceNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9091 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ membar(Assembler::LoadLoad|Assembler::LoadStore);
  
#line 12505 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void unnecessary_membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9104 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ block_comment("membar_acquire (elided)");
  
#line 12521 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9117 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ block_comment("membar_acquire");
    __ membar(Assembler::LoadLoad|Assembler::LoadStore);
  
#line 12538 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void membar_acquire_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9132 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ block_comment("membar_acquire_lock (elided)");
  
#line 12554 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void store_fenceNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9145 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ membar(Assembler::LoadStore|Assembler::StoreStore);
  
#line 12570 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void unnecessary_membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9158 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ block_comment("membar_release (elided)");
  
#line 12586 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9170 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ block_comment("membar_release");
    __ membar(Assembler::LoadStore|Assembler::StoreStore);
  
#line 12603 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void membar_storestoreNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9183 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ membar(Assembler::StoreStore);
  
#line 12619 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void membar_release_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9195 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ block_comment("membar_release_lock (elided)");
  
#line 12635 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void unnecessary_membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9209 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ block_comment("membar_volatile (elided)");
  
#line 12651 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 9222 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ block_comment("membar_volatile");
    __ membar(Assembler::StoreLoad);
    
#line 12668 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void castX2PNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9239 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ mov(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    }
  
#line 12686 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9254 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ mov(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    }
  
#line 12704 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convP2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9269 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ movw(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 12720 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convN2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9285 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ movw(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 12736 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void encodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9300 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ encode_heap_oop(d, s);
  
#line 12754 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void encodeHeapOop_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9313 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ encode_heap_oop_not_null(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 12770 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void decodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9325 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ decode_heap_oop(d, s);
  
#line 12788 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void decodeHeapOop_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9339 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ decode_heap_oop_not_null(d, s);
  
#line 12806 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void encodeKlass_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9357 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ encode_klass_not_null(dst_reg, src_reg);
  
#line 12824 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void decodeKlass_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9372 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    if (dst_reg != src_reg) {
      __ decode_klass_not_null(dst_reg, src_reg);
    } else {
      __ decode_klass_not_null(dst_reg);
    }
  
#line 12846 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void checkCastPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint checkCastPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castIINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castIINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void loadPLockedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 4197 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    if (index == -1) {
       if (disp != 0) {      
        __ lea(rscratch1, Address(base, disp));
        __ ldaxr(dst_reg, rscratch1);
      } else {
        // TODO
        // should we ever get anything other than this case?
        __ ldaxr(dst_reg, base);
      }
    } else {
      Register index_reg = as_Register(index);
      if (disp == 0) {
        __ lea(rscratch1, Address(base, index_reg, Address::lsl(scale)));
        __ ldaxr(dst_reg, rscratch1);
      } else {
        __ lea(rscratch1, Address(base, disp));
        __ lea(rscratch1, Address(rscratch1, index_reg, Address::lsl(scale)));
	__ ldaxr(dst_reg, rscratch1);
      }
    }
  
#line 12925 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storePConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// heap_top_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4226 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register src_reg = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* newval */);
    Register base = as_Register(opnd_array(1)->base(ra_,this,idx1));
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    if (index == -1) {
       if (disp != 0) {      
        __ lea(rscratch2, Address(base, disp));
        __ stlxr(rscratch1, src_reg, rscratch2);
      } else {
        // TODO
        // should we ever get anything other than this case?
        __ stlxr(rscratch1, src_reg, base);
      }
    } else {
      Register index_reg = as_Register(index);
      if (disp == 0) {
        __ lea(rscratch2, Address(base, index_reg, Address::lsl(scale)));
        __ stlxr(rscratch1, src_reg, rscratch2);
      } else {
        __ lea(rscratch2, Address(base, disp));
        __ lea(rscratch2, Address(rscratch2, index_reg, Address::lsl(scale)));
	__ stlxr(rscratch1, src_reg, rscratch2);
      }
    }
    __ cmpw(rscratch1, zr);
  
#line 12968 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeLConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4275 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ true, /*release*/ true);
  
#line 12988 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeIConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4282 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ true, /*release*/ true);
  
#line 13008 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4263 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ false, /*release*/ true);
  
#line 13028 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4290 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13038 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4256 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ false, /*release*/ true);
  
#line 13058 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4290 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13068 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4256 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ false, /*release*/ true);
  
#line 13088 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4290 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13098 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compareAndSwapNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4263 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ false, /*release*/ true);
  
#line 13118 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4290 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13128 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compareAndSwapIAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4282 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ true, /*release*/ true);
  
#line 13148 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4290 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13158 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compareAndSwapLAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4275 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ true, /*release*/ true);
  
#line 13178 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4290 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13188 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compareAndSwapPAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4275 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::xword, /*acquire*/ true, /*release*/ true);
  
#line 13208 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4290 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13218 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compareAndSwapNAcqNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 4282 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    guarantee(opnd_array(1)->index(ra_,this,idx1) == -1 && opnd_array(1)->disp(ra_,this,idx1) == 0, "impossible encoding");
    __ cmpxchg(as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
               Assembler::word, /*acquire*/ true, /*release*/ true);
  
#line 13238 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4290 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register res_reg = as_Register(opnd_array(0)->reg(ra_,this)/* res */);
    __ cset(res_reg, Assembler::EQ);
  
#line 13248 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_setINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 9683 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_xchgw(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13265 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_setLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 9692 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_xchg(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13282 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_setNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 9701 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_xchgw(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13299 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_setPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newv
  {
    MacroAssembler _masm(&cbuf);

#line 9710 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_xchg(opnd_array(0)->as_Register(ra_,this)/* prev */, opnd_array(2)->as_Register(ra_,this,idx2)/* newv */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13316 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_addLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 9721 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_add(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13333 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_addL_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 9732 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_add(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13350 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_addLiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 9742 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_add(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13367 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_addLi_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 9753 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_add(noreg, opnd_array(2)->constantL(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13384 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_addINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 9763 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_addw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13401 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_addI_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 9774 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_addw(noreg, opnd_array(2)->as_Register(ra_,this,idx2)/* incr */, as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13418 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_addIiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 9784 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_addw(opnd_array(0)->as_Register(ra_,this)/* newval */, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13435 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void get_and_addIi_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// incr
  {
    MacroAssembler _masm(&cbuf);

#line 9795 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ atomic_addw(noreg, opnd_array(2)->constant(), as_Register(opnd_array(1)->base(ra_,this,idx1)));
  
#line 13452 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9820 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13474 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9836 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13496 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovI_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9861 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
             zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13518 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUI_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9877 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
             zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13540 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovI_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 9893 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             zr,
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13562 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUI_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 9909 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             zr,
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13584 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovI_reg_zero_oneNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// one
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 9930 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    // equivalently
    // cset(as_Register($dst$$reg),
    //      negate_condition((Assembler::Condition)$cmp$$cmpcode));
    __ csincw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	     zr,
	     zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13609 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUI_reg_zero_oneNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// one
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 9949 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    // equivalently
    // cset(as_Register($dst$$reg),
    //      negate_condition((Assembler::Condition)$cmp$$cmpcode));
    __ csincw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	     zr,
	     zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13634 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9968 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13656 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9984 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13678 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovL_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 10002 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            zr,
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13700 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUL_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 10018 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            zr,
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13722 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovL_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10034 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
            zr,
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13744 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUL_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10050 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
            zr,
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13766 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10066 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13788 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10082 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13810 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovP_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 10100 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            zr,
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13832 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUP_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 10116 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            zr,
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13854 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovP_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10132 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
            zr,
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13876 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUP_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10148 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ csel(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
            zr,
            (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13898 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovN_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10164 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13920 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUN_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10180 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13942 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovN_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 10198 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             zr,
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13964 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUN_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 10214 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             zr,
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src */),
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 13986 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovN_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10230 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
             zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14008 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUN_zero_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10246 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src */),
             zr,
             (Assembler::Condition)opnd_array(1)->ccode());
  
#line 14030 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10263 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ fcsels(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              cond);
  
#line 14053 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10281 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ fcsels(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              cond);
  
#line 14076 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10299 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ fcseld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              cond);
  
#line 14099 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmovUD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10317 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ fcseld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
              as_FloatRegister(opnd_array(4)->reg(ra_,this,idx4)/* src2 */),
              as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              cond);
  
#line 14122 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10346 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14141 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 4465 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constant();
    // add has primary == 0, subtract has primary == 1
    if ((0x0 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ subw(dst_reg, src_reg, -con);
    } else {
      __ addw(dst_reg, src_reg, con);
    }
  
#line 14167 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addI_reg_imm_i2lNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 4465 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constant();
    // add has primary == 0, subtract has primary == 1
    if ((0x0 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ subw(dst_reg, src_reg, -con);
    } else {
      __ addw(dst_reg, src_reg, con);
    }
  
#line 14193 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10390 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14212 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addP_reg_reg_extNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10405 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw);
  
#line 14231 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addP_reg_reg_lslNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// scale
  {
    MacroAssembler _masm(&cbuf);

#line 10420 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lea(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	   Address(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
		   Address::lsl(opnd_array(3)->constant())));
  
#line 14251 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addP_reg_reg_ext_shiftNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// scale
  {
    MacroAssembler _masm(&cbuf);

#line 10435 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lea(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	   Address(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
		   Address::sxtw(opnd_array(3)->constant())));
  
#line 14271 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void lshift_extNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// scale
  {
    MacroAssembler _masm(&cbuf);

#line 10450 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sbfiz(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
          as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
          opnd_array(2)->constant()& 63, MIN(32, (-opnd_array(2)->constant()) & 63));
  
#line 14290 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addP_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 4479 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constantL();
    // add has primary == 0, subtract has primary == 1
    if ((0x0 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ sub(dst_reg, src_reg, -con);
    } else {
      __ add(dst_reg, src_reg, con);
    }
  
#line 14316 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10484 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14335 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 4479 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constantL();
    // add has primary == 0, subtract has primary == 1
    if ((0x0 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ sub(dst_reg, src_reg, -con);
    } else {
      __ add(dst_reg, src_reg, con);
    }
  
#line 14361 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void subI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10515 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14380 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void subI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 4465 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constant();
    // add has primary == 0, subtract has primary == 1
    if ((0x1 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ subw(dst_reg, src_reg, -con);
    } else {
      __ addw(dst_reg, src_reg, con);
    }
  
#line 14406 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void subL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10547 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14425 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void subL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 4479 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    Register src_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    int32_t con = (int32_t)opnd_array(2)->constantL();
    // add has primary == 0, subtract has primary == 1
    if ((0x1 /*primary()*/)) { con = -con; }
    if (con < 0) {
      __ sub(dst_reg, src_reg, -con);
    } else {
      __ add(dst_reg, src_reg, con);
    }
  
#line 14451 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void negI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10579 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ negw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	    as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 14469 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void negL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10595 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ neg(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	   as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 14487 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void mulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10611 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mulw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14506 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void smulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10626 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ smull(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	     as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
	     as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14525 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void smulI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 10626 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ smull(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	     as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
	     as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */));
  
#line 14544 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void mulLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10643 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mul(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14563 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void mulHiL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10659 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ smulh(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
	     as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14582 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void maddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10676 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ maddw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 14603 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void maddI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 10676 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ maddw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 14624 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void msubINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10692 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ msubw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
             as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 14645 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void maddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10710 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ madd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 14666 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void maddL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 10710 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ madd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src3 */));
  
#line 14687 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void msubLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src3
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10726 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ msub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src3 */));
  
#line 14708 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void divINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 4493 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
   Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
   Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
   Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivl(dst_reg, src1_reg, src2_reg, false, rscratch1);
  
#line 14728 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void signExtractNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// div2
  {
    MacroAssembler _masm(&cbuf);

#line 10752 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lsrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), 31);
  
#line 14746 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void div2RoundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// div2
  {
    MacroAssembler _masm(&cbuf);

#line 10763 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	      as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
	      as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
	      Assembler::LSR, 31);
  
#line 14767 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void div2Round_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// div2
  {
    MacroAssembler _masm(&cbuf);

#line 10763 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	      as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
	      as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
	      Assembler::LSR, 31);
  
#line 14788 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void divLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 4501 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
   Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
   Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
   Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivq(dst_reg, src1_reg, src2_reg, false, rscratch1);
  
#line 14808 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void signExtractLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// div2
  {
    MacroAssembler _masm(&cbuf);

#line 10788 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lsr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), 63);
  
#line 14826 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void div2RoundLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// div2
  {
    MacroAssembler _masm(&cbuf);

#line 10799 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	      as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
	      as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
	      Assembler::LSR, 63);
  
#line 14847 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void div2RoundL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// div2
  {
    MacroAssembler _masm(&cbuf);

#line 10799 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	      as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
	      as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
	      Assembler::LSR, 63);
  
#line 14868 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void modINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 4509 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
   Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
   Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
   Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivl(dst_reg, src1_reg, src2_reg, true, rscratch1);
  
#line 14888 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void modLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 4517 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
   Register dst_reg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
   Register src1_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
   Register src2_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    __ corrected_idivq(dst_reg, src1_reg, src2_reg, true, rscratch1);
  
#line 14908 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void lShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10843 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lslvw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14927 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void lShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10859 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lslw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            opnd_array(2)->constant()& 0x1f);
  
#line 14946 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void urShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10875 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lsrvw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 14965 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void urShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10891 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lsrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            opnd_array(2)->constant()& 0x1f);
  
#line 14984 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void rShiftI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10907 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ asrvw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15003 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void rShiftI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10923 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ asrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            opnd_array(2)->constant()& 0x1f);
  
#line 15022 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void lShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10944 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lslv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15041 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void lShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10960 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lsl(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            opnd_array(2)->constant()& 0x3f);
  
#line 15060 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void urShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10976 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lsrv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15079 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void urShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 10992 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lsr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constant()& 0x3f);
  
#line 15098 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void urShiftP_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 11008 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ lsr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constant()& 0x3f);
  
#line 15117 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void rShiftL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 11024 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ asrv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 15136 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void rShiftL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 11040 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ asr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           opnd_array(2)->constant()& 0x3f);
  
#line 15155 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void regL_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 11058 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              zr,
              Assembler::LSL, 0);
  
#line 15175 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void regI_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 11074 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              zr,
              Assembler::LSL, 0);
  
#line 15195 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 11091 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 15216 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11091 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 15237 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 11108 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 15258 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11108 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 15279 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 11125 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 15300 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11125 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 15321 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 11142 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 15342 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// m1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11142 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 15363 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// m1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11159 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 15384 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 11159 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 15405 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// m1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11176 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL, 0);
  
#line 15426 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// m1
  {
    MacroAssembler _masm(&cbuf);

#line 11176 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL, 0);
  
#line 15447 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11193 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 15470 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11193 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 15493 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11211 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 15516 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11211 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 15539 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11229 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 15562 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11229 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 15585 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11247 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 15608 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11247 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 15631 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11265 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 15654 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11265 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bicw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 15677 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11283 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 15700 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11283 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ bic(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 15723 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11301 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 15746 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11301 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 15769 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11301 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::LSR,
              opnd_array(4)->constant()& 0x1f);
  
#line 15792 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11301 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 15815 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11319 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 15838 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11319 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 15861 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11319 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::LSR,
              opnd_array(4)->constant()& 0x3f);
  
#line 15884 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11319 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 15907 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11337 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 15930 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11337 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 15953 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11337 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::ASR,
              opnd_array(4)->constant()& 0x1f);
  
#line 15976 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11337 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 15999 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11355 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 16022 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11355 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 16045 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11355 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::ASR,
              opnd_array(4)->constant()& 0x3f);
  
#line 16068 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11355 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 16091 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11373 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 16114 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11373 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 16137 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11373 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::LSL,
              opnd_array(4)->constant()& 0x1f);
  
#line 16160 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11373 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eonw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 16183 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11391 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 16206 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_not_reg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11391 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 16229 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src4
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11391 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
              Assembler::LSL,
              opnd_array(4)->constant()& 0x3f);
  
#line 16252 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_not_reg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11391 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eon(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 16275 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11409 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 16298 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11409 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 16321 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_URShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11427 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 16344 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_URShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11427 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 16367 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11445 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 16390 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11445 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 16413 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_RShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11463 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 16436 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_RShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11463 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 16459 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11481 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 16482 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11481 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ornw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 16505 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_LShift_not_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src4
  {
    MacroAssembler _masm(&cbuf);

#line 11499 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 16528 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_LShift_not_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src4
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11499 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orn(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 16551 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11518 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 16573 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11518 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 16595 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11537 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 16617 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11537 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 16639 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11556 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 16661 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11556 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 16683 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11575 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 16705 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11575 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 16727 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11594 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 16749 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndI_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11594 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 16771 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11613 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 16793 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AndL_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11613 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 16815 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11632 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 16837 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11632 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 16859 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11651 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 16881 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11651 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 16903 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11670 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 16925 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11670 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 16947 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11689 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 16969 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11689 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 16991 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11708 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 17013 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorI_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11708 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 17035 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11727 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 17057 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void XorL_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11727 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 17079 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11746 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17101 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11746 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 17123 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11765 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17145 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11765 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 17167 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11784 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17189 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11784 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 17211 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11803 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17233 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11803 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 17255 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11822 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 17277 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrI_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11822 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 17299 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11841 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 17321 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void OrL_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11841 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 17343 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddI_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11860 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17365 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddI_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11860 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x1f);
  
#line 17387 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddL_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11879 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17409 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddL_reg_URShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11879 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSR,
              opnd_array(2)->constant()& 0x3f);
  
#line 17431 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddI_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11898 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17453 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddI_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11898 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x1f);
  
#line 17475 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddL_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11917 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17497 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddL_reg_RShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11917 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::ASR,
              opnd_array(2)->constant()& 0x3f);
  
#line 17519 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddI_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11936 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 17541 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddI_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11936 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x1f);
  
#line 17563 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddL_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11955 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 17585 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddL_reg_LShift_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src3
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 11955 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
              Assembler::LSL,
              opnd_array(2)->constant()& 0x3f);
  
#line 17607 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubI_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11974 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17629 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubL_reg_URShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 11993 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17651 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubI_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 12012 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x1f);
  
#line 17673 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubL_reg_RShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 12031 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::ASR,
              opnd_array(3)->constant()& 0x3f);
  
#line 17695 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubI_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 12050 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x1f);
  
#line 17717 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubL_reg_LShift_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {
    MacroAssembler _masm(&cbuf);

#line 12069 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
              as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
              as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
              Assembler::LSL,
              opnd_array(3)->constant()& 0x3f);
  
#line 17739 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void sbfmLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift_count
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift_count
  {
    MacroAssembler _masm(&cbuf);

#line 12093 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int lshift = opnd_array(2)->constant(), rshift = opnd_array(3)->constant();
    int s = 63 - lshift;
    int r = (rshift - lshift) & 63;
    __ sbfm(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            r, s);
  
#line 17762 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void sbfmwINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift_count
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift_count
  {
    MacroAssembler _masm(&cbuf);

#line 12116 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int lshift = opnd_array(2)->constant(), rshift = opnd_array(3)->constant();
    int s = 31 - lshift;
    int r = (rshift - lshift) & 31;
    __ sbfmw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            r, s);
  
#line 17785 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void ubfmLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift_count
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift_count
  {
    MacroAssembler _masm(&cbuf);

#line 12139 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int lshift = opnd_array(2)->constant(), rshift = opnd_array(3)->constant();
    int s = 63 - lshift;
    int r = (rshift - lshift) & 63;
    __ ubfm(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            r, s);
  
#line 17808 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void ubfmwINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift_count
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift_count
  {
    MacroAssembler _masm(&cbuf);

#line 12162 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int lshift = opnd_array(2)->constant(), rshift = opnd_array(3)->constant();
    int s = 31 - lshift;
    int r = (rshift - lshift) & 31;
    __ ubfmw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            r, s);
  
#line 17831 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void ubfxwINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12181 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int rshift = opnd_array(2)->constant();
    long mask = opnd_array(3)->constant();
    int width = exact_log2(mask+1);
    __ ubfxw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), rshift, width);
  
#line 17853 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void ubfxLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12196 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int rshift = opnd_array(2)->constant();
    long mask = opnd_array(3)->constantL();
    int width = exact_log2(mask+1);
    __ ubfx(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), rshift, width);
  
#line 17875 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void ubfxIConvI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12214 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int rshift = opnd_array(2)->constant();
    long mask = opnd_array(3)->constant();
    int width = exact_log2(mask+1);
    __ ubfx(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), rshift, width);
  
#line 17897 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void extrOrLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12234 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ extr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            opnd_array(4)->constant()& 63);
  
#line 17917 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void extrOrL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    MacroAssembler _masm(&cbuf);

#line 12234 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ extr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
            opnd_array(2)->constant()& 63);
  
#line 17937 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void extrOrINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12249 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ extrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            opnd_array(4)->constant()& 31);
  
#line 17957 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void extrOrI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    MacroAssembler _masm(&cbuf);

#line 12249 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ extrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
            opnd_array(2)->constant()& 31);
  
#line 17977 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void extrAddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12264 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ extr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            opnd_array(4)->constant()& 63);
  
#line 17997 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void extrAddL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    MacroAssembler _masm(&cbuf);

#line 12264 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ extr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
            opnd_array(2)->constant()& 63);
  
#line 18017 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void extrAddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12279 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ extrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src2 */),
            opnd_array(4)->constant()& 31);
  
#line 18037 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void extrAddI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// lshift
  {
    MacroAssembler _masm(&cbuf);

#line 12279 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ extrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */),
            opnd_array(2)->constant()& 31);
  
#line 18057 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void rolL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 12295 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subw(rscratch1, zr, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ rorv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            rscratch1);
    
#line 18076 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void rolI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 12311 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subw(rscratch1, zr, as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    __ rorvw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            rscratch1);
    
#line 18095 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void rorL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 12363 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ rorv(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    
#line 18113 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void rorI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 12378 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ rorvw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
    
#line 18131 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 12429 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw);
   
#line 18149 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12429 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtw);
   
#line 18167 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubExtINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 12442 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw);
   
#line 18185 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_sxthNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12456 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxth);
   
#line 18205 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_sxth_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12456 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxth);
   
#line 18225 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_sxtbNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12469 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtb);
   
#line 18245 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_sxtb_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12469 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtb);
   
#line 18265 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_uxtbNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12482 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 18285 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_uxtb_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12482 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtb);
   
#line 18305 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_sxthNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12495 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxth);
   
#line 18325 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_sxth_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12495 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxth);
   
#line 18345 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_sxtwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12508 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtw);
   
#line 18365 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_sxtw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12508 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtw);
   
#line 18385 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_sxtbNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12521 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::sxtb);
   
#line 18405 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_sxtb_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12521 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::sxtb);
   
#line 18425 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_uxtbNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// lshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rshift
  {
    MacroAssembler _masm(&cbuf);

#line 12534 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 18445 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_uxtb_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lshift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rshift
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12534 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(4)->reg(ra_,this,idx4)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtb);
   
#line 18465 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_uxtb_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12548 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 18484 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_uxtb_and_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12548 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtb);
   
#line 18503 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_uxth_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12561 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth);
   
#line 18522 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtI_uxth_and_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12561 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ addw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxth);
   
#line 18541 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_uxtb_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12574 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 18560 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_uxtb_and_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12574 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtb);
   
#line 18579 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_uxth_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12587 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth);
   
#line 18598 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_uxth_and_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12587 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxth);
   
#line 18617 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_uxtw_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12600 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtw);
   
#line 18636 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void AddExtL_uxtw_and_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 12600 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ add(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(3)->reg(ra_,this,idx3)/* src1 */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src2 */), ext::uxtw);
   
#line 18655 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubExtI_uxtb_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12613 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 18674 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubExtI_uxth_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12626 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth);
   
#line 18693 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubExtL_uxtb_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12639 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtb);
   
#line 18712 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubExtL_uxth_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12652 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxth);
   
#line 18731 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void SubExtL_uxtw_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 12665 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     __ sub(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */), ext::uxtw);
   
#line 18750 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 12683 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fadds(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18769 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void addD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 12698 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ faddd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18788 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void subF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 12713 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fsubs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18807 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void subD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 12728 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fsubd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18826 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void mulF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 12743 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmuls(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18845 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void mulD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 12758 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmuld(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18864 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void divF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 12906 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fdivs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18883 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void divD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 12921 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fdivd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 18902 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void negF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 12936 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fnegs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 18919 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void negD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 12950 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fnegd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 18936 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 12963 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fabss(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 18953 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 12976 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fabsd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 18970 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void sqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 12989 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fsqrtd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 18987 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void sqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13002 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fsqrts(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19004 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void andI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13024 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	    as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
	    as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19023 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void andI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13039 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	    as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
	    (unsigned long)(opnd_array(2)->constant()));
  
#line 19042 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void orI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13056 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19061 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void orI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13071 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned long)(opnd_array(2)->constant()));
  
#line 19080 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void xorI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13088 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19099 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void xorI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13103 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eorw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned long)(opnd_array(2)->constant()));
  
#line 19118 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void andL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13121 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
	    as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
	    as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19137 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void andL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13136 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
            as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            (unsigned long)(opnd_array(2)->constantL()));
  
#line 19156 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void orL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13153 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19175 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void orL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13168 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orr(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           (unsigned long)(opnd_array(2)->constantL()));
  
#line 19194 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void xorL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13185 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 19213 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void xorL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 13200 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
           as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
           (unsigned long)(opnd_array(2)->constantL()));
  
#line 19232 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convI2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13215 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sbfm(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, 0, 31);
  
#line 19248 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convUI2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 13228 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ubfm(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, 0, 31);
  
#line 19265 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convL2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13241 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ movw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19281 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convI2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13258 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmpw(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), zr);
    __ cset(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Assembler::NE);
  
#line 19298 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convP2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13276 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmp(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), zr);
    __ cset(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Assembler::NE);
  
#line 19315 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convD2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13290 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fcvtd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19331 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convF2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13303 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fcvts(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19347 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13316 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fcvtzsw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19363 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convF2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13329 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fcvtzs(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19379 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13342 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ scvtfws(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19395 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convL2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13355 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ scvtfs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19411 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convD2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13368 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fcvtzdw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19427 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13381 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fcvtzd(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19443 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convI2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13394 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ scvtfwd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19459 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void convL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13407 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ scvtfd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19475 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveF2I_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13426 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ldrw(opnd_array(0)->as_Register(ra_,this)/* dst */, Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 19491 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveI2F_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13444 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ldrs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 19507 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveD2L_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13462 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ldr(opnd_array(0)->as_Register(ra_,this)/* dst */, Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 19523 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveL2D_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13480 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ldrd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), Address(sp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 19539 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveF2I_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13498 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ strs(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 19555 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveI2F_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13516 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ strw(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 19571 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveD2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13534 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ strd(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 19587 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveL2D_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13552 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ str(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, Address(sp, opnd_array(0)->disp(ra_,this,0)));
  
#line 19603 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13570 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmovs(opnd_array(0)->as_Register(ra_,this)/* dst */, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19619 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13588 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmovs(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 19635 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13606 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmovd(opnd_array(0)->as_Register(ra_,this)/* dst */, as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 19651 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void MoveL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 13624 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmovd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 19667 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void clearArray_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  {
    MacroAssembler _masm(&cbuf);

#line 13643 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ zero_words(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
  
#line 19684 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void clearArray_imm_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    MacroAssembler _masm(&cbuf);

#line 13658 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ zero_words(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, (u_int64_t)opnd_array(1)->constantL());
  
#line 19702 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowAddI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 13674 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmnw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 19719 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowAddI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 13687 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmnw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 19736 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowAddL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 13700 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmn(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 19753 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowAddL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 13713 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmn(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constantL());
  
#line 19770 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowSubI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 13726 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmpw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 19787 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowSubI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 13739 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmpw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 19804 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowSubL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 13752 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmp(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 19821 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowSubL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 13765 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmp(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constantL());
  
#line 19838 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowNegI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  {
    MacroAssembler _masm(&cbuf);

#line 13778 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmpw(zr, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */);
  
#line 19855 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowNegL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op1
  {
    MacroAssembler _masm(&cbuf);

#line 13791 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmp(zr, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */);
  
#line 19872 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowMulI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 13808 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ smull(rscratch1, opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
    __ subs(zr, rscratch1, rscratch1, ext::sxtw);      // NE => overflow
    __ movw(rscratch1, 0x80000000);                    // Develop 0 (EQ),
    __ cselw(rscratch1, rscratch1, zr, Assembler::NE); // or 0x80000000 (NE)
    __ cmpw(rscratch1, 1);                             // 0x80000000 - 1 => VS
  
#line 19893 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowMulI_reg_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 13830 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ smull(rscratch1, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* op2 */);
    __ subs(zr, rscratch1, rscratch1, ext::sxtw);      // NE => overflow
    __ br(cond == Assembler::VS ? Assembler::NE : Assembler::EQ, *L);
  
#line 19916 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowMulL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 13852 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mul(rscratch1, opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);   // Result bits 0..63
    __ smulh(rscratch2, opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */); // Result bits 64..127
    __ cmp(rscratch2, rscratch1, Assembler::ASR, 31);    // Top is pure sign ext
    __ movw(rscratch1, 0x80000000);                    // Develop 0 (EQ),
    __ cselw(rscratch1, rscratch1, zr, Assembler::NE); // or 0x80000000 (NE)
    __ cmpw(rscratch1, 1);                             // 0x80000000 - 1 => VS
  
#line 19938 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void overflowMulL_reg_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 13876 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    __ mul(rscratch1, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* op2 */);   // Result bits 0..63
    __ smulh(rscratch2, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* op2 */); // Result bits 64..127
    __ cmp(rscratch2, rscratch1, Assembler::ASR, 31);    // Top is pure sign ext
    __ br(cond == Assembler::VS ? Assembler::NE : Assembler::EQ, *L);
  
#line 19962 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compI_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4527 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmpw(reg1, reg2);
  
#line 19981 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compI_reg_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 4534 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int32_t val = opnd_array(2)->constant();
    if (val >= 0) {
      __ subsw(zr, reg, val);
    } else {
      __ addsw(zr, reg, -val);
    }
  
#line 20004 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compI_reg_immIAddSubNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4534 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int32_t val = opnd_array(2)->constant();
    if (val >= 0) {
      __ subsw(zr, reg, val);
    } else {
      __ addsw(zr, reg, -val);
    }
  
#line 20027 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compI_reg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4545 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    u_int32_t val = (u_int32_t)opnd_array(2)->constant();
    __ movw(rscratch1, val);
    __ cmpw(reg1, rscratch1);
  
#line 20047 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compU_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4527 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmpw(reg1, reg2);
  
#line 20066 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compU_reg_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 4534 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int32_t val = opnd_array(2)->constant();
    if (val >= 0) {
      __ subsw(zr, reg, val);
    } else {
      __ addsw(zr, reg, -val);
    }
  
#line 20089 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compU_reg_immIAddSubNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4534 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int32_t val = opnd_array(2)->constant();
    if (val >= 0) {
      __ subsw(zr, reg, val);
    } else {
      __ addsw(zr, reg, -val);
    }
  
#line 20112 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compU_reg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4545 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    u_int32_t val = (u_int32_t)opnd_array(2)->constant();
    __ movw(rscratch1, val);
    __ cmpw(reg1, rscratch1);
  
#line 20132 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4553 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmp(reg1, reg2);
  
#line 20151 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compL_reg_immL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 4560 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int64_t val = opnd_array(2)->constantL();
    if (val >= 0) {
      __ subs(zr, reg, val);
    } else if (val != -val) {
      __ adds(zr, reg, -val);
    } else {
    // aargh, Long.MIN_VALUE is a special case
      __ orr(rscratch1, zr, (u_int64_t)val);
      __ subs(zr, reg, rscratch1);
    }
  
#line 20178 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compL_reg_immLAddSubNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4560 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int64_t val = opnd_array(2)->constantL();
    if (val >= 0) {
      __ subs(zr, reg, val);
    } else if (val != -val) {
      __ adds(zr, reg, -val);
    } else {
    // aargh, Long.MIN_VALUE is a special case
      __ orr(rscratch1, zr, (u_int64_t)val);
      __ subs(zr, reg, rscratch1);
    }
  
#line 20205 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compL_reg_immLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4575 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    u_int64_t val = (u_int64_t)opnd_array(2)->constantL();
    __ mov(rscratch1, val);
    __ cmp(reg1, rscratch1);
  
#line 20225 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compUL_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4553 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmp(reg1, reg2);
  
#line 20244 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compUL_reg_immL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 4560 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int64_t val = opnd_array(2)->constantL();
    if (val >= 0) {
      __ subs(zr, reg, val);
    } else if (val != -val) {
      __ adds(zr, reg, -val);
    } else {
    // aargh, Long.MIN_VALUE is a special case
      __ orr(rscratch1, zr, (u_int64_t)val);
      __ subs(zr, reg, rscratch1);
    }
  
#line 20271 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compUL_reg_immLAddSubNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4560 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    int64_t val = opnd_array(2)->constantL();
    if (val >= 0) {
      __ subs(zr, reg, val);
    } else if (val != -val) {
      __ adds(zr, reg, -val);
    } else {
    // aargh, Long.MIN_VALUE is a special case
      __ orr(rscratch1, zr, (u_int64_t)val);
      __ subs(zr, reg, rscratch1);
    }
  
#line 20298 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compUL_reg_immLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4575 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    u_int64_t val = (u_int64_t)opnd_array(2)->constantL();
    __ mov(rscratch1, val);
    __ cmp(reg1, rscratch1);
  
#line 20318 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compP_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4583 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmp(reg1, reg2);
  
#line 20337 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compN_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 4590 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg1 = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    Register reg2 = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
    __ cmpw(reg1, reg2);
  
#line 20356 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void testP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 4597 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    __ cmp(reg, zr);
  
#line 20374 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void testN_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 4603 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
    __ cmpw(reg, zr);
  
#line 20392 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 14187 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fcmps(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 20409 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compF_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 14201 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fcmps(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), 0.0D);
  
#line 20426 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 14216 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fcmpd(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 20443 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compD_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 14230 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fcmpd(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), 0.0D);
  
#line 20460 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compF3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 14248 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label done;
    FloatRegister s1 = as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister s2 = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    Register d = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ fcmps(s1, s2);
    // installs 0 if EQ else -1
    __ csinvw(d, zr, zr, Assembler::EQ);
    // keeps -1 if less or unordered else installs 1
    __ csnegw(d, d, d, Assembler::LT);
    __ bind(done);
  
#line 20486 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compD3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 14276 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label done;
    FloatRegister s1 = as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    FloatRegister s2 = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */);
    Register d = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ fcmpd(s1, s2);
    // installs 0 if EQ else -1
    __ csinvw(d, zr, zr, Assembler::EQ);
    // keeps -1 if less or unordered else installs 1
    __ csnegw(d, d, d, Assembler::LT);
    __ bind(done);
  
#line 20512 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compF3_reg_immF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 14303 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label done;
    FloatRegister s1 = as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register d = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ fcmps(s1, 0.0D);
    // installs 0 if EQ else -1
    __ csinvw(d, zr, zr, Assembler::EQ);
    // keeps -1 if less or unordered else installs 1
    __ csnegw(d, d, d, Assembler::LT);
    __ bind(done);
  
#line 20537 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void compD3_reg_immD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 14330 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label done;
    FloatRegister s1 = as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
    Register d = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ fcmpd(s1, 0.0D);
    // installs 0 if EQ else -1
    __ csinvw(d, zr, zr, Assembler::EQ);
    // keeps -1 if less or unordered else installs 1
    __ csnegw(d, d, d, Assembler::LT);
    __ bind(done);
  
#line 20562 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpL3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 14359 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmp(opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* src2 */);
    __ csetw(opnd_array(0)->as_Register(ra_,this)/* dst */, Assembler::NE);
    __ cnegw(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(0)->as_Register(ra_,this)/* dst */, Assembler::LT);
  
#line 20581 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpLTMask_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  {
    MacroAssembler _masm(&cbuf);

#line 14380 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmpw(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* p */), as_Register(opnd_array(2)->reg(ra_,this,idx2)/* q */));
    __ csetw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), Assembler::LT);
    __ subw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), zr, as_Register(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 20600 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpLTMask_reg_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 14398 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ asrw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */), as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */), 31);
  
#line 20617 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void minI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 14421 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmpw(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             Assembler::LT);
  
#line 20639 "../generated/adfiles/ad_aarch64.cpp"
  }
}

uint minI_rRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 8);
}

void maxI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 14447 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ cmpw(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
    __ cselw(as_Register(opnd_array(0)->reg(ra_,this)/* dst */),
             as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
             as_Register(opnd_array(2)->reg(ra_,this,idx2)/* src2 */),
             Assembler::GT);
  
#line 20666 "../generated/adfiles/ad_aarch64.cpp"
  }
}

uint maxI_rRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 8, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 8);
}

void branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 4609 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Label *L = opnd_array(1)->label();
    __ b(*L);
  
#line 20688 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void branchConNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 4615 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Label *L = opnd_array(3)->label();
    __ br ((Assembler::Condition)opnd_array(1)->ccode(), *L);
  
#line 20707 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void branchConUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 4621 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Label *L = opnd_array(3)->label();
    __ br ((Assembler::Condition)opnd_array(1)->ccode(), *L);
  
#line 20726 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpI_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 14531 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ)
      __ cbzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 20750 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpL_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 14550 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ)
      __ cbz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 20774 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpP_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 14569 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ)
      __ cbz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 20798 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpN_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 14588 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ)
      __ cbzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 20822 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpP_narrowOop_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 14607 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ)
      __ cbzw(opnd_array(2)->as_Register(ra_,this,idx2)/* oop */, *L);
    else
      __ cbnzw(opnd_array(2)->as_Register(ra_,this,idx2)/* oop */, *L);
  
#line 20846 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpUI_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 14628 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ || cond == Assembler::LS)
      __ cbzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnzw(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 20870 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpUL_imm0_branchNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 14649 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    if (cond == Assembler::EQ || cond == Assembler::LS)
      __ cbz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
    else
      __ cbnz(opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, *L);
  
#line 20894 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpL_branch_signNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 14671 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond =
      ((Assembler::Condition)opnd_array(1)->ccode()== Assembler::LT) ? Assembler::NE : Assembler::EQ;
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, 63, *L);
  
#line 20916 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpI_branch_signNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 14689 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond =
      ((Assembler::Condition)opnd_array(1)->ccode()== Assembler::LT) ? Assembler::NE : Assembler::EQ;
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, 31, *L);
  
#line 20938 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpL_branch_bitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// labl
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 14708 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(5)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    int bit = exact_log2(opnd_array(3)->constantL());
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, bit, *L);
  
#line 20961 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpI_branch_bitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// labl
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 14727 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(5)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    int bit = exact_log2(opnd_array(3)->constant());
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, bit, *L);
  
#line 20984 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void far_cmpL_branch_signNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 14746 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond =
      ((Assembler::Condition)opnd_array(1)->ccode()== Assembler::LT) ? Assembler::NE : Assembler::EQ;
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, 63, *L, /*far*/true);
  
#line 21006 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void far_cmpI_branch_signNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// labl
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 14763 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(4)->label();
    Assembler::Condition cond =
      ((Assembler::Condition)opnd_array(1)->ccode()== Assembler::LT) ? Assembler::NE : Assembler::EQ;
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, 31, *L, /*far*/true);
  
#line 21028 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void far_cmpL_branch_bitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// labl
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 14781 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(5)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    int bit = exact_log2(opnd_array(3)->constantL());
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, bit, *L, /*far*/true);
  
#line 21051 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void far_cmpI_branch_bitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// labl
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 14799 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    Label* L = opnd_array(5)->label();
    Assembler::Condition cond = (Assembler::Condition)opnd_array(1)->ccode();
    int bit = exact_log2(opnd_array(3)->constant());
    __ tbr(cond, opnd_array(2)->as_Register(ra_,this,idx2)/* op1 */, bit, *L, /*far*/true);
  
#line 21074 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpL_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  {
    MacroAssembler _masm(&cbuf);

#line 14817 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ tst(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constantL());
  
#line 21092 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpI_andNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  {
    MacroAssembler _masm(&cbuf);

#line 14830 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ tstw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 21110 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpL_and_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  {
    MacroAssembler _masm(&cbuf);

#line 14841 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ tst(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 21128 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpI_and_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// op3
  {
    MacroAssembler _masm(&cbuf);

#line 14852 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ tstw(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 21146 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void branchLoopEndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 4615 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Label *L = opnd_array(3)->label();
    __ br ((Assembler::Condition)opnd_array(1)->ccode(), *L);
  
#line 21165 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void branchLoopEndUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// lbl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 4621 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Label *L = opnd_array(3)->label();
    __ br ((Assembler::Condition)opnd_array(1)->ccode(), *L);
  
#line 21184 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpFastLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {

#line 4785 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register oop = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* object */);
    Register box = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register disp_hdr = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    Register tmp = as_Register(opnd_array(4)->reg(ra_,this,idx4)/* tmp2 */);
    Label cont;
    Label object_has_monitor;
    Label cas_failed;

    assert_different_registers(oop, box, tmp, disp_hdr);

    // Load markOop from object into displaced_header.
    __ ldr(disp_hdr, Address(oop, oopDesc::mark_offset_in_bytes()));

    // Always do locking in runtime.
    if (EmitSync & 0x01) {
      __ cmp(oop, zr);
      return;
    }
    
    if (UseBiasedLocking && !UseOptoBiasInlining) {
      __ biased_locking_enter(box, oop, disp_hdr, tmp, true, cont);
    }

    // Handle existing monitor
    if ((EmitSync & 0x02) == 0) {
      // we can use AArch64's bit test and branch here but
      // markoopDesc does not define a bit index just the bit value
      // so assert in case the bit pos changes
#     define __monitor_value_log2 1
      assert(markOopDesc::monitor_value == (1 << __monitor_value_log2), "incorrect bit position");
      __ tbnz(disp_hdr, __monitor_value_log2, object_has_monitor);
#     undef __monitor_value_log2
    }

    // Set displaced_header to be (markOop of object | UNLOCK_VALUE).
    __ orr(disp_hdr, disp_hdr, markOopDesc::unlocked_value);

    // Load Compare Value application register.

    // Initialize the box. (Must happen before we update the object mark!)
    __ str(disp_hdr, Address(box, BasicLock::displaced_header_offset_in_bytes()));

    // Compare object markOop with mark and if equal exchange scratch1
    // with object markOop.
    if (UseLSE) {
      __ mov(tmp, disp_hdr);
      __ casal(Assembler::xword, tmp, box, oop);
      __ cmp(tmp, disp_hdr);
      __ br(Assembler::EQ, cont);
    } else {
      Label retry_load;
      if ((VM_Version::cpu_cpuFeatures() & VM_Version::CPU_STXR_PREFETCH))
        __ prfm(Address(oop), PSTL1STRM);
      __ bind(retry_load);
      __ ldaxr(tmp, oop);
      __ cmp(tmp, disp_hdr);
      __ br(Assembler::NE, cas_failed);
      // use stlxr to ensure update is immediately visible
      __ stlxr(tmp, box, oop);
      __ cbzw(tmp, cont);
      __ b(retry_load);
    }

    // Formerly:
    // __ cmpxchgptr(/*oldv=*/disp_hdr,
    //               /*newv=*/box,
    //               /*addr=*/oop,
    //               /*tmp=*/tmp,
    //               cont,
    //               /*fail*/NULL);

    assert(oopDesc::mark_offset_in_bytes() == 0, "offset of _mark is not 0");

    // If the compare-and-exchange succeeded, then we found an unlocked
    // object, will have now locked it will continue at label cont

    __ bind(cas_failed);
    // We did not see an unlocked object so try the fast recursive case.

    // Check if the owner is self by comparing the value in the
    // markOop of object (disp_hdr) with the stack pointer.
    __ mov(rscratch1, sp);
    __ sub(disp_hdr, disp_hdr, rscratch1);
    __ mov(tmp, (address) (~(os::vm_page_size()-1) | markOopDesc::lock_mask_in_place));
    // If condition is true we are cont and hence we can store 0 as the
    // displaced header in the box, which indicates that it is a recursive lock.
    __ ands(tmp/*==0?*/, disp_hdr, tmp);
    __ str(tmp/*==0, perhaps*/, Address(box, BasicLock::displaced_header_offset_in_bytes()));

    // Handle existing monitor.
    if ((EmitSync & 0x02) == 0) {
      __ b(cont);

      __ bind(object_has_monitor);
      // The object's monitor m is unlocked iff m->owner == NULL,
      // otherwise m->owner may contain a thread or a stack address.
      //
      // Try to CAS m->owner from NULL to current thread.
      __ add(tmp, disp_hdr, (ObjectMonitor::owner_offset_in_bytes()-markOopDesc::monitor_value));
      __ mov(disp_hdr, zr);

      if (UseLSE) {
        __ mov(rscratch1, disp_hdr);
        __ casal(Assembler::xword, rscratch1, rthread, tmp);
        __ cmp(rscratch1, disp_hdr);
      } else {
	Label retry_load, fail;
        if ((VM_Version::cpu_cpuFeatures() & VM_Version::CPU_STXR_PREFETCH))
          __ prfm(Address(tmp), PSTL1STRM);
	__ bind(retry_load);
        __ ldaxr(rscratch1, tmp);
	__ cmp(disp_hdr, rscratch1);
	__ br(Assembler::NE, fail);
        // use stlxr to ensure update is immediately visible
	__ stlxr(rscratch1, rthread, tmp);
	__ cbnzw(rscratch1, retry_load);
	__ bind(fail);
      }

      // Label next;
      // __ cmpxchgptr(/*oldv=*/disp_hdr,
      //               /*newv=*/rthread,
      //               /*addr=*/tmp,
      //               /*tmp=*/rscratch1,
      //               /*succeed*/next,
      //               /*fail*/NULL);
      // __ bind(next);

      // store a non-null value into the box.
      __ str(box, Address(box, BasicLock::displaced_header_offset_in_bytes()));

      // PPC port checks the following invariants
      // #ifdef ASSERT
      // bne(flag, cont);
      // We have acquired the monitor, check some invariants.
      // addw(/*monitor=*/tmp, tmp, -ObjectMonitor::owner_offset_in_bytes());
      // Invariant 1: _recursions should be 0.
      // assert(ObjectMonitor::recursions_size_in_bytes() == 8, "unexpected size");
      // assert_mem8_is_zero(ObjectMonitor::recursions_offset_in_bytes(), tmp,
      //                        "monitor->_recursions should be 0", -1);
      // Invariant 2: OwnerIsThread shouldn't be 0.
      // assert(ObjectMonitor::OwnerIsThread_size_in_bytes() == 4, "unexpected size");
      //assert_mem4_isnot_zero(ObjectMonitor::OwnerIsThread_offset_in_bytes(), tmp,
      //                           "monitor->OwnerIsThread shouldn't be 0", -1);
      // #endif
    }

    __ bind(cont);
    // flag == EQ indicates success
    // flag == NE indicates failure

  
#line 21353 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void cmpFastUnlockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {

#line 4943 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register oop = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* object */);
    Register box = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register disp_hdr = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    Register tmp = as_Register(opnd_array(4)->reg(ra_,this,idx4)/* tmp2 */);
    Label cont;
    Label object_has_monitor;
    Label cas_failed;

    assert_different_registers(oop, box, tmp, disp_hdr);

    // Always do locking in runtime.
    if (EmitSync & 0x01) {
      __ cmp(oop, zr); // Oop can't be 0 here => always false.
      return;
    }

    if (UseBiasedLocking && !UseOptoBiasInlining) {
      __ biased_locking_exit(oop, tmp, cont);
    }

    // Find the lock address and load the displaced header from the stack.
    __ ldr(disp_hdr, Address(box, BasicLock::displaced_header_offset_in_bytes()));

    // If the displaced header is 0, we have a recursive unlock.
    __ cmp(disp_hdr, zr);
    __ br(Assembler::EQ, cont);


    // Handle existing monitor.
    if ((EmitSync & 0x02) == 0) {
      __ ldr(tmp, Address(oop, oopDesc::mark_offset_in_bytes()));
      __ tbnz(disp_hdr, exact_log2(markOopDesc::monitor_value), object_has_monitor);
    }

    // Check if it is still a light weight lock, this is is true if we
    // see the stack address of the basicLock in the markOop of the
    // object.

      if (UseLSE) {
        __ mov(tmp, box);
        __ casl(Assembler::xword, tmp, disp_hdr, oop);
        __ cmp(tmp, box);
      } else {
	Label retry_load;
        if ((VM_Version::cpu_cpuFeatures() & VM_Version::CPU_STXR_PREFETCH))
          __ prfm(Address(oop), PSTL1STRM);
	__ bind(retry_load);
	__ ldxr(tmp, oop);
	__ cmp(box, tmp);
	__ br(Assembler::NE, cas_failed);
        // use stlxr to ensure update is immediately visible
	__ stlxr(tmp, disp_hdr, oop);
	__ cbzw(tmp, cont);
	__ b(retry_load);
      }

    // __ cmpxchgptr(/*compare_value=*/box,
    //               /*exchange_value=*/disp_hdr,
    //               /*where=*/oop,
    //               /*result=*/tmp,
    //               cont,
    //               /*cas_failed*/NULL);
    assert(oopDesc::mark_offset_in_bytes() == 0, "offset of _mark is not 0");

    __ bind(cas_failed);

    // Handle existing monitor.
    if ((EmitSync & 0x02) == 0) {
      __ b(cont);

      __ bind(object_has_monitor);
      __ add(tmp, tmp, -markOopDesc::monitor_value); // monitor
      __ ldr(rscratch1, Address(tmp, ObjectMonitor::owner_offset_in_bytes()));
      __ ldr(disp_hdr, Address(tmp, ObjectMonitor::recursions_offset_in_bytes()));
      __ eor(rscratch1, rscratch1, rthread); // Will be 0 if we are the owner.
      __ orr(rscratch1, rscratch1, disp_hdr); // Will be 0 if there are 0 recursions
      __ cmp(rscratch1, zr);
      __ br(Assembler::NE, cont);

      __ ldr(rscratch1, Address(tmp, ObjectMonitor::EntryList_offset_in_bytes()));
      __ ldr(disp_hdr, Address(tmp, ObjectMonitor::cxq_offset_in_bytes()));
      __ orr(rscratch1, rscratch1, disp_hdr); // Will be 0 if both are 0.
      __ cmp(rscratch1, zr);
      __ br(Assembler::NE, cont);
      // need a release store here
      __ lea(tmp, Address(tmp, ObjectMonitor::owner_offset_in_bytes()));
      __ stlr(rscratch1, tmp);
    }

    __ bind(cont);
    // flag == EQ indicates success
    // flag == NE indicates failure
  
#line 21463 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void safePointNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 14947 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ read_polling_page(as_Register(opnd_array(1)->reg(ra_,this,idx1)/* poll */), relocInfo::poll_type);
  
#line 21479 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void CallStaticJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 4645 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);

    address mark = __ pc();
    address addr = (address)opnd_array(1)->method();
    address call;
    if (!_method) {
      // A call to a runtime wrapper, e.g. new, new_typeArray_Java, uncommon_trap.
      call = __ trampoline_call(Address(addr, relocInfo::runtime_call_type), &cbuf);
    } else if (_optimized_virtual) {
      call = __ trampoline_call(Address(addr, relocInfo::opt_virtual_call_type), &cbuf);
    } else {
      call = __ trampoline_call(Address(addr, relocInfo::static_call_type), &cbuf);
    }
    if (call == NULL) {
      ciEnv::current()->record_failure("CodeCache is full"); 
      return;
    }

    if (_method) {
      // Emit stub for static call
      address stub = CompiledStaticCall::emit_to_interp_stub(cbuf, mark);
      if (stub == NULL) {
        ciEnv::current()->record_failure("CodeCache is full"); 
        return;
      }
    }
  
#line 21519 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4720 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      __ call_Unimplemented();
    }
  
#line 21531 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void CallStaticJavaDirectHandleNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// reg_mh_save
  {

#line 4674 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    relocInfo::relocType reloc;

    // RFP is preserved across all calls, even compiled calls.
    // Use it to preserve SP.
    __ mov(rfp, sp);

    address mark = __ pc();
    address addr = (address)opnd_array(1)->method();
    address call;
    if (!_method) {
      // A call to a runtime wrapper, e.g. new, new_typeArray_Java, uncommon_trap.
      call = __ trampoline_call(Address(addr, relocInfo::runtime_call_type), &cbuf);
    } else if (_optimized_virtual) {
      call = __ trampoline_call(Address(addr, relocInfo::opt_virtual_call_type), &cbuf);
    } else {
      call = __ trampoline_call(Address(addr, relocInfo::static_call_type), &cbuf);
    }
    if (call == NULL) {
      ciEnv::current()->record_failure("CodeCache is full"); 
      return;
    }

    if (_method) {
      // Emit stub for static call
      address stub = CompiledStaticCall::emit_to_interp_stub(cbuf, mark);
      if (stub == NULL) {
        ciEnv::current()->record_failure("CodeCache is full"); 
        return;
      }
    }

    // now restore sp
    __ mov(sp, rfp);
  
#line 21579 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4720 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      __ call_Unimplemented();
    }
  
#line 21591 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void CallDynamicJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 4711 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    address call = __ ic_call((address)opnd_array(1)->method());
    if (call == NULL) {
      ciEnv::current()->record_failure("CodeCache is full"); 
      return;
    }
  
#line 21611 "../generated/adfiles/ad_aarch64.cpp"
  }
  {

#line 4720 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      __ call_Unimplemented();
    }
  
#line 21623 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void CallRuntimeDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 4728 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a br (they
    // will be in a reachable segment) otherwise we have to use a blrt
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full"); 
        return;
      }
    } else {
      int gpcnt;
      int fpcnt;
      int rtype;
      getCallInfo(tf(), gpcnt, fpcnt, rtype);
      Label retaddr;
      __ adr(rscratch2, retaddr);
      __ lea(rscratch1, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ stp(zr, rscratch2, Address(__ pre(sp, -2 * wordSize)));
      __ blrt(rscratch1, gpcnt, fpcnt, rtype);
      __ bind(retaddr);
      __ add(sp, sp, 2 * wordSize);
    }
  
#line 21665 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void CallLeafDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 4728 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a br (they
    // will be in a reachable segment) otherwise we have to use a blrt
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full"); 
        return;
      }
    } else {
      int gpcnt;
      int fpcnt;
      int rtype;
      getCallInfo(tf(), gpcnt, fpcnt, rtype);
      Label retaddr;
      __ adr(rscratch2, retaddr);
      __ lea(rscratch1, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ stp(zr, rscratch2, Address(__ pre(sp, -2 * wordSize)));
      __ blrt(rscratch1, gpcnt, fpcnt, rtype);
      __ bind(retaddr);
      __ add(sp, sp, 2 * wordSize);
    }
  
#line 21707 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void CallLeafNoFPDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 4728 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);

    // some calls to generated routines (arraycopy code) are scheduled
    // by C2 as runtime calls. if so we can call them using a br (they
    // will be in a reachable segment) otherwise we have to use a blrt
    // which loads the absolute address into a register.
    address entry = (address)opnd_array(1)->method();
    CodeBlob *cb = CodeCache::find_blob(entry);
    if (cb) {
      address call = __ trampoline_call(Address(entry, relocInfo::runtime_call_type));
      if (call == NULL) {
        ciEnv::current()->record_failure("CodeCache is full"); 
        return;
      }
    } else {
      int gpcnt;
      int fpcnt;
      int rtype;
      getCallInfo(tf(), gpcnt, fpcnt, rtype);
      Label retaddr;
      __ adr(rscratch2, retaddr);
      __ lea(rscratch1, RuntimeAddress(entry));
      // Leave a breadcrumb for JavaFrameAnchor::capture_last_Java_pc()
      __ stp(zr, rscratch2, Address(__ pre(sp, -2 * wordSize)));
      __ blrt(rscratch1, gpcnt, fpcnt, rtype);
      __ bind(retaddr);
      __ add(sp, sp, 2 * wordSize);
    }
  
#line 21749 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void TailCalljmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// method_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 4769 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register target_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* jump_target */);
    __ br(target_reg);
  
#line 21767 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void TailjmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// ex_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 4775 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    Register target_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* jump_target */);
    // exception oop should be in r0
    // ret addr has been popped into lr
    // callee expects it in r3
    __ mov(r3, lr);
    __ br(target_reg);
  
#line 21789 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void CreateExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void RethrowExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 4759 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    __ far_jump(RuntimeAddress(OptoRuntime::rethrow_stub()));
  
#line 21818 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void RetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 4764 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    MacroAssembler _masm(&cbuf);
    __ ret(lr);
  
#line 21834 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void ShouldNotReachHereNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 15149 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    // TODO
    // implement proper trap call here
    __ brk(999);
  
#line 21852 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void partialSubtypeCheckNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  {

#line 4628 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     Register sub_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* sub */);
     Register super_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* super */);
     Register temp_reg = as_Register(R2_enc);
     Register result_reg = as_Register(opnd_array(0)->reg(ra_,this)/* result */);

     Label miss;
     MacroAssembler _masm(&cbuf);
     __ check_klass_subtype_slow_path(sub_reg, super_reg, temp_reg, result_reg,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
     if ((0x1 /*primary()*/)) {
       __ mov(result_reg, zr);
     }
     __ bind(miss);
  
#line 21881 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void partialSubtypeCheckVsZeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 4628 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

     Register sub_reg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* sub */);
     Register super_reg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* super */);
     Register temp_reg = as_Register(R2_enc);
     Register result_reg = as_Register(R5_enc);

     Label miss;
     MacroAssembler _masm(&cbuf);
     __ check_klass_subtype_slow_path(sub_reg, super_reg, temp_reg, result_reg,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
     if ((0x0 /*primary()*/)) {
       __ mov(result_reg, zr);
     }
     __ bind(miss);
  
#line 21911 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void string_compareNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  {
    MacroAssembler _masm(&cbuf);

#line 15203 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R10_enc));
  
#line 21932 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void string_indexofNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  {
    MacroAssembler _masm(&cbuf);

#line 15219 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                      opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */,
                      -1, opnd_array(0)->as_Register(ra_,this)/* result */);
  
#line 21959 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void string_indexof_conNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp3
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp4
  {
    MacroAssembler _masm(&cbuf);

#line 15238 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, zr,
                      opnd_array(5)->as_Register(ra_,this,idx5)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */,
                      opnd_array(7)->as_Register(ra_,this,idx7)/* tmp3 */, opnd_array(8)->as_Register(ra_,this,idx8)/* tmp4 */,
                      icnt2, opnd_array(0)->as_Register(ra_,this)/* result */);
  
#line 21987 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void string_equalsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {
    MacroAssembler _masm(&cbuf);

#line 15256 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ string_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
		      opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      as_Register(R10_enc));
  
#line 22007 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void array_equalsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  {
    MacroAssembler _masm(&cbuf);

#line 15271 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ char_arrays_equals(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                          opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(R10_enc));
  
#line 22025 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void encode_iso_arrayNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  {
    MacroAssembler _masm(&cbuf);

#line 15289 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ encode_iso_array(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
         opnd_array(0)->as_Register(ra_,this)/* result */, as_FloatRegister(V0_enc),  as_FloatRegister(V1_enc),
         as_FloatRegister(V2_enc),  as_FloatRegister(V3_enc));
  
#line 22045 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void tlsLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint tlsLoadPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void loadV4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3881 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister dst_reg = as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, MacroAssembler::S,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 22075 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadV8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3887 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister dst_reg = as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, MacroAssembler::D,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 22092 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void loadV16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3893 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister dst_reg = as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::ldr, dst_reg, MacroAssembler::Q,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 22109 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeV4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3974 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::str, src_reg, MacroAssembler::S,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 22127 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeV8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3980 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::str, src_reg, MacroAssembler::D,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 22145 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void storeV16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 3986 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    FloatRegister src_reg = as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src */);
    loadStore(MacroAssembler(&cbuf), &MacroAssembler::str, src_reg, MacroAssembler::Q,
       opnd_array(1)->opcode(), as_Register(opnd_array(1)->base(ra_,this,idx1)), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1));
  
#line 22163 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 15391 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22179 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 15403 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22195 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate8B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 15416 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B, opnd_array(1)->constant()& 0xff);
  
#line 22211 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate16B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 15428 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B, opnd_array(1)->constant()& 0xff);
  
#line 22227 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 15441 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22243 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 15453 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22259 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 15466 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H, opnd_array(1)->constant()& 0xffff);
  
#line 22275 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 15478 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H, opnd_array(1)->constant()& 0xffff);
  
#line 22291 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 15490 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22307 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 15502 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22323 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 15514 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S, opnd_array(1)->constant());
  
#line 22339 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 15526 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mov(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S, opnd_array(1)->constant());
  
#line 22355 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 15538 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22371 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate2L_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 15550 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
           as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */),
           as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 22389 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 15564 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22406 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 15577 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22423 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void replicate2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 15590 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 22440 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vadd8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15608 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22459 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vadd16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15622 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22478 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vadd4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15637 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22497 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vadd8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15651 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22516 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vadd2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15665 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22535 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vadd4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15679 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22554 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vadd2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15693 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ addv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22573 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vadd2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15707 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fadd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22592 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vadd4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15721 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fadd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22611 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vadd2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15734 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fadd(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22630 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsub8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15751 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22649 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsub16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15765 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22668 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsub4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15780 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22687 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsub8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15794 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22706 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsub2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15808 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22725 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsub4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15822 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22744 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsub2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15836 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ subv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22763 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsub2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15850 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fsub(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22782 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsub4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15864 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fsub(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22801 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsub2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15878 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fsub(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22820 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmul4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15895 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mulv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22839 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmul8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15909 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mulv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22858 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmul2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15923 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mulv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22877 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmul4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15937 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mulv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22896 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmul2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15951 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmul(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22915 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmul4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15965 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmul(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22934 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmul2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15979 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fmul(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 22953 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmla4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 15996 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 22973 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmla8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16010 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 22993 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmla2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16024 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 23013 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmla4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16038 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mlav(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 23033 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmls4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16055 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mlsv(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 23053 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmls8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16069 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mlsv(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 23073 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmls2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16083 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mlsv(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 23093 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vmls4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16097 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ mlsv(as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src1 */),
            as_FloatRegister(opnd_array(3)->reg(ra_,this,idx3)/* src2 */));
  
#line 23113 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vdiv2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16113 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fdiv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23132 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vdiv4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16127 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fdiv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23151 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vdiv2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16141 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ fdiv(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23170 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vand8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16158 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23189 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vand16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16172 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ andr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23208 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vor8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16189 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23227 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vor16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16203 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ orr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23246 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vxor8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16220 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23265 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vxor16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 16234 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src1 */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* src2 */));
  
#line 23284 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vshiftcntLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    MacroAssembler _masm(&cbuf);

#line 16247 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
  
#line 23300 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vshiftcntRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    MacroAssembler _masm(&cbuf);

#line 16257 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ dup(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B, as_Register(opnd_array(1)->reg(ra_,this,idx1)/* cnt */));
    __ negr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B, as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 23317 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16271 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23336 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll8B_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16271 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23355 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16285 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23374 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll16B_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16285 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23393 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16299 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23412 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16312 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23431 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll8B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16326 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 8) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 23456 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll16B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16345 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 8) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 23481 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsra8B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16365 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 8) sh = 7;
    sh = -sh & 7;
    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
  
#line 23502 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsra16B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16380 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 8) sh = 7;
    sh = -sh & 7;
    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
  
#line 23523 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl8B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16396 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 8) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), -sh & 7);
    }
  
#line 23548 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl16B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16415 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 8) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), -sh & 7);
    }
  
#line 23573 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16436 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23592 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll4S_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16436 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23611 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16450 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23630 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll8S_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16450 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23649 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16464 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23668 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16477 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23687 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16491 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 16) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 23712 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16510 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 16) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
    }
  
#line 23737 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsra4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16530 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 16) sh = 15;
    sh = -sh & 15;
    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
  
#line 23758 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsra8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16545 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 16) sh = 15;
    sh = -sh & 15;
    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), sh);
  
#line 23779 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16561 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 16) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4H,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), -sh & 15);
    }
  
#line 23804 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16580 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    int sh = (int)opnd_array(2)->constant()& 31;
    if (sh >= 16) {
      __ eor(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T16B,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    } else {
      __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T8H,
             as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */), -sh & 15);
    }
  
#line 23829 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16600 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23848 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll2I_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16600 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23867 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16614 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23886 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll4I_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16614 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23905 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16627 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23924 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16640 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 23943 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16653 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
           (int)opnd_array(2)->constant()& 31);
  
#line 23962 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16666 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
           (int)opnd_array(2)->constant()& 31);
  
#line 23981 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsra2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16679 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            -(int)opnd_array(2)->constant()& 31);
  
#line 24000 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsra4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16692 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            -(int)opnd_array(2)->constant()& 31);
  
#line 24019 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16705 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            -(int)opnd_array(2)->constant()& 31);
  
#line 24038 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16718 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T4S,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            -(int)opnd_array(2)->constant()& 31);
  
#line 24057 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16732 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 24076 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll2L_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16732 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 24095 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16745 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ushl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            as_FloatRegister(opnd_array(2)->reg(ra_,this,idx2)/* shift */));
  
#line 24114 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsll2L_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16758 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ shl(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
           as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
           (int)opnd_array(2)->constant()& 63);
  
#line 24133 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsra2L_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16771 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ sshr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            -(int)opnd_array(2)->constant()& 63);
  
#line 24152 "../generated/adfiles/ad_aarch64.cpp"
  }
}

void vsrl2L_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 16784 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    __ ushr(as_FloatRegister(opnd_array(0)->reg(ra_,this)/* dst */), __ T2D,
            as_FloatRegister(opnd_array(1)->reg(ra_,this,idx1)/* src */),
            -(int)opnd_array(2)->constant()& 63);
  
#line 24171 "../generated/adfiles/ad_aarch64.cpp"
  }
}

const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmCM0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmCM0_orderedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmB0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmC0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmI0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmL0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeimmP0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmN0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchntaNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2L_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2L_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2L_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2L_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadP_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadN_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadF_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeB_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeC_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeI_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeP_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeN_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeD_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* popCountI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* popCountL_memNode::memory_operand() const { return _opnds[1]; }
const TypePtr *load_fenceNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* load_fenceNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *unnecessary_membar_acquireNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* unnecessary_membar_acquireNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_acquireNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquireNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_acquire_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquire_lockNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *store_fenceNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* store_fenceNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *unnecessary_membar_releaseNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* unnecessary_membar_releaseNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_releaseNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_releaseNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_storestoreNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_storestoreNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_release_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_release_lockNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *unnecessary_membar_volatileNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* unnecessary_membar_volatileNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_volatileNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_volatileNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeIConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapINode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapIAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNAcqNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setINode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setLNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setNNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_setPNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addL_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLiNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addLi_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addINode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addI_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIiNode::memory_operand() const { return _opnds[1]; }
const MachOper* get_and_addIi_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadV4Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadV8Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadV16Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeV4Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeV8Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeV16Node::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // RegFlags
    false,  // VecS
    false,  // VecD
    false,  // VecX
    false,  // VecY
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    false,  // AbsI
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    true ,  // AryEq
    false,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    true ,  // ReverseBytesUS
    true ,  // ReverseBytesS
    false,  // CProj
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    true ,  // CMoveD
    true ,  // CMoveF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    true ,  // CMoveN
    true ,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CmpUL
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    true ,  // CompareAndSwapN
    true ,  // GetAndAddI
    true ,  // GetAndAddL
    true ,  // GetAndSetI
    true ,  // GetAndSetL
    true ,  // GetAndSetP
    true ,  // GetAndSetN
    false,  // Con
    true ,  // ConN
    true ,  // ConNKlass
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    false,  // CosD
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    true ,  // CountLeadingZerosI
    true ,  // CountLeadingZerosL
    true ,  // CountTrailingZerosI
    true ,  // CountTrailingZerosL
    true ,  // CreateEx
    true ,  // DecodeN
    true ,  // DecodeNKlass
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    false,  // DivModI
    false,  // DivModL
    true ,  // EncodeISOArray
    true ,  // EncodeP
    true ,  // EncodePKlass
    false,  // ExpD
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // Goto
    true ,  // Halt
    true ,  // If
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    false,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadUB
    true ,  // LoadUS
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    true ,  // LoadNKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadP
    true ,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    false,  // LogD
    false,  // Log10D
    false,  // Loop
    false,  // LoopLimit
    false,  // Mach
    false,  // MachProj
    true ,  // MaxI
    true ,  // MemBarAcquire
    true ,  // LoadFence
    true ,  // MemBarAcquireLock
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // StoreFence
    true ,  // MemBarReleaseLock
    true ,  // MemBarVolatile
    true ,  // MemBarStoreStore
    false,  // MergeMem
    true ,  // MinI
    false,  // ModD
    false,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    true ,  // MulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    false,  // Opaque1
    false,  // Opaque2
    false,  // Opaque3
    false,  // ProfileBoolean
    true ,  // OrI
    true ,  // OrL
    true ,  // OverflowAddI
    true ,  // OverflowSubI
    true ,  // OverflowMulI
    true ,  // OverflowAddL
    true ,  // OverflowSubL
    true ,  // OverflowMulL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // Phi
    true ,  // PopCountI
    true ,  // PopCountL
    false,  // PowD
    true ,  // PrefetchAllocation
    true ,  // PrefetchRead
    true ,  // PrefetchWrite
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    false,  // RoundDouble
    false,  // RoundFloat
    true ,  // SafePoint
    false,  // SafePointScalarObject
    false,  // SCMemProj
    false,  // SinD
    true ,  // SqrtD
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    true ,  // StoreN
    true ,  // StoreNKlass
    true ,  // StrComp
    true ,  // StrEquals
    true ,  // StrIndexOf
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    false,  // TanD
    true ,  // ThreadLocal
    false,  // Unlock
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    true ,  // AddVB
    true ,  // AddVS
    true ,  // AddVI
    true ,  // AddVL
    true ,  // AddVF
    true ,  // AddVD
    true ,  // SubVB
    true ,  // SubVS
    true ,  // SubVI
    true ,  // SubVL
    true ,  // SubVF
    true ,  // SubVD
    true ,  // MulVS
    true ,  // MulVI
    true ,  // MulVF
    true ,  // MulVD
    true ,  // DivVF
    true ,  // DivVD
    true ,  // LShiftCntV
    true ,  // RShiftCntV
    true ,  // LShiftVB
    true ,  // LShiftVS
    true ,  // LShiftVI
    true ,  // LShiftVL
    true ,  // RShiftVB
    true ,  // RShiftVS
    true ,  // RShiftVI
    true ,  // RShiftVL
    true ,  // URShiftVB
    true ,  // URShiftVS
    true ,  // URShiftVI
    true ,  // URShiftVL
    true ,  // AndV
    true ,  // OrV
    true ,  // XorV
    true ,  // LoadVector
    true ,  // StoreVector
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2L
    false,  // Pack2D
    true ,  // ReplicateB
    true ,  // ReplicateS
    true ,  // ReplicateI
    true ,  // ReplicateL
    true ,  // ReplicateF
    true ,  // ReplicateD
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractUB
    false,  // ExtractC
    false,  // ExtractS
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false   // ExtractD
};


bool Matcher::stack_direction() const { return false; }

int Compile::sync_stack_slots() const { return 2; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 2 +
              round_to((Compile::current()->in_preserve_stack_slots() +
                        Compile::current()->fixed_slots()),
                       stack_alignment_in_slots())); }

uint Compile::in_preserve_stack_slots() { return 4; }

uint Compile::out_preserve_stack_slots() { return SharedRuntime::out_preserve_stack_slots(); }

uint Compile::varargs_C_out_slots_killed() const { return frame::arg_reg_save_area_bytes/BytesPerInt; }

void Matcher::calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length, bool is_outgoing) {

#line 5159 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    // No difference between ingoing/outgoing just pass false
    SharedRuntime::java_calling_convention(sig_bt, regs, length, false);
  
#line 24641 "../generated/adfiles/ad_aarch64.cpp"

}

void Matcher::c_calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length) {

#line 5165 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    // This is obviously always outgoing
    (void) SharedRuntime::c_calling_convention(sig_bt, regs, NULL, length);
  
#line 24652 "../generated/adfiles/ad_aarch64.cpp"

}

OptoRegPair Matcher::return_value(uint ideal_reg, bool is_outgoing) {

#line 5172 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    // TODO do we allow ideal_reg == Op_RegN???
    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      R0_num,                            // Op_RegN
      R0_num,                            // Op_RegI
      R0_num,                            // Op_RegP
      V0_num,                            // Op_RegF
      V0_num,                            // Op_RegD
      R0_num                             // Op_RegL
    };
  
    static const int hi[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      OptoReg::Bad,                       // Op_RegN
      OptoReg::Bad,                      // Op_RegI
      R0_H_num,                          // Op_RegP
      OptoReg::Bad,                      // Op_RegF
      V0_H_num,                          // Op_RegD
      R0_H_num                           // Op_RegL
    };

    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 24688 "../generated/adfiles/ad_aarch64.cpp"

}

OptoRegPair Matcher::c_return_value(uint ideal_reg, bool is_outgoing) {

#line 5172 "/home/abuild/rpmbuild/BUILD/icedtea-3.13.0/openjdk/hotspot/src/cpu/aarch64/vm/aarch64.ad"

    // TODO do we allow ideal_reg == Op_RegN???
    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      R0_num,                            // Op_RegN
      R0_num,                            // Op_RegI
      R0_num,                            // Op_RegP
      V0_num,                            // Op_RegF
      V0_num,                            // Op_RegD
      R0_num                             // Op_RegL
    };
  
    static const int hi[Op_RegL + 1] = { // enum name
      0,                                 // Op_Node
      0,                                 // Op_Set
      OptoReg::Bad,                       // Op_RegN
      OptoReg::Bad,                      // Op_RegI
      R0_H_num,                          // Op_RegP
      OptoReg::Bad,                      // Op_RegF
      V0_H_num,                          // Op_RegD
      R0_H_num                           // Op_RegL
    };

    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 24724 "../generated/adfiles/ad_aarch64.cpp"

}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(R12_num); }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_method_oop_reg() { return OptoReg::Name(R12_num); }

int Matcher::interpreter_method_oop_reg_encode() { return _regEncode[interpreter_method_oop_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(R29_num); }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(R31_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

bool Compile::needs_clone_jvms() { return false; }

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DAARCH64=1
#ifndef AARCH64
#  error "AARCH64 must be defined"
#endif // AARCH64
// Check adlc -D_LP64=1
#ifndef _LP64
#  error "_LP64 must be defined"
#endif // _LP64
